package ch.ige.adr.server;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.CreateException;

import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.PostOrtVO;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.adr.vo.RegadrSelKritVO;
import ch.ige.util.ServiceLocator;
import ch.ige.util.exception.DBIntegFailException;
import ch.ige.util.exception.ServiceLocatorException;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;

/**
 * BusinessDelegate für Adressen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseDelegate.java,v 1.39 2007/07/03 15:20:54 nydegger Exp $
 */
public class AdresseDelegate
{
  private static AdresseDelegate adresseDelegate;
  private AdresseFacade facade;
  private static final Class HOME_CLASS = AdresseFacadeHome.class;

  /**
   * Konstruktor
   * @throws AdresseException
   */
  private AdresseDelegate() throws AdresseException
  {
    try
    {
      AdresseFacadeHome home = (AdresseFacadeHome)
          ServiceLocator.getInstance().getHome(AdresseFacadeHome.JNDI_NAME, HOME_CLASS);
      facade = home.create();
    }
    catch(ServiceLocatorException ex)
    {
      // ServiceLocaterException in Applikations-Exception umwandeln
      throw new AdresseException(ex.toString());
    }
    catch(CreateException ex)
    {
      // CreateException in Applikations-Exception umwandeln
      throw new AdresseException(ex.toString());
    }
    catch(RemoteException ex)
    {
      // RemoteException in Applikations-Exception umwandeln
      throw new AdresseException(ex.toString());
    }

  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * @return AdresseDelegate
   * @throws AdresseException
   */
  public static AdresseDelegate getInstance() throws AdresseException
  {
    if (adresseDelegate == null)
    {
      adresseDelegate = new AdresseDelegate();
    }
    return adresseDelegate;
  }

  /**
   * Gibt die URL der Bagis-Datenbank zurück.
   * @return URL der Bagis-Datenbank
   * @throws Exception
   */
  public String getBagisDbURL() throws Exception
  {
    return facade.getBagisDbURL();
  }

  /**
   * Kreiert eine Versandadresse.
   * @param versadrVO Value Object der Versandadresse
   * @return Adressnummer
   * @throws Exception
   */
  public int createVersadr(final VersadrVO versadrVO) throws Exception
  {
    return facade.createVersadr(versadrVO);
  }

  /**
   * Gibt das Value Object der Versandadresse zurück.
   * @param adrNr Adressnummer
   * @return Value Object der Versandadresse
   * @throws Exception
   */
  public VersadrVO getVersadr(final int adrNr) throws Exception
  {
    return facade.getVersadr(adrNr);
  }

  /**
   * Setzt die Versandadresse auf den Wert des Value Objects.
   * @param versadrVO Versandadresse
   * @throws Exception
   */
  public void setVersadr(final VersadrVO versadrVO) throws Exception
  {
    facade.setVersadr(versadrVO);
  }

  /**
   * Löscht die Versandadresse.
   * @param adrNr Adressnummer
   * @throws Exception
   */
  public void removeVersadr(final int adrNr) throws Exception
  {
    facade.removeVersadr(adrNr);
  }

  /**
   * Sperrt die Versandadresse
   * @param adrNr Adressnummer
   * @param gesperrt true - sperren, false - Sperrung aufheben
   * @param geaendBen Kürzel des Benuzter
   * @throws Exception
   */
  public void blockVersadr(final int adrNr, final boolean gesperrt, final String geaendBen) throws Exception
  {
    facade.blockVersadr(adrNr, gesperrt, geaendBen);
  }

  /**
   * Findet alle Adressen, auf welche die Selektionskriterien zutreffen.
   * @param adresseSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit AdresseAVO-Objekten mit den gefundenen Adressen
   * @throws Exception
   */
  public ArrayList findVersadr(final AdresseSelKritVO adresseSelKritVO) throws Exception
  {
    return facade.findVersadr(adresseSelKritVO);
  }

  /**
   * Liest alle Adr-Nummern aus der Tabelle Adresse
   * @param startAdrNr Anfangs-Adressnummer
   * @return ArrayList mit Integer-Objekten mit den Adressnummern
   * @throws Exception
   */
  public ArrayList getAdrNr(final int startAdrNr) throws Exception
  {
    return facade.getAdrNr(startAdrNr);
  }

  /**
   * Prüft die Existenz einer Adresse.
   * @param adrNr Adressnummer
   * @return true - LäAdresse existiert.
   * @throws Exception
   */
  public boolean existsVersadr(final int adrNr) throws Exception
  {
    return facade.existsVersadr(adrNr);
  }

   /**
   * Kreiert eine Registeradresse.
   * @param regadrVO Value Object der Registeradresse
   * @return Register-Adressnummer
   * @throws Exception
   */
  public int createRegadr(final RegadrVO regadrVO) throws Exception
  {
    return facade.createRegadr(regadrVO);
  }

  /**
   * Kreiert eine Registeradresse und eine Versandadresse.
   * @param regadrVO Value Object der Registeradresse
   * @param versadrVO Value Object der Versandadresse
   * @return Registeradressnummer
   * @throws Exception
   */
  public int createRegadr(final RegadrVO regadrVO, final VersadrVO versadrVO) throws Exception
  {
    return facade.createRegadr(regadrVO, versadrVO);
  }

  /**
   * Gibt das Value Object der Registeradresse zurück.
   * @param regadrNr Register-Adressnummer
   * @return Value Object der Registeradresse
   * @throws Exception
   */
  public RegadrVO getRegadr(final int regadrNr) throws Exception
  {
    return facade.getRegadr(regadrNr);
  }

  /**
   * Gibt die Registeradressnummer zurück.
   * @param regadrExistsVO Registeradresse
   * @return Registeradressnummer
   * @throws Exception
   */
  public int getRegadr(final RegadrExistsVO regadrExistsVO) throws Exception
  {
    return facade.getRegadr(regadrExistsVO);
  }

  /**
   * Setzt die Registeradresse auf den Wert des Value Objects.
   * @param regadrVO Registeradresse
   * @throws DBIntegFailException
   * @throws Exception
   */
  public void setRegadr(final RegadrVO regadrVO) throws Exception
  {
    facade.setRegadr(regadrVO);
  }

  /**
   * Erstellt eine Versandadresse und setzt die Adressnummer in der
   * Registeradresse ein und setzt die Registeradresse auf den Wert
   * des Value Objects.
   * @param regadrVO Registeradresse (wird mutiert!)
   * @param versadrVO Versandadresse (wird erstellt!)
   * @throws Exception
   */
  public void setRegadr(final RegadrVO regadrVO, final VersadrVO versadrVO) throws Exception
  {
    facade.setRegadr(regadrVO, versadrVO);
  }

  /**
   * Löscht die Registeradresse.
   * @param regadrNr Register-Adressnummer
   * @throws DBIntegFailException
   * @throws Exception
   */
  public void removeRegadr(final int regadrNr) throws DBIntegFailException, Exception
  {
    facade.removeRegadr(regadrNr);
  }

  /**
   * Findet alle Registeradressen, auf welche die Selektionskriterien zutreffen.
   * @param regadrSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit RegadrAVO-Objekten mit den gefundenen Registeradresse
   * @throws Exception
   */
  public ArrayList findRegadr(final RegadrSelKritVO regadrSelKritVO) throws Exception
  {
    return facade.findRegadr(regadrSelKritVO);
  }

  /**
   * Findet alle Länder, auf welche die Selektionskriterien zutreffen.
   * @param landCd Ländercode als Selektionskriterium
   * @param landBez Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit LandAVO-Objekten mit den gefundenen Ländern
   * @throws Exception
   */
  public ArrayList findLaender(final String landCd, final String landBez) throws Exception
  {
    return facade.findLaender(landCd, landBez);
  }

  /**
   * Prüft die Existenz eines Ländercodes.
   * @param landCd Ländercode, z.B. CH für Schweiz, F für Frankreich etc.
   * @return true - Ländercode existiert.
   * @throws Exception
   */
  public boolean existsLand(final String landCd) throws Exception
  {
    return facade.existsLand(landCd);
  }

  /**
   * Findet alle BLänder, auf welche die Selektionskriterien zutreffen.
   * @param blandCd Ländercode als Selektionskriterium
   * @param landBezD Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit BlandAVO-Objekten mit den gefundenen BLänder
   * @throws Exception
   */
  public ArrayList findBland(final String blandCd, final String landBezD) throws Exception
  {
    return facade.findBland(blandCd, landBezD);
  }

  /**
   * Prüft die Existenz eines BLändercodes.
   * @param blandCd BLändercode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return true - BLändercode existiert.
   * @throws Exception
   */
  public boolean existsBland(final String blandCd) throws Exception
  {
    return facade.existsBland(blandCd);
  }

  /**
   * Gibt den Landcode aufgrund des BLandcodes zurück.
   * @param blandCd BLandcode, z.B. CH für Schweiz, FR für Frankreich etc.
   * @return Landcode, z.B. CH für Schweiz, F für Frankreich etc.
   * @throws Exception
   */
  public String getLandCd(final String blandCd) throws Exception
  {
    return facade.getLandCd(blandCd);
  }

  /**
   * Findet alle Ortschaften mit plz oder ort.
   * Anfangsziffern bzw. Anfangsbuchstaben sind ausreichend.
   * @param plz Postleitzahl der Ortschaft
   * @param ort Name der Ortschaft
   * @return ArrayList mit PostOrtAVO-Objekten mit allen gefundenen Ortschaften
   * @throws Exception
   */
  public ArrayList findPostOrt(final String plz, final String ort) throws Exception
  {
    return facade.findPostOrt(plz, ort);
  }

  /**
  * Speicherung der Ortschaften in der Datenbank
  *
  * @param postOrtVO Benötigte Daten für die Normalisierung
  * @throws Exception
  * @author  Michael Zaugg
  */
  public void ladeOrtsverzeichnis(final PostOrtVO[] postOrtVO) throws Exception
  {
    facade.ladeOrtsverzeichnis(postOrtVO);
  }

  /**
   * Führt den Ortsabgleich durch.
   * @throws Exception
   */
  public void ortsabgleich() throws Exception
  {
    facade.ortsabgleich();
  }
}
