package ch.ige.adr.server;


/**
 * assemble sql query for a oracle text field.
 * note:
 * - all oracle text reseved words and characters not used as search
 *   option will be escaped by a \ for a single character and {} for words
 * - the user input for wildchards are '*' and '?'.
 *    they will be mapped to the oracle text wildchards '%' and '_'.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author Thomas Iten
 * @version $Id: AssemblerOracleText.java,v 1.2 2008/01/21 10:07:57 nydegger Exp $
 */
public class AssemblerOracleText 
{
  
  /**
   * oracle text reserved characters .
   * the first position has the reseved character, the second the escaped form
   * or the substitue character (see note 2).
   *
   * note:
   * the backslash must be the first character in the list, otherwise the
   * substitution dosen't work properly!
   *
   * note 2:
   * the reserved characters '*' and '?' have another meaning in this
   * application than intended by oracle text. in swissreg II they are used as
   * wildcards and have to be replaced by the oracle wildcard signs '%' and '_'.
   */
  private static final String RESERVED_CHARS[][] =
    { {"\\","\\\\"},
      {",","\\,"},
      {"&","\\&"},
      {"=","\\="},
      {"{","\\{"},
      {"}","\\}"},
      //{"(","\\("},                      // enabled, do NOT escape
      //{")","\\)"},                      // enabled, do NOT escape
      {"[","\\["},
      {"]","\\]"},
      {"-","\\-"},
      {";","\\;"},
      {"~","\\~"},
      {"|","\\|"},
      {"$","\\$"},
      {"!","\\!"},
      {">","\\>"},
      // {"%","\\%"},                     // enabled, do NOT escape
      {"_","\\_"},
      {"*","%"},                          // used as wildcard
      {"?","_"} };                        // used as wildcard

  /**
   * array of all oracle text reserved words (expect "OR", "AND, "NOT").
   * the first position has the reseved word, the second the escaped form.
   */
  private static final String RESERVED_WORDS[][] =
    { {"ABOUT", "{ABOUT}"},
      {"ACCUM","{ACCUM}"},
      {"BT","{BT}"},
      {"BTG","{BTG}"},
      {"BTI","{BTI}"},
      {"BTP","{BTP}"},
      {"EQUIV","{EQUIV}"},
      {"FUZZY","{FUZZY}"},
      {"HASPATH","{HASPATH}"},
      {"INPATH","{INPATH}"},
      {"MDATA","{MDATA}"},
      {"MINUS","{MINUS}"},
      {"NEAR","{NEAR}"},
      {"NT","{NT}"},
      {"NTG","{NTG}"},
      {"NTI","{NTI}"},
      {"NTP","{NTP}"},
      {"PT","{PT}"},
      {"RT","{RT}"},
      {"SQE","{SQE}"},
      {"SYN","{SYN}"},
      {"TR","{TR}"},
      {"TRSYN","{TRSYN}"},
      {"TT","{TT}"},
      {"WITHIN","{WITHIN}"} };

  public static final String  DOMAIN_CD_ALL         = "_ALL";
  public static final String  DOMAIN_CD_SEP         = "_SEP";

  public static final String  DOMAIN_CD_TRUE        = "_TRUE";
  public static final String  DOMAIN_CD_FALSE       = "_FALSE";

  public static final String  OPERATOR_NOT          = "NOT";
  public static final String  OPERATOR_AND          = "AND";
  public static final String  OPERATOR_OR           = "OR";

  public static final char    SEPARATOR             = ' ';    // token separator
  public static final char    SEPARATOR_ALTERNATIVE = ',';

  public static final char    GROUP_DELIMITER       = '"';    // group delimiter
  public static final char    RANGE_SIGN            = '-';    // indicate a range
  public static final char    WILDCARD_SINGLE       = '?';    // wildcard for a single char (1)
  public static final char    WILDCARD_MULTI        = '*';    // wildcard for multiple or none chars (0..n)


  private static final String COUNTRYCD_REGEXP      = "^[A-Z]{2}$";

  
  //
  // fields
  //
  private StringBuffer  buf   = new StringBuffer();     // internal store for oracle text value
  private boolean       first = true;                   // flag to indicate first assemble call

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public AssemblerOracleText()
  {
    buf = new StringBuffer();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public String getValue()
  {
    return buf.toString();
  }

  public void reset()
  {
    buf.delete(0,buf.length());
  }

  private void add(String value)
  {
    if (first)
    {
      first = false;
    }
    else
    {
      buf.append(SEPARATOR);
    }

    buf.append(value);
  }

  //---------------------------------------------------------------------------
  // public methods - AssemblerInterface
  //---------------------------------------------------------------------------

  public void assembleOr( String value )
  {
    add(OPERATOR_OR);
    assemble(value);
  }
  
  public void assembleAnd( String value )
  {
    add(OPERATOR_AND);
    assemble(value);
  }
  
  public void assembleAndNot( String value )
  {
    // note:
    // "expr1 and not expr2" will be in oracle text syntax "expr1 not expr2"
    assembleNot(value);
  }  
  
  public void assembleNot( String value )
  {
    add(OPERATOR_NOT);
    assemble(value);
  }
  
  public void assemble( String value )
  {
    //
    // escape single token
    //
    if (value.charAt(0) != GROUP_DELIMITER)
    {
      // escape reseved characters and substitue user to oracle wildcards
      for (int i = 0; i < RESERVED_CHARS.length; i++)
      {
        value = TextFormat.replace(value,RESERVED_CHARS[i][0],RESERVED_CHARS[i][1]);
      }

      // escape reseved words (do after escaping reserved characters!)
      for (int i = 0; i < RESERVED_WORDS.length; i++)
      {
        if (value.equalsIgnoreCase(RESERVED_WORDS[i][0]))
        {
          value = RESERVED_WORDS[i][1];
        }
      }
    }
    //
    // escape token group (word group)
    //
    else
    {
      // escape group as a whole expression by using {} as group delimiters
      value = "{" + value.substring(1,value.length()-1) + "}";
    }
    //
    // add value to the param stack
    //
    add(value);
  }
} // end class
