package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import ch.ige.art.vo.AltArtVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für alt_art
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: AltArtDAO.java,v 1.7 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class AltArtDAO
{

  /**
   * Selektiert einen Datensatz aus der Tabelle alt_art
   *
   * @param  artNr         ArtikelNr
   * @param  altArtNr      Folge Ereignis
   * @return AltArtVO      Value Object
   * @throws SQLException  Unerwarteter SQL Fehler
   */
  public AltArtVO select( String artNr, String altArtNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    AltArtVO altArtVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(alt_art_nr) as alt_art_nr, " + "trim(prior_cd) as prior_cd, " + "erst_dat, "
          + "trim(erst_ben) as erst_ben, " + "geaend_dat, " + "trim(geaend_ben) as geaend_ben, " + "u_version "
          + "FROM alt_art WHERE art_nr = RPAD(?, 18,' ') AND alt_art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, altArtNr );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        altArtVO = new AltArtVO();
        altArtVO.artNr = rs.getString( "art_nr" );
        altArtVO.altArtNr = rs.getString( "alt_art_nr" );
        altArtVO.priorCd = rs.getString( "prior_cd" );
        altArtVO.erstDat = rs.getDate( "erst_dat" );
        altArtVO.erstBen = rs.getString( "erst_ben" );
        altArtVO.geaendDat = rs.getDate( "geaend_dat" );
        altArtVO.geaendBen = rs.getString( "geaend_ben" );
        altArtVO.uVersion = rs.getString( "u_version" );
      }

      return altArtVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, AltArtDAO.class, query + "artNr: " + artNr + " altartNr: " + altArtNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Folgeereignisse eines Artikels aus der Tabelle alt_art
   *
   * @param  artNr        Artikel Nummer
   * @return ArrayList    ArrayList mit Value Objects
   * @throws SQLException Unerwarteter SQL Fehler
   */
  public ArrayList<AltArtVO> select( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<AltArtVO> al = new ArrayList<AltArtVO>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(alt_art_nr) as alt_art_nr, " + "trim(prior_cd) as prior_cd, " + "erst_dat, "
          + "trim(erst_ben) as erst_ben, " + "geaend_dat, " + "trim(geaend_ben) as geaend_ben, " + "u_version "
          + "FROM alt_art WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );

      rs = ps.executeQuery();

      while ( rs.next() )
      {
        AltArtVO altArtVO = new AltArtVO();
        altArtVO.artNr = rs.getString( "art_nr" );
        altArtVO.altArtNr = rs.getString( "alt_art_nr" );
        altArtVO.priorCd = rs.getString( "prior_cd" );
        altArtVO.erstDat = rs.getDate( "erst_dat" );
        altArtVO.erstBen = rs.getString( "erst_ben" );
        altArtVO.geaendDat = rs.getDate( "geaend_dat" );
        altArtVO.geaendBen = rs.getString( "geaend_ben" );
        altArtVO.uVersion = rs.getString( "u_version" );

        al.add( altArtVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, AltArtDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle alt_art ein
   *
   * @param     altArtVO         ValueObject AltArtVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( AltArtVO altArtVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO alt_art(" + "art_nr, " + "alt_art_nr, " + "prior_cd, " + "erst_dat, " + "erst_ben, " + "geaend_dat, " + "geaend_ben, "
          + "u_version )" + "VALUES( ?,?,?,?,?,?,?,?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, altArtVO.artNr );
      ps.setString( 2, altArtVO.altArtNr );
      ps.setString( 3, altArtVO.priorCd );
      if ( altArtVO.erstDat == null )
      {
        ps.setDate( 4, null );
      }
      else
      {
        ps.setDate( 4, new java.sql.Date( altArtVO.erstDat.getTime() ) );
      }
      ps.setString( 5, altArtVO.erstBen );
      if ( altArtVO.geaendDat == null )
      {
        ps.setDate( 6, null );
      }
      else
      {
        ps.setDate( 6, new java.sql.Date( altArtVO.geaendDat.getTime() ) );
      }
      ps.setString( 7, altArtVO.geaendBen );
      ps.setString( 8, altArtVO.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, AltArtDAO.class, query + altArtVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle alt_art
   *
   * @param     altArtVO      AltArtVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( AltArtVO altArtVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE alt_art SET " + "prior_cd = ?, " + "geaend_dat = ?, " + "geaend_ben = ?, " + "u_version = ? " + "WHERE art_nr = RPAD(?, 18,' ') "
          + "AND alt_art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, altArtVO.priorCd );
      if ( altArtVO.geaendDat == null )
      {
        ps.setNull( 2, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( altArtVO.geaendDat.getTime() ) );
      }
      ps.setString( 3, altArtVO.geaendBen );
      ps.setString( 4, altArtVO.uVersion );
      ps.setString( 5, altArtVO.artNr );
      ps.setString( 6, altArtVO.altArtNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, AltArtDAO.class, query + altArtVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht Datensätze aus der Tabelle alt_art
   *
   * @param     artNr          ArtikelNr      Primary Key
   * @param     altArtNr       Folge Ereignis Primary Key
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete( String artNr, String altArtNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "DELETE FROM alt_art " + "WHERE art_nr = RPAD(?, 18,' ') " + "AND alt_art_nr = RPAD(?, 18,' ') ";
      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, altArtNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, AltArtDAO.class, query + "artNr: " + artNr + " altartNr: " + altArtNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
