package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import ch.ige.art.vo.ArtextVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle Regformtext; es werden nur die nötigen Methoden
 * implementiert.
 *
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Markus Wild
 * @version $Id: ArtextDAO.java,v 1.8 2008/06/25 13:44:29 ruegsegger Exp $
 */
public class ArtextDAO
{

  /**
   * Selektierte alle Artikeltexte eines Artikels
   *
   * @param  artNr          ArtikelNr    Primary KEy
   * @return ArrayList      ArrayList mit Value Objects Artikeltext
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList<ArtextVO> select( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<ArtextVO> al = new ArrayList<ArtextVO>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(arttextart_cd) as arttextart_cd, " + "trim(sprach_cd) as sprach_cd, " + "erst_dat, "
          + "trim(erst_ben) as erst_ben, " + "geaend_dat, " + "trim(geaend_ben) as geaend_ben , " + "trim(text) as text, " + "u_version "
          + "FROM artext WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        ArtextVO artextVO = new ArtextVO();

        artextVO.artNr = rs.getString( "art_nr" );
        artextVO.artTextArtCd = rs.getString( "arttextart_cd" );
        artextVO.sprachCd = rs.getString( "sprach_cd" );
        artextVO.text = rs.getString( "text" );
        artextVO.erstDat = rs.getDate( "erst_dat" );
        artextVO.erstBen = rs.getString( "erst_ben" );
        artextVO.geaendDat = rs.getDate( "geaend_dat" );
        artextVO.geaendBen = rs.getString( "geaend_ben" );
        artextVO.uVersion = rs.getString( "u_version" );

        al.add( artextVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtextDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektieren von einem einzelnen Artikel Text
   *
   * @param     artNr Artikelnummer
   * @param     spracheCd Sprache (D,F,I,E)
   * @param     artTextArtCd Textart (zB. 'REG'
   * @return    ArtextVO ein einzelner Eintrag aus der Tabelle artext
   * @exception SQLException
   */
  public ArtextVO select( String artNr, String spracheCd, String artTextArtCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArtextVO artextVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "trim(arttextart_cd) as arttextart_cd, " + "trim(sprach_cd) as sprach_cd, " + "erst_dat, "
          + "trim(erst_ben) as erst_ben, " + "geaend_dat, " + "trim(geaend_ben) as geaend_ben, " + "trim(text) as text, " + "u_version "
          + "FROM artext WHERE art_nr = RPAD(?, 18,' ') AND arttextart_cd = RPAD(?, 3, ' ') AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, artTextArtCd );
      ps.setString( 3, spracheCd );

      rs = ps.executeQuery();

      if ( rs.next() )
      {
        artextVO = new ArtextVO();
        artextVO.text = rs.getString( "text" );
        artextVO.artTextArtCd = rs.getString( "arttextart_cd" );
        artextVO.sprachCd = rs.getString( "sprach_cd" );
        artextVO.artNr = rs.getString( "art_nr" );
        artextVO.uVersion = rs.getString( "u_version" );
      }
      return artextVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtextDAO.class, query + " art_nr: " + artNr + " sprach_cd: " + spracheCd + " arttextart_cd: " + artTextArtCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle artext ein
   *
   * @param     artextVO         ValueObject  ArtbezVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( ArtextVO artextVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO artext(" + "art_nr, " + "arttextart_cd, " + "sprach_cd, " + "text, " + "erst_dat, " + "erst_ben, " + "geaend_dat, " + "geaend_ben, "
          + "u_version) " + "VALUES( ?,?,?,?,?,?,?,?,?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artextVO.artNr );
      ps.setString( 2, artextVO.artTextArtCd );
      ps.setString( 3, artextVO.sprachCd );
      ps.setString( 4, artextVO.text );
      if ( artextVO.erstDat == null )
      {
        ps.setNull( 5, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 5, new java.sql.Date( artextVO.erstDat.getTime() ) );
      }
      ps.setString( 6, artextVO.erstBen );
      if ( artextVO.geaendDat == null )
      {
        ps.setNull( 7, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 7, new java.sql.Date( artextVO.geaendDat.getTime() ) );
      }
      ps.setString( 8, artextVO.geaendBen );
      ps.setString( 9, artextVO.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtextDAO.class, query + artextVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle artext
   *
   * @param     artextVO      ArtextVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( ArtextVO artextVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE artext SET " + "text = ?, " + "geaend_dat = ?, " + "geaend_ben = ?, " + "u_version = ? " + "WHERE art_nr = RPAD(?, 18,' ') "
          + "AND arttextart_cd = RPAD(?, 3, ' ') " + "AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artextVO.text );
      if ( artextVO.geaendDat == null )
      {
        ps.setNull( 2, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( artextVO.geaendDat.getTime() ) );
      }
      ps.setString( 3, artextVO.geaendBen );
      ps.setString( 4, artextVO.uVersion );

      ps.setString( 5, artextVO.artNr );
      ps.setString( 6, artextVO.artTextArtCd );
      ps.setString( 7, artextVO.sprachCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtextDAO.class, query + artextVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artext
   *
   * @param     artNr          ArtikelNr      Primary Key
   * @param     spracheCd      Sprache        Primary Key
   * @param     artTextArtCd   Preisart       Primary Key
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete( String artNr, String spracheCd, String artTextArtCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "DELETE FROM artext " + "WHERE art_nr = RPAD(?, 18,' ') " + "AND sprach_cd = RPAD(?, 3, ' ') " + "AND arttextart_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      ps.setString( 2, spracheCd );
      ps.setString( 3, artTextArtCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtextDAO.class, query + " art_nr: " + artNr + " sprach_cd: " + spracheCd + " arttextart_cd: " + artTextArtCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
