package ch.ige.epband.clientneu;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.epband.vo.StatistikVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Mit dieser Klasse wird eine Fehlerliste geführt.<br>
 * Name der verarbeiteten Zip-Datei muss im Name ersichtlich sein Epband_st320306 In dieses File werden
 * benutzerrelevante Informationen geschrieben. File wird nach der Verarbeitung dem Benutzer als Fehlerliste via Mail
 * zugestellt Es wurde ein RandomAccessFile gewaehlt, damit bei einem Restart der Applikation die eingetragenen
 * Information nicht ueberschrieben werden
 * </p>
 * <p>
 * Copyright: Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author A. Rueegsegger
 * @version $Id: EpbandLogFile.java,v 1.16 2008/02/04 08:45:33 wild Exp $
 */

public class EpbandLogFile
{
  private static final Logger LOG = Logger.getLogger( EpbandLogFile.class );

  private static final String DEFAULT_PATH = "c:\\data\\log\\Epband_temp.log";

  private static RandomAccessFile f = null;

  private static EpbandLogFile epbandLogFile;

  private long posPublDat;

  private long posPublWeek;

  private EpbandLogFile()
  {
    /**
     * Verhindern, dass das Objekt erstellt werden kann.
     */
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * 
   * @return EpbandDelegate
   */
  public static EpbandLogFile getInstance()
  {
    if ( epbandLogFile == null )
    {
      epbandLogFile = new EpbandLogFile();
    }
    return epbandLogFile;
  }

  /**
   * Erstellt ein Default-Logfile
   */
  private void createDefaultLogFile()
  {
    setLogFile( EpbandLogFile.DEFAULT_PATH );
    LOG.info( "Fehlerliste befindet sich auf: " + DEFAULT_PATH );
  }

  /**
   * File öffnen, bzw erstellen File-Pointer ans Ende der Datei setzte, damit geschriebene Daten nicht ueberschrieben
   * werden
   * 
   * @param pathname Fielpfad des Files
   */
  public void setLogFile( String pathname )
  {
    File file = new File( pathname );

    if ( !file.getParentFile().exists() )
    {
      file.getParentFile().mkdirs();
    }

    try
    {
      f = new RandomAccessFile( file, "rw" );
      f.seek( f.length() );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * File schliessen
   */
  public void close()
  {
    try
    {
      f.close();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * Eintrag ins File schreiben
   * 
   * @param line
   */
  public void write( String line )
  {
    try
    {
      if ( f == null )
      {
        createDefaultLogFile();
      }

      f.writeBytes( line + "\r\n" );
    }
    catch ( IOException ex )
    {
      LOG.error( "Eintrag kann nicht in Fehlerliste geschrieben werden: " + line );
      LOG.error( ex );
    }
  }

  /**
   * Publikatiosinformationen in Header schreiben Datum und Woche werden als Platzhalter eingetragen und am Ende der
   * Verarbeitung geschrieben Positionen werden zwischengespeichert
   */
  public void writePublInfo()
  {
    /** TODO publikationswoche und datum sollten bei einem fehler(restart) auch geschreiben werden */
    try
    {
      write( "Publikationsdatum: dd.MM.yyyy" );
      this.posPublDat = f.getFilePointer() - 12;

      write( "Publikationswoche: yyyyww" );
      this.posPublWeek = f.getFilePointer() - 8;
    }
    catch ( IOException ex )
    {
      System.out.println( ex );
    }
  }

  /**
   * Platzerhalter für Publikationsdatum und Publikationswoche ersetzten
   * 
   * @param publdat String
   * @param week String
   */
  public void writePublInfo( String publdat, String week )
  {
    try
    {
      long pos = f.getFilePointer();

      f.seek( posPublDat );
      write( publdat );

      f.seek( posPublWeek );
      write( week );

      f.seek( pos );
    }
    catch ( IOException ex )
    {
      System.out.println( ex );
    }
  }

  /**
   * Statistik schreiben
   * 
   * @param statistikVO
   */
  public void writeFooter( StatistikVO statistikVO )
  {
    epbandLogFile.write( "" );
    epbandLogFile.write( "Ende der Verarbeitung " + Util.dateToStringWithTime( new Date() ) );
    epbandLogFile.write( "\r\n" + statistikVO.toString() );
    epbandLogFile.write( "" );
    epbandLogFile.write( "=================================================================" );
  }

}
