package ch.ige.epband.clientneu;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.epband.server.EpbandDelegate;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.IpcVO;
import ch.ige.epband.vo.IpcrClassificationVO;
import ch.ige.epband.vo.PatIpcVO;
import ch.ige.util.MinimalDate;

/**
 * Gültig ab IPC-Reform - extrahiert die IPC-Klassen aus dem XML-Dokument
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author A. Rueegsegger
 * @version $Id: IpcrClassification.java,v 1.16 2008/05/26 09:02:34 ruegsegger Exp $
 */
public class IpcrClassification
{
  private XMLDoc xml;

  private EpbandVO epbandVO;

  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  private int countIpcCdH = 0; // Zähler für Hauptklassen

  private int countIpcCdN = 0; // Zähler für Nebenklassen

  private int countIpcCdF = 0; // Zähler für Fakultative Klasse

  private int countIpcCdI = 0; // Zähler für Indexierte Fakultative Klasse

  private static final Logger LOG = Logger.getLogger( IpcrClassification.class );

  /**
   * Setzen der lokalen Variablen
   * 
   * @param xml
   * @param epbandVO
   */
  public IpcrClassification( XMLDoc xmlDoc, EpbandVO epVO )
  {
    this.xml = xmlDoc;
    this.epbandVO = epVO;
  }

  /**
   * Extrahiert die IPC-Klassen aus dem XML und gibt sie in Form einer Liste zurück
   * 
   * @return List of PatIpcVO's
   */
  public ArrayList<PatIpcVO> getPatIpcVOs()
  {
    PatIpcVO patIpcVO = null;
    IpcVO ipcVO = null;
    ArrayList<PatIpcVO> patIpcAL = new ArrayList<PatIpcVO>();

    countIpcCdH = 0;
    countIpcCdN = 0;
    countIpcCdF = 0;
    countIpcCdI = 0;

    int anz = xml.getNumberOf( "classification-ipcr" );

    for ( int i = 1; i <= anz; i++ )
    {
      String tag = "classification-ipcr[" + i + "]";

      try
      {
        // Extrahiert PatIpc-Werte aus dem XML
        patIpcVO = extractPatIpcVO( tag );

        // Überprüft ob Symbol mit Gültig von vorhanden
        ipcVO = getIpcVO( patIpcVO );

        if ( patIpcVO.patIpcCd == PatIpcVO.IPC_CD_H )
        {
          patIpcVO.patIpcLnr = ++countIpcCdH;

          // Überprüft die Vorschriften für Hauptklassen
          checkFirstClass( patIpcVO, ipcVO.zusatz );
        }
        else if ( patIpcVO.patIpcCd == PatIpcVO.IPC_CD_N )
        {
          // Überprüft die Vorschriften für Nebenklassen
          checkLaterClass( patIpcVO, ipcVO.zusatz );
        }

        patIpcAL.add( patIpcVO );
      }
      catch ( Exception ex )
      {
        String message = "ERROR \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " IPC Klasse kann nicht gespeichert werden"
            + " Message: " + ex.getMessage();
        logFile.write( message );
      }
    }

    // Überprüft die Anzahl der Hauptklassen
    checkNumberOfFirstClass( countIpcCdH );

    return patIpcAL;
  }

  /**
   * Extrahiert die Werte aus dem XML und füllt sie in ein PatIpcVO ab
   * 
   * @param tag String XML-Element
   * @return PatIpcVO abgefülltes Object
   */
  private PatIpcVO extractPatIpcVO( String tag ) throws Exception
  {
    PatIpcVO patIpcVO = new PatIpcVO();

    IpcrClassificationVO ipcrClassificationVO = extractClassificationVO( tag );

    patIpcVO.ipcSymb = getIpcSymbol( ipcrClassificationVO );
    patIpcVO.patIpcCd = getPatIpcCd( ipcrClassificationVO );
    patIpcVO.ipcGueltigVon = getIpcGueltigVon( ipcrClassificationVO );
    patIpcVO.classLevel = getClassLevel( ipcrClassificationVO );
    patIpcVO.classValue = getClassValue( ipcrClassificationVO );
    patIpcVO.erfassDat = getErfassDat( ipcrClassificationVO );
    patIpcVO.source = getSource( ipcrClassificationVO );
    patIpcVO.geschichte = getStatus( ipcrClassificationVO );
    patIpcVO.generatingOffice = getOffice( ipcrClassificationVO );

    return patIpcVO;
  }

  /**
   * <B510EP> contains <classification-ipcr> which in turns contains all the new tags for each part of the
   * classification OR it can be stored in a fixed length field of 50 positions within the <text> data element.
   * 
   * @param tag String
   * @throws Exception
   * @return IpcrClassificationVO
   */
  private IpcrClassificationVO extractClassificationVO( String tag ) throws Exception
  {
    IpcrClassificationVO vo = null;

    if ( xml.exists( tag, "text" ) )
    {
      String text = xml.getValueOf( tag, "text" );
      vo = new IpcrClassificationVO( text );
    }
    else
    {
      throw new Exception( " Ungültiger Eintrag in classification-ipcr" );
    }
    return vo;
  }

  /**
   * Extrahiert den Wert eines IPC-Symbols aus dem XML
   * 
   * @param vo
   * @return String IPC-Symbol wie es in Bagis gespeichert wird
   */
  private String getIpcSymbol( IpcrClassificationVO vo ) throws Exception
  {
    IpcSymbol symbol = new IpcSymbol( vo.section, vo.ipcclass, vo.subclass );
    symbol.setGroupPart( vo.maingroup, vo.subgroup );

    return symbol.getIpcSymbol();
  }

  /**
   * Extrahiert den Code eines Ipc-Symbols aus dem XML-Element symbol-position Gültige Werte sind F=FirstClass und
   * L=LaterClass. Folgende Konvertierungen werden vorgenommen, bevor die Werte in Bagis gespeichert werden - F wird
   * umgewandelt in eine 2 (Hauptklasse) - L wird umgewandelt in eine 3 (Nebenklasse)
   * 
   * @param vo IpcrClassificationVO
   * @return int Code des Ipc-Symbols wie er in Bagis gespeichert wird
   */
  private int getPatIpcCd( IpcrClassificationVO vo ) throws Exception
  {
    int patIpcCd = 0;

    if ( vo.symbolPosition != null && vo.symbolPosition.equals( "F" ) )
    {
      patIpcCd = PatIpcVO.IPC_CD_H;
    }
    else if ( vo.symbolPosition != null && vo.symbolPosition.equals( "L" ) )
    {
      patIpcCd = PatIpcVO.IPC_CD_N;
    }
    else
    {
      patIpcCd = PatIpcVO.IPC_CD_H; // Defaultwert

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> keine oder ungültige Symbol-Position vorhanden. " + patIpcCd + " eingefügt";
      writeDebugMessage( line );
    }

    return patIpcCd;
  }

  /**
   * Extrahiert Attribut erfassDat aus dem XML-Element action-date Falls kein Action-Date vorhanden, wird das
   * Erteilungsdatum/Publikationsdatum oder das Anmeldedatum
   * 
   * @param vo Value Object
   * @return String Attribut erfassDat
   */
  private Date getIpcGueltigVon( IpcrClassificationVO vo ) throws Exception
  {
    Date ipcGueltigVon = parseDate( vo.ipcVersionIndicator );

    if ( ipcGueltigVon == null )
    {
      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein IPC-Version-Indicator oder ungültiges Datum vorhanden. ";

      writeDebugMessage( line );
    }
    return ipcGueltigVon;
  }

  /**
   * Extrahiert Attribut generatingOffice aus dem XML-Element generating-office Bezeichnet von welchem Land die
   * Klassifizierung für ein Patent gemacht wurde
   * 
   * @param vo IpcrClassificationVO
   * @return String Attribut generatingOffice
   */
  private String getOffice( IpcrClassificationVO vo ) throws Exception
  {
    String generatingOffice = vo.generatingOffice;

    if ( generatingOffice == null || generatingOffice.trim().equals( "" ) )
    {
      generatingOffice = new PatIpcVO().generatingOffice; // default

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein Generating-Office vorhanden. " + generatingOffice + " eingefügt";
      writeDebugMessage( line );
    }
    return generatingOffice;
  }

  /**
   * Extrahiert Attribut geschichte aus dem XML-Element classification-status Gültige Werte sind - B (Basic) - R
   * (Reclassified data) - V (various incidental changes) - D (data to be deleted)
   * 
   * @param vo Value Object
   * @return String Attribut geschichte
   */
  private String getStatus( IpcrClassificationVO vo ) throws Exception
  {
    String status = vo.classificationStatus;

    if ( status == null || status.trim().equals( "" ) )
    {
      status = new PatIpcVO().geschichte; // default

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein Classification-Status vorhanden. " + status + " eingefügt";
      writeDebugMessage( line );
    }
    return status;
  }

  /**
   * Extrahiert Attribut source aus dem XML-Element classification-data-source Gültige Werte sind - H (Human) - M
   * (Machine) - G (Generated by software)
   * 
   * @param vo Value Object
   * @return String Attribut source
   */
  private String getSource( IpcrClassificationVO vo ) throws Exception
  {
    String source = vo.classificationDataSource;

    if ( source == null || source.trim().equals( "" ) )
    {
      source = new PatIpcVO().source; // default

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> keine Classification-Data-Source vorhanden. " + source + " eingefügt";
      writeDebugMessage( line );
    }
    return source;
  }

  /**
   * Extrahiert Attribut classLevel aus dem XML-Element classification-level Gültige Werte sind - C (Core) - A
   * (Advanced) - S (Subclass)
   * 
   * @param vo
   * @return String Attribut classLevel
   */
  private String getClassLevel( IpcrClassificationVO vo ) throws Exception
  {
    String classLevel = vo.classificationLevel;

    if ( classLevel == null || classLevel.trim().equals( "" ) )
    {
      classLevel = new PatIpcVO().classLevel; // default

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein Class-Level vorhanden. " + classLevel + " eingefügt";
      writeDebugMessage( line );
    }
    return classLevel;
  }

  /**
   * Extrahiert Attribut classValue aus dem XML-Element classification-value Gültige Werte sind - I (inventive) - N
   * (non-inventive)
   * 
   * @param vo
   * @return String Attribut classValue
   */
  private String getClassValue( IpcrClassificationVO vo ) throws Exception
  {
    String classValue = vo.classificationValue;

    if ( classValue == null || classValue.trim().equals( "" )
        || !( classValue.equals( PatIpcVO.CLASS_VALUE_INVENTIVE ) || classValue.equals( PatIpcVO.CLASS_VALUE_NON_INVENTIVE ) ) )
    {
      classValue = new PatIpcVO().classValue; // default

      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein oder ungültiger Classification-Value vorhanden. " + classValue + " eingefügt";
      writeDebugMessage( line );
    }
    return classValue;
  }

  /**
   * Extrahiert Attribut erfassDat aus dem XML-Element action-date Falls kein Action-Date vorhanden, wird das
   * Erteilungsdatum/Publikationsdatum oder das Anmeldedatum
   * 
   * @param vo
   * @return String Attribut erfassDat
   */
  private Date getErfassDat( IpcrClassificationVO vo ) throws Exception
  {
    Date erfassDat = parseDate( vo.actionDate );

    if ( erfassDat == null )
    {
      String ipcSymbol = getIpcSymbol( vo );

      String line = " Bei IPC Klasse <" + ipcSymbol + "> kein oder ungültiges Action-Date vorhanden. ";

      String actionDate = xml.getValueOf( "B450", "date" );
      erfassDat = parseDate( actionDate );

      if ( erfassDat != null )
      {
        // Falls kein Action-Date vorhanden, wird das Erteilungsdatum/Publikationsdatum aus B450 DATE eingesetzt
        line += "Erteilungsdatum " + actionDate + " aus B450 eingefügt";
      }
      else
      {
        actionDate = xml.getValueOf( "B220", "date" );
        erfassDat = parseDate( actionDate );

        if ( erfassDat != null )
        {
          // Falls kein Action-Date und kein Erteilungsdatum wird das Anmeldedatum aus B220 DATE eingesetzt
          line += "Anmeldedatum " + actionDate + " aus B220 eingefügt";
        }
        else
        {
          // Ist weder Erteilungs- noch Erfassdatum vorhanden wird das Minimal-Date abgefüllt
          erfassDat = MinimalDate.getMinimalDate();
          line += "Minimaldate 15.10.1582 eingefügt";
        }
      }

      writeDebugMessage( line );
    }
    return erfassDat;
  }

  /**
   * Überprüft die Vorschriften die bei einer Hauptklasse gelten - eine Hauptklasse darf nicht indexiert sein. - eine
   * Hauptklasse muss inventive sein Bei Verletzungen wird ein Eintrag ins Logfile vorgenommen, die Werte werden
   * unverändert übernommen
   * 
   * @param patIpcVO PatIpcVO
   */
  private void checkFirstClass( PatIpcVO patIpcVO, String zusatz ) throws Exception
  {
    // Beim Auftreten einer indexierten Hauptklasse wird ein Fehler ins Logfile geschrieben.
    if ( zusatz.equals( IpcVO.ZUSATZ_INDEXING ) )
    {
      String message = "ERROR \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Eine Hauptklasse darf nicht indexiert sein. "
          + "IPC Klasse <" + patIpcVO.ipcSymb + "> ist indexiert";
      logFile.write( message );
    }
    // Beim Auftreten einer non-inventive Hauptklasse wird ein Fehler ins Logfile geschrieben
    if ( patIpcVO.classValue.equals( PatIpcVO.CLASS_VALUE_NON_INVENTIVE ) )
    {
      String message = "ERROR \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Eine Hauptklasse muss inventive sein. "
          + "IPC Klasse <" + patIpcVO.ipcSymb + "> ist non-inventive";
      logFile.write( message );
    }
  }

  /**
   * Überprüft die Vorschriften die bei einer Nebenklasse gelten - eine Nebenklasse darf nicht indexiert sein. - eine
   * Nebenklasse muss inventive sein Bei Verletzungen wird eine Konvertierung durchgeführt oder ein Eintrag ins Logfile
   * vorgenommen N(Nebenklasse) I(inventive) K,D(not indexed) -> N(Nebenklasse) I(inventive) okay N(Nebenklasse)
   * I(inventive) I (indexed) -> N(Nebenklasse) I(inventive) Logmeldung (debug) N(Nebenklasse) N(non-inventive) K (not
   * indexed) -> F(fakultative Klasse) N(non-inventive) N(Nebenklasse) N(non-inventive) I,D(indexed) -> I(indexierte
   * Klasse) N(non-inventive)
   * 
   * @param patIpcVO PatIpcVO
   */
  private void checkLaterClass( PatIpcVO patIpcVO, String zusatz ) throws Exception
  {
    if ( patIpcVO.classValue.equals( PatIpcVO.CLASS_VALUE_INVENTIVE ) )
    {
      patIpcVO.patIpcLnr = ++countIpcCdN;

      if ( zusatz.equals( IpcVO.ZUSATZ_INDEXING ) )
      {
        String line = " Eine Nebenklasse darf nicht indexiert sein. IPC Klasse <" + patIpcVO.ipcSymb + "> ist indexiert";
        writeDebugMessage( line );
      }
    }
    else
    {
      // Eine Nebenklasse darf nicht non-inventive sein
      // Konvertierung durchführen nach Code=F (fakultativ) da Typ classified
      if ( zusatz.equals( IpcVO.ZUSATZ_CLASSIFICATION ) )
      {
        patIpcVO.patIpcCd = PatIpcVO.IPC_CD_F;
        patIpcVO.patIpcLnr = ++countIpcCdF;
      }
      // Eine Nebenklasse darf nicht non-inventive sein
      // Konvertierung durchführen nach Code=I (indexiert) da Typ indexed or double purpose
      else
      {
        patIpcVO.patIpcCd = PatIpcVO.IPC_CD_I;
        patIpcVO.patIpcLnr = ++countIpcCdI;
      }
    }
  }

  /**
   * Überprüft die Anzahl erfasster Hauptklassen Ein EP muss genau eine Hauptklasse zugewiesen haben
   * 
   * @param countIpcCdH int Anzahl Hauptklassen
   */
  private void checkNumberOfFirstClass( int numberOfIpcCdH )
  {
    if ( numberOfIpcCdH < 1 )
    {
      String line = " keine IPC-Hauptklasse erfasst";
      writeDebugMessage( line );
    }
    else if ( numberOfIpcCdH > 1 )
    {
      String line = " mehr als eine IPC-Hauptklasse erfasst";
      writeDebugMessage( line );
    }
  }

  private Date parseDate( String actionDate )
  {
    try
    {
      return sdf.parse( actionDate );
    }
    catch ( ParseException pex )
    {
      return null;
    }
  }

  /**
   * Liest das IPC-Symbol und dessen Eingenschaften ab der Tabelle IPC Falls das Symbol nicht existiert wird ein
   * Eintrags ins Logfile vorgenommen
   * 
   * @param patIpcVO Value Object
   * @return IpcVO
   */
  private IpcVO getIpcVO( PatIpcVO patIpcVO ) throws Exception
  {
    IpcVO ipcVO = null;

    if ( patIpcVO.ipcGueltigVon != null )
    {
      ipcVO = EpbandDelegate.getInstance().getIpc( patIpcVO.ipcSymb, patIpcVO.ipcGueltigVon );
    }

    if ( ipcVO == null )
    {
      ipcVO = EpbandDelegate.getInstance().getYoungestIpc( patIpcVO.ipcSymb );

      if ( ipcVO != null )
      {
        // Logeintrag nur bei Patenten schreiben, bei Gesuchen sind zuviele davon betroffen
        if ( epbandVO.isPatent() )
        {
          String line = " IPC Klasse <" + patIpcVO.ipcSymb + "> mit Gültig von <"
              + ( patIpcVO.ipcGueltigVon == null ? "null" : sdf.format( patIpcVO.ipcGueltigVon ) ) + "> auf Bagis nicht vorhanden. Jüngstes Datum <"
              + sdf.format( ipcVO.gueltigVon ) + "> wird als Gültig von eingesetzt";
          writeDebugMessage( line );
        }
        patIpcVO.ipcGueltigVon = ipcVO.gueltigVon;
      }
      else
      {
        throw new Exception( " IPC Klasse <" + patIpcVO.ipcSymb + "> ist ungültig. " );
      }
    }

    return ipcVO;
  }

  /**
   * Schreibt eine Debug-Message ins Logfile, sofern der Log-Level im Config-File auf Debug gesetzt ist
   * 
   * @param line String Logmeldung
   */
  private void writeDebugMessage( String line )
  {
    if ( LOG.isDebugEnabled() )
    {
      String message = "DEBUG \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + line;
      logFile.write( message );
    }
  }
}
