package ch.ige.epband.util;

import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import ch.ige.util.IOUtil;

/**
 * standalone program to extract xml files with specific schutit numbers
 * browse zipfiles or textfiles for given schutit numbers and stores corresponding extracts as valid XML files
 * 
 * possible parameters see description of main method
 * 
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * 
 * @author ruegsegger
 * @version $Id: XMLExtractor.java,v 1.1 2008/05/27 07:52:47 ruegsegger Exp $
 */
public class XMLExtractor
{
  private static final Logger LOG = Logger.getLogger( XMLExtractor.class );

  private List<String> listSchutitNr;

  private File browseLocationDirectory;

  private File storeLocationDirectory;

  private RegexFileNameFilter filter;

  private String textFileName;

  /**
   * start program to extract xml files with specific schutit numbers
   * 
   * @param args     
   * args[0] - MANDATORY list of schutit numbers separated by comma [1023847, 1085395]    
   *           use schutit number without leading zero and without check number example BAGIS Schutztitel: 01023847-5 => 1023847
   * args[1] - OPTIONAL  URI to browse for - default value is [file:///c:/data/epband/browse]
   * args[2] - OPTIONAL  URI to store XML files found with corresponding schutit numbers - default value is [file:///c:/data/epband/found]
   * args[3] - OPTIONAL  regular expression for browsing files - default value is [^.*.zip$] all files with extension .zip
   */
  public static void main( String[] args )
  {
    try
    {
      LOG.info( "start program" );

      XMLExtractor extractor = new XMLExtractor();

      extractor.loadArguments( args );

      extractor.startExtraction();

      LOG.info( "end program" );
    }
    catch ( Exception ex )
    {
      LOG.error( ex.getMessage() );
      ex.printStackTrace();
    }
  }

  private void loadArguments( String[] args ) throws Exception
  {
    LOG.info( "loadArguments - begin" );

    if ( args == null || args.length == 0 )
    {
      LOG.info( "program started without arguments" );
    }
    else
    {
      StringBuffer arguments = new StringBuffer();

      for ( int i = 0; i < args.length; i++ )
      {
        arguments.append( args[i] );
        arguments.append( " " );
      }

      LOG.info( "program started with arguments: " + arguments.toString() );

      if ( args.length == 1 )
      {
        initStartParameters( args[0] );
      }
      else if ( args.length == 4 )
      {
        initStartParameters( args[0], args[1], args[2], args[3] );
      }
      else
      {
        throw new Exception( "invalid number of arguments" );
      }
    }

    LOG.info( "loadArguments - done" );
  }

  /**
   * initStartParameters load default values
   * 
   * @param eps list of schutit numbers separated by comma
   */
  private void initStartParameters( String eps ) throws Exception
  {
    initStartParameters( eps, "file:///c:/data/epband/browse", "file:///c:/data/epband/found", "^.*.zip$" );
  }

  /**
   * initStartParameters
   * 
   * @param eps - list of schutit numbers separated by comma
   * @param browseLocation - location to look after files 
   * @param storeLocation - location for storing XML files found with corresponding schutit numbers
   * @param regexFileName - regular expression for browsing specific files
   */
  private void initStartParameters( String eps, String browseLocation, String storeLocation, String regexFileName ) throws Exception
  {
    checkBrowseLocation( browseLocation );
    checkStoreLocation( storeLocation );

    filter = new RegexFileNameFilter( regexFileName );

    prepareSchutitNr( eps );
  }

  /**
   * filtering files to process
   * @throws Exception
   */
  private void startExtraction() throws Exception
  {
    File[] files = browseLocationDirectory.listFiles( filter );

    if ( files != null && files.length > 0 )
    {
      for ( File file: files )
      {
        processFile( file );
      }
    }
    else
    {
      LOG.info( "no files found to process with filter " + filter.getRegex() );
    }
  }

  /**
   * process file found with filter
   * 
   * @param file to process 
   * @throws Exception
   */
  private void processFile( File file ) throws Exception
  {
    if ( file.exists() && file.canRead() )
    {
      if ( file.getName().endsWith( ".zip" ) || file.getName().endsWith( ".ZIP" ) )
      {
        LOG.info( "process Zipfile: " + file.getPath() );
        processZipFile( file );
      }
      else if ( file.getName().endsWith( ".txt" ) || file.getName().endsWith( ".TXT" ) )
      {
        LOG.info( "process Textfile: " + file.getPath() );
        processTextFile( file );
      }
      else
      {
        throw new Exception( "file: " + file + " is not valid - extension has to be <zip> or <txt>" );
      }
    }
    else
    {
      throw new Exception( "file: " + file + " does not exist or is not readable" );
    }
  }

  /**
   * process zipfile by extracting zip entries
   * 
   * @param zipFile
   * @throws IOException
   * @throws Exception
   */
  private void processZipFile( File file )
  {
    ZipEntry entry;
    ZipFile zipFile;

    try
    {
      zipFile = new ZipFile( file );

      for ( Enumeration<?> e = zipFile.entries(); e.hasMoreElements(); )
      {
        entry = (ZipEntry) e.nextElement();
        LOG.info( "process zipfile entry: " + entry.getName() );

        setTextFileName( entry.getName() );

        InputStream is = zipFile.getInputStream( entry );

        processTextFile( is );

        is.close();
      }
    }
    catch ( Exception ex )
    {
      LOG.error( ex.getMessage() );
      ex.printStackTrace();
    }
  }

  /**
   * process textfile
   * 
   * @param textfile
   * @throws Exception
   */
  private void processTextFile( File textfile ) throws Exception
  {
    setTextFileName( textfile.getName() );

    InputStreamReader fr = new FileReader( textfile );
    readTextFile( fr );
  }

  /**
   * process textfile as inputstream
   * wird kein charset angegeben, wird der default genommen (hier cp1252)
   * was bei den sonderzeichen zu probleme führen kann
   * 
   * @param is inputstream of textfile
   * @throws Exception
   */
  private void processTextFile( InputStream is ) throws Exception
  {
    InputStreamReader ir = new InputStreamReader( is, "UTF-8" );
    readTextFile( ir );
  }

  /**
   * extract xml parts 
   * 
   * @param reader
   * @throws Exception
   */
  private void readTextFile( InputStreamReader reader ) throws Exception
  {
    StringBuffer sb = new StringBuffer();
    char[] ch = new char[8192];

    int j;

    while ( ( j = reader.read( ch ) ) > 0 )
    {
      sb.append( ch, 0, j );
    }

    reader.close();

    // -1 if not found
    int start = sb.indexOf( "<?xml" );
    int end = sb.indexOf( "</ep-bulletin>" );

    String xmlString = null;

    while ( start != -1 && end != -1 )
    {
      xmlString = sb.substring( start, end + 14 );
      processXMLFile( xmlString );

      start = sb.indexOf( "<?xml", end );
      end = sb.indexOf( "</ep-bulletin>", start );
    }
  }

  /**
   * process xml extract writing them as xml files if matching a schutit number
   * @param xmlString
   * @throws Exception
   */
  private void processXMLFile( String xmlString ) throws Exception
  {
    int i = 0;
    XMLDoc xml = new XMLDoc( xmlString );
    String schutitNr = xml.getValueOf( "B110" );

    if ( listSchutitNr.contains( schutitNr ) )
    {
      String xmlFileName = schutitNr + "_" + getTextFileNameWithoutExtension() + ".xml";

      File xmlFile = new File( storeLocationDirectory, xmlFileName );

      while ( xmlFile.exists() )
      {
        xmlFileName = schutitNr + "_" + getTextFileNameWithoutExtension() + "_" + i++ + ".xml";
        xmlFile = new File( storeLocationDirectory, xmlFileName );
      }

      LOG.info( "write file: " + xmlFile.getPath() );

      IOUtil.writeUTF8File( xmlFile, xmlString );
    }
  }

  /**
   * prepare schutit numbers as list
   * @param eps
   */
  private void prepareSchutitNr( String eps )
  {
    listSchutitNr = new ArrayList<String>();
    String[] ep = eps.split( "," );

    for ( int i = 0; i < ep.length; i++ )
    {
      listSchutitNr.add( ep[i] );
      LOG.info( "browse schutit number: " + ep[i] );
    }
  }

  /**
   * check if browse location exists and can be read
   * @param browseLocation
   * @throws Exception
   */
  private void checkBrowseLocation( String browseLocation ) throws Exception
  {
    browseLocationDirectory = new File( new URI( browseLocation ) );

    if ( !browseLocationDirectory.exists() || !browseLocationDirectory.canRead() || !browseLocationDirectory.isDirectory() )
    {
      throw new Exception( "location to browse: " + browseLocationDirectory + " does not exist or is not readable" );
    }
  }

  /**
   * check if store location exists otherwise create it
   * @param storeLocation
   * @throws Exception
   */
  private void checkStoreLocation( String storeLocation ) throws Exception
  {
    storeLocationDirectory = new File( new URI( storeLocation ) );

    if ( !storeLocationDirectory.exists() )
    {
      storeLocationDirectory.mkdirs();
    }

    if ( !storeLocationDirectory.exists() || !storeLocationDirectory.canRead() || !storeLocationDirectory.isDirectory() )
    {
      throw new Exception( "location to store: " + storeLocationDirectory + " does not exist or is not readable" );
    }
  }

  /**
   * set text file name - is part of xml file name to identify
   * @param fileName
   */
  private void setTextFileName( String fileName )
  {
    int beginIdx = 0;

    if ( fileName.indexOf( "/" ) > 0 )
    {
      beginIdx = fileName.lastIndexOf( "/" );
      beginIdx++;
    }

    textFileName = fileName.substring( beginIdx );
  }

  /**
   * Gets the textFileName.
   * @return the textFileName
   */
  public String getTextFileNameWithoutExtension()
  {
    int endIdx = textFileName.indexOf( "." );

    return textFileName.substring( 0, endIdx );
  }

  /**
   * FilenameFilter to filter browse location 
   * use regular expression for setting filter of filename  
   *
   * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
   * @author   ruegsegger
   * @version  $Id: XMLExtractor.java,v 1.1 2008/05/27 07:52:47 ruegsegger Exp $
   */
  public class RegexFileNameFilter implements FilenameFilter
  {
    private String regex;

    /**
     * sets regular expression used as filter filename
     * @param regexFileName regular expression of filename
     */
    public RegexFileNameFilter( String regexFileName )
    {
      this.regex = regexFileName;
    }

    // @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)    
    public boolean accept( File dir, String name )
    {
      return name.matches( regex );
    }

    public String getRegex()
    {
      return regex;
    }
  }
}
