package ch.ige.image;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.io.Reader;
import java.lang.management.ManagementFactory;
import java.nio.ByteBuffer;
import java.nio.InvalidMarkException;
import java.nio.MappedByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.FileChannel;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.nio.channels.FileChannel.MapMode;

import org.apache.log4j.Logger;

/**
 * IO Utilities. Note this is duplicated code from ch.ige.util.IOUtil.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Michael Locher
 * @version $Id: IOUtil.java,v 1.8 2009/04/15 07:23:36 locher Exp $
 */
public final class IOUtil
{

  private static final Logger LOG = Logger.getLogger( IOUtil.class );

  /**
   * Gives access to file contents via a {@link ByteBuffer}.
   * 
   * @param source the file to read
   * @param memoryMapped if true, the file will be memorymapped readonly
   * @param mapMode how the file is mapped
   * @return the contents of the file
   * @throws IOException if reading/mapping fails
   */
  public static ByteBuffer asByteBuffer( File source, boolean memoryMapped, MapMode mapMode ) throws IOException
  {
    if ( memoryMapped )
    {
      try
      {
        return asMappedByteBuffer( source, mapMode );
      }
      catch ( IOException e )
      {
        if ( isAddressSpaceExhaustedException( e ) )
        {
          LOG.info( "returning heap buffer instead of memory mapped buffer" );
          return asHeapByteBuffer( source );
        }
        else
        {
          throw e;
        }
      }
    }
    return asHeapByteBuffer( source );
  }

  public static ByteBuffer ensureWritable( ByteBuffer data )
  {
    if ( data != null && data.isReadOnly() )
    {
      int size = data.limit();
      ByteBuffer buf = data.isDirect() ? ByteBuffer.allocateDirect( size ) : ByteBuffer.allocate( size );
      buf.put( data.duplicate() );
      buf.rewind();
      return buf;
    }
    return data;
  }

  /**
   * Gives access to file contents via a {@link MappedByteBuffer}.
   * 
   * @param source source the file to read
   * @param mode the mapping mode
   * @return the mapped contents of the file
   * @throws IOException if mapping fails
   */
  private static MappedByteBuffer asMappedByteBuffer( File source, MapMode mode ) throws IOException
  {
    RandomAccessFile raf = new RandomAccessFile( source, MapMode.READ_ONLY.equals( mode ) ? "r" : "rw" );
    try
    {
      long length = source.length();
      FileChannel channel = raf.getChannel();
      try
      {
        try
        {
          MappedByteBuffer data = channel.map( mode, 0, length );
          data.load();
          return data;
        }
        catch ( IOException e )
        {
          if ( isAddressSpaceExhaustedException( e ) )
          {
            // currently, there is no way to free MappedByteBuffer resources except by garbage collection...
            LOG.error( "calling garbage collector with intent to collect existing MappedByteBuffers", e );
            try
            {
              ManagementFactory.getMemoryMXBean().gc();
            }
            catch ( Exception egc )
            {
              //failed to gc, throw the original exception
              throw e;
            }

            // try again
            MappedByteBuffer data = channel.map( mode, 0, length );
            data.load();
            return data;
          }
          throw e;
        }
      }
      finally
      {
        channel.close();
      }
    }
    finally
    {
      raf.close();
    }
  }

  private static boolean isAddressSpaceExhaustedException( IOException e )
  {
    return "Cannot allocate memory".equalsIgnoreCase( e.getMessage() );
  }

  private static ByteBuffer asHeapByteBuffer( File source ) throws IOException
  {
    FileInputStream fis = new FileInputStream( source );
    try
    {
      ByteBuffer buffer = ByteBuffer.allocate( (int) source.length() );
      FileChannel channel = fis.getChannel();
      try
      {
        do
        {
          channel.read( buffer );
        } while ( buffer.hasRemaining() );
        buffer.rewind();
        return buffer;
      }
      finally
      {
        channel.close();
      }
    }
    finally
    {
      fis.close();
    }
  }

  //-----------------------------------

  /**
   * Convert Reader to String.
   *
   * @param in
   *            the reader to read
   * @return the string or null if in was null
   * @throws IOException
   */
  public static String toString( Reader in ) throws IOException
  {
    if ( in == null )
    {
      return null;
    }
    StringBuilder sb = new StringBuilder();
    try
    {
      int cnt;
      char[] buff = new char[4096];
      while ( ( cnt = in.read( buff ) ) > -1 )
      {
        sb.append( buff, 0, cnt );
      }
    }
    finally
    {
      in.close();
    }

    return sb.toString();
  }

  /**
   * Convert InputStream to String using default encoding. Warning: Using the
   * default encoding is not a good idea.
   *
   * @param in
   *            the stream to read
   * @return the string or null if in was null
   * @throws IOException
   */
  public static String toString( InputStream in ) throws IOException
  {
    return IOUtil.toString( new InputStreamReader( in ) );
  }

  //-----------------------------------

  public static ByteBuffer toBuffer( InputStream in, boolean direct ) throws IOException
  {
    return toBuffer( Channels.newChannel( in ), direct, -1 );
  }

  public static ByteBuffer toBuffer( InputStream in, boolean direct, int estimatedSize ) throws IOException
  {
    return toBuffer( Channels.newChannel( in ), direct, estimatedSize );
  }

  public static ByteBuffer toBuffer( ReadableByteChannel in, boolean direct, int estimatedSize ) throws IOException
  {
    WritableByteBuffer tmp = estimatedSize > 0 ? new WritableByteBuffer( direct, estimatedSize ) : new WritableByteBuffer( direct );
    copy( in, tmp );
    return tmp.getBuffer();
  }

  public static void copy( ReadableByteChannel in, WritableByteBuffer out ) throws IOException
  {
    ByteBuffer buf = ByteBuffer.allocate( 4096 );
    while ( in.read( buf ) != -1 )
    {
      buf.flip();
      do
      {
        out.write( buf );
      } while ( buf.hasRemaining() );
      buf.rewind();
    }
  }

  public static class ReadableByteBuffer extends InputStream implements ReadableByteChannel
  {

    private final ByteBuffer buffer;

    private boolean open = true;

    public ReadableByteBuffer( ByteBuffer buffer )
    {
      this.buffer = buffer;
    }

    /**
     * Reopens the buffer for reading. 
     */
    public synchronized void reopen()
    {
      this.open = true;
      this.buffer.rewind();
    }

    // @see java.nio.channels.Channel#close()
    @Override
    @SuppressWarnings( "unused" )
    public synchronized void close() throws IOException
    {
      this.open = false;
    }

    // @see java.nio.channels.Channel#isOpen()
    public synchronized boolean isOpen()
    {
      return this.open;
    }

    private void ensureOpen() throws IOException
    {
      if ( !isOpen() )
      {
        throw new ClosedChannelException();
      }
    }

    // @see java.io.InputStream#available()
    @Override
    public synchronized int available() throws IOException
    {
      return this.buffer.remaining();
    }

    // @see java.io.InputStream#skip(long)
    @Override
    public long skip( long n ) throws IOException
    {
      long skip = n;
      int remaining = this.buffer.remaining();
      if ( n > remaining )
      {
        skip = Math.min( remaining, n );
      }
      this.buffer.position( (int) ( this.buffer.position() + skip ) );
      return skip;
    }

    // @see java.io.InputStream#read()
    @Override
    public synchronized int read() throws IOException
    {
      ensureOpen();
      if ( !this.buffer.hasRemaining() )
      {
        return -1;
      }
      return this.buffer.get() & 0xff;
    }

    // @see java.io.InputStream#read(byte[])
    @Override
    public int read( byte[] b ) throws IOException
    {
      return this.read( ByteBuffer.wrap( b ) );
    }

    // @see java.io.InputStream#read(byte[], int, int)
    @Override
    public int read( byte[] b, int off, int len ) throws IOException
    {
      ByteBuffer buf = ByteBuffer.wrap( b );
      buf.position( off ).limit( off + len );
      return this.read( buf );
    }

    // @see java.nio.channels.ReadableByteChannel#read(java.nio.ByteBuffer)
    public synchronized int read( ByteBuffer dst ) throws IOException
    {
      ensureOpen();
      int remaining = this.buffer.remaining();
      int requested = dst.remaining();

      int fetched = Math.min( remaining, requested );
      if ( fetched < 1 )
      {
        return -1;
      }

      ByteBuffer src = this.buffer;
      if ( fetched < remaining )
      {
        src = this.buffer.duplicate();
        int end = src.position() + fetched;
        src.limit( end );
        this.buffer.position( end );
      }
      dst.put( src );
      return fetched;
    }

    // @see java.io.InputStream#markSupported()
    @Override
    public boolean markSupported()
    {
      return true;
    }

    // @see java.io.InputStream#mark(int)
    @Override
    public synchronized void mark( int readlimit )
    {
      this.buffer.mark();
    }

    // @see java.io.InputStream#reset()
    @Override
    public synchronized void reset() throws IOException
    {
      try
      {
        this.buffer.reset();
      }
      catch ( InvalidMarkException e )
      {
        throw new IOException( "invalid mark" );
      }
    }

  } // inner-class

  public static class WritableByteBuffer extends OutputStream implements WritableByteChannel
  {
    private ByteBuffer buffer;

    private boolean open = true;

    private final boolean direct;

    public WritableByteBuffer( boolean direct )
    {
      this.direct = direct;
    }

    public WritableByteBuffer( boolean direct, int initialCapacity )
    {
      this.direct = direct;
      grow( initialCapacity );
    }

    public synchronized byte[] toByteArray()
    {
      ByteBuffer result = this.getBuffer();
      ByteBuffer array = ByteBuffer.allocate( result.remaining() );
      return array.put( result ).array();
    }

    public synchronized ByteBuffer getBuffer()
    {
      if ( this.buffer == null )
      {
        grow( 0 );
      }
      ByteBuffer view = this.buffer.duplicate();
      view.flip();
      return view;
    }

    // @see java.io.OutputStream#write(byte[])
    @Override
    public void write( byte[] b ) throws IOException
    {
      this.write( ByteBuffer.wrap( b ) );
    }

    // @see java.io.OutputStream#write(byte[], int, int)
    @Override
    public synchronized void write( byte[] b, int off, int len ) throws IOException
    {
      ByteBuffer buf = ByteBuffer.wrap( b );
      buf.position( off ).limit( off + len );
      this.write( buf );
    }

    // @see java.io.OutputStream#write(int)
    @Override
    public synchronized void write( int b ) throws IOException
    {
      ensureOpen();
      if ( !hasRoomFor( 1 ) )
      {
        grow( 1 );
      }
      this.buffer.put( (byte) b );
    }

    // @see java.nio.channels.WritableByteChannel#write(java.nio.ByteBuffer)
    public synchronized int write( ByteBuffer src ) throws IOException
    {
      ensureOpen();
      int len = src.remaining();
      if ( !hasRoomFor( len ) )
      {
        grow( len );
      }
      this.buffer.put( src );
      return len;
    }

    private void grow( int len )
    {
      ByteBuffer oldBuffer = this.buffer;
      int newSize = calcSize( this.buffer == null ? 0 : this.buffer.capacity(), len );
      this.buffer = this.direct ? ByteBuffer.allocateDirect( newSize ) : ByteBuffer.allocate( newSize );
      if ( oldBuffer != null )
      {
        oldBuffer.flip();
        this.buffer.put( oldBuffer );
      }
    }

    private boolean hasRoomFor( int len )
    {
      return this.buffer == null ? false : this.buffer.remaining() >= len;
    }

    private int calcSize( int currentCapacity, int minimumReqired )
    {
      final int minSize = currentCapacity + minimumReqired;
      int s = currentCapacity;
      if ( currentCapacity < 1 )
      {
        // start with this size
        s = 256;
      }
      while ( s < minSize )
      {
        s *= 2;
      }
      return s;
    }

    // @see java.nio.channels.Channel#close()
    @Override
    @SuppressWarnings( "unused" )
    public void close() throws IOException
    {
      this.open = false;
    }

    // @see java.nio.channels.Channel#isOpen()
    public boolean isOpen()
    {
      return this.open;
    }

    private void ensureOpen() throws IOException
    {
      if ( !isOpen() )
      {
        throw new ClosedChannelException();
      }
    }

  } // inner-class

  //-------------------------

  private static final int WIRE_CHUNK_SIZE = 64 * 1024;

  /**
   * Writes the buffer to the output.
   * @param out the destination
   * @param data the data
   * @throws IOException if writing fails
   */
  public static void writeBuffer( ObjectOutput out, ByteBuffer data ) throws IOException
  {
    if ( data == null )
    {
      out.writeInt( -1 ); // indicate with negative size
      return;
    }

    final ByteBuffer d = data.duplicate();
    final int size = d.remaining();
    int numFullChunks = size / WIRE_CHUNK_SIZE;
    int numChunks = ( size % WIRE_CHUNK_SIZE != 0 ) ? numFullChunks + 1 : numFullChunks;
    out.writeInt( size );
    out.writeInt( numChunks );
    out.writeBoolean( d.isDirect() );

    byte[] chunkBuffer = new byte[WIRE_CHUNK_SIZE];
    //proccess the full chunks
    for ( int i = 0; i < numFullChunks; i++ )
    {
      d.get( chunkBuffer );
      out.write( chunkBuffer );
    }
    //process the remaining
    if ( size > 0 )
    {
      int r = d.remaining();
      if ( r > 0 )
      {
        d.get( chunkBuffer, 0, r );
        out.write( chunkBuffer, 0, r );
      }
    }
  }

  /**
   * Reads a buffer from the input.
   * @param in the input
   * @return the buffer
   * @throws IOException if reading fails
   */
  public static ByteBuffer readBuffer( ObjectInput in ) throws IOException
  {
    final int size = in.readInt();
    if ( size < 0 )
    {
      return null;
    }
    final int numChunks = in.readInt();
    final int numFullChunks = Math.max( 0, numChunks - 1 );
    final boolean direct = in.readBoolean();
    final ByteBuffer buf = direct ? ByteBuffer.allocateDirect( size ) : ByteBuffer.allocate( size );
    final byte[] chunkBuffer = new byte[WIRE_CHUNK_SIZE];

    //proccess the full chunks
    for ( int i = 0; i < numFullChunks; i++ )
    {
      buf.put( readChunk( in, chunkBuffer, WIRE_CHUNK_SIZE ), 0, WIRE_CHUNK_SIZE );
    }
    //process the remaining
    if ( size > 0 )
    {
      int r = buf.remaining();
      if ( r > 0 )
      {
        buf.put( readChunk( in, chunkBuffer, r ), 0, r );
      }
    }
    buf.flip();
    return buf;
  }

  private static byte[] readChunk( ObjectInput in, byte[] chunkBuffer, int count ) throws IOException
  {
    in.readFully( chunkBuffer, 0, count );
    return chunkBuffer;
  }

}
