package ch.ige.image;

import java.awt.image.RenderedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.sql.Blob;
import java.sql.SQLException;

/**
 * Factory for images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: ImageFactory.java,v 1.12 2009/03/18 16:37:02 locher Exp $
 */
public interface ImageFactory
{

  /**
   * Wraps an AWT image for usage with this library. Assumes 72 DPI resolution.
   * 
   * @see BufferedImageWrapper
   * @param image the image to wrap.
   * @return an image with the same appearance
   */
  public Image wrapImage( java.awt.Image image );

  /**
   * Wraps an AWT image for usage with this library.
   * 
   * @see BufferedImageWrapper
   * @see MetadataBean
   * @param image the image to wrap
   * @param metadata the metadata for the resulting image
   * @return an image with the same appearance
   */
  public Image wrapImage( java.awt.Image image, Metadata metadata );

  /**
   * Wraps a Java2D image for usage with this library.
   * 
   * @see BufferedImageWrapper
   * @see MetadataBean
   * @param image the image to wrap
   * @param metadata the metadata for the resulting image
   * @return an image with the same appearance
   */
  public Image wrapRenderedImage( RenderedImage image, Metadata metadata );

  /**
   * Loads an image from a BLOB.
   * 
   * @param image the BLOB containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage loadBlob( Blob image ) throws IOException, ImageException, SQLException;

  /**
   * Loads an image from a File.
   * 
   * @param image the File containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage loadFile( File image ) throws IOException, ImageException;

  /**
   * Loads an image from an URL.
   * 
   * @param image the URL containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage loadURL( java.net.URL image ) throws IOException, ImageException;

  /**
   * Loads an image from a stream.
   * 
   * @param imageData the stream containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage loadStream( InputStream imageData ) throws IOException, ImageException;

  /**
   * Loads an image from a byte array.
   * 
   * @param imageData the array containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   * @deprecated use {@link #loadBytes(ByteBuffer)} instead.
   */
  @Deprecated
  public EncodedImage loadBytes( byte[] imageData ) throws ImageException;

  /**
   * Loads an image from a byte buffer.
   * 
   * @param imageData the buffer containing the image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage loadBytes( ByteBuffer imageData ) throws ImageException;

  /**
   * Loads an image from the encoding of an existing image.
   * 
   * @param image the existing encoded image.
   * @return the parsed image.
   * @throws ImageException if the image data is malformed.
   */
  public EncodedImage reload( EncodedImage image ) throws ImageException;

}