package ch.ige.image;

import java.awt.Dimension;
import java.awt.color.ICC_Profile;
import java.io.Serializable;

import ch.ige.image.format.Format;

/**
 * Representation of Image Metadata.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Image#getMetadata()
 * @author Michael Locher
 * @version $Id: Metadata.java,v 1.17 2008/05/20 16:53:00 galli Exp $
 */
public interface Metadata extends Serializable
{

  /**
   * Returns the format of the image.
   * 
   * @return the format of the image
   */
  public Format getFormat();

  /**
   * Returns the size of the image in pixels.
   * 
   * @return the Dimension of the image
   */
  public Dimension getSize();

  /**
   * Returns the size of the image in the requested unit.
   * 
   * @see Image#UNIT_MILLIMETER
   * @see Image#UNIT_PIXEL
   * @param unit one of the supported units.
   * @return the Dimension of the image
   * @throws MetadataException if metadata is missing
   */
  public Dimension getSize( String unit ) throws MetadataException;

  /**
   * Returns the resolution in dots per inch (DPI).
   * 
   * @return the DPI of the image
   * @throws MetadataException if the resolution is not available
   */
  public int getDPI() throws MetadataException;

  /**
   * Returns the colorspace of the image.
   * <p>
   * <code>
   * "XYZ" | "Lab" | "Luv" | 
   * "YCbCr" | "Yxy" | "YCCK" | "PhotoYCC" | "RGB" | "GRAY" | "HSV" | 
   * "HLS" | "CMYK" | "CMY" | "2CLR" | "3CLR" | "4CLR" | "5CLR" | 
   * "6CLR" | "7CLR" | "8CLR" | "9CLR" | "ACLR" | "BCLR" | "CCLR" | 
   * "DCLR" | "ECLR" | "FCLR"
   </code>
   * 
   * @return the colorspace of the image
   * @throws MetadataException it the colorspace is unknown
   */
  public String getColorspaceType() throws MetadataException;

  /**
   * Checks if an embedded ICC Profile is available.
   * 
   * @return true if an embedded ICC Profile is available.
   */
  public boolean hasEmbeddedICCProfile();

  /**
   * Returns the embedded ICC Profile.
   * 
   * @return the embedded ICC Profile or null if none is available
   */
  public ICC_Profile getEmbeddedICCProfile();

  /**
   * Returns the image mac.
   * 
   * @return the mac
   */
  public String getMAC();

  /**
   * Returns a part of the image metadata the image had before the conversion was made.
   * 
   * @return the original metadata
   */
  public String getOrigMetadata();

  /**
   * Returns the old ruleset name.
   * 
   * @return the ruleset name
   */
  public String getConversionRuleName();
}
