package ch.ige.image.color;

import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;

import org.apache.log4j.Logger;

/**
 * Grayscale operations. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: GrayScale.java,v 1.10 2008/08/07 17:10:04 locher Exp $
 */
public class GrayScale
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( GrayScale.class );

  /**
   * Recommended algorithm.
   */
  public static final Algorithm DEFAULT_ALGORITHM = Algorithm.LINEAR;

  /**
   * Grayscale algorithms. 
   */
  public static enum Algorithm
  {
    /**
     * Java linear gray.
     */
    LINEAR,

    /**
     * Kodak PhotoCD.
     */
    PHOTOCD,

    /**
     * BT.601 Video.
     */
    BT601,

    /**
     * BT.709 HD TV.
     */
    BT709,

    /**
     * IGE SETA (legacy) method. 
     */
    SETA;

  }

  /**
   * Performs a grayscale operation.
   * @param algorithm the algorithm to use
   * @param source the image to grayscale
   * @return the grayscaled image
   */
  public static BufferedImage grayscale( Algorithm algorithm, BufferedImage source )
  {
    switch( algorithm )
    {
      case LINEAR:
        return convertColorAndCombine( source, ColorSpace.getInstance( ColorSpace.CS_GRAY ) );
      case PHOTOCD:
        return convertColorAndCombine( source, ColorSpace.getInstance( ColorSpace.CS_PYCC ), 1.0D, 0.0D, 0.0D );
      case BT601:
        return convertColorAndCombine( source, SMPTEColorSpace.getSMPTE_C_ColorSpace(), 0.299D, 0.587D, 0.114D );
      case BT709:
        return convertColorAndCombine( source, ColorSpace.getInstance( ColorSpace.CS_sRGB ), 0.2126D, 0.7152D, 0.0722D );
      case SETA:
        return setaGrayscale( source );
      default:
        throw new IllegalArgumentException( "unknown algorithm" );
    }
  }

  private static BufferedImage convertColorAndCombine( BufferedImage source, ColorSpace targetSpace, double... bandcombineCoeffRGB )
  {
    BufferedImage intermediate;
    if ( ( source.getColorModel() instanceof ComponentColorModel )
        && ColorSpaceUtil.isSurelySameColorSpace( source.getColorModel().getColorSpace(), targetSpace ) )
    {
      intermediate = source;
    }
    else
    {
      intermediate = new ColorConvertOp( targetSpace, null ).filter( source, null );
    }
    if ( bandcombineCoeffRGB != null && bandcombineCoeffRGB.length == 3 )
    {
      return performJAIBandCombine( intermediate, bandcombineCoeffRGB );
    }
    else
    {
      return intermediate;
    }
  }

  private static BufferedImage setaGrayscale( BufferedImage img )
  {

    switch( img.getSampleModel().getNumBands() )
    {
      case 1:
        // already singleband - nothing to do
        return img;
      case 3:
        // RGB (really?)      
        return performJAIBandCombine( img, 0.299D, 0.587D, 0.114D );
      default:
        return performJAIColorConvert( img, new ComponentColorModel( ColorSpace.getInstance( ColorSpace.CS_GRAY ), new int[] { 8 }, false, false,
            Transparency.OPAQUE, DataBuffer.TYPE_BYTE ) );
    }
  }

  private static BufferedImage performJAIBandCombine( BufferedImage img, double... bandcombineCoeffRGB )
  {
    if ( img.getRaster().getNumBands() >= bandcombineCoeffRGB.length )
    {
      ParameterBlock pb = new ParameterBlock();
      pb.addSource( PlanarImage.wrapRenderedImage( img ) ).add( assembleBandcombineMatrix( img, bandcombineCoeffRGB ) );
      return JAI.create( "bandcombine", pb, null ).getAsBufferedImage();
    }
    else
    {
      LOG.error( "could not bandcombine input image" );
      return img;
    }
  }

  private static double[][] assembleBandcombineMatrix( BufferedImage img, double... bandcombineCoeffRGB )
  {
    int dim = img.getRaster().getNumBands();
    int type = img.getType();
    double[][] matrix = new double[1][dim + 1];
    switch( type )
    {
      case BufferedImage.TYPE_INT_BGR:
      case BufferedImage.TYPE_3BYTE_BGR:
      case BufferedImage.TYPE_4BYTE_ABGR:
      case BufferedImage.TYPE_4BYTE_ABGR_PRE:
        // blue,green,red
        matrix[0][0] = bandcombineCoeffRGB[2];
        matrix[0][1] = bandcombineCoeffRGB[1];
        matrix[0][2] = bandcombineCoeffRGB[0];
        break;
      default:
        // assume red,green,blue
        matrix[0][0] = bandcombineCoeffRGB[0];
        matrix[0][1] = bandcombineCoeffRGB[1];
        matrix[0][2] = bandcombineCoeffRGB[2];
    }
    return matrix;
  }

  private static BufferedImage performJAIColorConvert( BufferedImage img, ColorModel cm )
  {
    ParameterBlock pb = new ParameterBlock();
    pb.addSource( PlanarImage.wrapRenderedImage( img ) ).add( cm );
    ImageLayout layout = new ImageLayout();
    layout.setColorModel( cm ).setSampleModel( cm.createCompatibleSampleModel( img.getWidth(), img.getHeight() ) );
    return JAI.create( "ColorConvert", pb, new RenderingHints( javax.media.jai.JAI.KEY_IMAGE_LAYOUT, layout ) ).getAsBufferedImage();
  }

}
