package ch.ige.image.format;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.WritableByteChannel;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CodingErrorAction;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import ch.ige.image.IOUtil;
import ch.ige.image.format.png.PNGChunkFilter;
import ch.ige.image.format.png.ReorderChunksFilter;

/**
 * Utility for embedding MAC codes.
 * 
 * MAC tags which look like {@code IPI<hex coded hash>@YYYY-MM-DD} are embedded into the content (e.g. into the comment field of an image container).
 * 
 * <p>
 * Usage:
 * <ul>
 * <li>{@link #getTemplate(Date)} and embed it into your content.
 * <li> render your content, the template must not be mutated by this and should be recognizable as ASCII encoded characters within the rendered bytes.
 * <li> invoke {@link #sign(ByteBuffer)} on the rendered content to calculate the MAC.
 * <li> whenever you need to verify the integrity and authenticity of your content invoke {@link #verify(ByteBuffer)}.
 * </ul>
 * </p>
 * 
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: EmbeddedHashes.java,v 1.15 2009/04/15 07:23:36 locher Exp $
 */
public class EmbeddedHashes
{
  private static final int SHA1_HASH_LENGTH = 40;

  /**
   * WARNING: Do not change this key! it would render all signed images invalid.
   */
  private static final SecretKeySpec IMAGE_HANDLER_KEY = createKey( "Always Look on the Bright Side of Life" );

  /**
   * Used algorithm.
   */
  private static final String ALGORITHM = "HmacSHA1";

  /**
       * Provides the template tags which need to be embedded into the content.
       * @param time a timestamp (or {@literal null} for now)
       * @return the template
       */
  public static CharBuffer getTemplate( Date time )
  {
    return new Tag( SHA1_HASH_LENGTH, time ).getTag();
  }

  /**
   * Updates the embedded tags within the given buffer.
   * @param data the buffer with the rendered content
   * @return the signed buffer
   */
  public static ByteBuffer sign( ByteBuffer data )
  {
    //png are handled differently since their chunks are crc secured
    boolean isPNG = PNGFormat.isPNG( data );
    if ( isPNG )
    {
      return signPNGBuffer( IMAGE_HANDLER_KEY, data );
    }
    return signBuffer( IMAGE_HANDLER_KEY, data );
  }

  /**
   * Computes the MAC tag and adds it to the image.
   * @param data the buffer with the rendered content
   * @return the signed buffer
   */
  public static ByteBuffer signPNG( ByteBuffer data )
  {
    return signPNGBuffer( IMAGE_HANDLER_KEY, data );
  }

  private static ByteBuffer signPNGBuffer( SecretKey key, ByteBuffer data )
  {
    try
    {
      SignChunkFilter signer = new SignChunkFilter( key );
      ReorderChunksFilter reorder = new ReorderChunksFilter();
      return reorder.filter( signer.filter( reorder.filter( data ) ) );
    }
    catch ( InvalidKeyException e )
    {
      throw new Error( "invalid key", e );
    }
    catch ( NoSuchAlgorithmException e )
    {
      throw new Error( "invalid algorithm", e );
    }
  }

  private static ByteBuffer signBuffer( SecretKey key, ByteBuffer data )
  {
    try
    {

      // find tags
      List<Tag> tags = Tag.findTags( data, true );
      if ( tags.isEmpty() )
      {
        // TODO log no tags found
        return data;
      }

      // calculate hash
      CharBuffer hash = initAndCalc( tags, key, data );

      // write hash
      for ( Tag tag: tags )
      {
        tag.setHexHash( hash );
      }
    }
    catch ( InvalidKeyException e )
    {
      throw new Error( "invalid key", e );
    }
    catch ( NoSuchAlgorithmException e )
    {
      throw new Error( "invalid algorithm", e );
    }
    return data;
  }

  /**
   * Verifies the content. the provided buffer must be writable to do so.
   * 
   * @param data the buffer containing MAC tags
   * @return false if no tags are found, true if all tags are valid.
   * @throws SignatureException if the embedded MAC tags do not match with the content  
   */
  public static boolean verify( ByteBuffer data ) throws SignatureException
  {
    GregorianCalendar gc = new GregorianCalendar();
    gc.set( Calendar.YEAR, 1900 );
    return verify( data, gc.getTime() );
  }

  /**
   * Verifies the content. the provided buffer must be writable to do so.
   * 
   * @param data the buffer containing MAC tags
   * @param firstValidDate MAC's which are created before this date are invalild.
   * @return false if no tags are found, true if all tags are valid.
   * @throws SignatureException if the embedded MAC tags do not match with the content  
   */
  public static boolean verify( ByteBuffer data, Date firstValidDate ) throws SignatureException
  {
    try
    {
      //png are handled differently since their chunks are crc secured
      boolean isPNG = PNGFormat.isPNG( data );
      if ( isPNG )
      {
        return verifyPNG( data, firstValidDate );
      }

      final ByteBuffer writable = IOUtil.ensureWritable( data );

      // find tags
      List<Tag> tags = Tag.findTags( writable, true );
      if ( tags.isEmpty() )
      {
        return false;
      }

      // calculate hash
      CharBuffer hash = initAndCalc( tags, IMAGE_HANDLER_KEY, writable );

      // restore hashes and check
      Tag firstMiss = null;
      for ( Tag tag: tags )
      {
        tag.restoreRememberedHash();
        if ( !hash.equals( tag.getHexHash() ) || tag.getDate().before( firstValidDate ) )
        {
          firstMiss = tag;
        }
      }
      if ( firstMiss != null )
      {
        String msg = "expected: " + firstMiss.getHexHash().toString() + ", actual: " + hash.toString();
        throw new SignatureException( msg );
      }
      return true;
    }
    catch ( InvalidKeyException e )
    {
      throw new Error( "invalid key", e );
    }
    catch ( NoSuchAlgorithmException e )
    {
      throw new Error( "invalid algorithm", e );
    }
  }

  /**
   * Verifies the content of a PNG image.
   * 
   * @param data the buffer containing MAC tags
   * @param firstValidDate MAC's which are created before this date are invalild.
   * @return false if no tags are found, true if all tags are valid.
   * @throws SignatureException if the embedded MAC tags do not match with the content  
   */
  private static boolean verifyPNG( ByteBuffer data, Date firstValidDate ) throws SignatureException
  {
    try
    {
      VerifyChunkFilter verifier = new VerifyChunkFilter( IMAGE_HANDLER_KEY );
      return verifier.verify( data, firstValidDate );
    }
    catch ( InvalidKeyException e )
    {
      throw new Error( "invalid key", e );
    }
    catch ( NoSuchAlgorithmException e )
    {
      throw new Error( "invalid algorithm", e );
    }
  }

  // -------------------------------------------------------------------------

  /**
   * Creates a key from a shared secret passphrase.
   * @param keyphrase the passphrase
   * @return the key
   */
  private static SecretKeySpec createKey( CharSequence keyphrase )
  {
    try
    {
      StringBuilder input = new StringBuilder( keyphrase );
      byte[] part1 = getSHA256BytesOfUTF8EncodedText( input );
      input.append( '_' );
      input.reverse();
      byte[] part2 = getSHA256BytesOfUTF8EncodedText( input );

      if ( part1.length != 32 || part2.length != 32 )
      {
        throw new Error( "expected 32 bytes from SHA256" );
      }

      byte[] key = new byte[64];
      System.arraycopy( part1, 0, key, 0, 32 );
      System.arraycopy( part2, 0, key, 32, 32 );
      return new SecretKeySpec( key, ALGORITHM );

    }
    catch ( NoSuchAlgorithmException e )
    {
      throw new Error( "SHA256 required", e );
    }
    catch ( CharacterCodingException e )
    {
      throw new Error( "invalid encoding", e );
    }
  }

  //-------------------------------------------------------

  /**
   * Calculates the MAC for the provided data.
   * @param key the key to use for the MAC calculation
   * @param data the input data for the MAC calculation
   * @return the MAC bytes
   * @throws NoSuchAlgorithmException if needed cyypto algorithms are not available on the system
   * @throws InvalidKeyException if the provided key is unusable
   */
  private static byte[] createMAC( SecretKey key, ByteBuffer data ) throws NoSuchAlgorithmException, InvalidKeyException
  {
    Mac mac = Mac.getInstance( key.getAlgorithm() );
    mac.init( key );
    mac.update( data );
    return mac.doFinal();
  }

  protected static CharSequence formatAsHex( byte... digestBytes )
  {
    StringBuilder result = new StringBuilder( 2 * digestBytes.length );
    for ( byte b: digestBytes )
    {
      String next = Integer.toHexString( b & 0xFF );
      if ( next.length() < 2 )
      {
        result.append( '0' );
      }
      result.append( next );
    }
    return result;
  }

  private static byte[] getSHA256BytesOfUTF8EncodedText( CharSequence... texts ) throws NoSuchAlgorithmException, CharacterCodingException
  {
    MessageDigest digest = MessageDigest.getInstance( "SHA-256" );
    CharsetEncoder encoder = Charset.forName( "UTF-8" ).newEncoder().onMalformedInput( CodingErrorAction.REPORT ).onUnmappableCharacter(
        CodingErrorAction.REPORT );
    for ( CharSequence part: texts )
    {
      digest.update( encoder.encode( CharBuffer.wrap( part ) ) );
    }
    return digest.digest();
  }

  //-------------------------

  private static CharBuffer initAndCalc( List<Tag> tags, SecretKey key, ByteBuffer data ) throws NoSuchAlgorithmException, InvalidKeyException
  {
    // initialize tags
    for ( Tag tag: tags )
    {
      tag.rememberHashAndWriteTemplateHash();
    }
    return calcMAC( key, data );
  }

  private static CharBuffer calcMAC( SecretKey key, ByteBuffer data ) throws NoSuchAlgorithmException, InvalidKeyException
  {
    return CharBuffer.wrap( formatAsHex( createMAC( key, data.duplicate() ) ) );
  }

  /**
   * Tag in the format "IPI<hex coded hash>@YYYY-MM-DD"
   */
  private static class Tag
  {
    private static final Charset ASCII = Charset.forName( "ASCII" );

    private static final String PREFIX = "IPI";

    private static final byte[] PREFIX_ASCII;

    private static final char DATE_DELIM = '@';

    private static final byte DATE_DELIM_ASCII = 64;

    private static final String DATE_FORMAT = "yyyy-MM-dd";

    static
    {
      try
      {
        PREFIX_ASCII = PREFIX.getBytes( "ASCII" );
      }
      catch ( UnsupportedEncodingException e )
      {
        throw new Error( "ASCII should be available", e );
      }
    }

    private final int numDigits;

    private final Date time;

    private final ByteBuffer asciiHash;

    private ByteBuffer snapshot;

    public Tag( int numDigits, Date time )
    {
      assert numDigits > 0;
      this.asciiHash = null;
      this.numDigits = numDigits;
      this.time = time == null ? new Date() : time;
    }

    public Tag( CharBuffer hash, Date time )
    {
      this( ASCII.encode( hash ), time );
    }

    private Tag( ByteBuffer hash, Date time )
    {
      this.numDigits = hash.remaining();
      this.asciiHash = hash;
      this.time = time == null ? new Date() : time;
    }

    public static List<Tag> findTags( ByteBuffer data, boolean onlyFirst )
    {
      ByteBuffer buf = data.duplicate();
      buf.rewind();
      if ( onlyFirst )
      {
        Tag first = findNextTag( buf );
        return first == null ? Collections.<Tag>emptyList() : Collections.<Tag>singletonList( first );
      }
      List<Tag> tags = new ArrayList<Tag>( 1 );
      while ( true )
      {
        Tag current = findNextTag( buf );
        if ( current == null )
        {
          break;
        }
        tags.add( current );
      }
      return tags;

    }

    private static Tag findNextTag( ByteBuffer data )
    {
      if ( skipAfterPrefix( data ) )
      {
        int hashStart = data.position();
        if ( skipHexDigitsUpToDateDelimiter( data ) )
        {
          ByteBuffer hash = ( (ByteBuffer) data.duplicate().flip().position( hashStart ) ).slice();
          byte delim = data.get();
          assert delim == DATE_DELIM_ASCII;
          Date time = readASCII8601Date( data );
          if ( time == null )
          {
            return findNextTag( data );
          }
          return new Tag( hash, time );
        }
        return findNextTag( data );
      }
      return null;
    }

    private static boolean isValidASCIIHexDigit( byte candidate )
    {
      // 0-9: 48-57
      // A-F: 65-70
      // a-f: 97-102
      return ( candidate >= 48 && candidate <= 57 ) || ( candidate >= 97 && candidate <= 102 ) || ( candidate >= 65 && candidate <= 70 );
    }

    private static Date readASCII8601Date( ByteBuffer data )
    {
      byte[] dateBuf = new byte[10];
      data.get( dateBuf );
      try
      {
        return new SimpleDateFormat( DATE_FORMAT ).parse( new String( dateBuf, "ASCII" ) );
      }
      catch ( UnsupportedEncodingException e )
      {
        throw new Error( "ASCII encoding needed", e );
      }
      catch ( ParseException e )
      {
        return null;
      }
    }

    private static boolean skipAfterPrefix( ByteBuffer data )
    {
      int hitCount = 0;
      byte expected = PREFIX_ASCII[hitCount];
      while ( data.hasRemaining() )
      {
        byte next = data.get();
        if ( next == expected )
        {
          hitCount++;
          if ( hitCount == PREFIX_ASCII.length )
          {
            return true;
          }
          expected = PREFIX_ASCII[hitCount];
        }
        else if ( hitCount > 0 )
        {
          //already had a partial match, but not fully... recover.
          hitCount = 0;
          expected = PREFIX_ASCII[hitCount];
        }
      }
      return false;
    }

    private static boolean skipHexDigitsUpToDateDelimiter( ByteBuffer data )
    {
      while ( data.hasRemaining() )
      {
        byte next = data.get();
        if ( next == DATE_DELIM_ASCII )
        {
          data.position( data.position() - 1 );
          return true;
        }
        else if ( !isValidASCIIHexDigit( next ) )
        {
          return false;
        }
      }
      return false;
    }

    public Date getDate()
    {
      return this.time;
    }

    public CharBuffer getHexHash()
    {
      if ( this.asciiHash == null )
      {
        return createTemplateHash();
      }
      this.asciiHash.rewind();
      CharBuffer hash = ASCII.decode( this.asciiHash );
      return hash;
    }

    public CharBuffer getTag()
    {
      int l = PREFIX_ASCII.length + numDigits + 11;
      CharBuffer template = CharBuffer.allocate( l );
      template.put( PREFIX );
      template.put( this.getHexHash() );
      template.put( DATE_DELIM );
      this.putDate( template );
      template.rewind();
      return template;
    }

    // ---------------------------------------

    public void rememberHashAndWriteTemplateHash()
    {
      if ( this.asciiHash == null )
      {
        throw new IllegalStateException( "no hash available" );
      }
      this.snapshot = ByteBuffer.allocate( this.asciiHash.capacity() );
      this.asciiHash.rewind();
      this.snapshot.put( this.asciiHash );
      this.writeTemplateHash();
    }

    public void restoreRememberedHash()
    {
      if ( this.snapshot != null )
      {
        if ( this.asciiHash == null )
        {
          throw new IllegalStateException( "could not restore hash" );
        }
        this.asciiHash.rewind();
        this.snapshot.rewind();
        this.asciiHash.put( this.snapshot );
        this.snapshot = null;
      }
    }

    // ---------------------------------------

    public void setHexHash( CharSequence hexHash )
    {
      ByteBuffer hashHexBytes = ASCII.encode( CharBuffer.wrap( hexHash ) );
      if ( hashHexBytes.remaining() != this.numDigits )
      {
        throw new IllegalArgumentException( "invalid length" );
      }
      for ( int i = 0; i < this.numDigits; i++ )
      {
        if ( !isValidASCIIHexDigit( hashHexBytes.get( i ) ) )
        {
          throw new IllegalArgumentException( "invalid hash characters" );
        }
      }
      this.asciiHash.rewind();
      this.asciiHash.put( hashHexBytes );
      assert hashHexBytes.remaining() == 0;
    }

    @Override
    public String toString()
    {
      return this.getTag().toString();
    }

    public void writeTemplateHash()
    {
      this.setHexHash( this.createTemplateHash() );
    }

    private CharBuffer createTemplateHash()
    {
      CharBuffer hashBuf = CharBuffer.allocate( this.numDigits );
      for ( int i = 1; i <= this.numDigits; i++ )
      {
        hashBuf.put( Integer.toString( i % 10 ) );
      }
      hashBuf.rewind();
      return hashBuf;
    }

    private void putDate( CharBuffer dst )
    {
      Calendar cal = Calendar.getInstance();
      cal.setTime( time );
      dst.put( String.format( "%4d", cal.get( Calendar.YEAR ) ) );
      dst.put( '-' );
      dst.put( String.format( "%02d", 1 + cal.get( Calendar.MONTH ) ) );
      dst.put( '-' );
      dst.put( String.format( "%02d", cal.get( Calendar.DAY_OF_MONTH ) ) );
    }

  } // inner-class

  //-------------------------------------------------------

  private static abstract class SignatureFilter extends PNGChunkFilter
  {
    protected static final String PREFIX = Format.KEYWORD_MAC + ":";

    protected final Mac mac;

    public SignatureFilter( SecretKey key ) throws NoSuchAlgorithmException, InvalidKeyException
    {
      this.mac = Mac.getInstance( key.getAlgorithm() );
      this.mac.init( key );
    }

    // @see ch.ige.image.format.png.PNGChunkFilter#emitHeader(java.nio.ByteBuffer, java.nio.channels.WritableByteChannel)
    @Override
    protected void emitHeader( ByteBuffer buf, WritableByteChannel out ) throws IOException
    {
      super.emitHeader( buf, out );
      this.mac.reset();
    }

    protected Tag getTag( Date date )
    {
      return new Tag( getHash(), date );
    }

    protected CharBuffer getHash()
    {
      return CharBuffer.wrap( formatAsHex( mac.doFinal() ) );
    }

    protected boolean isPartOfChecksum( EmitableChunk c )
    {
      return !isTime( c ) && !isMACComment( c ) && !c.isIEND();
    }

    protected boolean isMACComment( EmitableChunk c )
    {
      return c.isType( Chunk.TEXT_TYPE ) && c.extractTextFromTextChunk().toString().startsWith( PREFIX );
    }

    protected boolean isTime( EmitableChunk c )
    {
      return c.isType( Chunk.TIME_TYPE );
    }

  } //inner-class

  private static class SignChunkFilter extends SignatureFilter
  {

    public SignChunkFilter( SecretKey key ) throws NoSuchAlgorithmException, InvalidKeyException
    {
      super( key );
    }

    @Override
    protected void handleChunk( EmitableChunk c ) throws IOException
    {
      if ( isPartOfChecksum( c ) )
      {
        mac.update( c.getDataArea() );
      }
      if ( shouldEmitWhenSigning( c ) )
      {
        c.emit();
      }

    }

    private boolean shouldEmitWhenSigning( EmitableChunk c )
    {
      return !isTime( c ) && !isMACComment( c );
    }

    @Override
    protected void beforeLast( WritableByteChannel out ) throws IOException
    {

      EmitableChunk.createTextChunk( "Comment", createSignatureComment(), out ).emit();
      EmitableChunk.createTimeChunkNow( out ).emit();

    }

    private StringBuilder createSignatureComment()
    {
      StringBuilder signature = new StringBuilder( PREFIX );
      signature.append( getTag( null ).getTag() );
      return signature;
    }

  } //inner-class

  private static class VerifyChunkFilter extends SignatureFilter
  {

    public VerifyChunkFilter( SecretKey key ) throws NoSuchAlgorithmException, InvalidKeyException
    {
      super( key );
    }

    public boolean verify( ByteBuffer data, Date firstValidDate ) throws SignatureException
    {
      // find tags
      List<Tag> tags = Tag.findTags( data, true );
      if ( tags.isEmpty() )
      {
        return false;
      }

      // compute the hash
      this.filter( data );
      CharBuffer computed = this.getHash();

      // check the declared hashes match the computed
      Tag firstMiss = null;
      for ( Tag tag: tags )
      {
        CharBuffer declared = tags.get( 0 ).getHexHash();
        if ( !computed.equals( declared ) || tag.getDate().before( firstValidDate ) )
        {
          firstMiss = tag;
        }
      }
      if ( firstMiss != null )
      {
        String msg = "expected: " + firstMiss.getHexHash().toString() + ", actual: " + computed.toString();
        throw new SignatureException( msg );
      }
      return true;
    }

    @Override
    protected void handleChunk( EmitableChunk c )
    {
      if ( isPartOfChecksum( c ) )
      {
        mac.update( c.getDataArea() );
      }
    }

  } //inner-class

}
