package ch.ige.image.format;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import javax.imageio.ImageWriteParam;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;

import org.apache.log4j.Logger;

import ch.ige.image.BitmapImage;
import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.Metadata;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.BuildInfo;

/**
 * JPEG Format.
 * <p>
 * Supports encoding with fixed quality (default=0.8) or adaption of the encoding quality to reach a given filesize.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: JPEGFormat.java,v 1.22 2009/03/19 13:26:20 locher Exp $
 */
public class JPEGFormat extends Format
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.22 $".hashCode();

  private static final Logger LOG = Logger.getLogger( JPEGFormat.class );

  public static final String IDENTIFIER = "jpeg";

  public static final String MIME = "image/jpeg";

  private float quality = 0.8f;

  public JPEGFormat()
  {
    super( IDENTIFIER );
  }

  @Override
  public String getMIMEType()
  {
    return MIME;
  }

  @Override
  public EncodingAdapter getEncodingAdapter( final Image image )
  {
    return new Format.EncodingAdapter()
    {
      private static final int MAX_ITERATIONS = 16;

      private static final int TOLERANCE = 256;

      private static final float MAX_QUALITY = 1.0f;

      private float nextQuality;

      private int iterationCount;

      @Override
      public RenderedImage getPreparedImage( RenderedImage image )
      {
        this.initQualitySequence();
        BufferedImage result = BitmapImage.paintBackdropColor( image, Color.WHITE );
        return result;
      }

      private void initQualitySequence()
      {
        if ( JPEGFormat.this.isSizeAdapting() )
        {
          this.nextQuality = 1.0f;
          this.iterationCount = 0;
        }
        else
        {
          this.nextQuality = JPEGFormat.this.getQuality();
        }
      }

      private float getNextQuality()
      {
        return this.nextQuality;
      }

      @Override
      public boolean resultIsOK( EncodedImage encoded )
      {
        if ( JPEGFormat.this.isSizeAdapting() )
        {
          return resultIsOKWhenSizeAdapting( encoded );
        }
        else
        {
          return super.resultIsOK( encoded );
        }
      }

      private boolean resultIsOKWhenSizeAdapting( EncodedImage encoded )
      {
        final int numBytesTooBig = encoded.getFilesize() - JPEGFormat.this.getTargetSize();
        if ( this.nextQuality >= MAX_QUALITY && numBytesTooBig < 0 )
        {
          // already at max quality and image still too small...
          return true;
        }
        final boolean hasAnotherTry = this.iterationCount < MAX_ITERATIONS;
        final boolean wasTooBig = numBytesTooBig > 0;
        final boolean wasTooSmall = numBytesTooBig + TOLERANCE < 0;
        if ( hasAnotherTry && ( wasTooBig || wasTooSmall ) )
        {
          this.iterationCount++;
          this.nextQuality += ( wasTooBig ? -1 : 1 ) * MAX_QUALITY / ( 1 << this.iterationCount );
          if ( this.nextQuality > MAX_QUALITY )
          {
            this.nextQuality = MAX_QUALITY;
          }
          return false;
        }
        return true;
      }

      @Override
      public void adaptWriteParams( ImageWriteParam params )
      {
        if ( params instanceof JPEGImageWriteParam )
        {
          JPEGImageWriteParam jpegParams = (JPEGImageWriteParam) params;
          jpegParams.setOptimizeHuffmanTables( true );
          jpegParams.setCompressionMode( ImageWriteParam.MODE_EXPLICIT );
          jpegParams.setCompressionQuality( this.getNextQuality() );
        }
      }

      @Override
      public void adaptMetadata( ConversionContext conversionContext, IIOMetadata streamMetadata, IIOMetadata imageMetadata )
      {
        try
        {
          final Metadata metadata = image.getMetadata();
          String dpi = Integer.toString( metadata.getDPI() );

          StringBuilder mergeDoc = new StringBuilder(
              "<javax_imageio_jpeg_image_1.0><JPEGvariety><app0JFIF majorVersion='1' minorVersion='1' resUnits='1' Xdensity='" );
          mergeDoc.append( dpi );
          mergeDoc.append( "' Ydensity='" );
          mergeDoc.append( dpi );
          mergeDoc.append( "' thumbWidth='0' thumbHeight='0' ></app0JFIF></JPEGvariety><markerSequence><com comment='ch.ige.image, " );
          mergeDoc.append( BuildInfo.CVS_TAG );
          mergeDoc.append( "'/><com comment='" );
          mergeDoc.append( Format.KEYWORD_MAC );
          mergeDoc.append( ":" );
          mergeDoc.append( EmbeddedHashes.getTemplate( null ) );
          mergeDoc.append( "' /></markerSequence></javax_imageio_jpeg_image_1.0>" );
          this.mergeMetadata( imageMetadata, "javax_imageio_jpeg_image_1.0", mergeDoc.toString() );
        }
        catch ( MetadataException e )
        {
          // no DPI information available... nothing to do
        }

        catch ( Exception e )
        {
          LOG.warn( "Setting of Resolution Metadata failed", e );
        }

        // --------------------------------------
        // now merge the conversion context
        // TODO solve this with a single merge
        if ( conversionContext != null )
        {
          try
          {
            this.mergeMetadata( imageMetadata, "javax_imageio_jpeg_image_1.0", createConversionContext( conversionContext ) );
          }
          catch ( Exception e )
          {
            LOG.info( "merge of conversion metadata failed", e );
            throw new RuntimeException( e );
          }
        }
      }

      private String createConversionContext( ConversionContext conversionContext )
      {
        StringBuilder mergeDoc = new StringBuilder( "<javax_imageio_jpeg_image_1.0><JPEGvariety></JPEGvariety><markerSequence><com comment='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_METADATA );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getOriginalMetadata() );
        mergeDoc.append( "'/><com comment='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_RULE_NAME );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getRulesName() );
        mergeDoc.append( "'/></markerSequence></javax_imageio_jpeg_image_1.0>" );
        return mergeDoc.toString();
      }
    };
  }

  protected boolean isSizeAdapting()
  {
    return this.quality < 0;
  }

  private int getTargetSize()
  {
    return (int) -this.quality * 1024;
  }

  private float getQuality()
  {
    return this.quality;
  }

  /**
   * Sets the targetsize when encoding.
   * 
   * @param targetSize requested size in kB
   */
  public void setTargetSize( int targetSize )
  {
    if ( targetSize <= 0 )
    {
      throw new IllegalArgumentException( "targetSize must be positive" );
    }
    this.quality = -targetSize;
  }

  /**
   * Sets the encoding quality.
   * 
   * @param quality encoding quality setting. 0.0 &lt; q &lt;=1.0
   */
  public void setQuality( float quality )
  {
    if ( quality <= 0.0 || quality >= 1.0 )
    {
      throw new IllegalArgumentException( "quality must be between 0.0 and 1.0" );
    }
    this.quality = quality;
  }

}
