package ch.ige.image.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.Semaphore;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.Metadata;
import ch.ige.image.format.Format;

/**
 * 
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: MetadataViewer.java,v 1.7 2009/04/21 14:40:23 locher Exp $
 */
public class MetadataViewer extends JPanel
{

  private static final long serialVersionUID = 1L;

  private JScrollPane scpMetadata = null;

  private JTable tblMetadata = null;

  private final MetadataTableModel model;

  /**
   * This is the default constructor
   */
  public MetadataViewer()
  {
    super();
    this.model = new MetadataTableModel();
    initialize();
  }

  /**
   * Gets the metadata.
   * @return the metadata
   */
  public Metadata getMetadata()
  {
    return this.model.getMetadata();
  }

  /**
   * Sets the metadata.
   * @param metadata The metadata to set
   */
  public void setMetadata( Metadata metadata )
  {
    this.model.setMetadata( metadata );
    this.firePropertyChange( "metadata", null, metadata );
  }

  /**
   * Sets the metadata.
   * @param img The metadata to set
   */
  public void setMetadataAndEncodedInfo( EncodedImage img )
  {
    this.model.setMetadataAndEncodedInfo( img );
    this.firePropertyChange( "metadata", null, img == null ? null : img.getMetadata() );
  }

  private void initialize()
  {
    GridBagConstraints gridBagConstraints = new GridBagConstraints();
    gridBagConstraints.fill = GridBagConstraints.BOTH;
    gridBagConstraints.gridy = 0;
    gridBagConstraints.weightx = 1.0;
    gridBagConstraints.weighty = 1.0;
    gridBagConstraints.gridx = 0;
    this.setSize( 623, 118 );
    this.setLayout( new GridBagLayout() );
    this.add( getScpMetadata(), gridBagConstraints );
  }

  /**
   * This method initializes scpMetadata	
   * 	
   * @return javax.swing.JScrollPane	
   */
  private JScrollPane getScpMetadata()
  {
    if ( scpMetadata == null )
    {
      scpMetadata = new JScrollPane();
      scpMetadata.setViewportView( getTblMetadata() );
    }
    return scpMetadata;
  }

  /**
   * This method initializes tblMetadata	
   * 	
   * @return javax.swing.JTable	
   */
  private JTable getTblMetadata()
  {
    if ( tblMetadata == null )
    {
      tblMetadata = new JTable( this.model );
    }
    return tblMetadata;
  }

  //-------------------------

  protected static class MetadataTableModel extends AbstractTableModel
  {
    private static final Logger LOG = Logger.getLogger( MetadataTableModel.class );

    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.7 $".hashCode();

    private volatile Metadata metadata;

    private final Semaphore metadataChangeLock = new Semaphore( 1 );

    private List<String> props = extractMetadata();

    private final ExecutorService extractor = Executors.newSingleThreadExecutor();

    protected final AtomicInteger ticketNumber = new AtomicInteger( 0 );

    /**
     * Gets the metadata.
     * @return the metadata
     */
    public Metadata getMetadata()
    {
      return this.metadata;
    }

    /**
     * Sets the metadata.
     * @param metadata The metadata to set
     */
    public void setMetadata( Metadata metadata )
    {
      assert SwingUtilities.isEventDispatchThread() : "processing in EDT required";
      try
      {
        this.metadataChangeLock.acquire();
      }
      catch ( InterruptedException e )
      {
        LOG.error( "setting metadata failed", e );
        return;
      }
      final int ticket = ticketNumber.incrementAndGet();
      try
      {
        this.metadata = metadata;
        final Future<List<String>> async = asyncExtractMetadata();
        this.props = Collections.emptyList();
        fireTableDataChanged();
        SwingUtilities.invokeLater( new Runnable()
        {
          public void run()
          {
            if ( ticket == ticketNumber.get() )
            {
              try
              {
                props = async.get();
              }
              catch ( InterruptedException e )
              {
                LOG.error( "extracting metadata failed", e );
              }
              catch ( ExecutionException e )
              {
                LOG.error( "extracting metadata failed", e.getCause() );
              }
              fireTableDataChanged();
            }
            else
            {
              LOG.debug( "metadata update skipped" );
            }
          }
        } );
      }
      finally
      {
        this.metadataChangeLock.release();
      }
    }

    public void setMetadataAndEncodedInfo( final EncodedImage img )
    {
      if ( img == null )
      {
        this.setMetadata( null );
        return;
      }

      assert SwingUtilities.isEventDispatchThread() : "processing in EDT required";
      try
      {
        this.metadataChangeLock.acquire();
      }
      catch ( InterruptedException e )
      {
        LOG.error( "setting metadata failed", e );
        return;
      }
      final int ticket = ticketNumber.incrementAndGet();
      try
      {
        this.metadata = img.getMetadata();
        final Future<List<String>> async = asyncExtractMetadata();
        this.props = Collections.emptyList();
        fireTableDataChanged();
        SwingUtilities.invokeLater( new Runnable()
        {
          public void run()
          {
            if ( ticket == ticketNumber.get() )
            {
              String fileSize;
              try
              {
                fileSize = String.format( "%d byte", img.getFilesize() );
              }
              catch ( Exception e )
              {
                fileSize = "?";
              }
              String digest;
              try
              {
                digest = img.getDigest();
              }
              catch ( Exception e )
              {
                digest = "?";
              }
              List<String> p;
              try
              {
                p = async.get();
                List<String> total = new ArrayList<String>( 4 + p.size() );
                total.add( "Filesize" );
                total.add( fileSize );
                total.add( "Digest" );
                total.add( digest );
                total.addAll( p );
                props = total;
              }
              catch ( InterruptedException e )
              {
                LOG.error( "extracting metadata failed", e );
              }
              catch ( ExecutionException e )
              {
                LOG.error( "extracting metadata failed", e.getCause() );
              }
              fireTableDataChanged();
            }
            else
            {
              LOG.debug( "metadata update skipped" );
            }
          }
        } );
      }
      finally
      {
        this.metadataChangeLock.release();
      }
    }

    private Future<List<String>> asyncExtractMetadata()
    {
      return extractor.submit( new Callable<List<String>>()
      {
        public List<String> call() throws Exception
        {
          return extractMetadata();
        }
      } );
    }

    protected List<String> extractMetadata()
    {
      final Metadata m = this.metadata; // Take a snapshot, since the setter is not locked during extraction
      if ( m == null )
      {
        return Collections.emptyList();
      }
      String mac = m.getMAC();
      String orig = m.getOrigMetadata();
      String rule = m.getConversionRuleName();
      Format format = m.getFormat();
      String icc = "?";
      try
      {
        icc = m.hasEmbeddedICCProfile() ? "yes" : "no";
      }
      catch ( Exception e )
      {
        //ignore
      }

      String sizePxString;
      try
      {
        Dimension d = m.getSize( Image.UNIT_PIXEL );
        sizePxString = d.width + " X " + d.height;
      }
      catch ( Exception e )
      {
        sizePxString = "?";
      }
      String sizeMmString;
      try
      {
        Dimension d = m.getSize( Image.UNIT_MILLIMETER );
        sizeMmString = d.width + " X " + d.height;
      }
      catch ( Exception e )
      {
        sizeMmString = "?";
      }
      String dpiString;
      try
      {
        dpiString = Integer.toString( m.getDPI() );
      }
      catch ( Exception e )
      {
        dpiString = "?";
      }
      String colorspace;
      try
      {
        colorspace = m.getColorspaceType();
      }
      catch ( Exception e )
      {
        colorspace = "?";
      }
      return Arrays.<String>asList( "IGE Authentication Code", mac == null ? "-" : mac, "Conversion Rule", rule == null ? "-" : rule, "Original Metadata",
          orig == null ? "-" : orig, "Format", format == null ? "?" : format.toString(), "Size (px)", sizePxString, "Size (mm)", sizeMmString, "DPI",
          dpiString, "Colorspace", colorspace, "Embedded ICC profile", icc );
    }

    // @see javax.swing.table.AbstractTableModel#getColumnName(int)
    @Override
    public String getColumnName( int column )
    {
      switch( column )
      {
        case 0:
          return "Property";
        case 1:
          return "Value";
        default:
          return "";
      }
    }

    // @see javax.swing.table.TableModel#getColumnCount()
    public int getColumnCount()
    {
      return 2;
    }

    // @see javax.swing.table.TableModel#getRowCount()
    public int getRowCount()
    {
      return this.props.size() / 2;
    }

    // @see javax.swing.table.TableModel#getValueAt(int, int)
    public Object getValueAt( int rowIndex, int columnIndex )
    {
      int idx = rowIndex * 2 + columnIndex;
      return ( idx < this.props.size() ) ? this.props.get( idx ) : "";

    }

  }

} //  @jve:decl-index=0:visual-constraint="10,10"
