package ch.ige.image.test;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.util.Arrays;

import ch.ige.image.BufferedImageWrapper;
import ch.ige.image.EncodedImage;
import ch.ige.image.EncodedImageWrapper;
import ch.ige.image.Image;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.ImageIOImage;
import ch.ige.image.Metadata;
import ch.ige.image.MetadataException;

/**
 * Test Imaging Operations of Image types.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 *
 * @author Michael Locher
 * @version $Id: ImageTest.java,v 1.14 2009/03/18 16:37:03 locher Exp $
 */
public class ImageTest extends AbstractImageTester
{

  public ImageTest( String name )
  {
    super( name );
  }

  private EncodedImage load( String name ) throws Exception
  {
    assertNotNull( ImageFactoryTest.class.getResource( name ) );
    return new ImageFactoryImpl().loadURL( ImageFactoryTest.class.getResource( name ) );
  }

  public void testCloneImage() throws Exception
  {
    Dimension RESIZE = new Dimension( 400, 100 );

    EncodedImage orig = this.load( "circle.png" );
    Image resized = orig.resize( RESIZE );

    EncodedImage origClone = (EncodedImage) orig.cloneImage();
    Image resizedClone = resized.cloneImage();

    assertTrue( origClone instanceof ImageIOImage );
    assertTrue( orig.getMetadata() != origClone.getMetadata() );
    assertTrue( orig.asRenderedImage() != origClone.asRenderedImage() );
    assertTrue( orig.asBufferedImage() != origClone.asBufferedImage() );
    assertTrue( orig.getData() != origClone.getData() );
    assertTrue( Arrays.equals( orig.getData(), origClone.getData() ) );

    assertEquals( RESIZE, resized.getMetadata().getSize() );
    assertTrue( resizedClone instanceof BufferedImageWrapper );
    assertTrue( resized.getMetadata() != resizedClone.getMetadata() );
    assertTrue( resized.asRenderedImage() != resizedClone.asRenderedImage() );
    assertTrue( resized.asBufferedImage() != resizedClone.asBufferedImage() );
  }

  public void testResize() throws Exception
  {
    EncodedImage loaded = this.load( "circle.png" );
    Image wrapped = new BufferedImageWrapper( loaded.asRenderedImage(), loaded.getMetadata() );
    this.resizeTest( loaded );
    this.resizeTest( wrapped );
  }

  private void resizeTest( Image img )
  {
    Dimension origSize = img.getMetadata().getSize();
    Dimension newSize = new Dimension( 100, 100 );

    Image resized = img.resize( newSize );
    assertEquals( newSize, resized.getMetadata().getSize() );

    BufferedImage origBI = img.asBufferedImage();
    assertEquals( origSize, new Dimension( origBI.getWidth(), origBI.getHeight() ) );

    BufferedImage resizedBI = resized.asBufferedImage();
    assertEquals( newSize, new Dimension( resizedBI.getWidth(), resizedBI.getHeight() ) );
  }

  public void testGrayscale() throws Exception
  {
    EncodedImage loaded = this.load( "circle.png" );
    Image gray = loaded.grayscaleAndEliminateAlpha();

    Metadata loadedMeta = loaded.getMetadata();
    Metadata grayMeta = gray.getMetadata();

    assertEquals( loadedMeta.getSize(), grayMeta.getSize() );
    assertEquals( loadedMeta.getDPI(), grayMeta.getDPI() );
    assertEquals( "GRAY", grayMeta.getColorspaceType() );
  }

  public void testReadTIFF_CYMK() throws Exception
  {
    Image loaded = this.load( "cmyk.tiff" );
    Metadata meta = loaded.getMetadata();

    assertEquals( new Dimension( 390, 89 ), meta.getSize() );
    assertEquals( "CMYK", meta.getColorspaceType() );
  }

  public void testRGBfromTIFF_CYMK() throws Exception
  {
    EncodedImage loaded = this.load( "cmyk.tiff" );
    Image rgb = loaded.asRGB();

    Metadata loadedMeta = loaded.getMetadata();
    Metadata rgbMeta = rgb.getMetadata();

    assertEquals( loadedMeta.getSize(), rgbMeta.getSize() );
    assertEquals( "RGB", rgbMeta.getColorspaceType() );
  }

  public void testRGBfromJPEG_CYMK() throws Exception
  {
    EncodedImage loaded = this.load( "cmyk.jpg" );
    Image rgb = loaded.asRGB();

    Metadata loadedMeta = loaded.getMetadata();
    Metadata rgbMeta = rgb.getMetadata();

    assertEquals( loadedMeta.getSize(), rgbMeta.getSize() );
    assertEquals( "RGB", rgbMeta.getColorspaceType() );
  }

  public void testRGBfromJPEG_Adobe98RGB() throws Exception
  {
    EncodedImage loaded = this.load( "icc_argb.jpg" );
    Image rgb = loaded.asRGB();

    Metadata loadedMeta = loaded.getMetadata();
    Metadata rgbMeta = rgb.getMetadata();

    assertEquals( loadedMeta.getSize(), rgbMeta.getSize() );
    assertEquals( "RGB", rgbMeta.getColorspaceType() );
  }

  public void testRGBfromTIFF_Adobe98RGB() throws Exception
  {
    EncodedImage loaded = this.load( "icc_argb.tiff" );
    Image rgb = loaded.asRGB();

    Metadata loadedMeta = loaded.getMetadata();
    Metadata rgbMeta = rgb.getMetadata();

    assertEquals( loadedMeta.getSize(), rgbMeta.getSize() );
    assertEquals( "RGB", rgbMeta.getColorspaceType() );
  }

  public void testSHA1Digest()
  {
    EncodedImage img = new EncodedImageWrapper( ByteBuffer.wrap( new byte[] {} ), null );
    assertEquals( "SHA1 of empty image", "da39a3ee5e6b4b0d3255bfef95601890afd80709", img.getDigest() );
    assertEquals( "length of hash", 40, img.getDigest().length() );
  }

  public void testResolution() throws MetadataException
  {
    // 320x320 @ 32DPI -> 10x10 inch -> 254x254 mm

    Image dummy = this.createDummyImage( 320, 320 );

    try
    {
      dummy.getMetadata().getDPI(); // provoke exception
      fail();
    }
    catch ( MetadataException ex )
    {
      // expected ...
    }

    dummy.getWritableMetadata().setDPI( 32 );
    assertEquals( 32, dummy.getMetadata().getDPI() );
    assertEquals( 320, dummy.getMetadata().getSize( Image.UNIT_PIXEL ).width );
    assertEquals( 320, dummy.getMetadata().getSize( Image.UNIT_PIXEL ).height );
    assertEquals( 254, dummy.getMetadata().getSize( Image.UNIT_MILLIMETER ).width );
    assertEquals( 254, dummy.getMetadata().getSize( Image.UNIT_MILLIMETER ).height );
  }

  public void testResolution2() throws Exception
  {
    EncodedImage loaded = this.load( "circle.png" );
    assertEquals( 72, loaded.getMetadata().getDPI() );
    assertEquals( 64, loaded.getMetadata().getSize( Image.UNIT_PIXEL ).width );
    assertEquals( 64, loaded.getMetadata().getSize( Image.UNIT_PIXEL ).height );
    assertEquals( 1 + (int) ( 64.0 / 72.0 * 25.4 ), loaded.getMetadata().getSize( Image.UNIT_MILLIMETER ).width );
    assertEquals( 1 + (int) ( 64.0 / 72.0 * 25.4 ), loaded.getMetadata().getSize( Image.UNIT_MILLIMETER ).height );
  }

}
