package ch.ige.imagehandler.profile;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ch.ige.imagehandler.rule.Rule;
import ch.ige.imagehandler.rule.RuleException;
import ch.ige.imagehandler.rule.RuleList;
import ch.ige.imagehandler.rule.RuleVisitor;
import ch.ige.imagehandler.rule.parameter.Environment;
import ch.ige.imagehandler.rule.parameter.SingleBinding;

/**
 * Default Profile Implementation.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: ProfileImpl.java,v 1.9 2009/04/14 13:28:42 locher Exp $
 */
public class ProfileImpl implements Profile
{

  private final Map<String, RuleList> rulelists;

  private Environment environment;

  public ProfileImpl()
  {
    this.rulelists = new HashMap<String, RuleList>();
  }

  /**
   * Adds a ruleset to the profile.
   * 
   * @param rules
   */
  public void addRules( RuleList rules )
  {
    this.rulelists.put( rules.getName(), rules );
  }

  public List<RuleList> getAllConverterRules()
  {
    List<RuleList> result = new ArrayList<RuleList>( this.rulelists.size() );
    for ( RuleList rule: this.rulelists.values() )
    {
      if ( rule.isConverter() )
      {
        result.add( rule );
      }
    }
    Collections.sort( result, new Comparator<RuleList>()
    {
      public int compare( RuleList rule1, RuleList rule2 )
      {
        return rule1.getName().compareTo( rule2.getName() );
      }
    } );
    return result;
  }

  public List<RuleList> getAllRules()
  {
    List<RuleList> result = new ArrayList<RuleList>( this.rulelists.values() );
    Collections.sort( result, new Comparator<RuleList>()
    {
      public int compare( RuleList rule1, RuleList rule2 )
      {
        return rule1.getName().compareTo( rule2.getName() );
      }
    } );
    return result;
  }

  public RuleList getRules( String name )
  {
    RuleList result = this.rulelists.get( name );
    if ( result == null )
    {
      throw new IllegalArgumentException( "no such rules defined: " + name );
    }
    return result;
  }

  /**
   * Assigns the global parameters.
   * 
   * @param environment the gloabl environment
   */
  public void setEnvironment( Environment environment )
  {
    this.environment = environment;
  }

  @Override
  public String toString()
  {
    StringBuilder result = new StringBuilder( "Profile:\n" );
    result.append( this.environment );
    result.append( '\n' );

    for ( RuleList ruleList: this.rulelists.values() )
    {
      result.append( ruleList.toString() );
      result.append( '\n' );
    }

    return result.toString();
  }

  /**
   * Shows the vistior the rulesets.
   * 
   * @param visitor the visitor.
   * @see "GOF Visitor Pattern"
   */
  protected void accept( RuleVisitor visitor ) throws RuleException
  {
    for ( RuleList ruleList: this.rulelists.values() )
    {
      ruleList.accept( visitor );
    }
  }

  /*
   * @see ch.ige.imagehandler.Profile#checkProfile()
   */
  public void checkProfile() throws RuleException
  {
    this.checkEnvironment();
    this.checkRuleParameters();
  }

  private void checkEnvironment()
  {
    this.checkDefaultDPISetting();
  }

  private void checkDefaultDPISetting()
  {
    int defaultDPI = 0;
    try
    {
      defaultDPI = Integer.parseInt( this.getEnvironmentValue( Profile.DEFAULT_DPI_NAME ) );

    }
    catch ( Exception e )
    {
      throw new IllegalArgumentException( "Default DPI not set" );
    }

    if ( defaultDPI < 1 )
    {
      throw new IllegalArgumentException( "Invalid Default DPI Setting" );
    }
  }

  private void checkRuleParameters() throws RuleException
  {
    this.accept( new RuleVisitor()
    {

      public void handleRule( Rule rule ) throws RuleException
      {
        rule.checkParameters();
      }

    } );
  }

  /**
   * Visits all its rules and registers itself as "parent" profile.
   * 
   * @see Rule#setProfile(Profile)
   */
  public void setRuleParents()
  {
    try
    {
      this.accept( new RuleVisitor()
      {
        public void handleRule( Rule rule ) throws RuleException
        {
          rule.setProfile( ProfileImpl.this );
        }
      } );
    }
    catch ( RuleException e )
    {
      throw new RuntimeException( "setting of rule parents failed", e );
    }

  }

  /*
   * @see ch.ige.imagehandler.Profile#getEnvironmentValue(java.lang.String)
   */
  public String getEnvironmentValue( String name )
  {
    return ( (SingleBinding) this.environment.lookup( name ) ).getValue();
  }

}
