package ch.ige.imagehandler.proxy;

import java.io.IOException;
import java.net.URL;
import java.nio.ByteBuffer;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageFactory;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.rule.RuleException;

/**
 * EJBFacade to the ImageHandler
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lang
 * @version  $Id: ImageHandlerFacadeBean.java,v 1.12 2009/03/19 13:26:22 locher Exp $
 * @ejb.bean 
 *  name="ImageHandlerFacade"  
 *  display-name="ImageHandlerFacade"
 *  view-type="both"
 *  jndi-name="ImageHandler"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="RequiresNew"
 */
public class ImageHandlerFacadeBean implements SessionBean, ImageHandler
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.12 $".hashCode();

  private static final Logger LOG = Logger.getLogger( ImageHandlerFacadeBean.class );

  private static final String PROFILE_URL = "http://localhost:8080/imageprofiles/imageprofile.xml";

  private transient ImageHandler imageHandler;

  // @see javax.ejb.SessionBean#ejbCreate()
  public void ejbCreate()
  {
    // nothing to do
  }

  // @see javax.ejb.SessionBean#ejbActivate()
  public void ejbActivate() throws EJBException
  {
    // nothing to do
  }

  // @see javax.ejb.SessionBean#ejbPassivate()
  public void ejbPassivate() throws EJBException
  {
    // nothing to do
  }

  // @see javax.ejb.SessionBean#ejbRemove()
  public void ejbRemove() throws EJBException
  {
    // nothing to do
  }

  // @see javax.ejb.SessionBean#setSessionContext(javax.ejb.SessionContext)
  public void setSessionContext( SessionContext sc ) throws EJBException
  {
    // nothing to do
  }

  private ImageHandler getImageHandler() throws IOException, SAXException, RuleException
  {
    if ( imageHandler == null )
    {
      imageHandler = ImageHandlerFactory.createLocalInstance( new URL( PROFILE_URL ) );
    }
    return imageHandler;
  }

  private EncodedImage parseImage( byte[] image )
  {
    try
    {
      ByteBuffer data = ByteBuffer.wrap( image );
      return this.getImageFactory().loadBytes( data );
    }
    catch ( Exception e )
    {
      LOG.error( "parse image failed", e );
      throw new EJBException( e );
    }
  }

  /**
   * Checks the image against a ruleset
   * @param image the image
   * @param rulesName the name of the ruleset to use to check the image
   * @return <code>true</code> if the image conforms to the rule, otherwise <code>false</code>
   * @ejb.interface-method
   */
  public boolean check( byte[] image, String rulesName )
  {
    EncodedImage img = parseImage( image );
    try
    {
      return getImageHandler().check( img, rulesName );
    }
    catch ( Exception e )
    {
      LOG.error( "check failed", e );
      throw new EJBException( e );
    }
  }

  /**
   * Converts the image.
   * @param imageData the image
   * @param rulesName the name of the rule to use to convert the image
   * @return the converted image
   * @ejb.interface-method
   */
  public byte[] convert( byte[] imageData, String rulesName )
  {
    if ( imageData == null )
    {
      throw new IllegalArgumentException( "Image is null" );
    }
    EncodedImage img = parseImage( imageData );
    Image converted = null;
    try
    {
      LOG.debug( "Transform the image according to the defined rules." );
      converted = this.getImageHandler().convert( img, rulesName );
      LOG.debug( "image is converted" );
      if ( converted instanceof EncodedImage )
      {
        return ( (EncodedImage) converted ).getData();
      }
      throw new RuntimeException( "output of rule is not an encoded image" );
    }
    catch ( Exception e )
    {
      LOG.error( "convert failed", e );
      throw new EJBException( e );
    }
    finally
    {
      LOG.debug( "Release image resources." );
      if ( img != null )
      {
        img.dispose();
      }
      if ( converted != null )
      {
        converted.dispose();
      }
    }
  }

  //---------------------------------------------
  // Imagehandler interface

  /**
   * Checks the image against a ruleset
   * @param image the image
   * @param rulesName the name of the ruleset to use to check the image
   * @return <code>true</code> if the image conforms to the rule, otherwise <code>false</code>
   * @ejb.interface-method
   */
  public boolean check( Image image, String rulesName )
  {
    try
    {
      return getImageHandler().check( image, rulesName );
    }
    catch ( Exception e )
    {
      LOG.error( "check failed", e );
      throw new EJBException( e );
    }
    finally
    {
      LOG.debug( "release image resources." );
      if ( image != null )
      {
        image.dispose();
      }
    }
  }

  /**
   * Converts an image.
   * @param image the old image
   * @param rulesName the name of the rule to use for the conversion
   * @return the converted image
   * @ejb.interface-method
   */
  public Image convert( Image image, String rulesName )
  {
    try
    {
      return this.getImageHandler().convert( image, rulesName );
    }
    catch ( Exception e )
    {
      LOG.error( "convert failed", e );
      throw new EJBException( e );
    }
    finally
    {
      LOG.debug( "release image resources." );
      if ( image != null )
      {
        image.dispose();
      }
    }
  }

  // @see ch.ige.imagehandler.ImageHandler#checkWithFeedback(ch.ige.image.Image, java.lang.String)
  public void checkWithFeedback( Image image, String rulesName ) throws MetadataException, RuleException
  {
    throw new UnsupportedOperationException();
  }

  // @see ch.ige.imagehandler.ImageHandler#getImageFactory()
  public ImageFactory getImageFactory()
  {
    try
    {
      return this.getImageHandler().getImageFactory();
    }
    catch ( Exception e )
    {
      LOG.error( "getImageFactory failed", e );
      throw new EJBException( e );
    }
  }
}