package ch.ige.imagehandler.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Serializable;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.util.Collection;
import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Semaphore;
import java.util.concurrent.atomic.AtomicLong;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.Metadata;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.BuildInfo;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Handles requests from ProxyingImageHandlers.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.proxy.ProxyingImageHandler
 * @see #doPost(HttpServletRequest, HttpServletResponse)
 * @author Michael Locher
 * @version $Id: ProxyServlet.java,v 1.27 2009/03/19 13:49:22 locher Exp $
 */
public class ProxyServlet extends HttpServlet implements ProxyServletParameters
{
  private static final Logger LOG = Logger.getLogger( ProxyServlet.class );

  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.27 $".hashCode();

  private Map<String, ImageHandler> handlerCache;

  private Map<String, Statistics> statistics;

  private volatile Throwable lastException = null;

  private long lastExceptionMillis;

  private final AtomicLong totalCount = new AtomicLong( 0 );

  private final AtomicLong totalTime = new AtomicLong( 0 );

  private transient Semaphore activeOperations;

  private transient ThreadLocal<ByteBuffer> receiveBuffer;

  /**
   * Creates the HandlerCache.
   * 
   * @see javax.servlet.GenericServlet#init()
   */
  @Override
  public void init( ServletConfig config ) throws ServletException
  {
    String configMaxConcurrentOperations = config.getInitParameter( "maxConcurrentOperations" );
    int maxConcurrentOperations = ( configMaxConcurrentOperations == null ) ? 5 : Integer.parseInt( configMaxConcurrentOperations );
    this.activeOperations = new Semaphore( maxConcurrentOperations, true );
    this.receiveBuffer = new ThreadLocal<ByteBuffer>();
    this.initCache();
    super.init();
  }

  /**
   * Nulls the cache.
   * 
   * @see javax.servlet.Servlet#destroy()
   */
  @Override
  public void destroy()
  {
    this.killCache();
    super.destroy();
  }

  /** see http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html */
  private void preventCaching( HttpServletRequest request, HttpServletResponse response )
  {
    String protocol = request.getProtocol();
    if ( "HTTP/1.0".equalsIgnoreCase( protocol ) )
    {
      response.setHeader( "Pragma", "no-cache" );
    }
    else if ( "HTTP/1.1".equalsIgnoreCase( protocol ) )
    {
      response.setHeader( "Cache-Control", "no-cache" ); // "no-store" works also
    }
    response.setDateHeader( "Expires", 0 );
  }

  // --------------------------------------------------------------------------------------------
  // Cached Imagehandlers

  private synchronized void killCache()
  {
    this.handlerCache = null;
    this.statistics = null;
  }

  private synchronized void initCache()
  {
    this.handlerCache = new ConcurrentHashMap<String, ImageHandler>();
    this.statistics = new ConcurrentHashMap<String, Statistics>();
    this.totalCount.set( 0 );
    this.totalTime.set( 0 );
  }

  private synchronized ImageHandler getHandler( URL profileURL ) throws ServletException
  {
    ImageHandler handler = this.handlerCache.get( profileURL.toString() );
    if ( handler == null )
    {
      try
      {
        handler = ImageHandlerFactory.createLocalInstance( profileURL );
        this.handlerCache.put( profileURL.toString(), handler );
      }
      catch ( Throwable e )
      {
        this.setLastException( e );
        throw new ServletException( "could not create imagehandler for " + profileURL.toString(), e );
      }
    }
    return handler;
  }

  // --------------------------------------------------------------------------------------------
  // GET Handler

  /**
   * Displays the cached profiles and statistics.
   * <p>
   * If CLEAR_CACHE_PARAM is set to "true" the cache will be flushed.
   * 
   * @see ProxyServlet#CLEAR_CACHE_PARAM
   * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse)
   */
  @Override
  public void doGet( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
  {
    this.preventCaching( request, response );
    URL requestURL = new URL( request.getScheme(), request.getServerName(), request.getServerPort(), request.getRequestURI() );

    if ( "true".equals( request.getParameter( CLEAR_CACHE_PARAM ) ) )
    {
      this.initCache();
      response.sendRedirect( requestURL.toExternalForm() );
      return;
    }
    response.setContentType( "text/html" );
    PrintWriter out = response.getWriter();

    String servletPath = request.getServletPath();
    String infoPath = request.getPathInfo();
    if ( infoPath != null )
    {
      this.renderInfoDIV( requestURL, servletPath, out );
    }
    else
    {
      this.renderInfoPage( requestURL, servletPath, out );
    }
  }

  private void renderInfoPage( URL requestURL, String servletPath, PrintWriter out )
  {
    out.println( "<html><head>" );
    out.println( "<script src='js/prototype.js' type='text/javascript'></script>" );
    out.println( "<script src='js/scriptaculous.js' type='text/javascript'></script>" );
    out.println( "<link rel='stylesheet' href='style.css' type='text/css' />" );
    out.println( "</head>" );
    out.println( "<body onload='new Ajax.PeriodicalUpdater(\"content\", \"." + servletPath
        + "/ajax/info-div\", {method: \"get\", frequency: 10, onSuccess: function() { new Effect.Highlight(\"title\", { startcolor: \"#E6B788\" }) } })' >" );
    out.println( "<h1 id='title'>Image Handler</h1>" );
    out.println( "<div id='version'><h2>Version</h2>" );
    out.println( "<p>Tag: " + BuildInfo.CVS_TAG + "</p>" );
    out.println( "<p>ID: " + BuildInfo.BUILD_ID + "</p></div><div id='content'>" );
    this.renderInfoDIV( requestURL, servletPath, out );
    out.println( "</div></body></html>" );
  }

  private void renderInfoDIV( URL requestURL, String servletPath, PrintWriter out )
  {
    this.showHandlerCache( out, servletPath, requestURL );
    this.showException( out );
  }

  private synchronized void showException( PrintWriter out )
  {
    if ( this.lastException != null )
    {
      out.println( "<div id='exception'><h2>Last Exception</h2>" );
      out.println( "<p>Thrown at " + new Date( this.lastExceptionMillis ).toString() + "</p>" );
      out.println( "<pre>" );
      this.lastException.printStackTrace( out );
      out.println( "</pre></div>" );
    }
  }

  @SuppressWarnings( "unused" )
  private void showHandlerCache( PrintWriter out, String servletPath, URL requestURL )
  {
    out.println( "<div id='statistics'><h2>Statistics</h2>" );
    Collection<Statistics> stats = this.getStatistics();
    if ( !stats.isEmpty() )
    {
      out.println( "<a class='clear' href='#none' onclick='new Effect.Shrink(\"content\", { afterFinish: function() { parent.location=\"." + servletPath + "?"
          + CLEAR_CACHE_PARAM + "=true\" } })'>Clear Cache</a>" );
      out.println( "<table><tr><th>Profile</th><th>Call</th><th>Count</th><th>Average Time</th></tr>" );
      for ( Statistics stat: stats )
      {
        stat.printTableRow( out );
      }
      out.println( "<tr class='total'><td colspan='2'>TOTAL</td><td class='numeric'>" + this.totalCount + "</td><td class='numeric'>" + this.totalTime.get()
          / this.totalCount.get() + " ms</td></tr></table>" );
    }
    else
    {
      out.println( "<p>No profile cached</p>" );
    }
    out.println( "<p>Ready image operation slots: " + this.activeOperations.availablePermits() + "</p>" );
    out.println( "<p>Queued image operations: " + this.activeOperations.getQueueLength() + "</p></div>" );
  }

  // --------------------------------------------------------------------------------------------
  // POST Handler

  /**
   * Processes an ImageHandler request.
   * <p>
   * The data of EncodedImages is transported in the request and response bodies. The remaining parameters are
   * transported with HTTP request/response header options.
   * 
   * @see javax.servlet.http.HttpServlet#doPost(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse)
   */
  @Override
  public void doPost( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
  {
    this.preventCaching( request, response );
    try
    {
      long startTime = System.currentTimeMillis();
      URL profileURL = new URL( request.getHeader( HEADER_PROFILE ) );
      ImageHandler handler = this.getHandler( profileURL );
      String rulesName = request.getHeader( HEADER_RULES );
      String operation = request.getHeader( HEADER_OPERATION );
      if ( OP_CHECK.equals( operation ) )
      {
        this.handleCheck( handler, rulesName, request, response );
      }
      else if ( OP_CONVERT.equals( operation ) )
      {
        this.handleConvert( handler, rulesName, request, response );
      }
      else
      {
        throw new ServletException( "operation not supported: " + operation );
      }
      long stopTime = System.currentTimeMillis();
      long serviceTime = stopTime - startTime;
      this.updateStatistics( profileURL, rulesName, operation, serviceTime );
      this.totalCount.incrementAndGet();
      this.totalTime.addAndGet( serviceTime );
    }
    catch ( Throwable t )
    {
      LOG.error( t );
      this.setLastException( t );
    }
  }

  private void handleCheck( ImageHandler handler, String rulesName, HttpServletRequest request, HttpServletResponse response ) throws ServletException,
      IOException
  {
    boolean slotAquired = false;
    try
    {
      ByteBuffer imageBuffer = requestToBuffer( request );
      try
      {
        this.aquireImageOperationSlot();
        slotAquired = true;
      }
      catch ( InterruptedException ex )
      {
        slotAquired = false;
      }
      handler.checkWithFeedback( this.readImage( handler.getImageFactory(), imageBuffer ), rulesName );
      response.addHeader( HEADER_RESULT, RESULT_OK );
      response.flushBuffer();
    }
    catch ( MetadataException e )
    {
      LOG.error( e );
      this.reportException( response, e );
    }
    catch ( RuleException e )
    {
      LOG.error( e );
      this.reportException( response, e );
    }
    catch ( ImageException e )
    {
      LOG.error( e );
      this.reportException( response, e );
    }
    catch ( Exception e )
    {
      LOG.error( e );
      this.reportException( response, e );
      throw new ServletException( "image check failed", e );
    }
    finally
    {
      if ( slotAquired )
      {
        this.releaseImageOperationSlot();
      }
    }
  }

  private void handleConvert( ImageHandler handler, String rulesName, HttpServletRequest request, HttpServletResponse response ) throws ServletException,
      IOException
  {
    try
    {
      ByteBuffer imageBuffer = requestToBuffer( request );
      Image result = performConvert( handler, rulesName, this.readImage( handler.getImageFactory(), imageBuffer ) );
      if ( result instanceof EncodedImage )
      {
        response.setHeader( HEADER_RESULT, RESULT_OK );
        this.sendMetadata( result.getMetadata(), response );
        ByteBuffer resultBuffer = resultToBuffer( (EncodedImage) result );
        result.dispose();
        result = null;
        sendImageData( response, resultBuffer );
      }
      else
      {
        this.reportException( response, new IllegalArgumentException( "result of convert must be encoded image" ) );
      }
    }
    catch ( ImageException e )
    {
      LOG.error( e );
      this.reportException( response, e );
    }
    catch ( RuleException e )
    {
      LOG.error( e );
      this.reportException( response, e );
    }
    catch ( Exception e )
    {
      LOG.error( e );
      this.reportException( response, e );
      throw new ServletException( "image convert failed", e );
    }
  }

  private Image performConvert( ImageHandler handler, String rulesName, Image image ) throws RuleException, ImageException
  {
    boolean slotAquired = false;
    try
    {
      try
      {
        this.aquireImageOperationSlot();
        slotAquired = true;
      }
      catch ( InterruptedException ex )
      {
        slotAquired = false;
      }
      long start = System.currentTimeMillis();
      Image converted = handler.convert( image, rulesName );
      long stop = System.currentTimeMillis();
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( rulesName + " convert time: " + ( stop - start ) + "ms" );
      }
      return converted;
    }
    finally
    {
      if ( slotAquired )
      {
        this.releaseImageOperationSlot();
      }
    }
  }

  private Image readImage( ImageFactory factory, ByteBuffer image ) throws ImageException
  {
    return factory.loadBytes( image );
  }

  private void sendMetadata( Metadata metadata, HttpServletResponse response )
  {
    try
    {
      response.setHeader( HEADER_META_FORMAT, metadata.getFormat().getName() );
      response.setHeader( HEADER_META_PIXELSIZE_X, Integer.toString( metadata.getSize().width ) );
      response.setHeader( HEADER_META_PIXELSIZE_Y, Integer.toString( metadata.getSize().height ) );
      try
      {
        response.setHeader( HEADER_META_DPI, Integer.toString( metadata.getDPI() ) );
      }
      catch ( MetadataException e )
      {
        // ignore
      }
      try
      {
        response.setHeader( HEADER_META_COLORSPACE, metadata.getColorspaceType() );
      }
      catch ( MetadataException e )
      {
        // ignore
      }
    }
    catch ( Exception e )
    {
      System.err.println( "could not send image metadata" );
      e.printStackTrace();
    }
  }

  private ByteBuffer resultToBuffer( EncodedImage encoded )
  {
    byte[] data = encoded.getData();
    ByteBuffer buf = this.getBuffer( data.length );
    buf.put( data );
    buf.flip();
    return buf;
  }

  private void sendImageData( HttpServletResponse response, ByteBuffer resultBuffer ) throws IOException
  {
    WritableByteChannel responseChannel = Channels.newChannel( response.getOutputStream() );
    while ( resultBuffer.hasRemaining() )
    {
      responseChannel.write( resultBuffer );
    }
    response.flushBuffer();
  }

  private void reportException( HttpServletResponse response, Exception e ) throws IOException
  {
    e.printStackTrace( System.err );
    setLastException( e );
    response.addHeader( HEADER_RESULT, RESULT_EXCEPTION );
    response.addHeader( HEADER_EXCEPTION_CLASS, e.getClass().getName() );
    response.getWriter().print( e.getMessage() );
    response.flushBuffer();
  }

  private synchronized void setLastException( Throwable t )
  {
    this.lastExceptionMillis = System.currentTimeMillis();
    this.lastException = t;
  }

  // --------------------------------------------------------------------------------------------
  // Limit Image Operations

  private void aquireImageOperationSlot() throws InterruptedException
  {
    this.activeOperations.acquire();
  }

  private synchronized void releaseImageOperationSlot()
  {
    this.activeOperations.release();
  }

  // --------------------------------------------------------------------------------------------
  // Receive Buffer

  private ByteBuffer requestToBuffer( HttpServletRequest request ) throws IOException
  {
    int len = request.getContentLength();
    ServletInputStream in = request.getInputStream();
    return ( len < 0 ) ? ByteBuffer.wrap( IOUtils.toByteArray( in ) ) : fetchToBuffer( len, in );
  }

  private ByteBuffer fetchToBuffer( int numBytes, InputStream in ) throws IOException
  {
    ByteBuffer buffer = this.getBuffer( numBytes );
    ReadableByteChannel inChannel = Channels.newChannel( in );
    int readCount;
    do
    {
      readCount = inChannel.read( buffer );
    } while ( readCount >= 0 && buffer.hasRemaining() );
    if ( buffer.hasRemaining() )
    {
      LOG.info( "input stream did contain less than the promised amount of data" );
    }
    else
    {
      int firstSurplusByte = in.read();
      if ( firstSurplusByte != -1 )
      {
        LOG.info( "input stream did contain more than the promised amount of data" );
        byte[] remainingSurplusBytes = IOUtils.toByteArray( in );
        ByteBuffer total = ByteBuffer.allocate( buffer.limit() + 1 + remainingSurplusBytes.length );
        buffer.flip();
        total.put( buffer );
        total.put( (byte) firstSurplusByte );
        total.put( remainingSurplusBytes );
        buffer = total;
      }
    }
    buffer.flip();
    return buffer;
  }

  private ByteBuffer getBuffer( int requiredSize )
  {
    ByteBuffer buf = this.receiveBuffer.get();
    if ( buf == null || buf.capacity() < requiredSize )
    {
      int newSize = ( 1 << 18 );
      while ( newSize < requiredSize )
      {
        newSize <<= 1;
      }
      LOG.debug( "allocate direct buffer with size " + newSize );
      buf = ByteBuffer.allocateDirect( newSize );
      this.receiveBuffer.set( buf );
    }
    buf.clear();
    buf.limit( requiredSize );
    return buf;
  }

  // --------------------------------------------------------------------------------------------
  // Statistics

  private static class Statistics implements Serializable
  {
    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.27 $".hashCode();

    private URL profileURL;

    private String rulesName;

    private String operation;

    private long totalTime = 0;

    private long count = 0;

    public Statistics( URL profileURL, String rulesName, String operation )
    {
      this.profileURL = profileURL;
      this.rulesName = rulesName;
      this.operation = operation;
    }

    public String key()
    {
      return createKey( this.profileURL, this.rulesName, this.operation );
    }

    public void register( long serviceTime )
    {
      this.count++;
      this.totalTime += serviceTime;
    }

    public void printTableRow( PrintWriter out )
    {
      out.println( "<tr><td><a href='" );
      out.println( this.profileURL.toExternalForm() );
      out.println( "'>" );
      out.println( this.profileURL.toExternalForm() );
      out.println( "</a></td><td>" );
      out.println( this.operation + " " + this.rulesName );
      out.println( "</td><td class='numeric'>" );
      out.println( this.count );
      out.println( "</td><td class='numeric'>" );
      out.println( this.totalTime / this.count );
      out.println( " ms</td></tr>" );
    }

    public static String createKey( URL profileURL, String rulesName, String operation )
    {
      return profileURL.toExternalForm() + rulesName + operation;
    }

  } // inner-class

  private synchronized Collection<Statistics> getStatistics()
  {
    return this.statistics.values();
  }

  private synchronized void updateStatistics( URL profileURL, String rulesName, String operation, long serviceTime )
  {
    Statistics stat = this.statistics.get( Statistics.createKey( profileURL, rulesName, operation ) );
    if ( stat == null )
    {
      stat = new Statistics( profileURL, rulesName, operation );
      this.statistics.put( stat.key(), stat );
    }
    stat.register( serviceTime );
  }

}
