package ch.ige.imagehandler.rule;

import ch.ige.image.Image;
import ch.ige.image.ImageException;

/**
 * Apply other rules depending on the outcome of a Checker-Rule.
 * 
 * <p>
 * <em>Example</em>
 * <pre>
 *    &lt;if&gt; 
 *      &lt;test&gt;
 *        &lt;!-- a checker rule, required --&gt;
 *      &lt;/test&gt;
 *      &lt;then&gt;
 *        &lt;!-- a rule, applied if the test was successfull --&gt;
 *      &lt;/then&gt;
 *      &lt;else&gt;
 *        &lt;!-- a rule, applied if the test was not successfull --&gt;
 *      &lt;/else&gt;
 *    &lt;/if&gt;
 * </pre>
 * 
 * <code>then</code> and <code>else</code> elements can be omitted. Omitted
 * elements default to the NoOP-Rule.
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.rule.NoOP
 * 
 * @author Michael Locher
 * @version $Id: IfRule.java,v 1.11 2009/04/14 13:28:42 locher Exp $
 */
public class IfRule<I extends Image, O extends Image> extends AbstractRule<I> implements FlowControl<I, O>
{

  private Checker<? super I> testRule;

  private Rule<? super I> thenRule;

  private Rule<? super I> elseRule;

  public IfRule()
  {
    this.testRule = new NoOP<I>();
    this.thenRule = new NoOP<I>();
    this.elseRule = new NoOP<I>();
  }

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "IF";
  }

  /**
   * Sets the <code>test</code> rule.
   * @param rule the checker
   */
  public void setTest( Checker<? super I> rule )
  {
    this.testRule = rule;
  }

  /**
   * Sets the <code>then</code> rule.
   * @param rule the rule
   */
  public void setThen( Rule<? super I> rule )
  {
    this.thenRule = rule;
  }

  /**
   * Sets the <code>else</code> rule.
   * @param rule the rule
   */
  public void setElse( Rule<? super I> rule )
  {
    this.elseRule = rule;
  }

  // @see ch.ige.imagehandler.rule.FlowControl#isConverter()
  public boolean isConverter()
  {
    return this.thenRule instanceof Converter && this.elseRule instanceof Converter;
  }

  @Override
  public String toString()
  {
    StringBuilder result = new StringBuilder( super.toString() );
    result.append( this.testRule.toString() );
    result.append( "THEN\n" );
    result.append( this.thenRule.toString() );
    result.append( "ELSE\n" );
    result.append( this.elseRule.toString() );
    return result.toString();
  }

  /* @see ch.ige.imagehandler.rule.Rule#accept(ch.ige.imagehandler.rule.RuleVisitor) */
  @Override
  public void accept( RuleVisitor visitor ) throws RuleException
  {
    visitor.handleRule( this );

    this.testRule.accept( visitor );
    this.thenRule.accept( visitor );
    this.elseRule.accept( visitor );
  }

  /**
   * Checks the image with the <code>then</code> or <code>else</code>
   * Checker.
   * 
   * @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image)
   */
  public void checkImage( I image ) throws RuleException
  {
    this.logCheck();
    try
    {
      this.testRule.checkImage( image );
      if ( this.thenRule instanceof Checker )
      {
        ( (Checker<I>) this.thenRule ).checkImage( image );
      }
    }
    catch ( RuleException e )
    {
      if ( this.elseRule instanceof Checker )
      {
        ( (Checker<I>) this.elseRule ).checkImage( image );
      }
    }
  }

  /**
   * Converts the image with the <code>then</code> or <code>else</code>
   * Converter.
   * 
   * @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image)
   */
  public O convert( I source ) throws RuleException, ImageException
  {
    this.logConvert();
    O result = (O) source;
    try
    {
      this.testRule.checkImage( source );
      if ( this.thenRule instanceof Converter )
      {
        result = ( (Converter<I, O>) this.thenRule ).convert( source );
      }
    }
    catch ( RuleException e )
    {
      if ( this.elseRule instanceof Converter )
      {
        result = ( (Converter<I, O>) this.elseRule ).convert( source );
      }
    }
    return result;
  }

}
