package ch.ige.imagehandler.test;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;

import org.xml.sax.SAXException;

import ch.ige.image.EncodedImage;
import ch.ige.image.ImageException;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Demonstrates the Usage of the ImageHandler.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: HandlerDemo.java,v 1.6 2009/03/23 13:55:36 locher Exp $
 */
public class HandlerDemo
{

  private static URL PROFILE = null;

  private static URL PROFILE_IN_JAR = null;

  private static URL PROXY = null;

  static
  {
    try
    {
      PROFILE = new URL( "http", "localhost", 8080, "/demo_profile.xml" );
      PROFILE_IN_JAR = HandlerDemo.class.getResource( "demo_profile.xml" );
      PROXY = new URL( "http", "localhost", 8080, "/proxy" );
    }
    catch ( MalformedURLException e )
    {
      // log and abort !
      throw new Error( e );
    }
  }

  private ImageHandler handler;

  private void createLocalHandler()
  {
    try
    {
      this.handler = ImageHandlerFactory.createLocalInstance( PROFILE_IN_JAR );
    }
    catch ( IOException e )
    {
      // URL not accessible
    }
    catch ( SAXException e )
    {
      // XML invalid
    }
    catch ( RuleException e )
    {
      // Profile XML is misconfigured
    }
  }

  private void createProxiedHandler()
  {
    try
    {
      this.handler = ImageHandlerFactory.createProxiedInstance( PROFILE, PROXY );
    }
    catch ( IOException e )
    {
      // URL not accessible
    }
    catch ( SAXException e )
    {
      // XML invalid
    }
    catch ( RuleException e )
    {
      // Profile XML is misconfigured
    }
  }

  private byte[] createThumbnail( ByteBuffer image )
  {
    EncodedImage img = null;
    EncodedImage thumb = null;
    try
    {

      // Step 1: Extract an image from the byte array.
      try
      {
        img = handler.getImageFactory().loadBytes( image );
      }
      catch ( ImageException e )
      {
        // image could not be read (e.g. invalid format, corruption, ...)
      }

      // Step 2: Transform the image accoring to the defined rules.
      try
      {
        thumb = (EncodedImage) this.handler.convert( img, "thumbnailDemo" );
      }
      catch ( RuleException e )
      {
        // Error when evaluating rules
      }
      catch ( ImageException e )
      {
        // Error when modifing image
      }

      // Step 3: Extract the bytes from the converted image.
      return thumb.getData();

    }
    finally
    {
      // Step 4: Release image resources.
      if ( img != null )
      {
        img.dispose();
      }
      if ( thumb != null )
      {
        thumb.dispose();
      }
    }
  }

  private void checkImage( ByteBuffer image )
  {
    EncodedImage img = null;
    try
    {

      // Step 1: Extract an image from the byte array.
      try
      {
        img = handler.getImageFactory().loadBytes( image );
      }
      catch ( ImageException e )
      {
        // image could not be read (e.g. invalid format, corruption, ...)
      }

      // Step 2: Check the image if it conforms to the rules.
      try
      {
        this.handler.checkWithFeedback( img, "checkDemo" );
      }
      catch ( MetadataException e )
      {
        // Metadata for the check is missing
      }
      catch ( RuleException e )
      {
        // Step 3a: Image does not conform
        String reason = e.getMessage();
        System.out.println( reason );
      }
      // Step 3b: Image does conform

    }
    finally
    {
      // Step 4: Release image resources.
      if ( img != null )
      {
        img.dispose();
      }
    }
  }

  private void checkImageSimple( ByteBuffer image )
  {
    EncodedImage img = null;
    try
    {

      // Step 1: Extract an image from the byte array.
      try
      {
        img = handler.getImageFactory().loadBytes( image );
        System.out.println( img.getDigest() );
      }
      catch ( ImageException e )
      {
        // image could not be read (e.g. invalid format, corruption, ...)
      }

      // Step 2: Check the image if it conforms to the rules.
      try
      {
        boolean imageConforms = this.handler.check( img, "checkDemo" );
        System.out.println( imageConforms ? "OK" : "NOK" );
      }
      catch ( MetadataException e )
      {
        // Metadata for the check is missing
      }

    }
    finally
    {
      // Step 3: Release image resources.
      if ( img != null )
      {
        img.dispose();
      }
    }
  }

}
