package ch.ige.ipcloadscheme.client;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;

import ch.ige.ipcloadscheme.BuildInfo;
import ch.ige.ipcloadscheme.client.image.FiguresProcess;
import ch.ige.ipcloadscheme.client.text.TextProcess;
import ch.ige.ipcloadscheme.client.transfer.Transfer;
import ch.ige.util.client.batch.ArgumentConverter;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReportMail;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;
import ch.ige.util.client.batch.OptionWithDefault;
import ch.ige.util.constants.DateTimeConstants;

/**
 * Processes the IPC schemes.
 *
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version  $Id: IPCLoad.java,v 1.25 2008/07/10 08:11:46 wild Exp $
 */
public class IPCLoad extends NewAbstractBatch
{
  /**
   * The name for this.
   */
  private static final String APPLICATION_NAME = "ipcloadscheme";

  /**
   * The languages that can be used for {@link #LANGUAGE}.
   */
  public static final Locale[] ACCEPTED_LANGUAGES = { Locale.GERMAN, Locale.FRENCH, Locale.ENGLISH };

  /**
   * Used when the batch run can't be initialized.
   */
  private static final BatchReturnCode INITIALIZATION_FAILED = new BatchReturnCode( 3, "Fehler bei der Initialisierung des Batch Job: {0} / Properties: {1}" );

  /**
   * The short option for the program mode.
   */
  private static final String PROGRAM_MODE = "m";

  /**
   * The long option for the program mode.
   */
  private static final String PROGRAM_MODE_LONG = "mode";

  /**
   * The short option for the date to process.
   */
  private static final String PROCESS_DATE = "d";

  /**
   * The long option for the date to process.
   */
  private static final String PROCESS_DATE_LONG = "date";

  /**
   * The short option for the language.
   */
  private static final String LANGUAGE = "l";

  /**
   * The long option for the language.
   */
  private static final String LANGUAGE_LONG = "language";

  /**
   * The short option for the file type to process.
   */
  private static final String FILE_TYPE = "f";

  /**
   * The long option for the file type to process.
   */
  private static final String FILE_TYPE_LONG = "file";

  /**
   * The short option for update or create.
   */
  private static final String SAVE_MODE = "s";

  /**
   * The long option for update or create.
   */
  private static final String SAVE_MODE_LONG = "save";

  /**
   * The CLI options to use.
   */
  protected static final Options OPTIONS;

  /**
   * Possible values for <code>PROGRAM_MODE</code>.
   */
  public static enum ProgramMode
  {
    /**
     * Process all in one step.
     */
    ALL,

    /**
     * Transfer files from ibis-server to disc.
     */
    TRANSFER,

    /**
     * Load IPC descriptions into database.
     */
    TEXT,

    /**
     * Load images into database.
     */
    IMAGE
  }

  /**
   * Possible values for <code>FILE_TYPE</code>.
   */
  public static enum FileType
  {
    /**
     * Transfers all file types. This is the default.
     */
    ALL,

    /**
     * Transfers only text files.
     */
    TEXT,

    /**
     * Transfers only image files.
     */
    IMG
  }

  /**
   * Possible values for <code>SAVE_MODE</code>.
   */
  public static enum SaveMode
  {
    /**
     * All IPC symbols are deleted and created anew.
     */
    CREATE,

    /**
     * Only the text of the IPC symbols is modified. This is the default.
     */
    UPDATE
  }

  static
  {
    Option programModeOption = new Option( PROGRAM_MODE, PROGRAM_MODE_LONG, true, "program mode" );
    programModeOption.setType( ProgramMode.class );
    programModeOption.setRequired( true );

    Option dateOption = new Option( PROCESS_DATE, PROCESS_DATE_LONG, true, "date to process" );
    dateOption.setType( Date.class );
    dateOption.setRequired( true );
    dateOption.setArgName( DateTimeConstants.IGE_DATE_FORMAT_SHORT );

    Option languageOption = new Option( LANGUAGE, LANGUAGE_LONG, true, "language (without param all languages are processed)" );
    languageOption.setType( Locale.class );
    languageOption.setRequired( false );
    StringBuilder languageArgument = new StringBuilder();
    for ( Locale language: ACCEPTED_LANGUAGES )
    {
      if ( languageArgument.length() > 0 )
      {
        languageArgument.append( "|" );
      }
      languageArgument.append( language.getLanguage() );
    }
    languageOption.setArgName( languageArgument.toString() );

    OptionWithDefault fileTypeOption = new OptionWithDefault( FILE_TYPE, FILE_TYPE_LONG, true,
        "type of file to transfer (without param both files are transfered)" );
    fileTypeOption.setType( FileType.class );
    fileTypeOption.setRequired( false );
    fileTypeOption.setDefaultValue( FileType.ALL );

    OptionWithDefault saveModeOption = new OptionWithDefault( SAVE_MODE, SAVE_MODE_LONG, true,
        "save mode (create=All IPC symbols are deleted and created anew / update=Only the text of the IPC symbols is modified (the default))" );
    saveModeOption.setType( SaveMode.class );
    saveModeOption.setRequired( false );
    saveModeOption.setDefaultValue( SaveMode.UPDATE );

    OPTIONS = new Options();
    OPTIONS.addOption( programModeOption );
    OPTIONS.addOption( dateOption );
    OPTIONS.addOption( languageOption );
    OPTIONS.addOption( fileTypeOption );
    OPTIONS.addOption( saveModeOption );
  }

  /**
   * The program mode from the command line.
   */
  private ProgramMode programMode;

  /**
   * The process date.
   */
  private Date processDate;

  /**
   * The file type from the command line.
   */
  private FileType fileType;

  /**
   * The save mode from the command line.
   */
  private SaveMode saveMode;

  /**
   * The languages from the command line.
   */
  private List<Locale> languages;

  /**
   * Creates this.
   */
  public IPCLoad()
  {
    super( APPLICATION_NAME, OPTIONS );
  }

  /**
   * Creates the batch run <strong>for tests</strong>.
   * @param applicationName The application name of this batch, used in logs etc.
   * @param options The command-line options or <code>null</code> if the command-line is ignored
   * @param endWithSystemExit If this is <code>true</code>, the batch run ends with a <code>System.exit()</code>,
   *        otherwise it just returns.
   */
  public IPCLoad( String applicationName, Options options, boolean endWithSystemExit )
  {
    super( applicationName, options, endWithSystemExit );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getPossibleReturnCodes()
  @Override
  protected BatchReturnCode[] getPossibleReturnCodes()
  {
    return BatchReturnCode.getBatchReturnCodes( IPCLoad.class );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getBatchReports()
  @Override
  public BatchReport[] getBatchReports() throws BatchException
  {
    String smtpServer = SchemeProperties.getInstance().getProperty( SchemeProperties.MAIL_SMTP_HOST );
    String from = SchemeProperties.getInstance().getProperty( SchemeProperties.MAIL_SEND_FROM );
    String to = SchemeProperties.getInstance().getProperty( SchemeProperties.MAIL_SEND_TO );
    String cc = SchemeProperties.getInstance().getProperty( SchemeProperties.MAIL_SEND_CC );
    String subject = BuildInfo.BUILD_ENV_NAME + " - " + "Batch Job: " + APPLICATION_NAME;

    return new BatchReport[] { new BatchReportMail( smtpServer, from, to, cc, subject ) };
  }

  /**
   * Initializes the batch job.
   * @param parsedCommandLine The parsed command line
   * @throws BatchException if the initialization fails
   */
  @SuppressWarnings( "unchecked" )
  private void init( Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.debug( "init - begin" );

    try
    {
      LOG.debug( "Properties \n" + SchemeProperties.getInstance().toString() );
    }
    catch ( Exception ex )
    {
      throw new BatchException( INITIALIZATION_FAILED.formatMessage( ex.getMessage(), SchemeProperties.getInstance().toString() ) );
    }

    this.programMode = (ProgramMode) parsedCommandLine.get( PROGRAM_MODE );
    this.processDate = (Date) parsedCommandLine.get( PROCESS_DATE );
    this.fileType = (FileType) parsedCommandLine.get( FILE_TYPE );
    this.saveMode = (SaveMode) parsedCommandLine.get( SAVE_MODE );
    this.languages = (List<Locale>) parsedCommandLine.get( LANGUAGE );

    LOG.debug( "init - done" );
  }

  /**
   * Starts the file transfer to load the ZIP files.
   * @throws BatchException if the processing fails
   */
  private void startTransfer() throws BatchException
  {
    Transfer transfer = new Transfer( this.processDate );
    transfer.process( this.fileType, this.languages );

    if ( transfer.isErrorOccured() )
    {
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR.formatMessage( transfer.getLogMessage() ) );
    }
  }

  /**
   * Processes the texts.
   * @throws BatchException if the processing fails
   */
  private void startTextProcess() throws BatchException
  {
    TextProcess textProcess = new TextProcess( this.processDate );
    textProcess.process( this.languages, this.saveMode );

    if ( textProcess.isErrorOccured() )
    {
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR.formatMessage( textProcess.getLogMessage() ) );
    }
  }

  /**
   * Processes the images.
   * @throws BatchException if the processing fails
   */
  private void startImageProcess() throws BatchException
  {
    FiguresProcess figureProcess = new FiguresProcess( this.processDate );
    figureProcess.process();

    if ( figureProcess.isErrorOccured() )
    {
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR.formatMessage( figureProcess.getLogMessage() ) );
    }
  }

  /**
   * Processes everything.
   * @throws BatchException if the processing fails
   * @see #startTransfer()
   * @see #startTextProcess()
   * @see #startImageProcess()
   */
  private void startAll() throws BatchException
  {
    StringBuilder errorMessage = new StringBuilder();

    try
    {
      startTransfer();
    }
    catch ( BatchException e )
    {
      errorMessage.append( e.getMessage() );
    }

    try
    {
      startImageProcess();
    }
    catch ( BatchException e )
    {
      errorMessage.append( e.getMessage() );
    }

    try
    {
      startTextProcess();
    }
    catch ( BatchException e )
    {
      errorMessage.append( e.getMessage() );
    }

    if ( errorMessage.length() > 0 )
    {
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR.formatMessage( errorMessage ) );
    }
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#checkCommandLine(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( { "unused", "unchecked" } )
  protected void checkCommandLine( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws IllegalArgumentException
  {
    // Check the language: By default, all languages are taken.
    Locale language = (Locale) parsedCommandLine.get( LANGUAGE );
    Option languageOption = null;
    for ( Option commandLineOption: (Collection<Option>) OPTIONS.getOptions() )
    {
      if ( commandLineOption.getOpt().equals( LANGUAGE ) )
      {
        languageOption = commandLineOption;
        break;
      }
    }
    if ( languageOption == null )
    {
      throw new IllegalArgumentException( "Found no language Option" );
    }

    List<Locale> allLanguages = new ArrayList<Locale>();
    if ( language == null )
    {
      allLanguages = Arrays.asList( ACCEPTED_LANGUAGES );
    }
    else
    {
      boolean languageFound = false;
      for ( Locale acceptedLanguage: ACCEPTED_LANGUAGES )
      {
        if ( language.equals( acceptedLanguage ) )
        {
          languageFound = true;
          break;
        }
      }

      if ( languageFound )
      {
        allLanguages.add( language );
      }
      else
      {
        throw new IllegalArgumentException( "Invalid language " + language.getLanguage() );
      }
    }
    ArgumentConverter.putOptionValue( parsedCommandLine, languageOption, allLanguages );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#run(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( "unused" )
  protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.debug( "### Load IPC Scheme - main... ###" );

    init( parsedCommandLine );

    switch( this.programMode )
    {
      case ALL:
        startAll();
        break;
      case TRANSFER:
        startTransfer();
        break;
      case IMAGE:
        startImageProcess();
        break;
      case TEXT:
        startTextProcess();
        break;
    }

    LOG.debug( "### Load IPC Scheme - main successfully done ###" );
    writeBatchReportLine( "Configuration" );
    writeBatchReportLine( SchemeProperties.getInstance().toString() );
  }

  /**
   * Runs this.
   * @param args The command line arguments, see {@link #OPTIONS}
   */
  public static void main( String[] args )
  {
    IPCLoad ipcLoad = new IPCLoad();
    NewAbstractBatch.start( ipcLoad, args );
  }

  protected String getBuildInfo()
  {
    return BuildInfo.BUILD_ID;
  }

  protected String getCvsTag()
  {
    return BuildInfo.CVS_TAG;
  }
}
