package ch.ige.ipcloadscheme.client;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.util.validation.Check;

/**
 * Handling Scheme Properties - Programmeigenschaften anhand Property Datei setzen
 * Pfad der Property Datei wird in der Batch-Prozedur gesetzt
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A.Rueegsegger
 * @version $Id: SchemeProperties.java,v 1.17 2008/07/01 06:27:02 lewis_lee Exp $
 */
public class SchemeProperties
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( SchemeProperties.class );

  /**
   * The key for finding the configuration properties.
   */
  public static final String CONFIG = "ipcloadscheme.config";

  /**
   * The HTTP proxy's host name.
   */
  public static final String HTTP_PROXY_HOST = "http.proxyHost";

  /**
   * The HTTP proxy's port.
   */
  public static final String HTTP_PROXY_PORT = "http.proxyPort";

  /**
   * The user for the HTTP proxy.
   */
  public static final String HTTP_USER = "http.user";

  /**
   * The password for the HTTP proxy.
   */
  public static final String HTTP_PW = "http.password";

  /**
   * A comma-separated list of host names that must be ignored by the HTTP proxy.
   */
  public static final String HTTP_NON_PROXY = "http.nonProxyHosts";

  /**
   * The mail host.
   */
  public static final String MAIL_SMTP_HOST = "mail.smtp.host";

  /**
   * The mail recipient.
   */
  public static final String MAIL_SEND_TO = "mail.send.to";

  /**
   * The mail copy recipient.
   */
  public static final String MAIL_SEND_CC = "mail.send.cc";

  /**
   * The mail sender.
   */
  public static final String MAIL_SEND_FROM = "mail.send.from";

  // Text in XML-Format
  public static final String DOWNLOAD_URL_TEXT = "download.url.text.";

  public static final String TEXT_PATHNAME = "text.pathname.";

  public static final String TEXT_FILE = "text.file.";

  public static final String LANG_FILE = "lang.file.";

  // Images
  public static final String IMAGE_URL = "image.url";

  public static final String IMAGE_ZIPFILE_TARGET = "image.zipfile.target";

  public static final String IMAGE_ZIPFILE_NAME = "image.zipfile.name";

  public static final String IMAGE_ZIPFILE_UNZIP = "image.zipfile.unzip";

  public static final String IMAGE_FILE_NAME = "image.file.name";

  /**
   * The path where the base configuration Properties file is found.
   */
  public static final String CONFIG_PATH = "config.path";

  /**
   * The language-dependent XSL file used to transform the XML document.
   */
  public static final String XSL_FILE = "xsl.file.";

  //TODO Is this still used?
  public static final String XSL_IMAGE = "xsl.image";

  /**
   * These keys are alyways set as System properties.
   */
  private static String[] SYSTEM_KEYS = { HTTP_PROXY_HOST, HTTP_NON_PROXY, HTTP_PROXY_PORT, HTTP_USER, HTTP_PW };

  /**
   * All the keys.
   */
  public static String[] KEYS = { CONFIG, HTTP_PROXY_HOST, HTTP_NON_PROXY, HTTP_PROXY_PORT, HTTP_USER, HTTP_PW, MAIL_SMTP_HOST, MAIL_SEND_TO, MAIL_SEND_CC,
      MAIL_SEND_FROM, TEXT_PATHNAME, DOWNLOAD_URL_TEXT, TEXT_FILE, LANG_FILE, IMAGE_URL, IMAGE_ZIPFILE_TARGET, IMAGE_ZIPFILE_NAME, IMAGE_ZIPFILE_UNZIP,
      IMAGE_FILE_NAME, CONFIG_PATH };

  /**
   * The process date is formatted to this within the file name.
   */
  private static final String PROCESS_DATE_FORMAT_FOR_FILE = "yyyyMMdd";

  /**
   * Formats the date to {@link #PROCESS_DATE_FORMAT_FOR_FILE}.
   */
  private static final SimpleDateFormat PROCESS_DATE_FORMATTER = new SimpleDateFormat( SchemeProperties.PROCESS_DATE_FORMAT_FOR_FILE );

  /**
   * The only instance of this.
   */
  private static final SchemeProperties INSTANCE = new SchemeProperties();

  /**
   * The properties as read from the file configured by {@link #CONFIG}.
   */
  private Properties fileProperties;

  /**
   * The only constructor.
   * It's private since this is a Signleton.
   */
  private SchemeProperties()
  {
    try
    {
      this.fileProperties = readProperties();
    }
    catch ( Exception e )
    {
      LOG.error( "Can't read properties from " + System.getProperty( CONFIG ), e );
    }

    for ( String systemKey: SYSTEM_KEYS )
    {
      String value = getProperty( systemKey );
      LOG.info( "Setting System Property '" + systemKey + "' to '" + value + "'" );
      System.setProperty( systemKey, value );
    }
  }

  /**
   * Gets the only instance of this.
   * @return The only instance
   */
  public static SchemeProperties getInstance()
  {
    return INSTANCE;
  }

  /**
   * Returns the processing date in the formatted form.
   * Normally, {@link #getProperty(String, Locale, Date)} should be used!
   * @param processingDate The processing date
   * @return The formatted processing date, used for file names etc.
   */
  public static String getFormattedProcessingDate( Date processingDate )
  {
    Check.notNull( processingDate, "processing date" );
    return PROCESS_DATE_FORMATTER.format( processingDate );
  }

  /**
   * Gets a property, either from the file configured in
   * {@link #CONFIG} or from a System property, which overrides the file property.
   * @param key The key to look for
   * @return The value for the property
   */
  public String getProperty( String key )
  {
    return getProperty( key, false );
  }

  /**
   * Gets a property, either from the file configured in
   * {@link #CONFIG} or from a System property, which overrides the file property.
   * @param key The key to look for
   * @param ignoreProcessingDate If this is <code>true</code>, the processing date in the value is left as is;
   *        if it's <code>false</code>, an {@link IllegalArgumentException} occurs, because {@link #getProperty(String, Locale, Date)}
   *        should be used
   * @return The value for the property
   */
  private String getProperty( String key, boolean ignoreProcessingDate )
  {
    Check.notEmpty( key, "property key" );

    if ( isLanguageKey( key ) )
    {
      throw new IllegalArgumentException( key + " is a language-dependent key, use it with a Locale" );
    }

    String valueToUse = null;
    String systemValue = System.getProperty( key );
    if ( StringUtils.isNotEmpty( systemValue ) && !systemValue.equalsIgnoreCase( "null" ) )
    {
      valueToUse = systemValue;
    }
    else
    {
      String fileValue = this.fileProperties.getProperty( key );
      if ( StringUtils.isNotEmpty( fileValue ) && !fileValue.equalsIgnoreCase( "null" ) )
      {
        valueToUse = fileValue;
      }
      else
      {
        throw new IllegalArgumentException( "Unknown property key " + key );
      }
    }

    if ( !ignoreProcessingDate && isProcessingDateValue( valueToUse ) )
    {
      throw new IllegalArgumentException( "Value '" + valueToUse + "' contains the processing date placeholder, use it with a Date" );
    }
    else
    {
      return valueToUse;
    }
  }

  /**
   * Gets a property that's language-dependent. 
   * @param key The key stub
   * @param language The language, which is appended to <code>key</code> using {@link Locale#getLanguage()}
   * @return The value for the property
   */
  public String getProperty( String key, Locale language )
  {
    return getProperty( key, language, false );
  }

  /**
   * Gets a property that's language-dependent. 
   * @param key The key stub
   * @param language The language, which is appended to <code>key</code> using {@link Locale#getLanguage()}
   * @param ignoreProcessingDate If this is <code>true</code>, the processing date in the value is left as is;
   *        if it's <code>false</code>, an {@link IllegalArgumentException} occurs, because {@link #getProperty(String, Locale, Date)}
   *        should be used
   * @return The value for the property
   */
  private String getProperty( String key, Locale language, boolean ignoreProcessingDate )
  {
    Check.notEmpty( key, "property key" );
    Check.notNull( language, "language" );

    return getProperty( getLanguageKey( key, language ), ignoreProcessingDate );
  }

  /**
   * Gets a property that's language-dependent and that contains a date placeholder to replace.
   * @param key The key stub
   * @param processingDate The processing date to insert into the value
   * @return The value for the property, including the processing date
   */
  public String getProperty( String key, Date processingDate )
  {
    Check.notNull( processingDate, "processing date" );

    String value = getProperty( key, true );
    return replaceProcessingDate( value, processingDate );
  }

  /**
   * Gets a property that's language-dependent and that contains a date placeholder to replace.
   * @param key The key stub
   * @param language The language, which is appended to <code>key</code> using {@link Locale#getLanguage()}
   * @param processingDate The processing date to insert into the value
   * @return The value for the property, including the processing date
   */
  public String getProperty( String key, Locale language, Date processingDate )
  {
    Check.notNull( processingDate, "processing date" );

    String value = getProperty( key, language, true );
    return replaceProcessingDate( value, processingDate );
  }

  /**
   * Replaces the processing date in the value.
   * @param value The value containing the processing date placeholder
   * @param processingDate The processing date
   * @return The value including the processing date
   */
  private String replaceProcessingDate( String value, Date processingDate )
  {
    if ( value.contains( PROCESS_DATE_FORMAT_FOR_FILE ) )
    {
      String formattedProcessingDate = getFormattedProcessingDate( processingDate );
      return value.replaceAll( PROCESS_DATE_FORMAT_FOR_FILE, formattedProcessingDate );
    }
    else
    {
      throw new IllegalArgumentException( "Value '" + value + "' contains no date to replace." );
    }
  }

  /**
   * Gets a language-dependent key.
   * @param key The key stub
   * @param language The language, which is appended to <code>key</code> using {@link Locale#getLanguage()}
   * @return The language-dependent key
   */
  private String getLanguageKey( String key, Locale language )
  {
    if ( isLanguageKey( key ) )
    {
      return key + language.getLanguage();
    }
    else
    {
      throw new IllegalArgumentException( "Illegal language-dependent key " + key );
    }
  }

  /**
   * Shows whether the key is language-dependent.
   * @param key The key to check
   * @return <code>true</code> if the key is language-dependent, otherwise <code>false</code>
   */
  private boolean isLanguageKey( String key )
  {
    return key.endsWith( "." );
  }

  /**
   * Shows whether the value contains a processing date placeholder.
   * @param value The value to check
   * @return <code>true</code> if the value contains a processing date placeholder, otherwise <code>false</code>
   */
  private boolean isProcessingDateValue( String value )
  {
    return value.contains( PROCESS_DATE_FORMAT_FOR_FILE );
  }

  /**
   * Reads al properties from the file configured in {@link #CONFIG}.
   * @return The properties from the file
   * @throws MalformedURLException if the file's URL is invalid
   * @throws IOException if the file can't be read
   */
  private static Properties readProperties() throws MalformedURLException, IOException
  {
    // Property ipcloadvalidity.config wird in Batch-Prozedur gesetzt
    String fileUrl = System.getProperty( CONFIG );
    LOG.debug( "Reading properties from " + fileUrl );

    URL propsFileURL = new URL( fileUrl );
    Properties properties = new Properties();
    properties.load( propsFileURL.openStream() );
    return properties;
  }

  // @see java.lang.Object#toString()
  @Override
  public String toString()
  {
    StringBuffer description = new StringBuffer();
    for ( String key: KEYS )
    {
      if ( isLanguageKey( key ) )
      {
        for ( Locale acceptedLanguage: IPCLoad.ACCEPTED_LANGUAGES )
        {
          description.append( "  " + getLanguageKey( key, acceptedLanguage ) + " = " + getProperty( key, acceptedLanguage, true ) + "\n" );
        }
      }
      else
      {
        description.append( "  " + key + " = " + getProperty( key, true ) + "\n" );
      }
    }
    return description.toString();
  }

}
