package ch.ige.ipcloadvalidity.client.process;


import java.io.FileInputStream;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.ipcloadvalidity.client.ValidityFile;
import ch.ige.ipcloadvalidity.util.StopWatch;
import ch.ige.ipcloadvalidity.vo.IpcVO;
import ch.ige.ipcloadvalidity.vo.StatistikVO;
import ch.ige.ipcloadvalidity.xml.generate.IpcrSymbol;
import ch.ige.ipcloadvalidity.xml.generate.IpcrValidityList;
import ch.ige.ipcloadvalidity.xml.generate.SubclassDescription;
import ch.ige.ipcloadvalidity.xml.generate.SubclassList;


/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: ReadFile.java,v 1.13 2007/01/24 14:44:30 ruegsegger Exp $
 */
public class ReadFile
{
  private static final Logger LOG = Logger.getLogger( ReadFile.class );
  private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
  private boolean errorOccured = false;
  private StringBuffer logMsg = new StringBuffer();
  private StatistikVO statistikVO = new StatistikVO();
  private Storage storage = new Storage();
  private StopWatch sw = new StopWatch();


  public ReadFile() throws Exception
  {
    sdf.setLenient(false);
  }


  public boolean isErrorOccured()
  {
    return errorOccured;
  }

  /**
   * Gibt die geloggten Debugmeldungen zurück
   * @return String
   */
  public String getLogMessage()
  {
    return this.logMsg.toString();
  }

  /**
   * Gibt die Statistik zurück
   * @return String
   */
  public String getStatistik()
  {
    return this.statistikVO.toString();
  }


  /**
   * Fügt übergebenen String der Fehlermeldung hinzu
   * @param message anzufügende Fehlermeldung
   */
  private void logAndAppendMsg(String message)
  {
    LOG.error(message);
    logMsg.append("\n");
    logMsg.append(message);
  }



  public void process(ValidityFile validityFile)
  {
    LOG.info("process file: " + validityFile.getName());

    try
    {
      InputStream is = new FileInputStream( validityFile );

      IpcrValidityList ipcrValidityList = IpcrValidityList.unmarshal(is);
      processList(ipcrValidityList, validityFile);
      is.close();
    }
    catch (IllegalStateException iex)
    {
      errorOccured = true;
      logAndAppendMsg( "Unguetiges Datum. Verarbeitung abgebrochen " + iex.getMessage());
    }
    catch ( Exception ex )
    {
      System.out.println(ex);
      errorOccured = true;
      logAndAppendMsg("Verarbeitung abgebrochen Exception: " + ex);
    }
  }


  private void processList(IpcrValidityList ipcrValidityList, ValidityFile validityFile) throws Exception
  {
    SubclassList subclasslist;
    SubclassDescription description;
    List listSymbol;
    IpcrSymbol ipcrSymbol;
    IpcSymbol symbol;
    IpcVO ipcVO;

    String section;
    String ipcclass;
    String subclass;

    List updates = new ArrayList();
    List inserts = new ArrayList();

    List list = ipcrValidityList.getSubclassList();
    LOG.info("size of list: " + list.size());

    String validityDateFrom = validityFile.getValidityDateFrom();
    LOG.info("process ipc symbols with validity date from: " + validityDateFrom );

    String validityDateTo = validityFile.getValidityDateToAsString();
    LOG.info("process ipc symbols with validity date to: " + validityDateTo );

    Date validTo = validityFile.getValidityDateToAsDate();

    for (int i = 0; i < list.size(); i++)
    {
       subclasslist = (SubclassList)list.get(i);
       description = subclasslist.getSubclassDescription();

       section = description.getSection();
       ipcclass = description.getIpcclass();
       subclass = description.getSubclass();

       listSymbol = subclasslist.getIpcrSymbol();

       for ( int j = 0; j < listSymbol.size(); j++ )
       {
         ipcrSymbol = ( IpcrSymbol ) listSymbol.get( j );

         // nur die zu verarbeitenden datensätze werden eingelesen
         // validity-date-to entpricht dem zu verarbeitenden gueltig bis datum
         if ( validityDateTo.equalsIgnoreCase( ipcrSymbol.getValidityDateTo() ) )
         {
           if ( !ipcrSymbol.getClassificationLevel().equals( IpcVO.CL_OBSOLETE ) )
           {
             symbol = new IpcSymbol( section, ipcclass, subclass );
             ipcVO = extractValues( ipcrSymbol, symbol );

             updates.add( ipcVO );
           }
         }
         else if ( validityDateFrom.equalsIgnoreCase( ipcrSymbol.getValidityDateFrom() ) )
         {
           if ( ipcrSymbol.getClassificationLevel().equals( IpcVO.CL_COMMON ) ||
                ipcrSymbol.getClassificationLevel().equals( IpcVO.CL_ADVANCED ) ||
                ipcrSymbol.getClassificationLevel().equals( IpcVO.CL_SUBCLASS ) )
           {
             symbol = new IpcSymbol( section, ipcclass, subclass );
             ipcVO = extractValues( ipcrSymbol, symbol );

             inserts.add( ipcVO );
           }
         }
       }
     }

     this.statistikVO = storage.save( updates, inserts, validTo );
     if ( storage.isErrorOccured() )
     {
       errorOccured = true;
     }
   }


  /**
   * Extrahiert und formatiert die Werte und füllt sie in IpcVO ab
   * @param ipcrSymbol IpcrSymbol beinhaltet zusätzliche Informationen eines IPC-Symbols
   * @param symbol IpcSymbol IPC-Symbol
   * @throws Exception ParserException
   * @return IpcVO abgefülltes IpcVO
   */
  private IpcVO extractValues(IpcrSymbol ipcrSymbol, IpcSymbol symbol) throws Exception
  {
    IpcVO ipcVO = new IpcVO();

    symbol.setGroupPart(ipcrSymbol.getMainGroup(), ipcrSymbol.getSubgroup() );

    ipcVO.ipcSymb = symbol.getIpcSymbolAsString();
    ipcVO.classLevel = ipcrSymbol.getClassificationLevel();
    ipcVO.zusatz = ipcrSymbol.getEntryType();
    ipcVO.corePred = ipcrSymbol.getCorePredecessor();

    try
    {
      if ( ipcrSymbol.getValidityDateFrom() != null &&
           !ipcrSymbol.getValidityDateFrom().equals( "" ) )
      {
        ipcVO.gueltigVon = sdf.parse( ipcrSymbol.getValidityDateFrom() );
      }
      if ( ipcrSymbol.getValidityDateTo() != null &&
           !ipcrSymbol.getValidityDateTo().equals( "" ) )
      {
        ipcVO.gueltigBis = sdf.parse( ipcrSymbol.getValidityDateTo() );
      }
    }
    catch ( ParseException ex )
    {
      throw new Exception("ParseException bei extractValues()" +
                          " ValidityDateFrom: " +  ipcrSymbol.getValidityDateFrom() +
                          " ValidityDateTo: " + ipcrSymbol.getValidityDateTo() +
                          " Message: " + ex.getMessage());
    }

    return ipcVO;
  }
}
