package ch.ige.jipps;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.jipps.pdf.PdfMerger;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.AuszugVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.xml.AuszugWriteXml;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Controls how the reports are printed. This is the main controller for the report system.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Priska Steiner
 * @author Eric Lewis
 * @version $Id: AuszugControl.java,v 1.135 2008/10/23 13:36:17 lewis_lee Exp $
 * @see AuszugAssembler
 * @see AuszugWriteXml
 */
public class AuszugControl extends ReportControl
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( AuszugControl.class );

  /**
   * Used to create the XML document
   */
  protected AuszugWriteXml writeXml;

  /**
   * Used to assemble the data
   */
  protected AuszugAssembler assembler;

  /**
   * The XSL file uses this to separate the different parts of the filename
   */
  private static final String FILE_SEPARATOR = "_";

  /**
   * Konstruktor
   */
  public AuszugControl()
  {
    this.writeXml = new AuszugWriteXml();
  }

  // @see ch.ige.jipps.ReportControl#makePrint(java.util.List)
  @Override
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    LOG.debug( "makePrint" );
    PrintJobVO printJobVO = null;

    int batchId = 0;
    // Creating several print jobs requires a batch ID
    if ( printJobs.size() > 1 )
    {
      try
      {
        batchId = getNextBatchId();
      }
      catch ( Exception ex )
      {
        LOG.error( "Can't create batch ID", ex );
      }
    }

    toPdf( toXml( loadReports( printJobs ) ) );

    this.assembler = new AuszugAssembler();
    for ( Iterator<PrintJobVO> it = printJobs.iterator(); it.hasNext(); )
    {
      printJobVO = it.next();
      if ( printJobVO != null )
      {
        try
        {
          ReportType reportType = ReportType.parse( printJobVO.regauszTyp );
          IPType ipType = IPType.parse( printJobVO.schutitTyp );

          // During the whole processing, something may have gone wrong
          if ( printJobVO.status == null )
          {
            printJobVO.status = "ok";
            LOG.debug( "Print job file contains PDF file with length "
                + ( ( printJobVO.printJobFile == null ) ? "<null>" : Integer.toString( printJobVO.printJobFile.length ) ) );
            // Alle benötigten Parameter fürs PdfVO in params füllen, was nicht gefüllt ist, bekommt Defaultwerte
            try
            {
              printJobVO.paperMode = this.assembler.getReportPaperWithDefault( reportType, ipType ).toDB();
              LOG.debug( "Paper mode for print job with ID <" + printJobVO.printJobId + "> is <" + printJobVO.paperMode + ">" );
            }
            catch ( SQLException ex )
            {
              LOG.error( "SQL-Exception", ex );
              printJobVO.status = "SQL-Exception: " + ex.getMessage();
            }

            int newPrintJobId = getNextPrintJobId();
            LOG.debug( "Got new print job ID <" + newPrintJobId + "> for batch ID <" + batchId + ">" );
            printJobVO.printJobId = newPrintJobId;
            printJobVO.batchId = batchId; // ist entweder 0 oder gefüllt

            storePdf( printJobVO );
            LOG.debug( "Saved PDF file "
                + ( ( printJobVO.printJobFile == null ) ? "<null>" : "with " + Integer.toString( printJobVO.printJobFile.length ) + " bytes" ) );
          }
        }
        catch ( Exception ex )
        {
          LOG.error( "Error during status setting", ex );
          printJobVO.status = ex.getMessage();
        }
        finally
        {
          addAnswer( printJobVO.printJobId, printJobVO.formTyp, printJobVO.reportId, printJobVO.status, printJobVO.formattedApplicationNumber,
              printJobVO.formattedIpNumber );
        }
      }
    }

    if ( printJobVO != null && printJobVO.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
    {
      if ( batchId > 0 )
      {
        sendBatchJob( printJobVO.batchId, printJobVO.watermark, printJobVO.druckerId, false, null, null );
      }
      else
      {
        sendPrintJob( printJobVO.printJobId, printJobVO.watermark, printJobVO.druckerId );
      }
    }

    return this.answers;
  }

  // @see ch.ige.jipps.ReportControl#makeXml(java.util.List)
  @Override
  public String[] makeXml( List<PrintJobVO> printJobs ) throws Exception
  {
    LOG.debug( "makeXml" );

    Map<PrintJobVO, String> printJobMap = toXml( loadReports( printJobs ) );
    // During the whole processing, something may have gone wrong
    for ( PrintJobVO printJob: printJobs )
    {
      if ( printJob.status != null )
      {
        throw new Exception( printJob.status );
      }
    }

    return printJobMap.values().toArray( new String[0] );
  }

  /**
   * Converts the reports in XML format into XSL-FO documents. <strong>Note that this is only used for testing!</strong>
   * @param printJobs The print jobs to render
   * @return An array of <code>String</code>s, each of them an XSL-FO document corresponding to its print job
   * @throws Exception if the conversion fails
   */
  public String[] makeXslFo( List<PrintJobVO> printJobs ) throws Exception
  {
    LOG.debug( "makeXslFo" );
    List<String> xslFoDocuments = new ArrayList<String>( printJobs.size() );
    setupPDFRenderer();

    Map<PrintJobVO, String> printJobMap = toXml( loadReports( printJobs ) );
    for ( Entry<PrintJobVO, String> entry: printJobMap.entrySet() )
    {
      String xmlDocument = ConvertText.prepareXmlForXsl( entry.getValue() );
      String xslFoDocument = this.pdfRenderer.renderXslFo( null, xmlDocument, getXslName( entry.getKey() ) );
      xslFoDocuments.add( xslFoDocument );
    }

    return xslFoDocuments.toArray( new String[0] );
  }

  /**
   * Creates a merged PDF stream for the given print jobs.
   * @param printJobs The print jobs to print
   * @return a merged PDF stream for the print jobs
   * @throws Exception if the PDFs can't be created
   */
  @Override
  public byte[] makePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    LOG.debug( "makePdf" );

    toPdf( toXml( loadReports( printJobs ) ) );
    // During the whole processing, something may have gone wrong
    for ( PrintJobVO printJob: printJobs )
    {
      if ( printJob.status != null )
      {
        throw new Exception( printJob.status );
      }
    }

    return PdfMerger.mergePdfs( getPdfFiles( printJobs ) );
  }

  /**
   * Loads the reports from the database for a number of print jobs.
   * @param printJobs The print jobs to handle
   * @return The print jobs, each pointing to its corresponding report VO
   */
  private Map<PrintJobVO, AuszugVO> loadReports( List<PrintJobVO> printJobs )
  {
    LOG.debug( "loadReports" );
    Map<PrintJobVO, AuszugVO> reports = new HashMap<PrintJobVO, AuszugVO>();

    this.assembler = new AuszugAssembler();
    for ( PrintJobVO printJobVO: printJobs )
    {
      LOG.debug( "Loading report: Print job " + printJobVO );

      try
      {
        IPKeyVO ipKey = IPKeyVO.create( printJobVO.reportId );
        ReportType reportType = ReportType.parse( printJobVO.regauszTyp );
        AuszugVO auszugVO = this.assembler.loadReport( ipKey, reportType, printJobVO.getBagisLanguage(), printJobVO.standDatum, printJobVO.maNr,
            printJobVO.printIgekopf );

        LOG.debug( "Got auszugVO with IP key <" + auszugVO.getIpHeader().getIpKey() + ">" );
        LOG.debug( "Setting print job language to report language " + auszugVO.getIpHeader().getReportLanguage() );
        printJobVO.sprache = BAGISLanguageDomain.parse( auszugVO.getIpHeader().getReportLanguage() ).toDB();
        printJobVO.formattedApplicationNumber = auszugVO.getIpHeader().getFormattedApplicationNumber();
        printJobVO.formattedIpNumber = auszugVO.getIpHeader().getFormattedIpNumber();
        printJobVO.watermark = auszugVO.getWatermark();
        reports.put( printJobVO, auszugVO );
      }
      catch ( Exception ex )
      {
        LOG.error( "Schutit-Key: " + printJobVO.reportId + ", Exception while loading VOs", ex );
        printJobVO.status = "Fehler beim Laden des Schutztitel-VO: " + ex.getMessage();
      }
    }
    return reports;
  }

  /**
   * Converts the reports to their XML equivalent.
   * @param printJobReports The print jobs, each pointing to its corresponding report VO
   * @return A copy of <code>reports</code>, but the value is now the XML for each print job
   */
  private Map<PrintJobVO, String> toXml( final Map<PrintJobVO, AuszugVO> printJobReports )
  {
    LOG.debug( "toXml" );
    Map<PrintJobVO, String> xmlReports = new HashMap<PrintJobVO, String>( printJobReports.size() );
    for ( Entry<PrintJobVO, AuszugVO> entry: printJobReports.entrySet() )
    {
      PrintJobVO printJobVO = entry.getKey();
      LOG.debug( "Converting to XML: Print job for IP key <" + printJobVO.reportId + ">" );
      AuszugVO auszugVO = printJobReports.get( printJobVO );
      xmlReports.put( printJobVO, null );
      String xml = null;

      try
      {
        xml = this.writeXml.toXml( auszugVO );
      }
      catch ( IOException ex )
      {
        LOG.error( "IOException while writing XML stream", ex );
        printJobVO.status = "Fehler beim Schreiben des XML-Streams: " + ex.getMessage();
      }
      catch ( JAXBException ex )
      {
        LOG.error( "JAXBException while writing XML stream", ex );
        printJobVO.status = "Fehler beim Schreiben des XML-Streams: " + ex.getMessage();
      }
      LOG.debug( "Got XML: " + xml );
      xmlReports.put( printJobVO, xml );

    }
    return xmlReports;
  }

  /**
   * Converts the reports in XML format into PDF files.
   * @param printJobXmlDocuments The print jobs, each pointing to its corresponding XML report
   * @return A copy of <code>reports</code>, but the value is now the PDF for each print job; additionally, it's also
   * saved in {@link PrintJobVO#printJobFile}
   */
  private Map<PrintJobVO, byte[]> toPdf( final Map<PrintJobVO, String> printJobXmlDocuments )
  {
    LOG.debug( "toPdf" );
    Map<PrintJobVO, byte[]> printJobPdfs = new HashMap<PrintJobVO, byte[]>();
    for ( Entry<PrintJobVO, String> entry: printJobXmlDocuments.entrySet() )
    {
      PrintJobVO printJobVO = entry.getKey();
      LOG.debug( "Converting to PDF: Print job for IP <" + printJobVO.reportId + ">" );
      byte[] pdfFile = null;
      try
      {
        String xml = printJobXmlDocuments.get( printJobVO );
        LOG.debug( "Got XML with length " + xml.length() );
        String xslName = getXslName( printJobVO );
        pdfFile = createPdf( xml, xslName );
        LOG.debug( "Got response from XSL servlet with length " + pdfFile.length );
        printJobVO.printJobFile = pdfFile;
      }
      catch ( Exception ex )
      {
        LOG.error( "Can't create PDF", ex );
        printJobVO.status = "Fehler beim Erzeugen des PDFs: " + ex.getMessage();
      }
      finally
      {
        // If an error occurs, the PDF file is null
        printJobPdfs.put( printJobVO, pdfFile );
      }
    }
    return printJobPdfs;
  }

  /**
   * Gets the XSL name for a print job.
   * @param printJobVO The print job
   * @return The full XSL name for the print job
   */
  protected String getXslName( PrintJobVO printJobVO )
  {
    StringBuilder xslName = new StringBuilder();
    xslName.append( printJobVO.regauszTyp );
    xslName.append( FILE_SEPARATOR );
    xslName.append( printJobVO.schutitTyp );
    LOG.debug( "getXslName: XSL name is " + xslName );
    return xslName.toString();
  }
}
