package ch.ige.jipps;

import java.rmi.RemoteException;
import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.mail.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.apache.log4j.Priority;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.key.IPIdentifier;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.patent.Patent;
import ch.ige.ipobjects.patent.dao.PatDocDAO;
import ch.ige.ipobjects.patent.model.PatentEvent;
import ch.ige.ipobjects.patent.vo.PatentDocumentEventKeyVO;
import ch.ige.ipobjects.patent.vo.PatentDocumentVO;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.ipobjects.time.PointInTimeCollection;
import ch.ige.ipobjects.time.visitors.TimeFilter;
import ch.ige.ipobjects.user.dao.MaDAO;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.AmvErrorDAO;
import ch.ige.jipps.dao.AmvLaufDAO;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.dao.DruckerDAO;
import ch.ige.jipps.dao.LoeschungDAO;
import ch.ige.jipps.dao.LoeschungErrorDAO;
import ch.ige.jipps.dao.PropertyJippsDAO;
import ch.ige.jipps.dao.SearchDAO;
import ch.ige.jipps.exception.LookupException;
import ch.ige.jipps.exception.PrinterReferencedException;
import ch.ige.jipps.exception.UnknownPrinterException;
import ch.ige.jipps.ftp.web.util.BibliographicDataControl;
import ch.ige.jipps.ftp.web.util.EPAUploadClientConfigWrapper;
import ch.ige.jipps.ftp.web.util.FTPServletConfigDAO;
import ch.ige.jipps.text.Watermark;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.jipps.vo.AmvLaufVO;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.DruckPeriVO;
import ch.ige.jipps.vo.DruckerVO;
import ch.ige.jipps.vo.LoeschungErrorVO;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.PrintjobSearchVO;
import ch.ige.jipps.vo.PropertyJippsVO;
import ch.ige.patdocutil.receipts.PatentDocumentVOWithPDF;
import ch.ige.util.constants.CancellationProcessingConstants;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.RegistrationState;
import ch.ige.util.enums.report.ReportRegistrationState;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.server.FailureRecovery;

/**
 * Contains the functionality for JIPPS. This is outsourced from the SessionBean, because otherwise it can't be tested
 * outside a J2EE server.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: JIPPSFacadeServicesImpl.java,v 1.122 2009/02/04 14:45:46 fabrizioli Exp $
 */
public class JIPPSFacadeServicesImpl implements JIPPSFacadeServices
{
  // TODO For better error handling, use Worklet from ipobjects (AbstractServiceImpl)

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( JIPPSFacadeServicesImpl.class );

  /**
   * If a JIPPS property has this suffix, it's supposed to contain a printer ID.
   */
  private static final String PRINTER_PROPERTY_SUFFIX = "_drucker";

  /**
   * Used to get the report types
   */
  private RegAuszTypSTypDAO regAuszTypSTypDAO;

  /**
   * Used to access a user's default printer
   */
  private MaDAO maDAO;

  /**
   * Used to access configuration values
   */
  private PropertyJippsDAO propertyJippsDAO;

  /**
   * Used to access the printer configuration
   */
  private DruckerDAO druckerDAO;

  /**
   * Used to access the patent documents
   */
  private PatDocDAO patdocDAO;

  /**
   * Used to access the configuration of the FTP Servlet and EPAUploadClient
   */
  private FTPServletConfigDAO ftpconfigDAO;

  /**
   * Used to access invoices
   */
  private BfakturDAO bfakturDAO;

  /**
   * Used for cancellations
   */
  private LoeschungDAO loeschungDAO;

  /**
   * Used for errors during cancellations
   */
  private LoeschungErrorDAO loeschungErrorDAO;

  /**
   * Used for trademark renewals 
   */
  private AmvLaufDAO amvLaufDAO;

  /**
   * Used for errors during trademark renewals
   */
  private AmvErrorDAO amvErrorDAO;

  /**
   * Used for searching print jobs
   */
  private SearchDAO searchDAO;

  /**
   * The only constructor
   */
  public JIPPSFacadeServicesImpl()
  {
    // Nothing to do
  }

  /**
   * Gets the RegAuszTypSTypDAO, creating it if it doesn't exist.
   * @return The RegAuszTypSTypDAO
   */
  private RegAuszTypSTypDAO getRegAuszTypSTypDAO()
  {
    if ( this.regAuszTypSTypDAO == null )
    {
      this.regAuszTypSTypDAO = new RegAuszTypSTypDAO();
    }
    return this.regAuszTypSTypDAO;
  }

  /**
   * Gets the MaDAO, creating it if it doesn't exist.
   * @return The MaDAO
   */
  private MaDAO getMaDAO()
  {
    if ( this.maDAO == null )
    {
      this.maDAO = new MaDAO();
    }
    return this.maDAO;
  }

  /**
   * Gets the PropertyJippsDAO, creating it if it doesn't exist.
   * @return The PropertyJippsDAO
   */
  private PropertyJippsDAO getPropertyJippsDAO()
  {
    if ( this.propertyJippsDAO == null )
    {
      this.propertyJippsDAO = new PropertyJippsDAO();
    }
    return this.propertyJippsDAO;
  }

  /**
   * Gets the PatDocDAO, creating it if it doesn't exist.
   * @return The PatDocDAO
   */
  private PatDocDAO getPatDocDAO()
  {
    if ( this.patdocDAO == null )
    {
      this.patdocDAO = new PatDocDAO();
    }
    return this.patdocDAO;
  }

  private FTPServletConfigDAO getFTPConfigDAO()
  {
    if ( this.ftpconfigDAO == null )
    {
      this.ftpconfigDAO = new FTPServletConfigDAO();
    }
    return this.ftpconfigDAO;
  }

  /**
   * Gets the DruckerDAO, creating it if it doesn't exist.
   * @return The DruckerDAO
   */
  private DruckerDAO getDruckerDAO()
  {
    if ( this.druckerDAO == null )
    {
      this.druckerDAO = new DruckerDAO();
    }
    return this.druckerDAO;
  }

  /**
   * Gets the LoeschungErrorDAO, creating it if it doesn't exist.
   * @return The LoeschungErrorDAO
   */
  private LoeschungErrorDAO getLoeschungErrorDAO()
  {
    if ( this.loeschungErrorDAO == null )
    {
      this.loeschungErrorDAO = new LoeschungErrorDAO();
    }
    return this.loeschungErrorDAO;
  }

  /**
   * Gets the LoeschungDAO, creating it if it doesn't exist.
   * @return The LoeschungDAO
   */
  private LoeschungDAO getLoeschungDAO()
  {
    if ( this.loeschungDAO == null )
    {
      this.loeschungDAO = new LoeschungDAO();
    }
    return this.loeschungDAO;
  }

  /**
   * Gets the BfakturDAO, creating it if it doesn't exist.
   * @return The BfakturDAO
   */
  private BfakturDAO getBfakturDAO()
  {
    if ( this.bfakturDAO == null )
    {
      this.bfakturDAO = new BfakturDAO();
    }
    return this.bfakturDAO;
  }

  /**
   * Gets the AmvLaufDAO, creating it if it doesn't exist.
   * @return The AmvLaufDAO
   */
  private AmvLaufDAO getAmvLaufDAO()
  {
    if ( this.amvLaufDAO == null )
    {
      this.amvLaufDAO = new AmvLaufDAO();
    }
    return this.amvLaufDAO;
  }

  /**
   * Gets the AmvErrorDAO, creating it if it doesn't exist.
   * @return The AmvErrorDAO
   */
  private AmvErrorDAO getAmvErrorDAO()
  {
    if ( this.amvErrorDAO == null )
    {
      this.amvErrorDAO = new AmvErrorDAO();
    }
    return this.amvErrorDAO;
  }

  /**
   * Gets the SearchDAO, creating it if it doesn't exist.
   * @return The SearchDAO
   */
  private SearchDAO getSearchDAO()
  {
    if ( this.searchDAO == null )
    {
      this.searchDAO = new SearchDAO();
    }
    return this.searchDAO;
  }

  /**
   * Gibt die Version der Servers (Jipps) bekannt.
   * 
   * @return String Versions-Nummer
   */
  public String getJippsVersion()
  {
    return BuildInfo.CVS_TAG;
  }

  /**
   * Creates the controller for this report. What kind of controller is created depends on
   * {@link ch.ige.jipps.vo.PrintJobVO#formTyp}.
   * 
   * @param printJobVO The print job to execute
   * @return The report controller that corresponds to the print job
   * @throws ClassNotFoundException if the report controller doesn't exist
   * @throws InstantiationException if the report controller can't be created
   * @throws IllegalAccessException if the report controller can't be created
   */
  private ReportControl createReportController( PrintJobVO printJobVO ) throws ClassNotFoundException, InstantiationException, IllegalAccessException
  {
    return (ReportControl) Class.forName( "ch.ige.jipps." + printJobVO.formTyp + "Control" ).newInstance();
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#send(List)
  public List<AnswerVO> createPrintjobs( List<PrintJobVO> printJobs )
  {
    try
    {
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "Print jobs:" );
        for ( PrintJobVO printJob: printJobs )
        {
          LOG.debug( printJob );
        }
      }

      LOG.info( "J I P P S - V E R S I O N\nBuild-ID:" + BuildInfo.BUILD_ID + "\nCVS-TAG:" + BuildInfo.CVS_TAG );
      ReportControl controller = createReportController( printJobs.get( 0 ) );
      return controller.makePrint( printJobs );
    }
    catch ( ClassNotFoundException fex )
    {
      return getAnswer( fex, Priority.FATAL );
    }
    catch ( IllegalAccessException iax )
    {
      return getAnswer( iax, Priority.ERROR );
    }
    catch ( InstantiationException iex )
    {
      return getAnswer( iex, Priority.ERROR );
    }
    catch ( ClassCastException cex )
    {
      return getAnswer( cex, Priority.ERROR );
    }
    catch ( Exception ex )
    {
      return getAnswer( ex, Priority.WARN );
    }
  }

  /**
   * Gets the answers if an Exception occurs.
   * @param e The exception to log and copy into the answer
   * @param logLevel Into what log level to log the message
   * @return A <code>List</code> of {@link ch.ige.jipps.vo.AnswerVO} objects,
   * the first contains the error message in {@link ch.ige.jipps.vo.AnswerVO#printStatus}
   */
  private List<AnswerVO> getAnswer( Exception e, Priority logLevel )
  {
    AnswerVO answerVO = new AnswerVO();
    List<AnswerVO> answers = new ArrayList<AnswerVO>();
    LOG.log( logLevel, e.getMessage(), e );
    answerVO.printStatus = e.getClass().getName() + ": " + e.getMessage();
    answers.add( answerVO );
    return answers;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPreviewData(int)
  public byte[] getPreviewData( int pdfId )
  {
    PdfShow pdfShow = new PdfShow();
    byte[] pdfArr = null;
    try
    {
      pdfArr = pdfShow.getPdfArr( pdfId );
    }
    catch ( LookupException lex )
    {
      LOG.error( "LookupException", lex );
    }
    return pdfArr;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getMergePdf(List)
  public byte[] getMergePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    try
    {
      ReportControl controller = createReportController( printJobs.get( 0 ) );
      byte[] mergedPdf = controller.makePdf( printJobs );
      LOG.debug( "Got merged PDF from controller with " + mergedPdf.length + " bytes" );
      return mergedPdf;
    }
    catch ( Exception ex )
    {
      LOG.error( "PDF-Datei konnte nicht erstellt werden", ex );
      throw ex;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getXml(List)
  public String[] getXml( List<PrintJobVO> printJobs ) throws Exception, EJBException
  {
    try
    {
      ReportControl controller = createReportController( printJobs.get( 0 ) );
      return controller.makeXml( printJobs );
    }
    catch ( Exception ex )
    {
      LOG.error( "XML files couldn't be created", ex );
      throw ex;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#setBfaktDruckstatus(int, String)
  public void setBfaktDruckstatus( final int bfaktId, final String statusDruck ) throws Exception
  {
    FailureRecovery failureRecovery = new FailureRecovery()
    {
      @SuppressWarnings( "synthetic-access" )
      @Override
      public Object executeService() throws Exception
      {
        String statusdruckBagis = null;

        try
        {
          statusdruckBagis = getBfakturDAO().selectBFakturDruckstatus( bfaktId );
          if ( ( statusdruckBagis != null ) && ( !statusdruckBagis.equals( PrintStatusConstants.PRINTED_STRING ) )
              && ( !statusdruckBagis.equals( statusDruck ) ) )
          {
            getBfakturDAO().updateBFakturDruckstatus( bfaktId, statusDruck );
          }
        }
        catch ( SQLException ex )
        {
          throw new Exception( "SQLFehler beim Update des BfakturDAO", ex );
        }
        return null;
      }
    };
    failureRecovery.run();
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#printAfterPreview(java.util.List, int, int)
  public void printAfterPreview( List<Integer> printJobIds, int printStatus, int printerId )
  {
    printAfterPreview( printJobIds, printStatus, printerId, null );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#printReportAfterPreview(ch.ige.jipps.vo.PrintJobVO, int, int)
  public void printReportAfterPreview( PrintJobVO printJob, int printStatus, int printerId )
  {
    try
    {
      // TODO This is an ugly hack to force the watermark to appear on the printed page after the client has requested a preview. The whole preview should be refactored!

      GenericIP ip = ServicesFactory.getInstance().findIP( IPKeyVO.create( printJob.reportId ) );
      Locale language = null;
      if ( printJob.sprache == PrintJobVO.USE_IP_LANGUAGE )
      {
        //Fetch the IP's language
        language = ip.getLocale();
      }
      else
      {
        language = BAGISLanguageDomain.parse( printJob.sprache ).toLocale();
      }

      String watermark = Watermark.getWatermark( ReportType.parse( printJob.regauszTyp ), ip, language, printJob.standDatum );

      List<Integer> printJobIds = new ArrayList<Integer>( 1 );
      printJobIds.add( printJob.printJobId );
      printAfterPreview( printJobIds, printStatus, printerId, watermark );
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception while printing a report after a preview", ex );
    }
  }

  /**
   * Prints a print job after a preview has been requested.
   * When showing a preview, the print job is in fact already created. To print it, only the print
   * status has to be changed, and the print jobs have to be sent to the print server.
   * @param printJobIds The IDs of the print jobs to print
   * @param printStatus The new print status, one of {@link PrintStatusConstants}
   * @param printerId The ID of the printer which prints the print jobs
   * @param watermark A possible watermark to print; is ignored if {@code null}
   */
  private void printAfterPreview( List<Integer> printJobIds, int printStatus, int printerId, String watermark )
  {
    try
    {
      PrintJobStoreServices printJobStore = PrintJobStoreFactory.getInstance();

      /*TODO marepen 1435: if printJobs.size > 1, then, if there is no batchId,
       * add one to every job in the list and send batch-jms.
       * this is already solved in LetterControl.makePrint() for direct printing without preview.
       * Actually, multiple letters cannot be previewed at the moment.
       * Therefore this is not yet necessary here.
       */

      printJobStore.updPrintJob( printJobIds, printStatus, printerId );
      LOG.debug( "Updated " + printJobIds.size() + " print jobs to new status " + printStatus );

      if ( printStatus == PrintStatusConstants.READY_FOR_PRINTING )
      {
        for ( Integer printJobId: printJobIds )
        {
          if ( LOG.isDebugEnabled() )
          {
            LOG.debug( "Sending JMS message for print job with ID " + printJobId + "; watermark is: " + watermark );
          }
          getJmsPublisherLocal().sendPrintJob( printJobId, watermark, Integer.valueOf( printerId ) );
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception bei Lookup auf JMSPublisherBean", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getReportTypes(IPType, RegistrationState)
  public Set<RegAuszTypSTypVO> getReportTypes( IPType ipType, RegistrationState registrationState )
  {
    try
    {

      ReportRegistrationState reportRegistrationState;

      if ( registrationState == RegistrationState.APPLICATION )
      {
        reportRegistrationState = ReportRegistrationState.ONLY_FOR_APPLICATION;
      }
      else if ( registrationState == RegistrationState.REGISTERED )
      {
        reportRegistrationState = ReportRegistrationState.ONLY_FOR_REGISTRATION;
      }
      else
      {
        reportRegistrationState = ReportRegistrationState.APPLICATION_AND_REGISTRATION;
      }

      return getRegAuszTypSTypDAO().findIpTypeReportTypes( ipType, reportRegistrationState );
    }
    catch ( SQLException e )
    {
      LOG.error( e.getMessage(), e );
      return Collections.emptySet();
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getMaDrucker(int, int)
  public Integer getMaDrucker( int employeeNumber )
  {
    try
    {
      return getMaDAO().findEmployee( Integer.valueOf( employeeNumber ) ).druckerId;
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getMaDrucker(int, String)
  public Integer getMaDrucker( String employeeUserName )
  {
    try
    {
      return getMaDAO().findEmployee( employeeUserName ).druckerId;
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getDrucker(int, int)
  public Set<DruckerVO> getPrinters( int printJobType, IPType ipType )
  {
    // Design and Trademark reports get all printers including the colourprinters
    if ( printJobType == PrintJobVO.AUSZUG_NUMERIC && ( ipType.equals( IPType.DESIGN ) || ipType.equals( IPType.TRADEMARK_CH ) ) )
    {
      return new LinkedHashSet<DruckerVO>( getAllJippsPrinters() );
    }
    else
    // everybody else gets only the black/white printers
    {
      return new LinkedHashSet<DruckerVO>( getBlackAndWhiteJippsPrinters() );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getDruckPeriListByDruckArt(int)
  public List<DruckPeriVO> getDruckPeriListByDruckArt( int druckArt )
  {
    List<DruckPeriVO> druckPeriList = null;
    int loeschungLaufId = 0;
    try
    {
      loeschungLaufId = getLoeschungDAO().getLoeschungLaufId( druckArt );
      druckPeriList = getLoeschungDAO().selectDruckPeriByDruckArt( druckArt, loeschungLaufId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return druckPeriList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getDruckPeriListByDruckArt(int, int)
  public List<DruckPeriVO> getDruckPeriListById( int druckArt, int id )
  {
    List<DruckPeriVO> druckPeriList = null;
    try
    {
      druckPeriList = getLoeschungDAO().selectDruckPeriByDruckArt( druckArt, id );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return druckPeriList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getLoeschungslaufIds(int, int)
  public List<LoeschungLaufVO> getLoeschungsLaeufe( Integer cancellationType, Integer maxNumber )
  {
    List<LoeschungLaufVO> fullList = null;
    try
    {
      fullList = getLoeschungDAO().getLoeschungLaeufe( cancellationType );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    if ( maxNumber == null )
    {
      return fullList;
    }
    List<LoeschungLaufVO> adjustedList = new ArrayList<LoeschungLaufVO>();
    if ( fullList != null )
    {
      int counter = 0;
      for ( LoeschungLaufVO loeschungLaufVO: fullList )
      {
        if ( counter >= maxNumber )
        {
          break;
        }
        adjustedList.add( loeschungLaufVO );
        counter++;
      }
    }
    return adjustedList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getLoeschungError(int)
  public LoeschungErrorVO getLoeschungError( int keyId )
  {
    LoeschungErrorVO errVO = null;
    try
    {
      errVO = getLoeschungErrorDAO().selectLoeschungError( keyId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return errVO;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#removeLoeschungError(List)
  public int removeLoeschungError( List<LoeschungErrorVO> loeschList )
  {
    int numberOfDeletedRows = 0;
    List<Integer> cancellationErrorIds = new ArrayList<Integer>( loeschList.size() );
    for ( LoeschungErrorVO cancellationError: loeschList )
    {
      cancellationErrorIds.add( cancellationError.keyId );
    }

    try
    {
      numberOfDeletedRows = getLoeschungErrorDAO().deleteLoeschungError( cancellationErrorIds );
    }
    catch ( UnitOfWorkException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }

    if ( numberOfDeletedRows > 0 )
    {
      return StatusConstants.OK;
    }
    else
    {
      return StatusConstants.ERROR;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getLoeschungErrorList()
  public List<LoeschungErrorVO> getLoeschungErrorList()
  {
    List<LoeschungErrorVO> loeschungErrorList = null;
    try
    {
      loeschungErrorList = getLoeschungErrorDAO().selectLoeschungError();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return loeschungErrorList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#renderLoeschungen(List, int, int, String, int)
  public String renderLoeschungen( List<DruckPeriVO> selList, int drucker, int pakete, String userName, int loeschungLaufId )
  {
    LoeschungControl lc = new LoeschungControl();
    String message = lc.renderLoeschungen( selList, drucker, pakete, userName, loeschungLaufId );
    return message;
  }

  /**
   * Holt die JIPPSDrucker Dies wird für den JippsClient benötigt. Die Methode getDruckerList macht das selbe über das
   * DruckerDAO. Da in dieser Methode ein two-phase commit gemacht wird muss dies über das PrintJobStoreBean ausgeführt
   * werden.
   * 
   * @return List mit den JIPPSDruckern
   */
  private List<DruckerVO> getAllJippsPrinters()
  {
    List<DruckerVO> printers = null;
    try
    {
      printers = getDruckerDAO().getAllPrinters();
    }
    catch ( SQLException e )
    {
      LOG.error( e.getMessage(), e );
    }
    return printers;
  }

  /**
   * Holt die JIPPSDrucker Dies wird für den JippsClient benötigt. Die Methode getDruckerList macht das selbe über das
   * DruckerDAO. Da in dieser Methode ein two-phase commit gemacht wird muss dies über das PrintJobStoreBean ausgeführt
   * werden.
   * 
   * @return List mit den JIPPSDruckern
   */
  private List<DruckerVO> getBlackAndWhiteJippsPrinters()
  {
    List<DruckerVO> printers = null;
    try
    {
      printers = getDruckerDAO().getBlackAndWhitePrinters();
    }
    catch ( SQLException e )
    {
      LOG.error( e.getMessage(), e );
    }
    return printers;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getMaNr(String)
  public int getMaNr( String employeeUserName )
  {
    try
    {
      return getMaDAO().findEmployee( employeeUserName ).maNr.intValue();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return 0;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#createMail(String, String)
  public void createMail( String meldung, String log )
  {
    try
    {
      SendMail mail = new SendMail( getMailSession() );
      String adressTo = getErrorAdressTo();
      String adressCC = getErrorAdressCC();
      String adressFrom = "JIPPSMAILER";
      String betreff = "Fehler bei der Erstellung der Löschungsanzeigen !";

      mail.sendMail( adressTo, adressCC, adressFrom, betreff, meldung );

      LOG.info( "Folgende Löschungs-Fehler wurden zur Behandlung " + " per Mail an: " + adressTo + " gesendet:\n" + log );
    }
    catch ( Exception ex )
    {
      LOG.error( "SendMail konnte nicht erzeugt werden", ex );
    }
  }

  /**
   * Gets the session for mailing.
   * @return The mail session
   * @throws NamingException
   * @throws ClassCastException
   */
  private Session getMailSession() throws NamingException, ClassCastException
  {
    Context ctx = new InitialContext();
    return (javax.mail.Session) ctx.lookup( "java:comp/env/mail/JavaMailSession" );
  }

  /**
   * Liest die Empfänger-Adresse aus der Property-Tabelle.
   * 
   * @return adressTo aus der Property-Tabelle.
   */
  private String getErrorAdressTo()
  {
    try
    {
      return getPropertyJippsDAO().select( PropertyConstants.ERROR_RECEIVER );
    }
    catch ( SQLException ex )
    {
      LOG.error( "Property (" + PropertyConstants.ERROR_RECEIVER + ") konnte nicht gelesen werden", ex );
      return null;
    }
  }

  /**
   * Liest die zusätzliche Empfänger-Adresse aus der Property-Tabelle.
   * 
   * @return adressCC aus der Property-Tabelle.
   */
  private String getErrorAdressCC()
  {
    try
    {
      return getPropertyJippsDAO().select( PropertyConstants.ERROR_RECEIVER_CC );
    }
    catch ( SQLException ex )
    {
      LOG.warn( "Property (" + PropertyConstants.ERROR_RECEIVER_CC + ") konnte nicht gelesen werden", ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#printEintrBesch()
  public int printEintrBesch()
  {
    return new EintrBeschControl().printEintrBesch( true );
  }

  // TODO Use constants for the AMV stuff

  // @see ch.ige.jipps.JIPPSFacadeServices#printAMVOne(boolean)
  public int printAMVOne( boolean hasCover )
  {
    return new AmvControl().startAMVLauf( hasCover );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#printAMVOne(boolean, int, boolean)
  public int printAMVOne( boolean hasCover, int amvLaufId, boolean sendMail )
  {
    return new AmvControl().startAMVLauf( hasCover, amvLaufId, sendMail, false );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#finishAMVOne(boolean, int, boolean)
  public void finishAMVOne( boolean hasCover, int amvLaufId, boolean sendMail )
  {
    new AmvControl().startAMVLauf( hasCover, amvLaufId, sendMail, true );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#printAMVChanges(boolean)
  public int printAMVChanges( boolean hasCover )
  {
    return new LetterBatchControl().printAMVChanges( hasCover );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getAmvLauf(int)
  public AmvLaufVO getAmvLauf( int amvLaufId ) throws EJBException
  {
    try
    {
      return getAmvLaufDAO().selectAMVLaufToId( amvLaufId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#findErrorList(int)
  public List<AmvErrorVO> findErrorList( int amvLaufId ) throws EJBException
  {
    try
    {
      return getAmvErrorDAO().selectAMVErrorToId( amvLaufId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#findAmvLauf(String)
  public List<AmvLaufVO> findAmvLauf( String typCd ) throws EJBException
  {
    try
    {
      return getAmvLaufDAO().select( typCd );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacade#getDruckerList()
  public List<DruckerVO> getPrinters()
  {
    List<DruckerVO> druckerList = null;
    try
    {
      druckerList = getDruckerDAO().getAllPrinters();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return druckerList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPrintJobSearchList(PrintjobSearchVO)
  public List<PrintjobSearchVO> getPrintJobSearchList( PrintjobSearchVO searchVO )
  {
    LOG.debug( "*** In getPrintJobSearchList" );
    List<PrintjobSearchVO> printjobSearchList = null;
    try
    {
      printjobSearchList = getSearchDAO().select( searchVO );
      LOG.debug( "*** searchDAO.select(searchVO) aufgerufen" );
      LOG.debug( "Grösse der Liste: " + printjobSearchList.size() );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return printjobSearchList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getDruckerById(int)
  public DruckerVO getPrinter( int druckerId )
  {
    DruckerVO druckerVO = null;
    try
    {
      druckerVO = getDruckerDAO().select( druckerId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return druckerVO;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getNextDruckerId()
  public int getNextDruckerId()
  {
    int nextDruckerId = 0;
    try
    {
      nextDruckerId = getDruckerDAO().selectNextId();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return nextDruckerId;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#selectControlServers()
  public Set<String> selectControlServers()
  {
    Set<String> controlServers = null;

    try
    {
      controlServers = getDruckerDAO().selectControlServers();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return controlServers;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#selectPrintJobTypes()
  public Set<String> selectPrintJobTypes()
  {
    Set<String> printJobTypes = null;

    try
    {
      printJobTypes = getSearchDAO().selectPrintJobTypes();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return printJobTypes;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#insertDrucker(DruckerVO)
  public void insertDrucker( DruckerVO druckerVO )
  {
    getDruckerDAO().insert( druckerVO );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#updateDrucker(DruckerVO)
  public void updateDrucker( DruckerVO druckerVO )
  {
    getDruckerDAO().update( druckerVO );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#deleteDrucker(int)
  public void deleteDrucker( int printerId ) throws EJBException
  {
    try
    {
      // Check whether there's a JIPPS property containing this printer ID.
      for ( PropertyJippsVO property: getPropertyJippsDAO().select() )
      {
        if ( property.key.endsWith( PRINTER_PROPERTY_SUFFIX ) )
        {
          if ( Integer.toString( printerId ).equals( property.value.trim() ) )
          {
            PrinterReferencedException printerReferencedException = new PrinterReferencedException( "Printer with ID " + printerId
                + " is still referenced in the JIPPS properties (key " + property.key + ")." );
            throw new EJBException( "Can't delete printer " + printerId, printerReferencedException );
          }
        }
      }

      // It's not referenced anywhere, go ahead and delete it!
      getDruckerDAO().delete( printerId );
    }
    catch ( SQLException e )
    {
      // Check whether the foreign key constraint was violated. This means that the printer is still used
      // in some print jobs, which must be shown by using a different exception.
      if ( e.getMessage().indexOf( "ORA-02292" ) > -1 )
      {
        PrinterReferencedException printerReferencedException = new PrinterReferencedException( "Printer with ID " + printerId
            + " is still referenced in other tables.", e );
        throw new EJBException( "Can't delete printer " + printerId, printerReferencedException );
      }
      else
      {
        throw new EJBException( "Can't delete printer " + printerId, e );
      }
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#removePrintJobs()
  public int removePrintJobs()
  {
    int status = 1;
    try
    {
      // delete the printjobs older than the given property
      String loeschPeri = getPropertyJippsDAO().select( PropertyConstants.PRINTJOB_DEL_PERI );
      long loeschen = ( System.currentTimeMillis() - ( ( new Long( loeschPeri ).longValue() ) * 86400000 ) );
      status = deleteJobs( new Date( loeschen ) );

      // additionally delete the amv-jobs older than the given property
      String amvLoeschPeri = getPropertyJippsDAO().select( PropertyConstants.PRINTJOB_AMV_DEL_PERI );
      long amvloeschen = ( System.currentTimeMillis() - ( ( new Long( amvLoeschPeri ).longValue() ) * 86400000 ) );
      status = deleteAmvJobs( new Date( amvloeschen ) );
    }
    catch ( SQLException ex )
    {
      status = StatusConstants.ERROR;
      LOG.error( "SQLException beim Lesen der PrintJob-LöschPeriode auf PropertyJipps", ex );
    }

    return status;
  }

  /**
   * Aufruf des PrintjJobStoreBeans für die Löschung
   * 
   * @param loeschDate Vergleichsdatum, was kleiner ist, als dieses, wird gelöscht
   * @return status
   */
  private int deleteJobs( Date loeschDate )
  {
    try
    {
      LOG.info( "delete Printjobs older than " + loeschDate.toString() );
      Integer deleteCount = PrintJobStoreFactory.getInstance().deleteJobs( loeschDate );
      LOG.info( deleteCount + " printjobs deleted." );
      return StatusConstants.OK;
    }
    catch ( EJBException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return StatusConstants.ERROR;
    }
  }

  /**
   * @param loeschDate Vergleichsdatum, was kleiner ist, als dieses, wird gelöscht
   * @return status
   */
  private int deleteAmvJobs( Date loeschDate )
  {
    new AmvControl().deleteAMVRunsOlderThanDate( loeschDate );
    return StatusConstants.OK;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getDbUrls()
  public List<String> getDbUrls()
  {
    List<String> dbURL = new ArrayList<String>();
    try
    {
      JIPPSInfoLocal jippsInfoLocal = getJippsInfoLocal();
      dbURL.add( jippsInfoLocal.getJippsDbUrl() );
      dbURL.add( jippsInfoLocal.getBagisDbUrl() );
      return dbURL;
    }
    catch ( NamingException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    catch ( CreateException cex )
    {
      LOG.error( cex.getMessage(), cex );
    }
    return dbURL;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#deleteProperty(String)
  public void deleteProperty( String key ) throws EJBException
  {
    try
    {
      // Note that for deleting (unlike insert/update), there's no check
      // whether the value refers to a printer. After all, it doesn't matter
      // whether it does or not: It's just deleted.
      getPropertyJippsDAO().delete( key );
    }
    catch ( UnitOfWorkException ex )
    {
      throw new EJBException( "Can't delete JIPPS property", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#insertProperty(PropertyJippsVO)
  public void insertProperty( PropertyJippsVO propertyJippsVO ) throws EJBException
  {
    try
    {
      checkPrinterProperty( propertyJippsVO );
      getPropertyJippsDAO().insert( propertyJippsVO );
    }
    catch ( UnitOfWorkException ex )
    {
      throw new EJBException( "Can't insert JIPPS property", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#updateProperty(PropertyJippsVO)
  public void updateProperty( PropertyJippsVO propertyJippsVO ) throws EJBException
  {
    try
    {
      checkPrinterProperty( propertyJippsVO );
      getPropertyJippsDAO().update( propertyJippsVO );
    }
    catch ( UnitOfWorkException ex )
    {
      throw new EJBException( "Can't update JIPPS property", ex );
    }
  }

  /**
   * Checks whether the property in <code>propertyJippsVO</code> references a printer
   * and whether that printer exists.
   * @param propertyJippsVO The property to check for a printer ID in its value
   * @throws EJBException if the printer ID in the value doesn't exist; this contains an {@link UnknownPrinterException}
   */
  protected void checkPrinterProperty( PropertyJippsVO propertyJippsVO ) throws EJBException
  {
    if ( propertyJippsVO.key.endsWith( PRINTER_PROPERTY_SUFFIX ) )
    {
      try
      {
        Integer printerId = Integer.valueOf( propertyJippsVO.value );
        DruckerVO printer = getPrinter( printerId.intValue() );

        if ( printer == null )
        {
          UnknownPrinterException unknownPrinterException = new UnknownPrinterException( "Supposed property key '" + propertyJippsVO.key
              + "' to be a printer, but found no printer with ID " + printerId );
          throw new EJBException( "Unknown printer " + propertyJippsVO.value, unknownPrinterException );
        }
        if ( !propertyJippsVO.description.contains( printer.druckerName ) )
        {
          StringBuffer sb = new StringBuffer();
          sb.append( printer.druckerName + " : " );
          sb.append( propertyJippsVO.description );

          propertyJippsVO.description = sb.toString();
        }
      }
      catch ( NumberFormatException e )
      {
        LOG.warn( "Supposed property " + propertyJippsVO.key + " to be a printer ID, but its value " + propertyJippsVO.value + " is not numeric.", e );
      }
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#updateVerarbStat(DruckPeriVO)
  public void updateVerarbStat( DruckPeriVO periVO )
  {
    try
    {
      getLoeschungDAO().updateDruckPeri( periVO.loeschungLaufId, periVO.druckPeri, CancellationProcessingConstants.PRINT_AGAIN_INTEGER );
    }
    catch ( UnitOfWorkException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPropertyList()
  public List<PropertyJippsVO> getPropertyList()
  {
    List<PropertyJippsVO> propList = null;
    try
    {
      propList = getPropertyJippsDAO().select();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return propList;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPropertyByKey(String)
  public PropertyJippsVO getPropertyByKey( String key )
  {
    PropertyJippsVO propVO = null;
    try
    {
      propVO = getPropertyJippsDAO().selectVO( key );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return propVO;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#updatePrintjobs(List, boolean)
  public void updatePrintjobs( List<PrintjobSearchVO> updateList, boolean isBatch )
  {
    try
    {
      if ( isBatch )
      {
        PrintjobSearchVO lastPrintJobSearch = null;
        for ( PrintjobSearchVO printJobSearch: updateList )
        {
          lastPrintJobSearch = printJobSearch;
          getSearchDAO().update( printJobSearch );
          LOG.debug( "*** updatePrintjobs auf SearchDAO wurde aufgerufen. (isBatch: " + isBatch + ")" );
        }
        if ( lastPrintJobSearch != null )
        {
          getJmsPublisherLocal().sendBatchJob( lastPrintJobSearch.batchId, null, lastPrintJobSearch.druckerId, false, null, null );
          LOG.debug( "*** jmsSendToPrintServer wurde aufgerufen.\n" + "DruckerId: " + lastPrintJobSearch.druckerId + "BatchId: " + lastPrintJobSearch.batchId );
        }
      }
      else
      {
        for ( PrintjobSearchVO printJobSearch: updateList )
        {
          getSearchDAO().update( printJobSearch );
          getJmsPublisherLocal().sendPrintJob( printJobSearch.printjobId, null, printJobSearch.druckerId );
          LOG.debug( "*** jmsSendToPrintServer wurde aufgerufen.\n" + "PrintJobId: " + printJobSearch.printjobId + "DruckerId: " + printJobSearch.druckerId );
        }
      }
    }
    catch ( Exception e )
    {
      LOG.error( e.getMessage(), e );
    }
  }

  /**
   * Gets the JIPPS info local interface.
   * 
   * @return The JIPPS info local interface
   * @throws NamingException
   * @throws CreateException
   */
  private static JIPPSInfoLocal getJippsInfoLocal() throws NamingException, CreateException
  {
    // TODO Cache initial context as member variable?
    Context ctx = new InitialContext();
    JIPPSInfoLocalHome home = (JIPPSInfoLocalHome) ctx.lookup( JIPPSInfoLocalHome.COMP_NAME );
    return home.create();
  }

  /**
   * Gets the JMS publisher local interface.
   * 
   * @return The JMS publisher local interface
   * @throws NamingException
   * @throws CreateException
   */
  private static JMSPublisherLocal getJmsPublisherLocal() throws NamingException, CreateException
  {
    // TODO Cache initial context as member variable?
    Context ctx = new InitialContext();
    JMSPublisherLocalHome home = (JMSPublisherLocalHome) ctx.lookup( JMSPublisherLocalHome.COMP_NAME );
    return home.create();
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getInstalledPrinters()
  @SuppressWarnings( "unchecked" )
  public List<String> getInstalledPrinters()
  {
    try
    {
      return getJmsPublisherLocal().getInstalledPrintersFromPrintservice();
    }
    catch ( Exception e )
    {
      LOG.error( e.getMessage(), e );
      return null;
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getJippsPrintersAsMap()
  public Map<Integer, String> getPrintersAsMap() throws RemoteException
  {
    try
    {
      return getDruckerDAO().getPrintersAsMap();
    }
    catch ( SQLException e )
    {
      throw new RemoteException( e.getMessage(), e );
    }
  }

  //@see ch.ige.jipps.JIPPSFacadeServices#getAllPatdocIds(java.util.Date,java.lang.Integer)
  public List<PatentDocumentEventKeyVO> getAllPatdocIds( final java.util.Date publicationDate, final Integer patentNumber )
  {
    try
    {
      // all patdocIds are retrieved, also not published ones, is used by patdocFeederclient, therefore not moeved to FTPServlet.
      return getPatDocDAO().selectPatdocIds( publicationDate, patentNumber );
    }
    catch ( SQLException e )
    {
      LOG.error( "Error in looking up the patdoc_ids for " + publicationDate + " patentNumber " + patentNumber + ":" + e.getMessage(), e );
      throw new EJBException( "Error in looking up the patdoc_ids for " + publicationDate + " patentNumber " + patentNumber + ":" + e.getMessage(), e );
    }
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPublishedPatentDocumentsWithPDF(java.util.Map)
  public List<PatentDocumentVOWithPDF> getPublishedPatentDocumentsWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsPrintJobs )
      throws RemoteException
  {
    return this.getPatentDocumentsWithPDF( patdocIdsPrintJobs, true );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPatentDocumentsWithPDF(Map<PatentDocumentEventKeyVO, PrintJobVO>, boolean)
  public List<PatentDocumentVOWithPDF> getPatentDocumentsWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsWithPrintJob, boolean published )
  {
    return this.getST36DocumentsWithPDF( patdocIdsWithPrintJob, published, false );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getBiblioDataWithPDF(Map<PatentDocumentEventKeyVO, PrintJobVO>, boolean)
  public List<PatentDocumentVOWithPDF> getBiblioDataWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsWithPrintJob, boolean published )
  {
    return this.getST36DocumentsWithPDF( patdocIdsWithPrintJob, published, true );
  }

  private List<PatentDocumentVOWithPDF> getST36DocumentsWithPDF( Map<PatentDocumentEventKeyVO, PrintJobVO> patdocIdsWithPrintJob, boolean published,
      boolean biblioDataOnly )
  {
    LOG.debug( " Starting request for generation of " + patdocIdsWithPrintJob.size() + " documents" );
    List<PatentDocumentVOWithPDF> result = new ArrayList<PatentDocumentVOWithPDF>( patdocIdsWithPrintJob.size() );
    PatentSpecificationControl patentSpecificationControl = null;
    if ( biblioDataOnly )
    {
      try
      {
        patentSpecificationControl = new BibliographicDataControl();
      }
      catch ( Exception e )
      {
        LOG.error( "Error in generating the transformer " + e.getMessage(), e );
        throw new IllegalStateException( "Cannot generate the bibliographic data, tranformer creation failed " + e.getMessage() );
      }
    }
    else
    {
      patentSpecificationControl = new PatentSpecificationControl();
    }
    for ( Entry<PatentDocumentEventKeyVO, PrintJobVO> entry: patdocIdsWithPrintJob.entrySet() )
    {
      PatentDocumentEventKeyVO patdocId = entry.getKey();
      try
      {
        IPIdentifier schutitKey = IPKeyVO.create( patdocId.getIPKey() );
        LOG.debug( " Starting request for generation of pdf for " + schutitKey );
        Patent ipInstance = ServicesFactory.getStandaloneServices().findPatent( schutitKey );
        PointInTimeCollection<PatentDocumentVO> patentdocs = TimeFilter.<PatentDocumentVO>sameTime( patdocId.getEreigIDat() )
            .filter( ipInstance.getDocuments() );
        PointInTimeCollection<Event> patentdocEvents = TimeFilter.<Event>sameTime( patdocId.getEreigIDat() ).filter( ipInstance.getEvents() );
        assert ( patentdocs != null );
        assert ( patentdocEvents != null );
        if ( patentdocEvents.size() != 1 )
        {
          // there exists a patent document without an event assigned to it.
          LOG.warn( "No valid event was found for " + entry );
          continue;
        }
        else if ( published )
        {
          PatentEvent event = PatentEvent.wrap( patentdocEvents.get( 0 ) );
          if ( event.isOnWaitlist() || !event.isPublished() )
          {
            LOG.warn( "The patent document found for " + entry + " is not published." );
            continue;
          }
          if ( !event.getPubl_dat().equals( event.getDate() ) )
          {
            LOG
                .warn( "The patent document found for " + entry + " is not published on the expected date " + event.getDate() + " but on "
                    + event.getPubl_dat() );
            continue;
          }
        }
        if ( patentdocs.size() != 1 )
        {
          throw new IllegalArgumentException( "The patent document must exist when starting the request for generating the pdf" );
        }
        byte[] pdfContent = new byte[10];

        if ( biblioDataOnly )
        {
          PatentDocumentVO biblioVo = (PatentDocumentVO) patentSpecificationControl.getReport( entry.getValue().reportId );
          LOG.debug( "Created xml, length in bytes: " + ( ( biblioVo != null && biblioVo.getXml() != null ) ? biblioVo.getXml().length() : "null" ) );
          patentdocs.set( 0, biblioVo );
        }
        else
        // TODO both
        {
          pdfContent = patentSpecificationControl.makePdf( Collections.singletonList( entry.getValue() ) );
          LOG.debug( "Created pdf, length in bytes: " + pdfContent.length );

        }

        assert ( pdfContent.length > 0 );
        PatentDocumentVOWithPDF receipt = new PatentDocumentVOWithPDF( patentdocs.get( 0 ), pdfContent, ipInstance.getRegisterNumber(), ipInstance
            .getLanguage().toString() );
        result.add( receipt );
      }
      catch ( Exception e )
      {
        LOG.error( "Error in generating the PDF of : " + entry + ", reason: " + e.getMessage(), e );
        throw new EJBException( e.getMessage(), e );
      }
    }

    LOG.debug( "Patent Docs found " + result.size() );
    return result;
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#getPrintJobsByBfaktId(java.lang.Integer)
  public List<Integer> getPrintJobsByBfaktId( Integer bfaktId )
  {
    LOG.debug( "get all printjobs for bfaktId " + bfaktId );
    return PrintJobStoreFactory.getInstance().getPrintJobsByBfaktId( bfaktId );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#deletePrintjob(int)
  public void deletePrintjob( int printJobId )
  {
    PrintJobStoreFactory.getInstance().deletePrintJob( printJobId );
  }

  // @see ch.ige.jipps.JIPPSFacadeServices#deletePrintjobs(java.util.List)
  public void deletePrintjobs( List<Integer> printjobIds )
  {
    PrintJobStoreFactory.getInstance().deletePrintJobs( printjobIds );
  }

  public EPAUploadClientConfigWrapper getConfigurationForEPAUploadClient() throws RemoteException
  {
    try
    {
      return this.getFTPConfigDAO().selectClientConfig();
    }
    catch ( SQLException e )
    {
      String msg = "Error in retrieving the epaupload client configuration from DB " + e.getMessage();
      LOG.error( msg, e );
      throw new EJBException( msg, e );
    }
  }

}
