package ch.ige.jipps;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.jipps.dao.PropertyJippsDAO;

/**
 * JMSPublisherBean: Bean sendet die Druckaufträge an den JPrintServer via JMS.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Marc Bouquet
 * @version $Id: JMSPublisherBean.java,v 1.29 2007/12/20 11:01:32 lewis_lee Exp $ <!-- begin-xdoclet-definition -->
 * @ejb.bean name="JMSPublisher" description="An EJB named JMSPublisher" display-name="JMSPublisher" view-type="both"
 *           jndi-name="JMSPublisher" type="Stateless" transaction-type="Container"
 * @ejb.transaction type="Supports" <!-- end-xdoclet-definition -->
 * 
 * @ejb.resource-ref
 *   res-ref-name="jms/PrintjobQueue"
 *   res-type="javax.jms.Destination"
 *   res-auth="Container"
 *   
 * @jboss.resource-ref
 *   res-ref-name="jms/PrintjobQueue"
 *   jndi-name="jms/JIPPS.PrintJobQueue"
 * 
 * @ejb.resource-ref
 *   res-ref-name="jms/PrintjobTopic"
 *   res-type="javax.jms.Destination"
 *   res-auth="Container"
 *   
 * @jboss.resource-ref
 *   res-ref-name="jms/PrintjobTopic"
 *   jndi-name="jms/JIPPS.PrintJobTopic"
 * 
 * @ejb.resource-ref
 *   res-ref-name="jms/ConnectionFactory"
 *   res-type="javax.jms.ConnectionFactory"
 *   res-auth="Container"
 *   
 * @jboss.resource-ref
 *   res-ref-name="jms/ConnectionFactory"
 *   jndi-name="jms/IGETopicConnectionFactory"
 */
public class JMSPublisherBean implements SessionBean
{
  private static final Logger LOG = Logger.getLogger( JMSPublisherBean.class );

  private static final int JMS_DELIVERY_MODE = javax.jms.DeliveryMode.PERSISTENT;

  private static final int JMS_PRIORITY = javax.jms.Message.DEFAULT_PRIORITY;

  private static final long JMS_TIME_TO_LIVE = javax.jms.Message.DEFAULT_TIME_TO_LIVE;

  private static final long JMS_REPLY_TIME_TO_LIVE = 5000;

  private Context ctx = null;

  private ConnectionFactory connectionFactory = null;

  private Connection connection = null;

  private Session session = null;

  private MessageProducer producer = null;

  private Destination destination = null;

  private PropertyJippsDAO propertyDAO;

  // the name of the Jipps-Property for the destination type
  private static final String JMS_DESTINATION_TYPE_PROPERTY_NAME = "jms_destination_type";

  // the value of the property if its a topic
  private static final String JMS_DESTINATION_TYPE_TOPIC_VALUE = "topic";

  private static final String CONNECTION_FACTORY_NAME = "java:comp/env/jms/ConnectionFactory";

  private static final String DESTINATION_NAME_QUEUE = "java:comp/env/jms/PrintjobQueue";

  private static final String DESTINATION_NAME_TOPIC = "java:comp/env/jms/PrintjobTopic";

  SessionContext sessionContext;

  public void ejbCreate()
  {
    lookupJNDI();
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }

  private void lookupJNDI()
  {
    if ( connection == null )
    {
      try
      {
        ctx = new InitialContext();
        connectionFactory = (ConnectionFactory) ctx.lookup( CONNECTION_FACTORY_NAME );
        if ( StringUtils.equals( getPropertyDAO().select( JMS_DESTINATION_TYPE_PROPERTY_NAME ), JMS_DESTINATION_TYPE_TOPIC_VALUE ) )
        {
          LOG.debug( "Messages will be sent to " + DESTINATION_NAME_TOPIC );
          destination = (Destination) ctx.lookup( DESTINATION_NAME_TOPIC );
        }
        else
        {
          LOG.debug( "Messages will be sent to " + DESTINATION_NAME_QUEUE );
          destination = (Destination) ctx.lookup( DESTINATION_NAME_QUEUE );
        }
      }
      catch ( Exception nex )
      {
        LOG.error( "ejbCreate - Exception occured", nex );
      }
    }
  }

  private void closeJMSConnection()
  {
    try
    {
      if ( producer != null )
      {
        producer.close();
      }
      if ( session != null )
      {
        session.close();
      }
    }
    catch ( JMSException jex )
    {
      LOG.error( "ejbRemove - JmsException occured", jex );
    }
    finally
    {
      if ( connection != null )
      {
        try
        {
          connection.close();
        }
        catch ( JMSException e )
        {
          LOG.error( e.getMessage(), e );
        }
      }
    }
  }

  /**
   * Sends one print job to the print service.
   * @param printJobId The unique ID of the print job to print
   * @param watermark An additional text used as watermark (not the environment watermark)
   * @param printerId The printer on which the print job is printed
   * @throws Exception if the JMS sending fails
   * @ejb.interface-method
   */
  public void sendPrintJob( Integer printJobId, String watermark, Integer printerId ) throws Exception
  {
    this.connection = this.connectionFactory.createConnection();
    this.session = this.connection.createSession( false, Session.AUTO_ACKNOWLEDGE );
    this.connection.start();
    this.producer = this.session.createProducer( this.destination );
    TextMessage textMessage = this.session.createTextMessage();
    textMessage.clearBody();
    textMessage.setStringProperty( "printJobId", Integer.toString( printJobId ) );
    textMessage.setStringProperty( "watermark", watermark );
    textMessage.setStringProperty( "druckerId", Integer.toString( printerId ) );
    this.producer.send( textMessage, JMS_DELIVERY_MODE, JMS_PRIORITY, JMS_TIME_TO_LIVE );
    closeJMSConnection();
  }

  /**
   * Sends all print jobs of a batch run to the print service and also maybe saves them.
   * @param batchId The unique ID of the batch run to print
   * @param watermark An additional text used as watermark (not the environment watermark)
   * @param printerId The printer on which the print job is printed
   * @param saveInFileSystem If this is <code>true</code>, the print job files are saved in <code>filePath</code>
   *        using the filename <code>reportName</code> 
   * @param reportName The file name stub to use when print jobs are saved
   * @param filePath The file path where print jobs are saved (PDF files)
   * @throws Exception if the JMS sending fails
   * @ejb.interface-method
   */
  public void sendBatchJob( Integer batchId, String watermark, Integer printerId, boolean saveInFileSystem, String reportName, String filePath )
      throws Exception
  {
    this.connection = this.connectionFactory.createConnection();
    this.session = this.connection.createSession( false, Session.AUTO_ACKNOWLEDGE );
    this.connection.start();
    this.producer = this.session.createProducer( this.destination );
    TextMessage textMessage = this.session.createTextMessage();
    textMessage.clearBody();
    textMessage.setStringProperty( "watermark", watermark );
    textMessage.setStringProperty( "druckerId", Integer.toString( printerId ) );
    textMessage.setStringProperty( "batchId", Integer.toString( batchId ) );
    if ( saveInFileSystem )
    {
      textMessage.setStringProperty( "isSave", "1" );
    }
    textMessage.setStringProperty( "reportName", reportName );
    textMessage.setStringProperty( "path", filePath );
    this.producer.send( textMessage, JMS_DELIVERY_MODE, JMS_PRIORITY, JMS_TIME_TO_LIVE );
    closeJMSConnection();
  }

  /**
   * Gets all printers that are installed on the available print server(s).
   * @return The printers installed on the print servers
   * @throws Exception if JMS fails
   * @throws JMSException if the JMS sending fails
   * @ejb.interface-method
   * @ejb.transaction type="NotSupported"
   */
  public List<String> getInstalledPrintersFromPrintservice() throws Exception
  {
    try
    {
      this.connection = this.connectionFactory.createConnection();
      this.session = this.connection.createSession( false, Session.AUTO_ACKNOWLEDGE );
      this.connection.start();
      this.producer = this.session.createProducer( this.destination );
      this.producer.setDeliveryMode( JMS_DELIVERY_MODE );
      this.producer.setTimeToLive( JMS_REPLY_TIME_TO_LIVE );

      Destination replyDestination = this.session.createTemporaryQueue();
      MessageConsumer replyConsumer = this.session.createConsumer( replyDestination );

      TextMessage textMessage = this.session.createTextMessage();
      textMessage.clearBody();
      textMessage.setJMSReplyTo( replyDestination );
      this.producer.send( textMessage );

      // wait for the response ...
      ObjectMessage objectMessage = (ObjectMessage) replyConsumer.receive( JMS_REPLY_TIME_TO_LIVE + 2000 );
      if ( objectMessage != null )
      {
        LOG.debug( "Reading the object contained in the reply-message ..." );
        return (ArrayList<String>) objectMessage.getObject();
      }
      else
      {
        throw new Exception( "No JMSReply received before timeout." );
      }
    }
    finally
    {
      closeJMSConnection();
    }
  }

  /**
   * Gets the property DAO.
   * @return The property DAO
   */
  private PropertyJippsDAO getPropertyDAO()
  {
    if ( this.propertyDAO == null )
    {
      this.propertyDAO = new PropertyJippsDAO();
    }
    return this.propertyDAO;
  }
}
