package ch.ige.jipps;

import java.io.IOException;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.user.dao.MaDAO;
import ch.ige.ipobjects.user.vo.MaVO;
import ch.ige.jipps.constants.LetterTypeConstants;
import ch.ige.jipps.constants.XslConstants;
import ch.ige.jipps.dao.BfaktPosDAO;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.exception.ExceptionMessage;
import ch.ige.jipps.pdf.PdfMerger;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.BfakturVO;
import ch.ige.jipps.vo.DruckerVO;
import ch.ige.jipps.vo.LetterVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.xml.HinterlegungWriteXml;
import ch.ige.jipps.xml.LetterWriteXml;
import ch.ige.util.constants.InvoiceConstants;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.printer.ReportPaper;
import ch.ige.util.validation.Check;

/**
 * Controls the printing of letters.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Priska Steiner
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: LetterControl.java,v 1.117 2009/01/12 16:11:50 lewis_lee Exp $
 * @see LetterAssembler
 * @see LetterWriteXml
 */
public class LetterControl extends ReportControl
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( LetterControl.class );

  /**
   * The name of the XSL stylesheet to use for creating the PDF.
   */
  protected String xslName = "";

  /**
   * The number of errors that occured.
   */
  protected int numberOfErrors = 0;

  /**
   * If <code>true</code>, saves files in the file system.
   */
  protected boolean saveInFileSystem = false;

  /**
   * Additional JMS property for publication confirmation.
   */
  protected String reportName = null;

  /**
   * Additional JMS property for publication confirmation.
   */
  protected String path = null;

  /**
   * The constructor
   */
  public LetterControl()
  {
    // empty constructor
  }

  // @see ch.ige.jipps.ReportControl#makePrint(java.util.List)
  @Override
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    return makePrint( printJobs, true );
  }

  /**
   * Prints the given print jobs.
   * @param printJobs The print jobs which define what to print
   * @param sendToJms If this is <code>true</code>, the print jobs are also sent to JMS for printing,
   *        if it's <code>false</code>, no JMS queue is fed
   * @return The answer for every print job
   */
  List<AnswerVO> makePrint( List<PrintJobVO> printJobs, boolean sendToJms )
  {
    PrintJobVO printJob = null;

    if ( printJobs.size() == 0 )
    {
      return Collections.emptyList();
    }
    // if more than one job and not yet a batchId, then convert it to a batch
    else if ( ( printJobs.size() > 1 ) && ( printJobs.get( 0 ).batchId == 0 ) )
    {
      LOG.debug( "This jobList contains more than one printjob, but no batchId so far. Therefore these jobs are converted to a batch." );
      int newBatchId = getNextBatchId();
      for ( PrintJobVO printJobVO: printJobs )
      {
        printJobVO.batchId = newBatchId;
      }
    }

    boolean isBatch = ( printJobs.get( 0 ).batchId ) > 0;

    for ( Iterator<PrintJobVO> printJobIterator = printJobs.iterator(); printJobIterator.hasNext(); )
    {
      printJob = printJobIterator.next();

      String status = AnswerVO.STATUS_OK;
      List<LetterVO> letters = Collections.emptyList();
      List<Integer> printJobIds = new ArrayList<Integer>();

      try
      {
        setPublicationConfirmationDate( printJob );
        // The report ID is in fact the ID of the invoice to load
        letters = new LetterAssembler().loadLetterVO( printJob.reportId, printJob.erstDatum, printJob.standDatum, !printJob.preview );
      }
      catch ( Exception ex )
      {
        this.numberOfErrors++;
        status = "Error while loading the LetterVO: " + ex.getMessage();
      }

      // Render all letters
      for ( LetterVO letterVO: letters )
      {
        try
        {
          String reportXml = createXML( letterVO );
          printJob.printJobFile = render( reportXml, this.xslName );
        }
        catch ( Exception e )
        {
          printJob.status = e.getMessage();
        }

        printJob.printJobId = getNextPrintJobId();
        printJob.paperMode = ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB();
        if ( letters.size() > 1 ) // if this letter is part of a potzahler-group
        {
          LOG.debug( "this letter has siblings; therefore annotate the printjob " + printJob.printJobId );
          printJob.hasSiblings = true;
        }
        printJob.formattedApplicationNumber = letterVO.jippsFakturVO.gesuchNr;
        printJob.formattedIpNumber = letterVO.jippsFakturVO.schutitNr;

        storePdf( printJob );

        if ( sendToJms && !isBatch && printJob.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
        {
          sendPrintJob( printJob.printJobId, null, printJob.druckerId );
          LOG.info( "JMS-Message for single job was sent: " + printJob );
        }

        printJobIds.add( Integer.valueOf( printJob.printJobId ) );
      }
      addAnswer( printJobIds, printJob.formTyp, null, status, printJob.formattedApplicationNumber, printJob.formattedIpNumber );
    }

    if ( sendToJms && printJob != null && isBatch && printJob.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
    {
      sendBatchJob( printJob.batchId, null, printJob.druckerId, this.saveInFileSystem, this.reportName, this.path );
      LOG.info( "JMS-Message for batchjob was sent: " + printJob );
    }

    return this.answers;
  }

  // @see ch.ige.jipps.ReportControl#makePdf(java.util.List)
  @Override
  public byte[] makePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    if ( printJobs.get( 0 ).archiveCopy )
    {
      return mergeExistingPdfs( printJobs );
    }
    else
    {
      return createAndMergePdfs( printJobs );
    }
  }

  /**
   * @param printJobs List of Printjobs
   * @return all the pdfs merged into one single bytearray.
   * @throws SQLException
   */
  private byte[] mergeExistingPdfs( List<PrintJobVO> printJobs ) throws SQLException
  {
    List<byte[]> pdfFiles = new ArrayList<byte[]>();

    PrintjobDAO printJobDAO = DAOPool.get( PrintjobDAO.class );
    for ( PrintJobVO printJob: printJobs )
    {
      pdfFiles.add( printJobDAO.readPrintjob( printJob.printJobId ).printJobFile );
    }
    return PdfMerger.mergePdfs( pdfFiles );
  }

  /**
   * Creates the PDFs from the print jobs and merges them to one big PDF.
   * @param printJobs The print jobs to execute
   * @return The created and merged PDF files
   * @throws Exception if the creation fails
   */
  private byte[] createAndMergePdfs( List<PrintJobVO> printJobs ) throws Exception
  {
    List<byte[]> pdfFiles = new ArrayList<byte[]>();

    for ( PrintJobVO printJob: printJobs )
    {
      List<LetterVO> letters = Collections.emptyList();
      try
      {
        if ( printJob.hasPotZahl )
        {
          setPublicationConfirmationDate( printJob );
          letters = new LetterAssembler().loadLetterVO( printJob.reportId, printJob.erstDatum, printJob.standDatum, !printJob.preview );
        }
        else
        {
          letters = new LetterAssembler().loadLetterVO( printJob.reportId, printJob.adrNr, !printJob.preview );
        }
      }
      catch ( Exception ex )
      {
        this.numberOfErrors++;
        throw new Exception( "Fehler beim Laden des Schreibens", ex );
      }

      // Render all letters
      for ( LetterVO letterVO: letters )
      {
        letterVO.printIgekopf = printJob.printIgekopf;

        try
        {
          String reportXml = createXML( letterVO );
          printJob.printJobFile = render( reportXml, this.xslName );
        }
        catch ( Exception e )
        {
          printJob.status = e.getMessage();
        }

        if ( printJob.status == null )
        {
          pdfFiles.add( printJob.printJobFile );
        }

        if ( printJob.status != null )
        {
          LOG.error( "Error while rending the letter PDF: " + printJob.status );
          // Hand over the error to the GUI
          throw new Exception( "Fehler beim Erzeugen der PDF-Datei." );
        }
      }
    }

    return PdfMerger.mergePdfs( pdfFiles );
  }

  /**
   * Sets the publication confirmation date into {@link PrintJobVO#erstDatum} if it doesn't exist.
   * This only applies if the letter is a {@link LetterTypeConstants#PUBLICATION_CONFIRMATION_417}
   * or a {@link LetterTypeConstants#PUBLICATION_CONFIRMATION_416}.
   * @param printJob The print job whose publication confirmation date is maybe set
   */
  private void setPublicationConfirmationDate( PrintJobVO printJob )
  {
    if ( printJob.erstDatum == null && printJob.formTyp.equals( PrintJobVO.LETTER )
        && LetterTypeConstants.isTrademarkPublicationConfirmation( printJob.schutitTyp ) )
    {
      try
      {
        printJob.erstDatum = DAOPool.get( BfaktPosDAO.class ).getPublicationDate( printJob.reportId );
      }
      catch ( SQLException e )
      {
        LOG.error( "Error: Can't get the publication date for invoice " + printJob.reportId, e );
      }
    }
  }

  /**
   * Prints the publication confirmation for an IP after it has been published on Swissreg. The <code>eventDates</code>
   * contain the dates of all published events. These events are contained in invoices, and every invoice will be
   * printed as publication confirmation. The printer where this is printed depends on the user who created the invoice.
   * @param ipKey The IP for which the publication confirmation is printed
   * @param eventDates The dates of the events, each pointing to its publication date
   * @return The answer objects containing the status of every invoice print job
   */
  public List<AnswerVO> printPublicationConfirmation( Integer ipKey, Map<EreigIDat, Date> eventDates )
  {
    try
    {
      List<PrintJobVO> printJobs = createPublicationConfirmationPrintJobs( ipKey, eventDates );
      if ( printJobs.isEmpty() )
      {
        return Collections.emptyList();
      }
      else
      {
        return makePrint( printJobs );
      }
    }
    catch ( SQLException e )
    {
      LOG.error( e.getMessage(), e );
    }
    return Collections.emptyList();
  }

  /**
   * Does the same as {@link #printPublicationConfirmation(Integer, Map)}, except it only creates the PDF file and
   * doesn't print it.
   * @param ipKey The IP for which the publication confirmation is printed
   * @param eventDates The dates of the events, each pointing to its publication date
   * @return A merged PDF file with all rendered print jobs
   * @throws Exception if the rendering fails
   */
  public byte[] createPublicationConfirmationPdf( Integer ipKey, Map<EreigIDat, Date> eventDates ) throws Exception
  {
    try
    {
      List<PrintJobVO> printJobs = createPublicationConfirmationPrintJobs( ipKey, eventDates );
      if ( printJobs.isEmpty() )
      {
        return null;
      }
      else
      {
        return makePdf( printJobs );
      }
    }
    catch ( SQLException e )
    {
      LOG.error( e.getMessage(), e );
    }
    return null;
  }

  /**
   * Creates the print jobs for the publication confirmation.
   * @param ipKey The IP for which the publication confirmation is printed
   * @param eventDates The dates of the events, each pointing to its publication date
   * @return A <code>List</code> of print jobs with the invoice letters
   * @throws SQLException if the print jobs selection fails
   */
  protected List<PrintJobVO> createPublicationConfirmationPrintJobs( Integer ipKey, Map<EreigIDat, Date> eventDates ) throws SQLException
  {
    // Check parameters
    Check.notNegative( ipKey, "IP key" );
    Check.notNull( eventDates, "event dates" );
    if ( eventDates.isEmpty() )
    {
      return Collections.emptyList();
    }

    BfakturDAO bfakturDAO = DAOPool.get( BfakturDAO.class );
    MaDAO maDAO = DAOPool.get( MaDAO.class );

    // Keep only the events that match the given event dates.
    // Note that the registration event is ignored because it
    // is handled in a batch run.
    GenericIP ip = ServicesFactory.getInstance().findIP( IPKeyVO.create( ipKey ) );
    Event registrationEvent = ip.getRegistrationEvent();
    List<EreigIDat> publishedEvents = new ArrayList<EreigIDat>();
    Map<EreigIDat, Date> eventPublicationDates = new HashMap<EreigIDat, Date>();
    for ( Event event: ip.getEvents() )
    {
      if ( eventDates.containsKey( event.getEreig_idat() ) && ( registrationEvent != null && !event.sameTime( registrationEvent ) ) )
      {
        publishedEvents.add( event.getEreig_idat() );
        eventPublicationDates.put( event.getEreig_idat(), eventDates.get( event.getEreig_idat() ) );
      }
    }
    if ( publishedEvents.isEmpty() )
    {
      return Collections.emptyList();
    }

    Map<BfakturVO, Set<EreigIDat>> invoices = bfakturDAO.selectInvoices( InvoiceConstants.FORM_SWISSREG_PUBLICATION_CONFIRMATION,
        PrintStatusConstants.READY_FOR_PRINTING_STRING, ipKey, publishedEvents );
    if ( invoices.isEmpty() )
    {
      return Collections.emptyList();
    }

    BfakturVO[] allInvoices = invoices.keySet().toArray( new BfakturVO[] {} );
    MaVO employee = maDAO.findEmployee( allInvoices[0].employeeNumber );

    Integer employeePrinter = null;
    try
    {
      List<DruckerVO> printers = PrintJobStoreFactory.getInstance().selectPrinter();
      for ( DruckerVO printer: printers )
      {
        if ( printer.druckerId == employee.druckerId )
        {
          employeePrinter = Integer.valueOf( printer.druckerId );
          break;
        }
      }
    }
    catch ( Exception e )
    {
      SQLException result = new SQLException( "Couldn't get printers" );
      result.initCause( e );
      throw result;
    }
    if ( employeePrinter == null )
    {
      throw new IllegalStateException( "Found no JIPPS printer ID for user " + employee );
    }

    return createPrintJobs( invoices, eventDates, false, employeePrinter.intValue() );
  }

  /**
   * Creates the PDF file for the given XML and XSL.
   * 
   * @param xml The XML document to convert
   * @param printJobXslName The name of the XSL stylesheet to use for PDF creation
   * @return The PDF file
   * @throws Exception if the creation fails; it contains the status of the conversion,
   *         which can be used as error message when not <code>null</code>
   */
  private byte[] render( String xml, String printJobXslName ) throws Exception
  {
    try
    {
      byte[] pdf = createPdf( xml, printJobXslName );
      return pdf;
    }
    catch ( Exception ex )
    {
      String message = "Exception in LetterControl.render()";
      LOG.error( message, ex );
      throw new Exception( message + ": " + ExceptionMessage.getMessage( ex ) );
    }
  }

  /**
   * Creates the XML document.
   * @param letterVO The letter to convert to XML
   * @return The XML document
   * @throws Exception if the creation fails; it contains the status of the conversion,
   *         which can be used as error message when not <code>null</code>
   */
  private String createXML( LetterVO letterVO ) throws Exception
  {
    String reportXml = null;
    try
    {
      // Application confirmations are created differently.
      if ( LetterTypeConstants.isTrademarkApplicationConfirmation( letterVO.jippsFakturVO.formartCd )
          || letterVO.jippsFakturVO.formartCd.equals( LetterTypeConstants.APPLICATION_CONFIRMATION_105 ) )
      {
        reportXml = new HinterlegungWriteXml().toXml( letterVO );
        this.xslName = XslConstants.HINTERLEGUNG;
      }
      else
      {
        reportXml = new LetterWriteXml().toXml( letterVO );
        this.xslName = XslConstants.LETTER;
      }
    }
    catch ( IOException ex )
    {
      String message = "IOException beim Schreiben des XMLStreams";
      LOG.error( message, ex );
      throw new Exception( message + ": " + ExceptionMessage.getMessage( ex ) );
    }
    catch ( JAXBException ex )
    {
      String message = "JAXBException beim Schreiben des XMLStreams";
      LOG.error( message, ex );
      throw new Exception( message + ": " + ExceptionMessage.getMessage( ex ) );
    }

    LOG.debug( "Got XML:\n" + reportXml );
    LOG.debug( "XSL name is " + this.xslName );

    return reportXml;
  }

  /**
   * Creates print jobs for a number of invoices (<code>BFaktur</code> in German).
   * @param invoices The invoices to convert to print jobs
   * @param batchId If this is part of a bigger batch run, this contains the batch ID to use;
   *        if it's <code>null</code>, a new batch ID is generated
   * @param hasCover If this is <code>true</code>, a cover sheet is inserted, otherwise none is inserted
   * @param printerId The ID of the printer to print this print job on
   * @return The invoice print jobs, ready to be printed
   */
  protected List<PrintJobVO> createPrintJobs( Collection<BfakturVO> invoices, Integer batchId, boolean hasCover, int printerId )
  {
    Integer batchIdToUse = batchId;
    if ( batchIdToUse == null )
    {
      batchIdToUse = getNextBatchId();
    }
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();

    for ( BfakturVO invoice: invoices )
    {
      printJobs.add( createPrintJob( invoice, hasCover, printerId, batchIdToUse ) );
    }
    return printJobs;
  }

  /**
   * Creates print jobs for a number of invoices (<code>BFaktur</code> in German).
   * @param invoices The invoices which point to their event dates
   * @param eventDates The event dates to print, which point to their publication date
   * @param hasCover If this is <code>true</code>, a cover sheet is inserted, otherwise none is inserted
   * @param printerId The ID of the printer to print this print job on
   * @return A <code>List</code> of print jobs, ready to be printed
   */
  private List<PrintJobVO> createPrintJobs( Map<BfakturVO, Set<EreigIDat>> invoices, Map<EreigIDat, Date> eventDates, boolean hasCover, int printerId )
  {
    for ( BfakturVO invoice: invoices.keySet() )
    {
      Date publicationDate = null;
      for ( EreigIDat eventDate: invoices.get( invoice ) )
      {
        Date possiblePublicationDate = eventDates.get( eventDate );
        if ( possiblePublicationDate != null )
        {
          if ( publicationDate != null )
          {
            throw new IllegalStateException( "Found multiple publication dates for invoice " + invoice );
          }
          else
          {
            publicationDate = possiblePublicationDate;
          }
        }
      }

      if ( publicationDate == null )
      {
        throw new IllegalStateException( "Found no publication date for invoice " + invoice );
      }
      else
      {
        invoice.publicationDate = publicationDate;
      }
    }
    return createPrintJobs( invoices.keySet(), null, hasCover, printerId );
  }

  /**
   * Gets the invoice IDs out of the given invoices.
   * @param invoices The invoices whose IDs are extracted
   * @return The invoice IDs of <code>invoices</code>
   */
  public static List<Integer> getInvoiceIds( Collection<BfakturVO> invoices )
  {
    List<Integer> invoiceIds = new ArrayList<Integer>( invoices.size() );
    for ( BfakturVO invoice: invoices )
    {
      invoiceIds.add( Integer.valueOf( invoice.bfaktId ) );
    }
    return invoiceIds;
  }

  /**
   * Creates print jobs for an invoice.
   * @param invoice The invoice to convert
   * @param hasCover If this is <code>true</code>, a cover sheet is inserted, otherwise none is inserted
   * @param printerId The ID of the printer to print this print job on
   * @param batchId The ID of the batch run
   * @return The print job corresponding to <code>invoice</code>
   */
  private PrintJobVO createPrintJob( BfakturVO invoice, boolean hasCover, int printerId, int batchId )
  {
    PrintJobVO printJobVO = new PrintJobVO();
    printJobVO.batchId = batchId;
    printJobVO.formTyp = PrintJobVO.LETTER;
    printJobVO.druckerId = printerId;
    printJobVO.reportId = invoice.bfaktId;
    printJobVO.paperMode = ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB();
    printJobVO.anzKopien = invoice.anzKopie;
    printJobVO.druckStatus = PrintStatusConstants.READY_FOR_PRINTING;
    printJobVO.senderUser = PrintJobVO.SYSTEM;
    printJobVO.senderApp = PrintJobVO.SYSTEM;
    printJobVO.sprache = PrintJobVO.USE_IP_LANGUAGE;
    printJobVO.coverCd = hasCover;
    printJobVO.erstDatum = new Timestamp( invoice.publicationDate.getTime() );
    printJobVO.standDatum = invoice.printDate;
    return printJobVO;
  }
}
