package ch.ige.jipps;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.ipobjects.exceptions.ImageDiscardedException;
import ch.ige.ipobjects.patent.PatentImage;
import ch.ige.ipobjects.patent.dao.PatDocDAO;
import ch.ige.ipobjects.patent.vo.PatentDocumentVO;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.constants.St36Constants;
import ch.ige.jipps.pdf.PdfAreaTreeRenderer;
import ch.ige.jipps.pdf.PdfMerger;
import ch.ige.jipps.pdf.PdfRenderer;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.printer.ReportPaper;

/**
 * Creates one or more patent specifications as a PDF.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Esther Bandschapp
 * @version $Id: PatentSpecificationControl.java,v 1.25 2009/03/23 09:51:14 locher Exp $
 */
public class PatentSpecificationControl extends ReportControl
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PatentSpecificationControl.class );

  /**
   * The default status for print jobs
   */
  private static final String STATUS_OK = "ok";

  /**
   * The default constructor
   */
  public PatentSpecificationControl()
  {
    super();
  }

  // @see ch.ige.jipps.ReportControl#createPdfRenderer()
  @Override
  protected PdfRenderer createPdfRenderer()
  {
    return new PdfAreaTreeRenderer();
  }

  // @see ch.ige.jipps.ReportControl#makePdf( List<PrintJobVO> )
  @Override
  public byte[] makePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    List<byte[]> pdfFiles = new LinkedList<byte[]>(); // ordered by index, fast adding
    String xslName = St36Constants.getSt36XslName( true );

    for ( PrintJobVO printJobVO: printJobs )
    {
      AbstractVO reportVO = getReport( printJobVO.reportId );
      Integer patdocSchemaId = getPatdocSchemaId( reportVO );
      String xslDirectory = St36Constants.getSt36SchemaDirectory( patdocSchemaId );
      printJobVO.listData = getXmlDocument( reportVO );

      saveImagesToDisk( reportVO );

      printJobVO.printJobFile = createPdf( printJobVO.listData, patdocSchemaId, xslDirectory, xslName );
      pdfFiles.add( printJobVO.printJobFile );
    }
    return PdfMerger.mergePdfs( pdfFiles );
  }

  /**
   * Gets the report to transform.
   * @param reportId The report ID, can be a patent document ID or a patent search report ID
   * @return The report to transform
   * @throws SQLException if the report can't be found
   */
  protected AbstractVO getReport( Integer reportId ) throws SQLException
  {
    if ( reportId == null )
    {
      throw new IllegalArgumentException( "patDocId must not be null" );
    }
    PatentDocumentVO patent = new PatDocDAO().select( reportId );
    if ( patent == null )
    {
      throw new IllegalArgumentException( "This patdoc id does not exist, no database entry could be found" );
    }
    return patent;
  }

  /**
   * Gets the ID of the used patent document schema.
   * @param vo The report loaded by {@link #getReport(Integer)}.
   * @return The ID of the patent document schema
   */
  protected Integer getPatdocSchemaId( AbstractVO vo )
  {
    return ( (PatentDocumentVO) vo ).getPatdocschema_id();
  }

  /**
   * Gets the XML document contained in the VO.
   * @param vo The report loaded by {@link #getReport(Integer)}.
   * @return The XML document in the VO
   */
  protected String getXmlDocument( AbstractVO vo )
  {
    return ( (PatentDocumentVO) vo ).getXml();
  }

  // @see ch.ige.jipps.ReportControl#makePrint( List<PrintJobVO> )
  @Override
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    String status = StringUtils.EMPTY;

    // If there's a batch ID in the first print job, it's a batch run
    boolean isBatchRun = ( printJobs.get( 0 ) ).batchId > 0;

    PrintJobVO printJobVO = null;
    Iterator<PrintJobVO> printJobIterator = printJobs.iterator();
    while ( printJobIterator.hasNext() )
    {
      printJobVO = printJobIterator.next();
      AbstractVO reportVO = null;
      try
      {
        reportVO = getReport( printJobVO.reportId );
        printJobVO.listData = getXmlDocument( reportVO );
        saveImagesToDisk( reportVO );
      }
      catch ( SQLException e )
      {
        // Should not recover from this
        throw new RuntimeException( "Error getting the PatDoc from the database", e );
      }
      catch ( IOException e )
      {
        // Should not recover from this
        throw new RuntimeException( "Error whilst saving the PatDoc images to the disk", e );
      }
      printJobVO.printIgekopf = false;
      printJobVO.printJobId = getNextPrintJobId();
      printJobVO.paperMode = ReportPaper.PAPER_PLAIN.toDB();
      printJobVO.anzKopien = PrintJobVO.NO_COPIES;

      String xslName = St36Constants.getSt36XslName( true );
      Integer patdocSchemaId = getPatdocSchemaId( reportVO );
      String xslDirectory = St36Constants.getSt36SchemaDirectory( patdocSchemaId );
      status = render( printJobVO, patdocSchemaId, xslDirectory, xslName );
      storePdf( printJobVO );

      // Send to JMS when the print job is ready for printing (not in batch)
      if ( !isBatchRun && printJobVO.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
      {
        sendPrintJob( printJobVO.printJobId, null, printJobVO.druckerId );
        logJmsMessage( isBatchRun, printJobVO, "(isSave) = 0 , (reportName) = null , (path) = null" );
      }
      printJobVO.formattedIpNumber = new Integer( printJobVO.reportId ).toString(); //Patdoc Id

      List<Integer> printJobIds = new ArrayList<Integer>();
      printJobIds.add( printJobVO.printJobId );
      addAnswer( printJobIds, printJobVO.formTyp, null, status, printJobVO.formattedApplicationNumber, printJobVO.formattedIpNumber );

    }

    if ( isBatchRun && printJobVO != null && printJobVO.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
    {
      sendBatchJob( printJobVO.batchId, null, printJobVO.druckerId, false, null, null );
      logJmsMessage( isBatchRun, printJobVO, null );
    }
    return this.answers;
  }

  /**
   * Saves the images to the local disk (FOP_BASEDIR)
   * @param vo The VO containing the images
   * @throws IOException In case the path is wrong
   * @throws SQLException In case something with the images goes wrong
   */
  protected void saveImagesToDisk( AbstractVO vo ) throws IOException, SQLException
  {
    List<? extends PatentImage> images = ( (PatentDocumentVO) vo ).getImages();
    for ( PatentImage patentImage: images )
    {
      try
      {
        Blob data = patentImage.getImage();
        saveImageToDisk( patentImage.getFilename(), data );
      }
      catch ( ImageDiscardedException e )
      {
        e.throwAsSystemException();
      }
    }
  }

  /**
   * Saves one image to the local disk.
   * @param filename The filename to use
   * @param image The image to write
   * @throws IOException In case the path is wrong
   * @throws SQLException In case something with the images goes wrong
   */
  protected final static void saveImageToDisk( String filename, Blob image ) throws IOException, SQLException
  {
    //TODO: please use {@link LOBUtils#copyBLOBtoFile(File, Blob)} !!!
    String path = St36Constants.getSt36BaseDirectory() + filename;
    File imageFile = new File( path );
    FileOutputStream out = new FileOutputStream( imageFile.getPath() );
    out.write( image.getBytes( 1l, (int) image.length() ) );
    out.flush();
    out.close();
    //TODO Resource leak! use try-finally
  }

  /**
   * Renders the XML document into a PDF and saves it in the <code>printJob</code>.
   * @param printJob The print job containing XML in {@link PrintJobVO#listData}; its {@link PrintJobVO#printJobFile}
   * is filled with the PDF
   * @param schemaId The schema ID which indicates which schema should be used
   * @param xslDirectory The directory where the XSL is stored
   * @param xslName The name of the XSL to use for conversion
   * @return The print status, which is either {@link #STATUS_OK} or which contains the error message
   */
  private String render( PrintJobVO printJob, final int schemaId, final String xslDirectory, final String xslName )
  {
    String status = STATUS_OK;
    try
    {
      LOG.debug( "Patdoc Data " + printJob.formTyp + ": " + printJob.listData );
      printJob.printJobFile = createPdf( printJob.listData, schemaId, xslDirectory, xslName );
    }
    catch ( Exception ex )
    {
      LOG.error( "Rendering of PDF failed", ex );
      status = "Exception in PatentSpecificationControl.render(): " + ex.getMessage();
    }
    return status;
  }

  /**
   * Creates a PDF document by doing an XSL transformation on an XML document.
   * @param patentXml The XML document to transform
   * @param schemaId The schema ID which indicates which schema should be used
   * @param xslDirectory The directory where the XSL is stored
   * @param xslName The XSL stylesheet to use
   * @return A PDF file as bytes containing the result of the transformation or <code>null</code> if the
   * transformation failed
   * @throws Exception If pdf cannot be created for various reasons.
   */
  private byte[] createPdf( final String patentXml, final int schemaId, final String xslDirectory, final String xslName ) throws Exception
  {
    try
    {
      setupPDFRenderer();
      PdfAreaTreeRenderer renderer = ( (PdfAreaTreeRenderer) this.pdfRenderer );
      renderer.updatePDFRenderer( schemaId );

      // First, build an area tree
      ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

      renderer.buildAreaTree( patentXml, xslDirectory, xslName, outputStream );
      File temporary = File.createTempFile( "st36_area_tree", FileConstants.EXTENSION_XML );

      File areaTreeFile = new File( PathConfiguration.getPath( PropertyConstants.JIPPS_PATH_IMAGE )
          + StringUtils.substringAfterLast( temporary.toString(), File.separator ) );

      FileOutputStream fos = new FileOutputStream( areaTreeFile );
      BufferedOutputStream bos = new BufferedOutputStream( fos );

      bos.write( outputStream.toByteArray() );
      bos.close();
      fos.close();
      outputStream.close();

      ByteArrayOutputStream pdfStream = new ByteArrayOutputStream();

      renderer.renderPdf( patentXml, xslDirectory, PdfAreaTreeRenderer.createUrl( areaTreeFile.toString() ), xslName, pdfStream );
      return pdfStream.toByteArray();
    }
    catch ( Exception ex )
    {
      LOG.error( "START XML-STREAM:" + patentXml + "ENDE XML-STREAM", ex );
      throw ex;
    }
  }

  /**
   * Logs the information sent to JMS. TODO: Duplicated code from ListenControl -> share it!
   * @param isBatchRun Shows whether the print job is part of a batch run
   * @param printJob The print job
   * @param additionalMessage Any additional information
   */
  private static void logJmsMessage( boolean isBatchRun, PrintJobVO printJob, String additionalMessage )
  {
    StringBuffer message = new StringBuffer();
    message.append( "*** JMS-Nachricht (" );
    if ( isBatchRun )
    {
      message.append( "Batch" );
    }
    else
    {
      message.append( "Original" );
    }
    message.append( ") wurde gesendet:" );
    message.append( "(isBatch) = " );
    message.append( isBatchRun );
    message.append( "(PrintJobId) = " );
    message.append( printJob.printJobId );
    message.append( "(DruckerId) = " );
    message.append( printJob.druckerId );
    message.append( "(BatchId) = " );
    message.append( printJob.batchId );
    if ( additionalMessage != null )
    {
      message.append( additionalMessage );
    }
    LOG.info( message.toString() );
  }

}
