package ch.ige.jipps;

import java.io.IOException;
import java.io.OutputStream;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.constants.LanguageConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Servlet which can be called in order to get a report, a letter or a patdoc via a browser.
 * <p>
 * Copyright (c) 2006-2007, Eidgenössisches Institut für Geistiges Eigentum
 * @author Esther Bandschapp
 * @version $Id: PdfRenderServlet.java,v 1.92 2009/03/23 09:51:14 locher Exp $
 */

public class PdfRenderServlet extends HttpServlet
{
  /**
   * The facade we use
   */
  private JIPPSFacade jippsFacade;

  /**
   * The home facade we use
   */
  private JIPPSFacadeHome jippsFacadeHome;

  /**
   * Indicator whether this is a report, a letter or a patdoc Based on this indicator it is decided which control to use
   */
  private ControlType controlType;

  /**
   * The enumeration of allowed Types
   * <p>
   * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
   * </p>
   * @author Esther Bandschapp
   * @version $Id: PdfRenderServlet.java,v 1.92 2009/03/23 09:51:14 locher Exp $
   */
  enum ControlType
  {
    /**
     * The report
     */
    REPORT
    {
      public String getFormType()
      {
        return "Report";
      }
    },

    /**
     * The letter
     */
    LETTER
    {
      public String getFormType()
      {
        return "Letter";
      }
    },

    /**
     * The patdoc
     */
    PATDOC
    {
      public String getFormType()
      {
        return "PatentSpecification";
      }
    },

    /**
     * None of the above, which ends in an error
     */
    NONE;

    /**
     * The method to override
     * @return A String with the exact letters to call the Control
     */
    public String getFormType()
    {
      return StringUtils.EMPTY;
    }
  }

  static
  {
// activate for testing:
// PathConfiguration.setPathsForLocalTest();
  }

  /**
   * The constructor
   */
  public PdfRenderServlet()
  {
    this.jippsFacade = getJippsRemote();
  }

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PdfRenderServlet.class );

  /**
   * Implements the doGet method of the request
   * @param request The HttpServletRequest
   * @param response The HttpServletResponse
   * @throws IOException if anything goes wrong
   */
  public void doGet( HttpServletRequest request, HttpServletResponse response ) throws IOException
  {
    List<PrintJobVO> printJobs = null;
    byte[] pdfContent = null;
    OutputStream out = null;

    this.controlType = getReportTyp( request );

    try
    {
      response.setContentType( "application/pdf" );

      // Careful with name
      response.setHeader( "Content-disposition", "attachment; filename=" + getFileName( request, this.controlType ).toString() );
      printJobs = createPrintJobs( request, this.controlType );
      out = response.getOutputStream();
      pdfContent = this.jippsFacade.getMergePdf( printJobs );
      out.write( pdfContent, 0, pdfContent.length );
      out.close();
    }
    catch ( ParseException ex )
    {
      LOG.error( "[pdfRenderServlet] A ParseException occured", ex );
      response.sendError( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
    }
    catch ( Exception ex )
    {
      LOG.error( "[pdfRenderServlet] An Exception occured", ex );
      response.sendError( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
    }
    finally
    {
      if ( out != null )
      {
        out.close();
      }
    }
  }

  /**
   * Produces a filename depending on request parameters
   * @param request The <code>HttpServletRequest</code>
   * @param rTyp The <code>ControlType</code> for this instance
   * @return The filename
   */
  private StringBuffer getFileName( HttpServletRequest request, ControlType rTyp )
  {
    StringBuffer fileName = new StringBuffer( checkStringAndAssign( request, "control" ) );
    fileName.append( "_" );

    switch( rTyp )
    {
      case REPORT:
        fileName.append( checkStringAndAssign( request, "ipKey" ) );
        fileName.append( "_" + ControlType.REPORT.getFormType() );
        break;
      case LETTER:
        fileName.append( checkStringAndAssign( request, "bfakturNr" ) );
        break;
      case PATDOC:
        fileName.append( checkStringAndAssign( request, "patdocId" ) );
        break;
      case NONE:
        //ignore
        break;
    }
    return fileName;
  }

  /**
   * Implements the doPost method of the request, which in this case just redirects to the doGet method
   * @param request The HttpServletRequest
   * @param response The HttpServletResponse
   * @throws IOException if anything goes wrong
   */
  public void doPost( HttpServletRequest request, HttpServletResponse response ) throws IOException
  {
    this.doGet( request, response );
  }

  /**
   * Creates the print jobs
   * @param request The HttpServletRequest
   * @param rTyp The <code>ControlType</code> for this instance
   * @return A List with the print jobs
   * @throws ParseException
   */
  private List<PrintJobVO> createPrintJobs( HttpServletRequest request, ControlType rTyp ) throws ParseException
  {
    int reportId = 0;
    IPType ipType = IPType.ALL;
    ReportType reportType = ReportType.NO_TYPE;
    String formType = "";
    switch( rTyp )
    {
      case REPORT:
        reportId = checkIntAndAssign( request, "ipKey" );
        ipType = IPType.parse( checkIntAndAssign( request, "ipType" ) );
        reportType = ReportType.parse( checkIntAndAssign( request, "reportType" ) );
        formType = getFormType( reportType );
        break;
      case LETTER:
        reportId = checkIntAndAssign( request, "bfakturNr" );
        formType = checkStringAndAssign( request, "control" );
        break;
      case PATDOC:
        reportId = checkIntAndAssign( request, "patdocId" );
        formType = checkStringAndAssign( request, "control" );
        break;
      case NONE:
        //ignore
        break;
    }
    int registerKz = getRegisterKz( reportType );
    int userHw = 3;

    String currentDate = request.getParameter( "currentDate" ).toString();
    String[] languages = request.getParameterValues( "lang" );

    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    if ( languages == null )
    {
      languages = new String[] { "de" };
    }
    for ( int language = 0; language < languages.length; language++ )
    {
      PrintJobVO printJob = new PrintJobVO();

      printJob.formTyp = formType;
      printJob.reportId = reportId;
      printJob.schutitTyp = ipType.toDB();
      printJob.printIgekopf = false;
      printJob.maNr = userHw;
      printJob.regauszTyp = reportType.toDB();
      printJob.registerKz = registerKz;
      printJob.sprache = getLanguageCode( languages[language] );
      printJob.standDatum = DateConverter.stringToDate( currentDate );

      printJobs.add( printJob );
      LOG.debug( "[pdfRenderServlet] added printJob: " + printJob.toString() );
    }
    return printJobs;
  }

  /**
   * Returns the register_kz corresponding to the report type.
   * @param reportType The type of the report
   * @return An <code>int</code> which stands for the register_kz which can be either 1, -1 or 0. 0 = ip is still in
   * application mode <br/> 1 = ip is registered <br/> -1 = ip has electronically applied, but is not in application
   * mode yet
   */
  private int getRegisterKz( ReportType reportType )
  {
    /** TODO Is there a logic we can take from database or so? Way too much concrete business stuff in here */
    switch( reportType )
    {
      case DOSSIER_REPORT_AFTER_APPLICATION:
      case SEARCH_REPORT_AFTER_APPLICATION:
        return IpRegisterStateConstants.APPLICATION_STATE;

      case REGISTER_REPORT_AFTER_REGISTRATION:
      case CURRENT_STATE:
      case DESIGN_IMAGE_LIST:
      default:
        return IpRegisterStateConstants.REGISTRATION_STATE;
    }
  }

  /**
   * Returns the form type corresponding to the report type.
   * @param reportType The type of the report
   * @return A <code>String</code> with the form type
   */
  private String getFormType( ReportType reportType )
  {
    /** TODO Is there a logic we can take from database or so? Way too much concrete business stuff in here */
    switch( reportType )
    {
      case DESIGN_IMAGE_LIST:
        return PrintJobVO.DESIGN_BILDER;

      case REGISTER_REPORT_AFTER_REGISTRATION:
      case DOSSIER_REPORT_AFTER_APPLICATION:
      case CURRENT_STATE:
      case SEARCH_REPORT_AFTER_APPLICATION:
      default:
        return PrintJobVO.AUSZUG;
    }
  }

  /**
   * Calls the bean
   * @return A <code>JIPPSFacade</code>
   */
  private JIPPSFacade getJippsRemote()
  {
    try
    {
      Context context = new InitialContext();
      this.jippsFacadeHome = (JIPPSFacadeHome) context.lookup( JIPPSFacadeHome.JNDI_NAME );
      this.jippsFacade = this.jippsFacadeHome.create();
    }
    catch ( CreateException cex )
    {
      LOG.error( "CreateException: ", cex );
    }
    catch ( RemoteException rex )
    {
      LOG.error( "RemoteException: ", rex );
    }
    catch ( NamingException e )
    {
      LOG.error( "NamingException: ", e );
    }
    return this.jippsFacade;

  }

  /**
   * Takes a parameter name and checks that the value it is not empty and transforms it into an int
   * @param request The <code>HttpServletRequest</code>
   * @param parameterName The parameter name
   * @return An <code>int</code> which represents the parameter
   */
  private int checkIntAndAssign( HttpServletRequest request, String parameterName )
  {
    if ( StringUtils.isNotEmpty( request.getParameter( parameterName ) ) )
    {
      return Integer.parseInt( request.getParameter( parameterName ).toString() );
    }
    return 0;
  }

  /**
   * Takes a parameter name and checks that the value it is not empty
   * @param request The <code>HttpServletRequest</code>
   * @param parameterName The parameter name
   * @return The parameter value, or an empty String
   */
  private String checkStringAndAssign( HttpServletRequest request, String parameterName )
  {
    if ( StringUtils.isNotEmpty( request.getParameter( parameterName ) ) )
    {
      return request.getParameter( parameterName ).toString();
    }
    return "";
  }

  /**
   * Takes a <code>HttpServletRequest</code> and checks for a parameter name and transforms the value into a
   * <code>ControlType</code>
   * @param request The <code>HttpServletRequest</code>
   * @return The <code>ControlType</code> to use
   */
  private ControlType getReportTyp( HttpServletRequest request )
  {
    String formTyp = checkStringAndAssign( request, "control" );
    if ( StringUtils.isEmpty( formTyp ) )
    {
      throw new IllegalArgumentException( "Must have a form type like Report, Letter or Patdoc" );
    }
    else if ( formTyp.equalsIgnoreCase( ControlType.REPORT.getFormType() ) )
    {
      return ControlType.REPORT;
    }
    else if ( formTyp.equalsIgnoreCase( ControlType.LETTER.getFormType() ) )
    {
      return ControlType.LETTER;
    }
    else if ( formTyp.equalsIgnoreCase( ControlType.PATDOC.getFormType() ) )
    {
      return ControlType.PATDOC;
    }
    else
    {
      return ControlType.NONE;
    }

  }

  /**
   * Takes a String and transforms it into a language code
   * @param lang The language as a String
   * @return The language code
   */
  private int getLanguageCode( String lang )
  {
    if ( lang.equalsIgnoreCase( LanguageConstants.FRENCH_CODE_EP ) )
    {
      return LanguageConstants.FRENCH_NUMERIC_CODE;
    }
    if ( lang.equalsIgnoreCase( LanguageConstants.ITALIAN_CODE_EP ) )
    {
      return LanguageConstants.ITALIAN_NUMERIC_CODE;
    }
    return LanguageConstants.GERMAN_NUMERIC_CODE;
  }
}
