package ch.ige.jipps;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

/**
 * The factory for getting a print job store service.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: PrintJobStoreFactory.java,v 1.1 2007/03/07 14:09:30 lewis_lee Exp $
 */
public class PrintJobStoreFactory
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PrintJobStoreFactory.class );
  
  /**
   * Shows whether this is running on a J2EE server
   */
  private static boolean runsInJ2EEMode = false;

  /**
   * Contains the print job store within a J2EE server
   */
  private static PrintJobStoreLocal j2eePrintJobStore = null;

  /**
   * Contains the local print job store for testing etc.
   */
  private static PrintJobStoreServices localPrintJobStore = null;

  /**
   * Hide this from public access, use {@link #getInstance()}.
   *
   */
  private PrintJobStoreFactory()
  {
    // Nothing to do
  }

  /**
   * Gets the print job store, either as a bean (on the J2EE server)
   * or a local class (for local tests).
   * @return The print job store
   */
  public final static PrintJobStoreServices getInstance()
  {
    if ( runsInJ2EEMode )
    {
      LOG.debug( "PrintJobStore created for J2EE server" );
      if ( PrintJobStoreFactory.j2eePrintJobStore == null )
      {
        throw new IllegalStateException( "Factory not initalized" );
      }
      return PrintJobStoreFactory.j2eePrintJobStore;
    }
    else
    {
      LOG.debug( "PrintJobStore created for standalone use" );
      return getStandalonePrintJobStore();
    }
  }

  /**
   * Gets the print job store for local use.
   * @return The print job store
   */
  public final static PrintJobStoreServices getStandalonePrintJobStore()
  {
    if ( localPrintJobStore == null )
    {
      localPrintJobStore = new PrintJobStoreServicesImpl();
    }
    return localPrintJobStore;
  }

  /**
   * Gets the print job store for use within a J2EE server.
   * This must only be called by other beans that use this, in <code>ejbCreate()</code>.
   * @return The print job store
   * @throws CreateException
   * @throws NamingException
   */
  public final static PrintJobStoreLocal getEJBPrintJobStore() throws CreateException, NamingException
  {
    if ( PrintJobStoreFactory.j2eePrintJobStore == null )
    {
      runsInJ2EEMode = true;
      InitialContext context = new InitialContext();
      PrintJobStoreLocalHome printJobStoreLocalHome = (PrintJobStoreLocalHome) context.lookup( PrintJobStoreLocalHome.COMP_NAME );
      PrintJobStoreFactory.j2eePrintJobStore = printJobStoreLocalHome.create();
    }
    
    return PrintJobStoreFactory.j2eePrintJobStore;
  }
}
