package ch.ige.jipps;

import java.net.URL;
import java.rmi.RemoteException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.jipps.constants.XslConstants;
import ch.ige.jipps.pdf.PdfRenderer;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.BfakturVO;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * The base class for all report controllers.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Priska Steiner
 * @author Johannes Lang
 * @author Marc Bouquet
 * @author Elmar Isler
 * @author Kurt Jegge
 * @author Eric Lewis
 * @version $Id: ReportControl.java,v 1.123 2008/05/16 13:58:25 lewis_lee Exp $
 */
public abstract class ReportControl
{
  private JMSPublisherHome home = null;

  private JMSPublisher jmsPublisherBean = null;

  protected URL url = null;

  /**
   * The answers to the print job requests.
   */
  protected List<AnswerVO> answers;

  /**
   * Renders the PDF files.
   */
  protected PdfRenderer pdfRenderer = null;

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( ReportControl.class );

  /**
   * The constructor
   */
  public ReportControl()
  {
    this.answers = new ArrayList<AnswerVO>();
  }

  /**
   * Prints the given print jobs. This is an abstract implementation and must be implemented in subclasses.
   * @param printJobs The print jobs which define what to print
   * @return The answer for every print job
   */
  @SuppressWarnings( "unused" )
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    addAnswer( (Integer) null, null, null, "Methode nicht implementiert", null, null );
    return this.answers;
  }

  /**
   * Creates a merged PDF file from the print jobs. This is an abstract implementation and must be implemented in
   * subclasses.
   * @param printJobs The print jobs which define what to print
   * @return This returns <code>null</code>
   * @throws Exception if the creation fails
   */
  @SuppressWarnings( "unused" )
  public byte[] makePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    return null;
  }

  /**
   * Abstract implementation of {@link ch.ige.jipps.JIPPSFacadeServices#getXml(List)}. This must be implemented in
   * subclasses. See the interface for documentation.
   * @param printJobs The print jobs which define what to print
   * @return This returns <code>null</code>
   * @throws Exception if the creation fails
   */
  @SuppressWarnings( "unused" )
  public String[] makeXml( List<PrintJobVO> printJobs ) throws Exception
  {
    return null;
  }

  /**
   * Factory method for the PdfRenderer. Override to instantiate a different subclass of PdfRenderer.
   * @return the new PdfRenderer instance
   */
  protected PdfRenderer createPdfRenderer()
  {
    return new PdfRenderer();
  }

  /**
   * Prepares the PDF renderer if that hasn't happened already.
   */
  protected void setupPDFRenderer()
  {
    if ( this.pdfRenderer == null )
    {
      this.pdfRenderer = createPdfRenderer();
    }
  }

  /**
   * Creates a PDF document by doing an XSL transformation on an XML document.
   * @param reportXml The XML document to transform
   * @param xslName The XSL stylesheet to use
   * @return A PDF file as bytes containing the result of the transformation or <code>null</code> if the
   * transformation failed
   * @throws Exception If pdf cannot be created for various reasons.
   */
  protected byte[] createPdf( final String reportXml, final String xslName ) throws Exception
  {
    try
    {
      String xslReadyReport = null;
      if ( !( xslName.equals( XslConstants.STAMMBLATT ) || xslName.equals( XslConstants.FORMULAR ) ) )
      {
        xslReadyReport = ConvertText.prepareXmlForXsl( reportXml );
      }
      else
      {
        xslReadyReport = reportXml;
      }
      setupPDFRenderer();
      return this.pdfRenderer.renderPdf( xslReadyReport, xslName );
    }
    catch ( Exception ex )
    {
      LOG.error( "START XML-STREAM:" + reportXml + "ENDE XML-STREAM", ex );
      throw ex;
    }
  }

  /**
   * Gets the PDF files in the same sequence as the print jobs.
   * @param printJobs The print jobs, containing the PDF in {@link PrintJobVO#printJobFile}
   * @return A <code>List</code> of PDF files
   */
  protected List<byte[]> getPdfFiles( List<PrintJobVO> printJobs )
  {
    LOG.debug( "getPdfFiles" );
    List<byte[]> allPdfFiles = new ArrayList<byte[]>();
    for ( PrintJobVO printJobVO: printJobs )
    {
      byte[] pdfFile = printJobVO.printJobFile;
      String pdfFileLength = ( pdfFile == null ) ? "<null>" : Integer.toString( pdfFile.length );
      LOG.debug( "Got PDF file with length " + pdfFileLength );

      allPdfFiles.add( pdfFile );
    }
    return allPdfFiles;
  }

  /**
   * Füllt die Übergebenen Werte ins VO und ruft das Bean auf zu deren Speicherung
   * @param printJobVO enthält die Angaben zum Drucken
   */
  protected void storePdf( PrintJobVO printJobVO )
  {
    printJobVO.erstDatum = new Timestamp( System.currentTimeMillis() );
    try
    {
      PrintJobStoreFactory.getInstance().insPrintJob( printJobVO );
    }
    catch ( EJBException e )
    {
      LOG.error( e.getMessage(), e );
    }
  }

  /**
   * Löscht die Printjobs mit der übergebenen Batch-Id aus der Tabelle JIPPS_PRINTJOBS
   * @param batchId int Batch-ID
   */
  protected void deletePrintJobs( int batchId )
  {
    try
    {
      PrintJobStoreFactory.getInstance().deletePrintJobs( batchId );
    }
    catch ( EJBException e )
    {
      LOG.error( e.getMessage(), e );
    }
  }

  /**
   * Löscht den Printjob mit der übergebenen Bfakt_id und Batch-Id aus der Tabelle JIPPS_PRINTJOBS
   * @param reportId int Report-ID
   * @param batchId int Batch-ID
   */
  protected void deletePrintJobs( int reportId, int batchId )
  {
    try
    {
      PrintJobStoreFactory.getInstance().deletePrintJob( reportId, batchId );
    }
    catch ( EJBException e )
    {
      LOG.error( e.getMessage(), e );
    }
  }

  /**
   * Gets the next available print job ID.
   * @return The print job ID to use for a new print job
   */
  protected int getNextPrintJobId()
  {
    int id = 0;

    try
    {
      id = PrintJobStoreFactory.getInstance().getNextPrintJobId();
    }
    catch ( EJBException e )
    {
      LOG.error( e.getMessage(), e );
    }

    return id;
  }

  /**
   * Gets the next available batch ID.
   * @return The batch ID to use for a new batch run
   */
  protected int getNextBatchId()
  {
    int id = 0;

    try
    {
      id = PrintJobStoreFactory.getInstance().getNextBatchId();
    }
    catch ( EJBException e )
    {
      LOG.error( e.getMessage(), e );
    }
    return id;
  }

  /**
   * @see ch.ige.jipps.JMSPublisherBean#sendPrintJob(Integer, String, Integer)
   */
  public void sendPrintJob( Integer printJobId, String watermark, Integer printerId )
  {
    try
    {
      Context context = new InitialContext();
      this.home = (JMSPublisherHome) context.lookup( JMSPublisherHome.JNDI_NAME );
      this.jmsPublisherBean = this.home.create();
      this.jmsPublisherBean.sendPrintJob( printJobId, watermark, printerId );
    }
    catch ( CreateException cex )
    {
      LOG.error( "CreateException bei Lookup auf JMSPublisherBean", cex );
    }
    catch ( NamingException nex )
    {
      LOG.error( "NamingException bei Lookup auf JMSPublisherBean", nex );
    }
    catch ( RemoteException rex )
    {
      LOG.error( "RemoteException bei Lookup auf JMSPublisherBean", rex );
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception bei Lookup auf JMSPublisherBean", ex );
    }
  }

  /**
   * @see ch.ige.jipps.JMSPublisherBean#sendBatchJob(Integer, String, Integer, boolean, String, String)
   */
  public void sendBatchJob( Integer batchId, String watermark, Integer printerId, boolean saveInFileSystem, String reportName, String filePath )
  {
    try
    {
      Context context = new InitialContext();
      this.home = (JMSPublisherHome) context.lookup( JMSPublisherHome.JNDI_NAME );
      this.jmsPublisherBean = this.home.create();
      this.jmsPublisherBean.sendBatchJob( batchId, watermark, printerId, saveInFileSystem, reportName, filePath );
    }
    catch ( CreateException cex )
    {
      LOG.error( "CreateException bei Lookup auf JMSPublisherBean", cex );
    }
    catch ( NamingException nex )
    {
      LOG.error( "NamingException bei Lookup auf JMSPublisherBean", nex );
    }
    catch ( RemoteException rex )
    {
      LOG.error( "RemoteException bei Lookup auf JMSPublisherBean", rex );
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception bei Lookup auf JMSPublisherBean", ex );
    }
  }

  /**
   * Gets the JIPPS properties service. Depending on how it was initialized, it works locally (for tests) or in a JEE
   * server.
   * @return The JIPPS properties service
   */
  protected JIPPSPropertiesServices getJippsProperties()
  {
    return JIPPSPropertiesFactory.getInstance();
  }

  /**
   * Creates the answer for a print job and adds it to {@link #answers}.
   * @param printJobIds The IDs of the print job
   * @param printJobType The print job type which defines what controller is used
   * @param reportId The report's ID, which may for instance be the IP key
   * @param printStatus The print status, either {@link ch.ige.jipps.vo.AnswerVO#STATUS_OK} or the error message
   * @param applicationNumber The number of the application, if it already exists
   * @param ipNumber The IP number, if it exists
   */
  protected void addAnswer( List<Integer> printJobIds, String printJobType, Integer reportId, String printStatus, String applicationNumber, String ipNumber )
  {
    AnswerVO answerVO = new AnswerVO();
    answerVO.formTyp = printJobType;
    answerVO.printStatus = printStatus;
    answerVO.listPrintId.addAll( printJobIds );
    if ( reportId != null )
    {
      answerVO.schutitKey = reportId.intValue();
    }
    answerVO.gesuchNr = applicationNumber;
    answerVO.schutitNr = ipNumber;
    this.answers.add( answerVO );
  }

  /**
   * Creates the answer for a print job and adds it to {@link #answers}.
   * @param printJobId The ID of the print job
   * @param printJobType The print job type which defines what controller is used
   * @param reportId The report's ID, which may for instance be the IP key
   * @param printStatus The print status, either {@link ch.ige.jipps.vo.AnswerVO#STATUS_OK} or the error message
   * @param applicationNumber The number of the application, if it already exists
   * @param ipNumber The IP number, if it exists
   */
  protected void addAnswer( Integer printJobId, String printJobType, Integer reportId, String printStatus, String applicationNumber, String ipNumber )
  {
    List<Integer> printJobIds = new ArrayList<Integer>();
    printJobIds.add( printJobId );
    addAnswer( printJobIds, printJobType, reportId, printStatus, applicationNumber, ipNumber );
  }

  /**
   * Creates the answer for a print job and adds it to {@link #answers}.
   * @param printJobId The ID of the print job
   * @param printJobType The print job type which defines what controller is used
   * @param reportId The report's ID, which may for instance be the IP key
   * @param printStatus The print status, either {@link ch.ige.jipps.vo.AnswerVO#STATUS_OK} or the error message
   * @param applicationNumber The number of the application, if it already exists
   * @param ipNumber The IP number, if it exists
   */
  protected void addAnswer( int printJobId, String printJobType, int reportId, String printStatus, String applicationNumber, String ipNumber )
  {
    Integer thePrintJobId = new Integer( 0 );
    if ( printJobId != 0 )
    {
      thePrintJobId = new Integer( printJobId );
    }
    Integer theReportId = new Integer( 0 );
    if ( reportId != 0 )
    {
      theReportId = new Integer( reportId );
    }
    addAnswer( thePrintJobId, printJobType, theReportId, printStatus, applicationNumber, ipNumber );
  }

  /**
   * Logs the invoices including their IDs.
   * @param invoices The invoices to log
   * @param invoiceDescription The description of the kind of invoices, for instance <code>cancellation letters</code>
   */
  protected void logInvoices( List<BfakturVO> invoices, String invoiceDescription )
  {
    LOG.info( "Got " + invoices.size() + " " + invoiceDescription + "." );
    if ( LOG.isDebugEnabled() )
    {
      List<Integer> ids = new ArrayList<Integer>( invoices.size() );
      for ( BfakturVO cancellation: invoices )
      {
        ids.add( cancellation.bfaktId );
      }
      LOG.debug( invoiceDescription + " contain IDs: " + StringUtils.join( ids.iterator(), ", " ) );
    }
  }
}
