package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.AmvErrorVO;
import ch.ige.util.validation.Check;

/**
 * Accesses the table <code>JIPPS_AMV_ERROR</code>.
 * If during a batch run of trademark renewals, an error occurs, then the error message is written into this.
 * 
 * <p>Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: AmvErrorDAO.java,v 1.18 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class AmvErrorDAO extends AbstractJippsDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<AmvErrorVO> VO_TYPE = AmvErrorVO.class;

  /**
   * Selects all errors for a batch run
   */
  private static final SQLStatement<AmvErrorVO> SELECT_BATCH_RUN = SQL.select( "SELECT amv_lauf_id, adr_nr, schutit_key, error_msg, created_dat "
      + "FROM jipps_amv_error WHERE amv_lauf_id = ?" );

  /**
   * Inserts an error
   */
  private static final SQLStatement<Integer> INSERT = SQL.insert( "INSERT INTO jipps_amv_error ( amv_lauf_id, adr_nr, schutit_key, error_msg ) "
      + "VALUES ( ?, ?, ?, ? )" );

  /**
   * Deletes all errors of a batch run
   */
  private static final SQLStatement<Integer> DELETE_WHOLE_BATCH_RUN = SQL.update( "DELETE FROM jipps_amv_error WHERE amv_lauf_id = ?" );

  /**
   * Deletes just one error
   */
  private static final SQLStatement<Integer> DELETE_ONE_ERROR = DELETE_WHOLE_BATCH_RUN.append( " AND adr_nr = ? AND schutit_key = ?" );

  /**
   * Gets all errors that occured during the trademark renewal batch run.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @return The errors that occured during the batch run
   * @throws SQLException if the query fails
   */
  public List<AmvErrorVO> selectAMVErrorToId( int amvBatchRunId ) throws SQLException
  {
    return selectVOList( 50, VO_TYPE, SELECT_BATCH_RUN, new Integer( amvBatchRunId ) );
  }

  /**
   * Inserts an error.
   * @param error The error to insert
   */
  @SuppressWarnings( "unchecked" )
  public void insert( final AmvErrorVO error )
  {
    Check.notNull( error, "AMV error" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_error_insert";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( INSERT, error.amvLaufId, error.adrNr, error.schutitKey, error.errorMsg );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }
    } );
  }

  /**
   * Deletes all errors that match the criteria.
   * @param amvBatchRunId The unique ID of the AMV batch run
   * @param addressNumber  The unique ID of the address to which the renewal is sent 
   * @param ipKey The unique ID of the IP (trademark in this case)
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final int amvBatchRunId, final int addressNumber, final int ipKey )
  {
    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_error_delete_one";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( DELETE_ONE_ERROR, amvBatchRunId, addressNumber, ipKey );
        verifyMinimalUpdateCount( count, 1, "Update count must be at least 1" );
      }
    } );
  }

  /**
   * Deletes all errors of a trademark renewal batch run.
   * @param amvBatchRunId The unique ID of the AMV batch run
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final int amvBatchRunId )
  {
    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "amv_error_delete";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( DELETE_WHOLE_BATCH_RUN, amvBatchRunId );
        verifyMinimalUpdateCount( count, 1, "Update count must be at least 1" );
      }
    } );
  }

}
