package ch.ige.jipps.dao;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import oracle.sql.BFILE;

import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.validation.Check;

/**
 * PrintjobDAO for use with letters; is implemented as extra DAO because of the BFILE-stuff
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   lang
 * @version  $Id: LetterPrintjobDAO.java,v 1.24 2008/05/09 09:34:17 lang Exp $
 */
public class LetterPrintjobDAO extends AbstractJippsDAO
{
  protected static final Logger LOG = Logger.getLogger( LetterPrintjobDAO.class );

  // this storedfunction is used to write letter-pdfs into Oracle-BFILE-Objects
  private static final String STOREDFUNCTION_WRITEBFILE = "begin ? := jipps.write_bfile(?,?,?); end;";

  private static final SQLStatement<PrintJobVO> BASE_SELECT = SQL.select( "SELECT printjob_id AS print_job_id, batch_id, drucker_id, paper_mode, anz_kopien, "
      + "druck_status, senderapp AS sender_app, senderuser AS sender_user, erst_dat AS erst_datum, typ AS form_typ, report_id, "
      + "printjob_bfile AS print_job_file, cover_cd FROM printjob " );

  private static final SQLStatement<PrintJobVO> SELECT_PRINT_JOB = BASE_SELECT.append( "WHERE printjob_id = ?" );

  private static final SQLStatement<Integer> SELECT_PRINTJOBS_BY_BFAKTID = SQL
      .select( "SELECT printjob_id FROM printjob WHERE report_id = ? ORDER BY printjob_id" );

  private static final SQLStatement<PrintJobVO> SELECT_BATCH_PRINT_JOBS = BASE_SELECT.append( "WHERE batch_id = ? ORDER BY printjob_id" );

  private static final SQLStatement<Integer> DELETE_PRINTJOB_BY_REPORTID = SQL.update( "DELETE FROM printjob WHERE report_id = ?" );

  private static final SQLStatement<Integer> INSERT_PRINTJOB = SQL.insert( "INSERT INTO printjob "
      + "(printjob_id, batch_id, drucker_id, paper_mode, anz_kopien, druck_status, senderapp, senderuser, erst_dat, typ, report_id, cover_cd)"
      + "VALUES (?,?,?,?,?,?,?,?,sysdate,?,?,?)" );

  public LetterPrintjobDAO()
  {
    super();
  }

  /**
   * read a letter printjob from the database; needs extra handling because of the Oracle-Bfile.
   * @param printJobId the printjobId
   * @return bytearray containing the printjob-pdf
   * @throws SQLException
   */
  public PrintJobVO readPrintjob( final Integer printJobId ) throws SQLException
  {
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "read Letter-Printjob from database for printjobId " + printJobId );
    }
    return (PrintJobVO) this.getQueryRunner().query( SELECT_PRINT_JOB.toString(), printJobId, new BFileHandler() );
  }

  /**
   * Gets all print jobs for a given bfaktId.
   * @param bfaktId The unique identifier of the batch run
   * @return A <code>List</code> of print jobs for the batch run
   * @throws SQLException if the print jobs can't be fetched
   */
  public List<Integer> readPrintJobsByBfaktId( final Integer bfaktId ) throws SQLException
  {
    Check.notNegative( bfaktId, "bfaktID" );
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "read Letter-Printjobs from database for bfaktId " + bfaktId );
    }
    return (List<Integer>) this.getQueryRunner().query( SELECT_PRINTJOBS_BY_BFAKTID.toString(), bfaktId, new PrintJobIdListHandler() );
  }

  /**
   * Gets all print jobs for a given batch run.
   * @param batchId The unique identifier of the batch run
   * @return A <code>List</code> of print jobs for the batch run
   * @throws SQLException if the print jobs can't be fetched
   */
  public List<PrintJobVO> readPrintjobs( final Integer batchId ) throws SQLException
  {
    Check.notNegative( batchId, "batch ID" );
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "read Letter-Printjobs from database for batchId " + batchId );
    }
    return (List<PrintJobVO>) this.getQueryRunner().query( SELECT_BATCH_PRINT_JOBS.toString(), batchId, new BFileListHandler() );
  }

  /**
   * everything has to be done "manually" (instead of using the IGEQueryRunner)
   * for 3 reasons:
   * 1: insert of row and call to storedfunction have to be done in the same transaction.
   * 2: Calls to storedfunctions are not supported in dbutils.
   * 3: BFile-Attributes are not supported by dbutils.
   * @param printJob printjob to insert
   * @throws SQLException 
   */
  public void writePrintjob( final PrintJobVO printJob ) throws SQLException
  {
    // if overwrite is true, "deprecated" printjobs are deleted first to avoid zombies.
    //TODO if a letter with overwrite=true, that has siblings, is printed again, the old group should be deleted, but actually it isn't.
    if ( printJob.overwrite && !printJob.hasSiblings )
    {
      LOG.debug( "existing printjob has to be overwritten ..." );
      int updatecount = this.getQueryRunner().update( DELETE_PRINTJOB_BY_REPORTID, printJob.reportId );
      LOG.debug( "number of printjobs deleted: " + updatecount );
    }

    BigDecimal storedFunctionReturnValue;
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "write Letter-Printjob to database for printjobId " + printJob.printJobId );
    }
    Connection con = null;
    CallableStatement cstmt = null;
    try
    {
      con = this.getDataSource().getConnection();

      PreparedStatement pstmt = con.prepareStatement( INSERT_PRINTJOB.toString() );
      pstmt.setInt( 1, printJob.printJobId );
      pstmt.setInt( 2, printJob.batchId );
      pstmt.setInt( 3, printJob.druckerId );
      pstmt.setInt( 4, printJob.paperMode );
      pstmt.setInt( 5, printJob.anzKopien );
      pstmt.setInt( 6, printJob.druckStatus );
      pstmt.setString( 7, printJob.senderApp );
      pstmt.setString( 8, printJob.senderUser );
      pstmt.setString( 9, printJob.formTyp );
      pstmt.setInt( 10, printJob.reportId );
      pstmt.setBoolean( 11, printJob.coverCd );
      pstmt.executeUpdate();

      cstmt = con.prepareCall( STOREDFUNCTION_WRITEBFILE );
      cstmt.setInt( 2, printJob.printJobId );
      cstmt.setInt( 3, printJob.reportId );
      cstmt.setBytes( 4, printJob.printJobFile );
      cstmt.registerOutParameter( 1, oracle.jdbc.OracleTypes.NUMBER );
      cstmt.execute();
      storedFunctionReturnValue = (BigDecimal) cstmt.getObject( 1 );
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "storedfunction returned " + storedFunctionReturnValue );
      }
      if ( storedFunctionReturnValue.intValue() != 0 )
      {
        throw new SQLException( "StoredFunction returned " + storedFunctionReturnValue );
      }
    }
    finally
    {
      try
      {
        if ( cstmt != null )
        {
          cstmt.close();
        }
        if ( con != null )
        {
          con.close();
        }
      }
      catch ( Exception e )
      {
        LOG.error( "Error: " + e.getMessage(), e );
      }
    }
  }

  /**
   * @param rs
   * @return a PrintJobVO containing the data from the given resultSet
   * @throws SQLException
   */
  public PrintJobVO getPrintjobVOFromResultSet( ResultSet rs ) throws SQLException
  {
    PrintJobVO printjobVO = new PrintJobVO();
    printjobVO.printJobId = rs.getInt( "print_job_id" );
    printjobVO.batchId = rs.getInt( "batch_id" );
    printjobVO.druckerId = rs.getInt( "drucker_id" );
    printjobVO.paperMode = rs.getInt( "paper_mode" );
    printjobVO.anzKopien = rs.getInt( "anz_kopien" );
    printjobVO.druckStatus = rs.getInt( "druck_status" );
    printjobVO.senderApp = rs.getString( "sender_app" );
    printjobVO.senderUser = rs.getString( "sender_user" );
    printjobVO.formTyp = rs.getString( "form_typ" );
    printjobVO.erstDatum = rs.getTimestamp( "erst_datum" );
    printjobVO.reportId = rs.getInt( "report_id" );
    printjobVO.coverCd = ( rs.getInt( "cover_cd" ) == 1 );
    printjobVO.printJobFile = getByteArrayFromBFile( rs, "print_job_file" );
    return printjobVO;
  }

  /**
   * ResultSetHandler for single BFile-Rows
   */
  class BFileHandler implements ResultSetHandler
  {
    public Object handle( ResultSet rs ) throws SQLException
    {
      if ( rs.next() )
      {
        return getPrintjobVOFromResultSet( rs );
      }
      else
      {
        return null;
      }
    }

  }

  /**
   * ResultSetHandler for Lists of BFile-Rows
   */
  class BFileListHandler implements ResultSetHandler
  {
    public Object handle( ResultSet rs ) throws SQLException
    {
      List<PrintJobVO> list = new ArrayList<PrintJobVO>();
      while ( rs.next() )
      {
        list.add( getPrintjobVOFromResultSet( rs ) );
      }
      return list;
    }
  }

  /**
   * ResultSetHandler for Lists of PrintjobIds
   */
  class PrintJobIdListHandler implements ResultSetHandler
  {
    public Object handle( ResultSet rs ) throws SQLException
    {
      List<Integer> list = new ArrayList<Integer>();
      while ( rs.next() )
      {
        list.add( rs.getInt( "printjob_id" ) );
      }
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "number of printjobs found: " + list.size() );
      }
      return list;
    }
  }

  /**
   * Returns a byte array contained in an Oracle bfile column.
   * @param rs The result set
   * @param columnName the name of the column containing the bfile
   * @return The Oracle BFile content
   * @throws SQLException if the conversion fails
   */
  protected byte[] getByteArrayFromBFile( ResultSet rs, String columnName ) throws SQLException
  {
    byte[] byteArray = null;
    BFILE bfile;
    bfile = (BFILE) rs.getObject( columnName );
    bfile.openFile();
    try
    {
      byteArray = IOUtils.toByteArray( bfile.getBinaryStream() );
    }
    catch ( IOException e )
    {
      LOG.error( "Error: " + e.getMessage(), e );
    }
    bfile.closeFile();
    return byteArray;
  }
}
