package ch.ige.jipps.dao;

import java.sql.SQLException;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.util.validation.Check;

/**
 * Accesses the table <code>LOESCHUNG_LAUF</code>.
 * 
 * <p>Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: LoeschungLaufDAO.java,v 1.30 2008/05/22 09:23:30 lewis_lee Exp $
 */
public class LoeschungLaufDAO extends AbstractJippsDAO
{
  /**
   * This is always inserted as a constant, since it seems to have lost its meaning.
   */
  private static final String RUNNING_NUMBER = "99";

  /**
   * The type of value object to return
   */
  private static final Class<LoeschungLaufVO> VO_TYPE = LoeschungLaufVO.class;

  /**
   * The name of the cancellation run ID sequence
   */
  private static final String CANCELLATION_RUN_ID_SEQUENCE = "seq_loeschung_lauf_id";

  /**
   * Selects a cancellation run
   */
  private static final SQLStatement<LoeschungLaufVO> SELECT = SQL
      .select( "SELECT loeschung_lauf_id, lauf_nr, verarb_start, verarb_ende, jipps_stat, anz_bfakt_gel, "
          + "anz_loesch_gen, created_dat AS created_date, changed_dat AS changed_date, druck_art " + "FROM loeschung_lauf " + "WHERE loeschung_lauf_id = ?" );

  /**
   * Inserts a cancellation run
   */
  private static final SQLStatement<Integer> INSERT = SQL
      .insert( "INSERT INTO loeschung_lauf " + "(loeschung_lauf_id, lauf_nr, verarb_start, jipps_stat, "
          + "anz_bfakt_gel, created_dat, changed_dat, druck_art) " + "VALUES (" + CANCELLATION_RUN_ID_SEQUENCE
          + ".nextval, ?, sysdate, ?, ?, sysdate, sysdate, ?)" );

  /**
   * Updates a cancellation run
   */
  private static final SQLStatement<Integer> UPDATE = SQL.update( "UPDATE loeschung_lauf "
      + " SET verarb_ende = sysdate, jipps_stat = ?, anz_loesch_gen = ?, changed_dat = sysdate " + "WHERE loeschung_lauf_id = ? " );

  /**
   * Deletes a cancellation run
   */
  private static final SQLStatement<Integer> DELETE = SQL.update( "DELETE FROM loeschung_lauf WHERE loeschung_lauf_id = ?" );

  /**
   * Selects a cancellation run.
   * @param cancellationRunId The ID of the cancellation run
   * @return The cancellation run or <code>null</code> if none is found
   * @throws SQLException
   */
  public LoeschungLaufVO select( int cancellationRunId ) throws SQLException
  {
    Integer theCancellationRunId = Integer.valueOf( cancellationRunId );
    Check.notNegative( theCancellationRunId, "cancellation run ID" );
    return selectVO( VO_TYPE, SELECT, theCancellationRunId );
  }

  /**
   * Inserts a cancellation run.
   * The following variables are set to the current database date/time:
   * <ul>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#verarbStart}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#createdDate}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#changedDate}</li>
   * </ul>
   * The following variables are ignored and set to <code>null</code> in the database:
   * <ul>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#verarbEnde}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#anzLoeschGen}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#changedDate}</li>
   * </ul>
   * @param cancellationRun The cancellation run to insert
   * @return The ID of the inserted cancellation run
   */
  public Integer insert( final LoeschungLaufVO cancellationRun )
  {
    Check.notNull( cancellationRun, "cancellation run" );

    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_lauf_insert";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        String cancellationRunTypeString = "";
        if ( cancellationRun.druckArt != null )
        {
          cancellationRunTypeString = cancellationRun.druckArt.toString();
        }
        int count = runner.update( INSERT, RUNNING_NUMBER, cancellationRun.jippsStat, cancellationRun.anzBfaktGel, cancellationRunTypeString );
        verifyUpdateCount( count, 1, "update count must be 1" );
        this.setResult( selectCurrentSequenceValue( runner, CANCELLATION_RUN_ID_SEQUENCE ) );
      }
    } );
  }

  /**
   * Updates a cancellation run.
   * The following variables are set to the current database date/time:
   * <ul>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#verarbEnde}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#changedDate}</li>
   * </ul>
   * Only the following variables are updated:
   * <ul>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#jippsStat}</li>
   *  <li>{@link ch.ige.jipps.vo.LoeschungLaufVO#anzLoeschGen}</li>
   * </ul>
   * @param cancellationRun The cancellation run to update
   */
  @SuppressWarnings( "unchecked" )
  public void update( final LoeschungLaufVO cancellationRun )
  {
    Check.notNull( cancellationRun, "cancellation run" );
    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_lauf_update";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int numberOfUpdates = runner.update( UPDATE, cancellationRun.jippsStat, cancellationRun.anzLoeschGen, cancellationRun.loeschungLaufId );
        verifyUpdateCount( numberOfUpdates, 1, "Couldn't udpate cancellation run with ID " + cancellationRun.loeschungLaufId );
      }
    } );
  }

  /**
   * Deletes the whole cancellation run including the cancellations
   * and cancellation errors (by cascading delete).
   * @param cancellationRunId The ID of the cancellation run
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final int cancellationRunId )
  {
    Check.notNegative( cancellationRunId, "cancellation run ID" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_lauf_delete";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int numberOfUpdates = runner.update( DELETE, Integer.valueOf( cancellationRunId ) );
        verifyUpdateCount( numberOfUpdates, 1, "Couldn't delete cancellation run with ID " + cancellationRunId );
      }
    } );
  }
}
