package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.dao.CachingDAO;
import ch.ige.dbutils.handlers.VOMapHandler;
import ch.ige.dbutils.handlers.VOMapHandler.MapBuilder;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.PropertyJippsVO;
import ch.ige.util.validation.Check;

/**
 * DAO for Table <code>PROPERTY_JIPPS</code>. Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Johannes Lang
 * @version $Id: PropertyJippsDAO.java,v 1.26 2009/01/20 15:56:52 lewis_lee Exp $
 */
public class PropertyJippsDAO extends AbstractJippsDAO implements CachingDAO
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PropertyJippsDAO.class );

  /**
   * The number of properties in the DB
   */
  private static final int EXPECTED_NUMBER_OF_PROPERTIES = 60;

  /**
   * Selects all properties
   */
  private static final SQLStatement<PropertyJippsVO> SELECT_PROPERTIES = SQL.select( "SELECT key, value, description FROM property_jipps" );

  /**
   * Inserts one property
   */
  private static final SQLStatement<Integer> INSERT = SQL.insert( "INSERT INTO property_jipps (key, value, description) VALUES(?, ?, ?)" );

  /**
   * Updates one property
   */
  private static final SQLStatement<Integer> UPDATE = SQL.update( "UPDATE property_jipps SET value = ?, description = ? WHERE key = ?" );

  /**
   * Deletes one property
   */
  private static final SQLStatement<Integer> DELETE = SQL.update( "DELETE FROM property_jipps WHERE key = ?" );

  /**
   * Builds the properties map based on the property key.
   */
  private static final MapBuilder<PropertyJippsVO, String, PropertyJippsVO> MAP_BUILDER = new VOMapHandler.OneToOneMapBuilder<String, PropertyJippsVO>( "key",
      false );

  /**
   * The cached properties
   */
  private static Map<String, PropertyJippsVO> jippsProperties;

  /**
   * Shows that the properties have changed
   */
  private static boolean dataChanged = true;

  /**
   * Gets the Properties from the database, which may be cached.
   * 
   * @return A <code>Map</code> where the key is the property key and the value is the whole property
   * @throws SQLException if the properties can't be loaded
   */
  private Map<String, PropertyJippsVO> getProperties() throws SQLException
  {
    if ( dataChanged )
    {
      LOG.debug( "Load JIPPS Properties from database ..." );
      jippsProperties = selectPropertiesFromDB();
      dataChanged = false;
    }
    return jippsProperties;
  }

  /**
   * Loads the properties from the database into the local cache.
   * @return A <code>Map</code> where the key is the property key, and the value is the whole property
   * @throws SQLException if the properties can't be loaded
   */
  private Map<String, PropertyJippsVO> selectPropertiesFromDB() throws SQLException
  {
    return this.selectVOMap( EXPECTED_NUMBER_OF_PROPERTIES, PropertyJippsVO.class, SELECT_PROPERTIES, MAP_BUILDER );
  }

  /**
   * Loads all property values from the database.
   * 
   * @return A <code>List</code> of {@link ch.ige.jipps.vo.PropertyJippsVO} objects
   * @exception SQLException if the properties can't be loaded
   */
  public List<PropertyJippsVO> select() throws SQLException
  {
    List<PropertyJippsVO> properties = new ArrayList<PropertyJippsVO>();
    properties.addAll( getProperties().values() );
    return properties;
  }

  /**
   * Gets the property value specified by <code>key</code> either from the System Properties or from the database.
   * This allows a local test to run without database by setting a property in the System Properties.
   * 
   * @param key The property key to get
   * @return The property value for <code>key</code>
   * @throws SQLException if the property can't be fetched
   */
  public String select( String key ) throws SQLException
  {
    Check.notEmpty( key, "key" );
    if ( System.getProperty( key ) == null || System.getProperty( key ).length() == 0 )
    {
      PropertyJippsVO property = selectVO( key );
      if ( property == null )
      {
        return null;
      }
      else
      {
        return property.value;
      }
    }
    else
    {
      return System.getProperty( key );
    }
  }

  /**
   * Gets the property specified by <code>key</code>.
   * 
   * @param key The property key to get
   * @return The full property containing <code>key</code>
   * @throws SQLException if the property can't be fetched
   */
  public PropertyJippsVO selectVO( String key ) throws SQLException
  {
    Check.notEmpty( key, "key" );
    return getProperties().get( key );
  }

  /**
   * Inserts a property into the database.
   * 
   * @param jippsPropertyVO The full property to insert
   */
  @SuppressWarnings( "unchecked" )
  public void insert( final PropertyJippsVO jippsPropertyVO )
  {
    Check.notNull( jippsPropertyVO, "property" );
    this.execute( new SingleDAOUnitOfWork()
    {
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int updateCount = runner.update( INSERT, jippsPropertyVO.key, jippsPropertyVO.value, jippsPropertyVO.description );
        verifyUpdateCount( updateCount, 1, "update count must be 1" );
        dataChanged = true;
      }
    } );
  }

  /**
   * Updates a property in the database.
   * 
   * @param jippsPropertyVO The full property to update
   */
  @SuppressWarnings( "unchecked" )
  public void update( final PropertyJippsVO jippsPropertyVO )
  {
    Check.notNull( jippsPropertyVO, "property" );
    this.execute( new SingleDAOUnitOfWork()
    {
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int updateCount = runner.update( UPDATE, jippsPropertyVO.value, jippsPropertyVO.description, jippsPropertyVO.key );
        verifyUpdateCount( updateCount, 1, "update count must be 1" );
        dataChanged = true;
      }
    } );
  }

  /**
   * Deletes a property from the database.
   * 
   * @param key The property key
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final String key )
  {
    Check.notEmpty( key, "key" );

    this.execute( new SingleDAOUnitOfWork()
    {
      public void execute( IGEQueryRunner queryRunner ) throws SQLException
      {
        int updatedRows = queryRunner.update( DELETE, key );
        verifyUpdateCount( updatedRows, 1, "update count must be 1 for property with " + key );
        dataChanged = true;
      }
    } );
  }

  // @see ch.ige.dbutils.dao.CachingDAO#clearCache()
  public void clearCache()
  {
    jippsProperties = null;
    dataChanged = true;
  }

  // @see ch.ige.dbutils.dao.CachingDAO#getCacheSize()
  public int getCacheSize()
  {
    return jippsProperties == null ? 0 : jippsProperties.size();
  }

  // @see ch.ige.dbutils.dao.CachingDAO#describeCachedContent()
  public CharSequence describeCachedContent()
  {
    return jippsProperties == null ? "" : jippsProperties.toString();
  }
}
