package ch.ige.jipps.dao;

import java.sql.Date;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.PrintjobSearchVO;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.validation.Check;

/**
 * Searches for print jobs in the table <code>PRINTJOB</code>. 
 * <p>Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: SearchDAO.java,v 1.30 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class SearchDAO extends AbstractJippsDAO
{
  //TODO Refactor this: It should only use PrintJobVOs

  /**
   * The type of value object to return
   */
  private static final Class<PrintJobVO> VO_TYPE = PrintJobVO.class;

  /**
   * Selects all existing print job types
   */
  private static final SQLStatement<PrintJobVO> SELECT_PRINT_JOB_TYPES = SQL.select( "SELECT DISTINCT LOWER(typ) AS form_typ FROM printjob" );

  /**
   * Selects print jobs with a number of criteria
   */
  private static final String BASE_SELECT = "SELECT printjob_id, report_id, batch_id, drucker_id, senderuser AS sender, LOWER(typ) AS formtyp, "
      + "druck_status, erst_dat, cover_cd FROM printjob";

  /**
   * Adds a <code>WHERE</code> to the SQL query.
   */
  private static final String WHERE_CLAUSE = " WHERE ";

  /**
   * Updates the print job to be ready to print
   */
  private static final SQLStatement<Integer> UPDATE = SQL.update( "UPDATE printjob SET drucker_id = ?, druck_status = ? WHERE printjob_id = ?" );

  /**
   * Searches for print jobs depending on the content of <code>searchVO</code>.
   * The following variables are used for searching:
   * <ul>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#printjobId} (if > 0)</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#reportId} (if > 0)</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#batchId} (if >= 0, use {@link ch.ige.jipps.vo.PrintjobSearchVO#DUMMY_BATCH_ID} for no search)</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#sender} (if not <code>null</code>)</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#formtyp} (if not <code>null</code>, independent of case)</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#druckStatus} (if not {@link ch.ige.jipps.vo.PrintjobSearchVO#DUMMY_PRINT_STATUS})</li>
   * <li>{@link ch.ige.jipps.vo.PrintjobSearchVO#erstDat} (if not <code>null</code>)</li>
   * </ul>
   * @param searchVO Contains the data for searching
   * @return All print jobs that match the search criteria
   * @throws SQLException
   */
  public List<PrintjobSearchVO> select( PrintjobSearchVO searchVO ) throws SQLException
  {
    //TODO Why does this return PrintjobSearchVOs instead of PrintJobVOs?
    Check.notNull( searchVO, "print job search" );

    StringBuilder query = new StringBuilder( BASE_SELECT );
    if ( searchVO.printjobId > 0 )
    {
      appendSearchCriteria( query, "printjob_id", null, searchVO.printjobId, null );
    }
    if ( searchVO.reportId > 0 )
    {
      appendSearchCriteria( query, "report_id", null, searchVO.reportId, null );
    }
    if ( searchVO.batchId >= 0 )
    {
      appendSearchCriteria( query, "batch_id", null, searchVO.batchId, null );
    }
    if ( searchVO.sender != null )
    {
      appendSearchCriteria( query, "senderuser", searchVO.sender, -1, null );
    }
    if ( searchVO.formtyp != null )
    {
      appendSearchCriteria( query, "LOWER(typ)", searchVO.formtyp.toLowerCase(), -1, null );
    }
    if ( searchVO.druckStatus != PrintjobSearchVO.DUMMY_PRINT_STATUS )
    {
      appendSearchCriteria( query, "druck_status", null, searchVO.druckStatus, null );
    }
    if ( searchVO.erstDat != null )
    {
      appendSearchCriteria( query, "erst_dat", null, -1, searchVO.erstDat );
    }

    SQLStatement<PrintjobSearchVO> statement = SQL.select( query.toString() );
    return selectVOList( 50, PrintjobSearchVO.class, statement );
  }

  /**
   * Appends a search criteria to the query.
   * @param query The query to append to
   * @param columnName The name of the column
   * @param stringValue A possible String value
   * @param intValue A possible int value
   * @param dateValue A possible Date value
   */
  private void appendSearchCriteria( StringBuilder query, String columnName, String stringValue, int intValue, Date dateValue )
  {
    if ( query.length() == BASE_SELECT.length() )
    {
      query.append( SearchDAO.WHERE_CLAUSE );
    }
    if ( query.length() > ( BASE_SELECT.length() + SearchDAO.WHERE_CLAUSE.length() ) )
    {
      query.append( " AND " );
    }
    if ( dateValue != null )
    {
      // Search between the given date and the next day
      Calendar endDate = Calendar.getInstance();
      endDate.setTime( dateValue );
      endDate.add( Calendar.DAY_OF_YEAR, +1 );

      SimpleDateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd" );
      String formattedStartDate = dateFormat.format( dateValue );
      String formattedEndDate = dateFormat.format( endDate.getTime() );

      query.append( columnName );
      query.append( " >= TO_DATE('" );
      query.append( formattedStartDate );
      query.append( "', 'YYYY-MM-DD') AND " );
      query.append( columnName );
      query.append( " < to_date('" );
      query.append( formattedEndDate );
      query.append( "', 'YYYY-MM-DD')" );
    }
    else
    {
      query.append( columnName );
      query.append( " = " );
      if ( stringValue != null )
      {
        query.append( "'" );
        query.append( stringValue );
        query.append( "'" );
      }
      else if ( intValue > -2 )
      {
        query.append( Integer.toString( intValue ) );
      }
    }
  }

  /**
   * Returns the distinct types of print jobs.
   * @return The distinct types of print jobs in lowercase
   * @throws SQLException if the print job types can't be fetched
   */
  public Set<String> selectPrintJobTypes() throws SQLException
  {
    List<PrintJobVO> printJobs = selectVOList( TEN, VO_TYPE, SELECT_PRINT_JOB_TYPES );
    Set<String> printJobTypes = new HashSet<String>( printJobs.size() );
    for ( PrintJobVO printJob: printJobs )
    {
      printJobTypes.add( printJob.formTyp );
    }
    return printJobTypes;
    /**
     * Aendert eine bestehende Zeile in der Tabelle Printjob, damit sie nochmal gedruckt wird.
     * @param searchVO ValueObject mit Daten zum Printjob
     * @throws SQLException
     */
  }

  /**
   * Updates the given print job to be printed once more, updating the printer ID
   * at the same time.
   * @param searchVO Contains the data for searching; only {@link ch.ige.jipps.vo.PrintjobSearchVO#printjobId}
   *        and {@link ch.ige.jipps.vo.PrintjobSearchVO#druckerId} are used
   */
  @SuppressWarnings( "unchecked" )
  public void update( final PrintjobSearchVO searchVO )
  {
    Check.notNull( searchVO, "print job search" );
    Check.notNegative( searchVO.printjobId, "print job ID" );
    this.execute( new SingleDAOUnitOfWork()
    {
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int updateCount = runner.update( UPDATE, Integer.valueOf( searchVO.druckerId ), PrintStatusConstants.READY_FOR_PRINTING, Integer
            .valueOf( searchVO.printjobId ) );
        verifyUpdateCount( updateCount, 1, "update count must be 1" );
      }
    } );
  }
}
