package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.dao.BfaktPosDAO;
import ch.ige.jipps.vo.BfaktPosVO;
import ch.ige.util.IGEDate;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.BfaktPosDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Johannes Lang
 * @author Eric Lewis
 * @version $Id: BfaktPosDAOTest.java,v 1.17 2008/07/23 13:17:37 lewis_lee Exp $
 */
public class BfaktPosDAOTest extends BaseDAOTestTemplate<BfaktPosDAO, BfaktPosVO>
{
  /**
   * Some well-known invoices
   */
  private static int[] INVOICE_IDS = new int[] { 4042820, 4042821, 4042822, 4042823 };

  /**
   * The IP keys corresponding to {@link #INVOICE_IDS}.
   */
  private static int[] IP_KEYS_FOR_INVOICES = new int[] { 1200625887, 1200625917, 1200625921, 1200557097 };

  /**
   * An invoice that points to several IPs and also contains publication texts.
   */
  private static int INVOICE_WITH_SEVERAL_IPS = 5133685;

  /**
   * The IP numbers corresponding to {@link #INVOICE_WITH_SEVERAL_IPS}.
   */
  private static String[] IP_NUMBERS_FOR_INVOICE_WITH_SEVERAL_IPS = { "P-322901", "371327", "374611", "374636", "471236", "473031", "505001", "510611", "511816" };

  /**
   * The invoice that confirms an electronic publication.
   */
  private static int INVOICE_FOR_PUBLICATION_CONFIRMATION = 6757462;

  /**
   * The only constructor
   * @param args Any arguments
   */
  public BfaktPosDAOTest( String args )
  {
    super( args );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<BfaktPosDAO> getDAOClass()
  {
    return BfaktPosDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfaktPosDAO#select(int)}.
   * @throws Exception if the test fails 
   */
  public void testSelect() throws Exception
  {
    List<BfaktPosVO> invoiceLineItems = getDAO().select( -1 );
    assertTrue( "Checking result for unknown invoice ID", invoiceLineItems.isEmpty() );

    invoiceLineItems = getDAO().select( INVOICE_IDS[0] );
    assertEquals( "Checking result for known invoice ID", 1, invoiceLineItems.size() );

    BfaktPosVO invoiceLineItem = invoiceLineItems.get( 0 );
    assertEquals( "Checking invoice ID", 4042820, invoiceLineItem.invoiceId );
    assertEquals( "Checking invoice line number", 10, invoiceLineItem.lineItemNumber );
    assertEquals( "Checking invoice event BAGIS date", "2001-03-14#05", invoiceLineItem.eventDate.toString() );
    assertEquals( "Checking invoice text number", 0, invoiceLineItem.textNumber );
    assertEquals( "Checking invoice IP number", "482682", invoiceLineItem.ipNumber );
    assertEquals( "Checking invoice number", 1, invoiceLineItem.numberOfArticles );
    assertEquals( "Checking invoice deadline", null, invoiceLineItem.deadline );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfaktPosDAO#select(int)} for ordering and event text.
   * @throws Exception if the test fails 
   */
  public void testSelectOrderAndText() throws Exception
  {
    List<BfaktPosVO> invoiceLineItems = getDAO().select( INVOICE_WITH_SEVERAL_IPS );

    assertEquals( "Checking number of line items", IP_NUMBERS_FOR_INVOICE_WITH_SEVERAL_IPS.length, invoiceLineItems.size() );

    int previousLineNumber = Integer.MIN_VALUE;
    for ( int i = 0; i < invoiceLineItems.size(); i++ )
    {
      BfaktPosVO invoiceLineItem = invoiceLineItems.get( i );
      assertTrue( "Checking line number", invoiceLineItem.lineItemNumber > previousLineNumber );
      assertEquals( "Checking IP number", IP_NUMBERS_FOR_INVOICE_WITH_SEVERAL_IPS[i], invoiceLineItem.ipNumber );
      assertEquals( "Checking invoice ID", INVOICE_WITH_SEVERAL_IPS, invoiceLineItem.invoiceId );
      assertEquals( "Checking event date", "2003-11-24#10", invoiceLineItem.eventDate.toString() );
      assertEquals( "Checking invoice number", 1, invoiceLineItem.numberOfArticles );
      assertEquals( "Checking invoice deadline", null, invoiceLineItem.deadline );
      assertEquals( "Checking publication text", "Teillizenz für Zigaretten.", invoiceLineItem.publicationText );

      previousLineNumber = invoiceLineItem.lineItemNumber;
    }
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfaktPosDAO#getRegistrationIpKeys(java.util.List)}.
   * @throws Exception if the test fails 
   */
  public void testGetRegistrationIpKeys() throws Exception
  {
    try
    {
      getDAO().getRegistrationIpKeys( null );
      fail( "null List of invoice IDs must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<Integer> ipKeys = getDAO().getRegistrationIpKeys( new ArrayList<Integer>() );
    assertNotNull( "Checking IP keys exist", ipKeys );
    assertTrue( "Checking for empty IP keys", ipKeys.isEmpty() );

    List<Integer> invoiceIds = new ArrayList<Integer>();
    for ( int invoiceId: INVOICE_IDS )
    {
      invoiceIds.add( Integer.valueOf( invoiceId ) );
    }
    ipKeys = getDAO().getRegistrationIpKeys( invoiceIds );
    assertEquals( "Checking number of IP keys", IP_KEYS_FOR_INVOICES.length, ipKeys.size() );

    for ( int i = 0; i < IP_KEYS_FOR_INVOICES.length; i++ )
    {
      int expectedIpKey = IP_KEYS_FOR_INVOICES[i];
      Integer actualIpKey = ipKeys.get( i );
      assertEquals( "Checking IP key", expectedIpKey, actualIpKey.intValue() );
    }
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfaktPosDAO#getInvoiceIdLists(List, int)}.
   */
  public void testGetInvoiceIdLists()
  {
    List<Integer> list = new ArrayList<Integer>();
    list.add( 1 );
    list.add( 2 );
    list.add( 3 );
    list.add( 4 );
    List<List<Integer>> returnList = getDAO().getInvoiceIdLists( list, 5 );
    assertTrue( returnList.size() == 1 );
    assertTrue( returnList.get( 0 ).size() == 4 );
    assertEquals( "Checking third list entry", 3, returnList.get( 0 ).get( 2 ).intValue() );

    list.add( 5 );
    returnList = getDAO().getInvoiceIdLists( list, 5 );
    assertTrue( returnList.size() == 1 );
    assertTrue( returnList.get( 0 ).size() == 5 );
    assertEquals( "Checking fifth list entry", 5, returnList.get( 0 ).get( 4 ).intValue() );

    list.add( 6 );
    returnList = getDAO().getInvoiceIdLists( list, 5 );
    assertTrue( returnList.size() == 2 );
    assertTrue( returnList.get( 0 ).size() == 5 );
    assertTrue( returnList.get( 1 ).size() == 1 );
    assertEquals( "Checking first list entry of second list", 6, returnList.get( 1 ).get( 0 ).intValue() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfaktPosDAO#getPublicationDate(int)}.
   * @throws Exception if the test fails
   */
  public void testGetPublicationDate() throws Exception
  {
    Timestamp publicationDate = getDAO().getPublicationDate( INVOICE_FOR_PUBLICATION_CONFIRMATION );
    assertNotNull( "Checking that publication date exists", publicationDate );
    assertEquals( "Checking publication date", "07.07.2008", IGEDate.dateToString( publicationDate ) );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected BfaktPosVO getSerializableVO() throws SQLException
  {
    try
    {
      BfaktPosVO invoiceLineItem = new BfaktPosVO();
      invoiceLineItem.numberOfArticles = 3;
      invoiceLineItem.invoiceId = 662;
      invoiceLineItem.lineItemNumber = 6;
      invoiceLineItem.eventDate = EreigIDat.from( 2005010322 );
      invoiceLineItem.deadline = DateConverter.stringToSqlDate( "03.08.2001" );
      invoiceLineItem.ipNumber = "518412";
      invoiceLineItem.textNumber = 523;
      return invoiceLineItem;
    }
    catch ( ParseException e )
    {
      throw new SQLException( "Couldn't parse date: " + e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( BfaktPosVO originalItem, BfaktPosVO serializedItem )
  {
    return originalItem.numberOfArticles == serializedItem.numberOfArticles && originalItem.invoiceId == serializedItem.invoiceId
        && originalItem.lineItemNumber == serializedItem.lineItemNumber && originalItem.eventDate.equals( serializedItem.eventDate )
        && originalItem.textNumber == serializedItem.textNumber && originalItem.deadline.equals( serializedItem.deadline )
        && originalItem.ipNumber.equals( serializedItem.ipNumber );
  }
}
