package ch.ige.jipps.dao.test;

import java.sql.Date;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.vo.AmvVO;
import ch.ige.jipps.vo.BfakturVO;
import ch.ige.jipps.vo.LoeschungVO;
import ch.ige.util.collection.MapConverter;
import ch.ige.util.constants.InvoiceConstants;
import ch.ige.util.constants.PrintExecutionConstants;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.date.DateFormatter;

/**
 * Tests {@link ch.ige.jipps.dao.BfakturDAO}.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: BfakturDAOTest.java,v 1.27 2009/04/21 15:23:13 lewis_lee Exp $
 */
public class BfakturDAOTest extends BaseDAOTestTemplate<BfakturDAO, BfakturVO>
{
  /**
   * An existing invoice.
   */
  private static final int INVOICE_ID = 6031380;

  /**
   * An existing invoice which has a print date.
   */
  private static final int INVOICE_ID_WITH_PRINT_DATE = 6000308;

  /**
   * An existing invoice whose publication date is taken from the event
   * that published the trademark.
   */
  private static final int INVOICE_ID_FOR_E_PUBLICATION = 6759069;

  /**
   * A trademark which has events contained in invoices.
   */
  private static final Integer TRADEMARK = Integer.valueOf( 2100335988 );

  /**
   * The event dates which are contained in the invoice 436.
   */
  private static final int[] EVENT_DATES_436 = new int[] { 2005122205 };

  /**
   * The event dates which are contained in the invoice 437.
   */
  private static final int[] EVENT_DATES_437 = new int[] { 2005122210, 2005122215, 2005122220 };

  /**
   * Default constructor.
   * 
   * @param name The test's name
   */
  public BfakturDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<BfakturDAO> getDAOClass()
  {
    return BfakturDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectBFaktIdLoeschung(java.lang.String, java.lang.String)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectBFaktIdLoeschung() throws Exception
  {
    try
    {
      getDAO().selectBFaktIdLoeschung( null, PrintStatusConstants.READY_FOR_PRINTING_STRING );
      fail( "Must fail with null print type" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectBFaktIdLoeschung( PrintExecutionConstants.BATCH_PRINTING, null );
      fail( "Must fail with null print status" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<LoeschungVO> cancellations = getDAO().selectBFaktIdLoeschung( PrintExecutionConstants.BATCH_PRINTING, PrintStatusConstants.READY_FOR_PRINTING_STRING );
    assertNotNull( "Checking for empty result", cancellations );
    assertTrue( "Checking for result size", cancellations.size() >= 3 );

    LoeschungVO cancellation = cancellations.get( 0 );
    assertTrue( "Checking first invoice ID", cancellation.bfaktId > 5000000 );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectBFakturByForm(Integer, String)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectBFakturByForm() throws Exception
  {
    try
    {
      getDAO().selectBFakturByForm( null, PrintStatusConstants.READY_FOR_PRINTING_STRING );
      fail( "Must fail with null form type" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectBFakturByForm( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION, null );
      fail( "Must fail with null print status" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<BfakturVO> invoices = getDAO().selectBFakturByForm( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION,
        PrintStatusConstants.READY_FOR_PRINTING_STRING );
    assertNotNull( "Checking for empty result", invoices );
    assertTrue( "Checking for result size", invoices.size() >= 1 );

    BfakturVO invoice = findInvoiceById( invoices, INVOICE_ID );
    assertNotNull( "Checking for known invoice ID", invoice );
    assertEquals( "Checking first number of copies", 1, invoice.anzKopie );
    assertEquals( "Checking employee number", 125, invoice.employeeNumber.intValue() );
    assertEquals( "Checking IP key", 1200016048, invoice.ipKey.intValue() );

    // Check for publication date using ePublication
    invoices = getDAO().selectBFakturByForm( InvoiceConstants.FORM_ELECTRONIC_PUBLICATION_CONFIRMATION, PrintStatusConstants.READY_FOR_PRINTING_STRING );
    assertNotNull( "Checking for empty result", invoices );
    if ( invoices.isEmpty() )
    {
      // They may have already been printed, so get a result in any case.
      invoices = getDAO().selectBFakturByForm( InvoiceConstants.FORM_ELECTRONIC_PUBLICATION_CONFIRMATION, PrintStatusConstants.PRINTED_STRING );
    }
    assertTrue( "Checking for result size", invoices.size() >= 1 );

    invoice = findInvoiceById( invoices, INVOICE_ID_FOR_E_PUBLICATION );
    assertNotNull( "Checking for known invoice ID", invoice );
    assertEquals( "Checking publication date", "08.07.2008", DateFormatter.shortFormat( invoice.publicationDate ) );
    assertEquals( "Checking first number of copies", 0, invoice.anzKopie );
    assertEquals( "Checking employee number", 131, invoice.employeeNumber.intValue() );
    assertEquals( "Checking IP key", 1202175468, invoice.ipKey.intValue() );
  }

  /**
   * Finds the required invoice ID in the given List of invoices.
   * @param invoices A <code>List</code> of {@link ch.ige.jipps.vo.BfakturVO} objects
   * @param invoiceId The invoice ID to look for
   * @return The requested invoice or <code>null</code> if it wasn't found
   */
  private BfakturVO findInvoiceById( final List<BfakturVO> invoices, final int invoiceId )
  {
    BfakturVO invoiceFound = null;
    for ( BfakturVO invoice: invoices )
    {
      if ( invoice.bfaktId == invoiceId )
      {
        invoiceFound = invoice;
        break;
      }
    }
    return invoiceFound;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectBFakturDruckstatus(int)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectBFakturDruckstatus() throws Exception
  {
    assertNull( "Checking unknown invoice ID", getDAO().selectBFakturDruckstatus( -33 ) );

    String printStatus = getDAO().selectBFakturDruckstatus( INVOICE_ID );
    assertEquals( "Checking print status", PrintStatusConstants.READY_FOR_PRINTING_STRING, printStatus );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectInvoice(int)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectInvoice() throws Exception
  {
    assertNull( "Checking unknown invoice ID", getDAO().selectInvoice( -33 ) );

    BfakturVO invoice = getDAO().selectInvoice( INVOICE_ID );
    assertEquals( "Checking invoice ID", INVOICE_ID, invoice.bfaktId );
    assertEquals( "Checking number of copies", 1, invoice.anzKopie );
    assertEquals( "Checking print status", PrintStatusConstants.READY_FOR_PRINTING_STRING, invoice.printStatus );
    assertEquals( "Checking print date", "2006-07-11", invoice.printDate.toString() );
    assertEquals( "Checking employee number", 125, invoice.employeeNumber.intValue() );
    assertEquals( "Checking IP key", 1200016048, invoice.ipKey.intValue() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectInvoices(Integer, String, Integer, List)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectInvoices() throws Exception
  {
    List<EreigIDat> eventDates436 = convertEventDates( EVENT_DATES_436 );
    List<EreigIDat> eventDates437 = convertEventDates( EVENT_DATES_437 );

    try
    {
      getDAO().selectInvoices( null, PrintStatusConstants.PRINTED_STRING, null, eventDates436 );
      fail( "Must fail with null print status" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectInvoices( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION, null, null, eventDates436 );
      fail( "Must fail with null print status" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectInvoices( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION, "", null, eventDates436 );
      fail( "Must fail with empty print status" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectInvoices( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION, PrintStatusConstants.PRINTED_STRING, null, eventDates436 );
      fail( "Must fail with null IP key" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().selectInvoices( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION, PrintStatusConstants.PRINTED_STRING, -1, eventDates436 );
      fail( "Must fail with negative IP key" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    Map<BfakturVO, Set<EreigIDat>> invoiceEvents = getDAO().selectInvoices( InvoiceConstants.FORM_PAPER_PUBLICATION_CONFIRMATION,
        PrintStatusConstants.PRINTED_STRING, TRADEMARK, Collections.<EreigIDat>emptyList() );
    assertEquals( "Checking that no event dates result in no invoices", 0, invoiceEvents.size() );

    invoiceEvents = getDAO().selectInvoices( 436, PrintStatusConstants.PRINTED_STRING, TRADEMARK, eventDates436 );
    assertEquals( "Checking number of invoices", 1, invoiceEvents.size() );
    BfakturVO[] invoices = invoiceEvents.keySet().toArray( new BfakturVO[] {} );
    BfakturVO invoice = invoices[0];
    assertEquals( "Checking invoice ID", 5855528, invoice.bfaktId );
    assertEquals( "Checking number of copies", 1, invoice.anzKopie );
    assertEquals( "Checking print status", PrintStatusConstants.PRINTED_STRING, invoice.printStatus );
    assertEquals( "Checking print date", "2006-01-03", invoice.printDate.toString() );
    assertEquals( "Checking employee number", 949, invoice.employeeNumber.intValue() );
    assertEquals( "Checking IP key", 2100335988, invoice.ipKey.intValue() );

    checkEventDates( eventDates436, invoiceEvents.get( invoice ) );

    invoiceEvents = getDAO().selectInvoices( 437, PrintStatusConstants.PRINTED_STRING, TRADEMARK, eventDates437 );
    assertEquals( "Checking number of invoices", 1, invoiceEvents.size() );
    invoices = invoiceEvents.keySet().toArray( new BfakturVO[] {} );
    invoice = invoices[0];
    assertEquals( "Checking invoice ID", 5855530, invoice.bfaktId );
    assertEquals( "Checking number of copies", 1, invoice.anzKopie );
    assertEquals( "Checking print status", PrintStatusConstants.PRINTED_STRING, invoice.printStatus );
    assertEquals( "Checking print date", "2006-01-03", invoice.printDate.toString() );
    assertEquals( "Checking employee number", 949, invoice.employeeNumber.intValue() );
    assertEquals( "Checking IP key", 2100335988, invoice.ipKey.intValue() );

    checkEventDates( eventDates437, invoiceEvents.get( invoice ) );
  }

  /**
   * Converts integers to event dates.
   * @param eventDates The event dates to convert
   * @return The converted event dates
   */
  private List<EreigIDat> convertEventDates( int[] eventDates )
  {
    List<EreigIDat> eventDateObjects = new ArrayList<EreigIDat>();
    for ( int eventDate: eventDates )
    {
      eventDateObjects.add( EreigIDat.from( eventDate ) );
    }
    return eventDateObjects;
  }

  /**
   * Checks the event dates for an invoice.
   * @param expectedEventDates The event dates as expected
   * @param actualEventDates The event dates as loaded
   */
  private void checkEventDates( List<EreigIDat> expectedEventDates, Set<EreigIDat> actualEventDates )
  {
    int foundEventDates = 0;
    for ( EreigIDat eventDate: actualEventDates )
    {
      if ( expectedEventDates.contains( eventDate ) )
      {
        foundEventDates++;
      }
    }
    assertEquals( "Checking that all event dates were found", expectedEventDates.size(), foundEventDates );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#updateBFakturDruckstatus(int, java.lang.String)}.
   * 
   * @throws Exception if the test fails
   */
  public void testUpdateBFakturDruckstatus() throws Exception
  {
    try
    {
      getDAO().updateBFakturDruckstatus( INVOICE_ID, null );
      fail( "Null print status must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().updateBFakturDruckstatus( INVOICE_ID, "" );
      fail( "Empty print status must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    getDAO().updateBFakturDruckstatus( INVOICE_ID, InvoiceConstants.PRINT_AGAIN );
    String printStatus = getDAO().selectBFakturDruckstatus( INVOICE_ID );
    assertEquals( "Checking new print status", InvoiceConstants.PRINT_AGAIN, printStatus );

    getDAO().updateBFakturDruckstatus( INVOICE_ID, PrintStatusConstants.READY_FOR_PRINTING_STRING );
    printStatus = getDAO().selectBFakturDruckstatus( INVOICE_ID );
    assertEquals( "Checking if print status has been reset", PrintStatusConstants.READY_FOR_PRINTING_STRING, printStatus );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#updateBFakturBfaktDat(int, java.sql.Date)}.
   * 
   * @throws Exception if the test fails
   */
  public void testUpdateBFakturBfaktDat() throws Exception
  {
    try
    {
      getDAO().updateBFakturBfaktDat( INVOICE_ID_WITH_PRINT_DATE, null );
      fail( "Null print date must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    Date oldDate = DateConverter.stringToSqlDate( "24.05.2006" );
    Date newDate = DateConverter.stringToSqlDate( "25.05.2006" );

    getDAO().updateBFakturBfaktDat( INVOICE_ID_WITH_PRINT_DATE, newDate );
    BfakturVO invoice = getDAO().selectInvoice( INVOICE_ID_WITH_PRINT_DATE );
    assertEquals( "Checking print date", "2006-05-25", invoice.printDate.toString() );

    getDAO().updateBFakturBfaktDat( INVOICE_ID_WITH_PRINT_DATE, oldDate );
    invoice = getDAO().selectInvoice( INVOICE_ID_WITH_PRINT_DATE );
    assertEquals( "Checking print date", "2006-05-24", invoice.printDate.toString() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakturDAO#selectBFaktIdAMV(int, int, int)}.
   * 
   * @throws Exception if the test fails
   */
  public void testSelectBFaktIdAMV() throws Exception
  {
    List<AmvVO> trademarkRenewals = getDAO().selectBFaktIdAMV( -3, 2222222, 9999 );
    assertNotNull( "Checking List for unknown data", trademarkRenewals );
    assertEquals( "Checking List for unknown data", 0, trademarkRenewals.size() );

    // This is just put into the VO
    int pseudoAMVrun = 9999;
    int invoiceType = Integer.parseInt( InvoiceConstants.TYPE_INVOICE );
    int printStatus = PrintStatusConstants.READY_FOR_PRINTING;

    trademarkRenewals = getDAO().selectBFaktIdAMV( invoiceType, printStatus, pseudoAMVrun );
    assertNotNull( "Checking for empty result", trademarkRenewals );
    assertTrue( "Checking for result size", trademarkRenewals.size() >= 40 );

    Map<String, AmvVO> trademarkRenewalMap = MapConverter.toMap( trademarkRenewals, String.class, "schutitNr" );
    AmvVO trademarkRenewal = trademarkRenewalMap.get( "00108/2005" );
    assertNotNull( "Checking for trademark renewal", trademarkRenewal );
    assertEquals( "Checking IP key", 1201324985, trademarkRenewal.schutitKey );
    assertEquals( "Checking IP number", "00108/2005", trademarkRenewal.schutitNr );
    assertEquals( "Checking AMV run number", pseudoAMVrun, trademarkRenewal.amvLaufId );
    assertEquals( "Checking invoice number", 5570446, trademarkRenewal.bfaktId );
    assertEquals( "Checking processing status", 0, trademarkRenewal.verarbStat );
    assertEquals( "Checking address number", 500877, trademarkRenewal.adrNr );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected BfakturVO getSerializableVO()
  {
    BfakturVO invoice = new BfakturVO();
    invoice.anzKopie = 3;
    invoice.bfaktId = 51;
    return invoice;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( BfakturVO originalInvoice, BfakturVO serializedInvoice )
  {
    return originalInvoice.anzKopie == serializedInvoice.anzKopie && originalInvoice.bfaktId == serializedInvoice.bfaktId;
  }
}
