package ch.ige.jipps.dao.test;

import java.util.List;
import java.util.Set;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.DruckerDAO;
import ch.ige.jipps.vo.DruckerVO;

/**
 * Tests {@link ch.ige.jipps.dao.DruckerDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Eric Lewis
 * @version  $Id: DruckerDAOTest.java,v 1.9 2008/05/21 15:11:34 lewis_lee Exp $
 */
public class DruckerDAOTest extends BaseDAOTestTemplate<DruckerDAO, DruckerVO>
{
  /**
   * The unit under test
   */
  private DruckerDAO druckerDAO;

  /**
   * The printer to use for the test, may be deleted at {@link #tearDown()}
   */
  private DruckerVO testPrinter;

  /**
   * Shows whether {@link #testPrinter} should be deleted at {@link #tearDown()}
   */
  private boolean deletePrinter = false;

  /**
   * The ID for the printer <code>Thun</code>
   */
  private static final int THUN_PRINTER_ID = 15;

  /**
   * The minimum of printers at the moment
   */
  private static final int MINIMUM_NUMBER_OF_PRINTERS = 4;

  /**
   * The only constructor
   * @param name The test's name
   */
  public DruckerDAOTest( String name )
  {
    super( name );
    this.druckerDAO = new DruckerDAO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<DruckerDAO> getDAOClass()
  {
    return DruckerDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#getAllPrinters()}.
   * @throws Exception if the test fails
   */
  public void testSelectAll() throws Exception
  {
    List<DruckerVO> printers = this.druckerDAO.getAllPrinters();
    assertTrue( "Checking number of printers", printers.size() >= MINIMUM_NUMBER_OF_PRINTERS );
    boolean foundPrinter = false;
    for ( DruckerVO printer: printers )
    {
      if ( printer.druckerId == THUN_PRINTER_ID )
      {
        foundPrinter = true;
        checkPrinter( getKnownPrinter(), printer );
        break;
      }
    }
    assertTrue( "Checking for printer with ID " + THUN_PRINTER_ID, foundPrinter );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#select(int)}.
   * @throws Exception if the test fails
   */
  public void testSelectOne() throws Exception
  {
    DruckerVO printer = this.druckerDAO.select( 257234715 );
    assertNull( "Checking unknown printer", printer );

    printer = this.druckerDAO.select( THUN_PRINTER_ID );
    assertNotNull( "Checking that printer exists", printer );
    checkPrinter( getKnownPrinter(), printer );
  }

  /**
   * Checks the printer.
   * @param expectedPrinter The printer as expected
   * @param actualPrinter The printer to check
   */
  private void checkPrinter( DruckerVO expectedPrinter, DruckerVO actualPrinter )
  {
    assertEquals( "Checking printer ID", expectedPrinter.druckerId, actualPrinter.druckerId );
    assertEquals( "Checking printer name", expectedPrinter.druckerName, actualPrinter.druckerName );
    assertEquals( "Checking control server", expectedPrinter.controlServer, actualPrinter.controlServer );
    assertEquals( "Checking room", expectedPrinter.raum, actualPrinter.raum );
    assertEquals( "Checking color capability", expectedPrinter.farbe, actualPrinter.farbe );
    assertEquals( "Checking paper tray for IPI paper", expectedPrinter.igePaperId, actualPrinter.igePaperId );
    assertEquals( "Checking paper tray for plain white paper", expectedPrinter.whitePaperId, actualPrinter.whitePaperId );
    assertEquals( "Checking paper tray for cover paper", expectedPrinter.coverPaperId, actualPrinter.coverPaperId );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#selectNextId()}.
   * @throws Exception if the test fails
   */
  public void testSelectNextId() throws Exception
  {
    int nextId = this.druckerDAO.selectNextId();
    assertTrue( "Checking for next printer ID", nextId >= 19 );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#selectControlServers()}.
   * @throws Exception if the test fails
   */
  public void testSelectControlServers() throws Exception
  {
    Set<String> controlServers = this.druckerDAO.selectControlServers();
    assertNotNull( "Checking that the data exists", controlServers );
    assertEquals( "Checking size", 1, controlServers.size() );
    assertEquals( "Checking content", "IPIESA1081", controlServers.iterator().next() );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#insert(ch.ige.jipps.vo.DruckerVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    this.testPrinter = getSerializableVO();
    this.deletePrinter = true;
    this.druckerDAO.insert( this.testPrinter );

    DruckerVO savedPrinter = this.druckerDAO.select( this.testPrinter.druckerId );
    checkPrinter( this.testPrinter, savedPrinter );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#update(ch.ige.jipps.vo.DruckerVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    this.testPrinter = getSerializableVO();
    this.deletePrinter = true;
    this.druckerDAO.insert( this.testPrinter );

    DruckerVO savedPrinter = this.druckerDAO.select( this.testPrinter.druckerId );
    checkPrinter( this.testPrinter, savedPrinter );

    this.testPrinter.druckerName = "Opaljevski";
    this.testPrinter.controlServer = "ControlTheController";
    this.testPrinter.raum = "To inifinity and beyond";
    this.testPrinter.farbe = false;
    this.testPrinter.igePaperId = 63;
    this.testPrinter.whitePaperId = 592;
    this.testPrinter.coverPaperId = 90;

    this.druckerDAO.update( this.testPrinter );
    savedPrinter = this.druckerDAO.select( this.testPrinter.druckerId );
    checkPrinter( this.testPrinter, savedPrinter );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.DruckerDAO#delete(int)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    this.testPrinter = getSerializableVO();
    this.deletePrinter = true;
    this.druckerDAO.insert( this.testPrinter );

    this.druckerDAO.delete( this.testPrinter.druckerId );
    assertNull( "Checking that printer was deleted", this.druckerDAO.select( this.testPrinter.druckerId ) );
    this.deletePrinter = false;

    try
    {
      this.druckerDAO.delete( 5712736 );
      fail( "Deleting unknown printer must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }
  }

  /**
   * Gets a well-known printer
   * @return A well-known printer
   */
  private DruckerVO getKnownPrinter()
  {
    DruckerVO knownPrinter = new DruckerVO();
    knownPrinter.druckerId = 15;
    knownPrinter.druckerName = "155_Maennertreu";
    knownPrinter.controlServer = "IPIESA1081";
    knownPrinter.raum = "155";
    knownPrinter.farbe = false;
    knownPrinter.igePaperId = 1;
    knownPrinter.whitePaperId = 11;
    knownPrinter.coverPaperId = 11;
    return knownPrinter;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected DruckerVO getSerializableVO()
  {
    DruckerVO newPrinter = new DruckerVO();
    newPrinter.druckerId = 99;
    newPrinter.druckerName = "Rubinovicz";
    newPrinter.controlServer = "YouTakeMySelfControl";
    newPrinter.raum = "Up, up in space";
    newPrinter.farbe = true;
    newPrinter.igePaperId = 123;
    newPrinter.whitePaperId = 456;
    newPrinter.coverPaperId = 789;
    return newPrinter;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( DruckerVO originalPrinter, DruckerVO serializedPrinter )
  {
    return originalPrinter.druckerId == serializedPrinter.druckerId && originalPrinter.druckerName.equals( serializedPrinter.druckerName )
        && originalPrinter.controlServer.equals( serializedPrinter.controlServer ) && originalPrinter.raum.equals( serializedPrinter.raum )
        && originalPrinter.farbe == serializedPrinter.farbe && originalPrinter.igePaperId == serializedPrinter.igePaperId
        && originalPrinter.whitePaperId == serializedPrinter.whitePaperId && originalPrinter.coverPaperId == serializedPrinter.coverPaperId;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.deletePrinter && this.testPrinter != null )
    {
      this.druckerDAO.delete( this.testPrinter.druckerId );
    }
    super.tearDown();
  }
}
