package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Serializable;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.UnavailableException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import ch.ige.ftp.IGEFTPClientAdapter;
import ch.ige.ftp.jakarta.IGEFTPClientAdapterJakarta;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfigDAO;
import ch.ige.jipps.ftp.web.util.FTPServletConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

/**
 * Servlet accepting the request to extract the data from BAGIS and generate the PDFs ({@link #doPost(HttpServletRequest, HttpServletResponse)} and for sending the data
 * to the EPA server ({@link #doGet(HttpServletRequest, HttpServletResponse)}.
 * <p>
 * <ol>Parameters for the {@link #doPost(HttpServletRequest, HttpServletResponse)} method</ol>
 * <li>{@link FTPServletConfiguration#PATENT_NUMMER_REQUEST_PARAMETER} for the patent number</li>
 * <li>{@link FTPServletConfiguration#PUBL_DATE_REQUEST_PARAMETER} for the publication date</li>
 * <li>{@link FTPServletConfiguration#EMPLOYEE_REQUEST_PARAMETER} for the user name</li>
 * <li>{@link FTPServletConfiguration#DATASOURCE_FOLDER_REQUEST_PARAMETER} for the folder name where to store the data locally in order to retrieve them in a second request</li>
 * </ol>
 *  * <ol>Parameters for the {@link #doGet(HttpServletRequest, HttpServletResponse)} method</ol>
 * <li>{@link FTPServletConfiguration#DESTINATION_REQUEST_PARAMETER} for the destination where to upload the data</li>
 * <li>{@link FTPServletConfiguration#DATASOURCE_FOLDER_REQUEST_PARAMETER} for the folder name from where to read the data that have to be sent to the server</li>
 * </ol>
 * 
 * 
 */
public class FTPServlet extends javax.servlet.http.HttpServlet implements javax.servlet.Servlet
{

  private RequestProcessor requestProcessor;

  @Override
  public void init( ServletConfig c ) throws ServletException
  {
    super.init( c );

    try
    {
      ServletContext context = c.getServletContext();
      initRequestProcessor( context );
      setConfiguration( context );
    }
    catch ( Exception e )
    {
      LOG.error( "Error: " + e.getMessage(), e );
      throw new UnavailableException( "The servlet was not correctly initialized" );
    }



  }

  private void initRequestProcessor( ServletContext context ) throws Exception
  {
    this.requestProcessor = new RequestProcessor( context );

  }

  private void setConfiguration( ServletContext context ) throws SQLException
  {
    FTPServletConfigDAO dao = new FTPServletConfigDAO();

    Map<String, FTPAccessConfigVO> ftpLoginParameters = new HashMap<String, FTPAccessConfigVO>( 2 );
    String intranet_ftp_url = "";

    // intranet data copy configuration 
    FTPAccessConfigVO vo = dao.selectIntranetConfig();
    ftpLoginParameters.put( FTPServletConfiguration.SHARE_DESTINATION, vo );
    intranet_ftp_url = vo.getFTPUrl();
    if ( intranet_ftp_url == null || vo.getFTPHost() == null || vo.getFTPUser() == null || vo.getFTPPasswort() == null || vo.getFTPServerPath() == null )
    {
      throw new IllegalArgumentException( "Intranet FTP parameter not correctly initialized" );
    }

    // epa ftp server configuration
    vo = dao.selectEPAConfig();
    ftpLoginParameters.put( FTPServletConfiguration.EPA_DESTINATION, vo );
    if ( vo.getFTPHost() == null || vo.getFTPUser() == null || vo.getFTPPasswort() == null || vo.getFTPServerPath() == null )
    {
      throw new IllegalArgumentException( "EPA FTP parameter not correctly initialized" );
    }

    context.setAttribute( FTPServletConfiguration.CONTEXT_FTPLOGIN_PARAMTERER_MAP, ftpLoginParameters );
    context.setAttribute( FTPServletConfiguration.CONTEXT_SHARE_DATA_URL, intranet_ftp_url );

    FTPServletConfigVO configVo = dao.selectServletConfig();
    LOG.debug( "Configuration for upload temp dir " + configVo.getPathToTemporaryUploadDir() );
    LOG.debug( "Configuration for download temp dir " + configVo.getPathToTemporaryDownloadDir() );
    if ( configVo.getPathToTemporaryUploadDir() == null || configVo.getPathToTemporaryDownloadDir() == null )
    {
      throw new IllegalArgumentException( "tmp dir configaration not correctly initialized" );
    }
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_UPLOAD, configVo.getPathToTemporaryUploadDir() );
    FTPServletConfiguration.addConfigurationEntry( FTPServletConfiguration.TMP_DIR_DOWNLOAD, configVo.getPathToTemporaryDownloadDir() );

  }



  protected void doGet( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
  {

    LOG.debug( "Starting the GET request.." );

    String nextPage;
    try
    {
      nextPage = requestProcessor.execute( request, response );
    }
    catch ( Exception e )
    {
      LOG.error( e.getMessage(), e );
      throw new ServletException( e );
    }

    ResultBean bean = (ResultBean) request.getAttribute( FTPServletConfiguration.SESSION_PARAMETER_RESULT_BEAN );

    if ( nextPage != null )
    {

      generateOutput( response, bean );

    }
    else
    {
      {
        LOG.error( "no nextPage evaluated" );
        throw new ServletException( "no nextPage evaluated" );
      }
    }

//    RequestDispatcher dispatcher = request.getRequestDispatcher( nextPage );
//    dispatcher.forward( request, response );

  }

  protected void doPost( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException
  {
    LOG.debug( "Starting the POST request.." );
  }

  // TODO use jsp for representing the data from the bean
  private void generateOutput( HttpServletResponse response, ResultBean bean ) throws IOException
  {
    LOG.debug( "Generating the output for bean " + bean );
    response.setContentType( "text/html" );
    PrintWriter writer = response.getWriter();
    String firstLine = bean.getServerUrl();
    List<? extends Serializable> receipts = bean.getReceipts();
    if ( receipts == null || receipts.isEmpty() )
    {
      receipts = bean.getUploadedFilesPaths();
    }
    String emptyResultText = null;
    if ( !bean.getErrorMessages().isEmpty() )
    {
      StringBuilder emptyResult = new StringBuilder();
      for ( String error: bean.getErrorMessages() )
      {
        emptyResult.append( error );
        emptyResult.append( "\n" );
      }
      emptyResultText = emptyResult.toString();
    }

    if ( firstLine != null )
    {
      writer.print( firstLine );
      writer.print( "\n" );
    }
    if ( receipts != null && receipts.size() > 0 )
    {
      for ( Serializable receipt: receipts )
      {
        writer.print( FTPServletConfiguration.modifyReceiptForOutput( receipt ) );
        writer.print( "\n" );
      }
    }
    else if ( emptyResultText != null )
    {
      writer.print( emptyResultText );
    }
  }

  private List<String> list( String date, String[] ftpParameters ) throws Exception
  {

    List<String> fileList = new ArrayList<String>( 64 );

    assert ( ftpParameters[0] != null );
    assert ( ftpParameters[1] != null );
    assert ( ftpParameters[2] != null );
    assert ( ftpParameters[3] != null );
    IGEFTPClientAdapter client = new IGEFTPClientAdapterJakarta( ftpParameters[0], ftpParameters[1], ftpParameters[2] );

    client.login();
    client.setRelativeServerPath( ftpParameters[3], false );
    client.setRelativeServerPath( date, false );
    fileList = client.listData( null ); // PDF
    client.setRelativeServerPath( "..", false );
    client.setRelativeServerPath( "XML-FULL", false );
    client.setRelativeServerPath( date.substring( 0, 4 ), false );
    fileList.addAll( client.listData( null ) ); // XML
    client.logout();
    return fileList;
  }

  private String extractRequestParameter( HttpServletRequest request, HttpServletResponse response, String parameterName, boolean required ) throws IOException
  {
    String parameterValue = request.getParameter( parameterName );
    LOG.debug( parameterName + " set to " + parameterValue );
    if ( required && ( parameterValue == null || parameterValue.length() == 0 ) )
    {
      String msg = "No " + parameterName + " available: Cannot execute ftp request";
      LOG.warn( msg );
      response.sendError( 400, msg );
    }
    return parameterValue;

  }

}
