package ch.ige.jipps.ftp.web.util;

import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URI;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.ipobjects.patent.PatentDocument;
import ch.ige.ipobjects.patent.PatentImage;
import ch.ige.ipobjects.patent.dao.PatDocDAO;
import ch.ige.ipobjects.patent.vo.PatentDocumentVO;
import ch.ige.jipps.PatentSpecificationControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.transformation.CachingTransformerFactory;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * Creates one or more bibliographic data for patent specifications as a PDF.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author fabrizioli
 * @version $Id: BibliographicDataControl.java,v 1.4 2009/02/24 15:16:23 fabrizioli Exp $
 */
public class BibliographicDataControl extends PatentSpecificationControl
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( BibliographicDataControl.class );

  /**
   * The markup in the filename characterizing an image file for the bibliographic data
   */
  private static final String MARKUP_FOR_BIBLIO_IMAGE = "_ia";

  /**
   * Used to transform the XML document
   */
  private static Transformer transformer;

  /**
   * The property to set the transformer factory
   */
  private static final String TRANSFORMER_FACTORY_PROPERTY_NAME = "javax.xml.transform.TransformerFactory";

  /**
   * The class of the transformer factory to use
   */
  private static final String TRANSFORMER_FACTORY_PROPERTY_VALUE = CachingTransformerFactory.class.getName();

  /**
   * The path of the stylesheet to use
   */
  private String xslPath;

  /**
   * The default constructor
   * @throws TransformerConfigurationException 
   */
  public BibliographicDataControl() throws TransformerConfigurationException
  {
    super();
    this.xslPath = PathConfiguration.getVerifiedPathAsUrl( PropertyConstants.JIPPS_PATH_XSL, false );
    LOG.debug( "Create the BiblioControl for " + this.xslPath );
    if ( this.transformer == null )
    {
      this.transformer = this.createTransformer( new StreamSource( this.xslPath + FTPServletConfiguration.XSLT_BIBLIO_NAME + FileConstants.EXTENSION_XSL ) );
    }
  }

  /**
   * Gets the report to transform.
   * @param reportId The report ID, a patent document ID 
   * @return The report to transform
   * @throws SQLException if the report can't be found
   */
  protected AbstractVO getReport( Integer reportId ) throws SQLException
  {
    if ( reportId == null )
    {
      throw new IllegalArgumentException( "patDocId must not be null" );
    }
    PatentDocumentVO patent = null;
    try
    {
      patent = generateBibliographicData( new PatDocDAO().select( reportId ) );
    }
    catch ( TransformerException e )
    {
      LOG.error(
          "Error in generating the ST36 xml for the bibliographic data for " + ( patent == null ? null : patent.getPatdoc_id() ) + ": " + e.getMessage(), e );
    }
    catch ( InvalidPatentDocumentException e )
    {
      LOG.error(
          "Error in validating the ST36 xml for the bibliographic data for " + ( patent == null ? null : patent.getPatdoc_id() ) + ": " + e.getMessage(), e );
    }
    if ( patent == null )
    {
      throw new IllegalArgumentException( "This patdoc id does not exist, no database entry could be found" );
    }

    return patent;
  }

  /**
  * @param patentdocument, from where to generate the bibliographic data from
  * @return bibliographic data
   * @throws TransformerException 
   * @throws TransformerException 
  */
  private PatentDocumentVO generateBibliographicData( PatentDocumentVO patdoc ) throws TransformerException, InvalidPatentDocumentException
  {
    if ( patdoc == null || patdoc.getXml() == null )
    {
      throw new IllegalArgumentException( "Cannot generate the bibliodata because patdoc is null or empty" );
    }
    StringWriter transformationWriter = new StringWriter();
    this.transformer.transform( new StreamSource( new StringReader( patdoc.getXml() ) ), new StreamResult( transformationWriter ) );
    PatentDocumentVO biblioData = new PatentDocumentVO( patdoc.getPatdoc_id(), patdoc.getEreig_idat(), patdoc.getPatdocschema_id(), transformationWriter
        .getBuffer().toString(), patdoc.getCountry_cd(), patdoc.getKind_cd(), false );
    //this.validate( biblioData );
    List<PatentImage> imagesForBiblio = new ArrayList<PatentImage>( 2 );
    for ( PatentImage image: patdoc.getImages() )
    {
      if ( image.getFilename().contains( MARKUP_FOR_BIBLIO_IMAGE ) )
      {
        imagesForBiblio.add( image );
      }
    }
    biblioData.setImages( imagesForBiblio );
    return biblioData;
  }

  /**
   * Creates the Transformer for XSLT. Leave private!
   * @param xslName The XSL file (without <code>.xsl</code> to use
   * @return The Transformer for XSLT
   * @throws TransformerConfigurationException if the Transformer can't be created
   */
  private Transformer createTransformer( StreamSource source ) throws TransformerConfigurationException
  {
    // Enforces the use of JIPPS's own TransformerFactory
    System.setProperty( TRANSFORMER_FACTORY_PROPERTY_NAME, TRANSFORMER_FACTORY_PROPERTY_VALUE );
    TransformerFactory transformerFactory = TransformerFactory.newInstance();
    return transformerFactory.newTransformer( source );
  }

  @Override
  public String[] makeXml( List<PrintJobVO> printJobs ) throws Exception
  {
    String[] result = new String[printJobs.size()];
    for ( int i = 0; i < printJobs.size(); i++ )
    {
      result[i] = this.getXmlDocument( this.getReport( printJobs.get( i ).reportId ) );
    }
    return result;
  }

  private void validate( PatentDocument document ) throws InvalidPatentDocumentException
  {
    //public static boolean validate( Schema schema, XMLDocument document, ErrorHandler handler ) throws VerifierConfigurationException, SAXException, IOException
    try
    {

      SchemaFactory schemaFactory = SchemaFactory.newInstance( "http://www.w3.org/2001/XMLSchema" );
      String schemaPath = "http://schema.ige.ch/ch-patent-doc-v1-2-ige1-1.xsd";
      //URL schemaSource = new URL( schemaPath );
      Source schemaSource = new StreamSource( this.xslPath + "ch-patent-doc-v1-2-ige1-1-bibl.xsd" );
      LOG.info( "Validating with " + schemaSource.getSystemId() );
      File file = new File( new URI( schemaSource.getSystemId() ) );
      LOG.info( "Validating with " + file.getAbsolutePath() );
//      Schema schema = schemaFactory.newSchema( file );
//      Validator validator = schema.newValidator();
//      validator.validate( new StreamSource( new StringReader( document.getXml() ) ) );

//      VerifierFactory factory = VerifierFactory.newInstance( "http://www.w3.org/2001/XMLSchema" );
//      Source schemaSource = new StreamSource( this.xslPath + "ch-patent-doc-v1-2-ige1-1-bibl.xsd" );
//      LOG.info( "Validating with " + schemaSource.getSystemId() );
//      File file = new File( "C:/jp/projects/jipps/xsl/ch-patent-doc-v1-2-ige1-1-bibl.xsd" );//new URI( schemaSource.getSystemId() ) );
//      Schema schema = factory.compileSchema( file );
//      Verifier verifier = schema.newVerifier();
      InputSource result = new InputSource();
      result.setCharacterStream( new StringReader( document.getXml() ) );
//      verifier.verify( result );

      // Load up the document
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      //factory.setValidating( true );
      factory.setSchema( schemaFactory.newSchema( file ) );
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document doc = builder.parse( result );

    }
    catch ( Exception e )
    {
      throw new InvalidPatentDocumentException( "Document not valid " + document.getIdentifier(), e );
    }
  }
}
