package ch.ige.jipps.history.description;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Locale;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.history.ValueHistory;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.design.DesignPublicationDeferment;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.date.LanguageDateFormatter;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.PublicationTarget;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.server.replace.P2d2;

/**
 * Creates the text for the publication of an event.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: PublicationText.java,v 1.34 2008/05/21 13:49:21 lewis_lee Exp $
 */
public class PublicationText
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PublicationText.class );

  /**
   * The variable used for the publication number
   */
  private final static String PUBLICATION_NUMBER_VARIABLE = P2d2.TAG_PREFIX + "publ_nr";

  /**
   * The variable used for the publication date
   */
  private final static String PUBLICATION_DATE_VARIABLE = P2d2.TAG_PREFIX + "publ_dat";

  /**
   * The variable used for the publication year
   */
  private final static String PUBLICATION_YEAR_VARIABLE = P2d2.TAG_PREFIX + "publ_jahr";

  /**
   * The year when the publication name changed
   */
  private final static int YEAR_OF_PUBLICATION_CHANGE = 2002;

  /**
   * The issue when the publication name changed
   */
  private final static int ISSUE_OF_PUBLICATION_CHANGE = 12;

  /**
   * Gets the publication text for an event.
   *
   * <h2>Online publication</h2>
   * This applies when an event is published on Swissreg.
   * 
   * <h3>All IP types except design</h3>
   * <h4>All reports</h4>
   * If a publication date exists, it's always shown, <strong>regardless of whether the publication is definitive or not</strong><br>
   * <code>Published on Swissreg on &lt;dd.MM.yyyy&gt;</code><br><br>
   * If no publication date exists:<br>
   * <code>Not yet published</code>
   * 
   * <h3>Design</h3>
   * Same as above, but a Design that's being deferred gets no publication text at all.
   *
   * <h2>Publication in a magazine</h2>
   * 
   * <h3>Patent and SPC</h3>
   * <h4>Register report</h4>
   * If a publication date exists, it's always shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   * <h4>Dossier report</h4>
   * If a publication date exists, it's always shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   *
   * <h3>Trademark</h3>
   * <h4>Register report, published in SHAB</h4>
   * If the publication <strong>is set as being definitive</strong>, it's shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If the publication <strong>is not set as being definitive</strong>, it's not shown:<br>
   * <code>Not yet published in &lt;publication&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   * <h4>Register report, published with Swissreg (not yet activated)</h4>
   * If a publication date exists and it lies in the <strong>past</strong>, it's shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If a publication date exists and it lies in the <strong>future</strong>, it's not shown:<br>
   * <code>Not yet published in &lt;publication&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   * <h4>Dossier report</h4>
   * If a publication date exists, it's always shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   *
   * <h3>Design</h3>
   * <h4>Register report</h4>
   * If a publication date exists and it lies in the <strong>past</strong>, it's shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If a publication date exists and it lies in the <strong>future</strong>, it's not shown.<br>
   * <code>Not yet published</code><br><br>
   * <h4>Dossier report</h4>
   * If a publication date exists, it's always shown:<br>
   * <code>Published in &lt;publication&gt;, issue &lt;number&gt; of &lt;year&gt;</code><br><br>
   * If no publication date exists, nothing is shown.
   *
   * @param ip The IP containing all general data
   * @param currentState The history is built for this date
   * @param event The event which may be published
   * @param language The report language
   * @param reportType The report type
   * @param throwExceptionForWrongState If this is <code>true</code>, a wrong state for an event results in an
   *        <code>IllegalStateException</code>, if it's <code>false</code>, it results in a log message
   * @return The publication text for the event or <code>null</code> if it doesn't have to be published
   * @throws SQLException if the report text can't be fetched
   * @throws Exception if the P2D2 replacement fails
   */
  public static String getPublicationText( GenericIP ip, PointInTime currentState, Event event, Locale language, ReportType reportType,
      boolean throwExceptionForWrongState ) throws SQLException, Exception
  {
    LOG.debug( "Event is " + event );
    // If there's no publication anyway, don't bother...
    if ( !event.isPubl_pflicht() && event.getPubl_dat() == null )
    {
      LOG.debug( "Event must not be published" );
      return null;
    }

    if ( event.getPublicationTarget().equals( PublicationTarget.SWISSREG2 ) )
    {
      return getOnlinePublicationText( ip, currentState, event, language, throwExceptionForWrongState );
    }
    else if ( event.getPublicationTarget().equals( PublicationTarget.PAPER ) )
    {
      return getPublicationTextForMagazine( ip, currentState, event, language, reportType, throwExceptionForWrongState );
    }
    else
    {
      LOG.debug( "Event is published neither in Swissreg nor in a magazine" );
      return null;
    }
  }

  /**
   * Gets the publication text for an event that's published online.
   * @param ip The IP containing all general data
   * @param currentState The history is built for this date
   * @param event The event which may be published
   * @param language The report language
   * @param throwExceptionForWrongState If this is <code>true</code>, a wrong state for an event results in an
   *        <code>IllegalStateException</code>, if it's <code>false</code>, it results in a log message
   * @return The publication text for the event or <code>null</code> if it doesn't have to be published
   * @throws SQLException if the report text can't be fetched
   * @throws Exception if the P2D2 replacement fails
   */
  private static String getOnlinePublicationText( GenericIP ip, PointInTime currentState, Event event, Locale language, boolean throwExceptionForWrongState )
      throws SQLException, Exception
  {
    if ( !event.isPubl_pflicht() )
    {
      if ( event.getPubl_dat() == null )
      {
        return null;
      }
      else
      {
        showWrongState( throwExceptionForWrongState, "Found event for IP " + ip.getIPKey() + " which must not be published, but which has a publication date: "
            + event );
      }
    }

    // Deferred designs never get any publication text!
    if ( Types.DESIGN.isInstance( ip ) )
    {
      Design design = (Design) ip;
      ValueHistory<HistoryItem<DesignPublicationDeferment>> defermentState = design.getDefermentState();
      if ( defermentState != null )
      {
        HistoryItem<DesignPublicationDeferment> historyItem = defermentState.getItemContaining( currentState );
        DesignPublicationDeferment deferment = historyItem.getValue();
        if ( deferment.equals( DesignPublicationDeferment.ACTIVE ) )
        {
          return null;
        }
      }
    }

    RegFormTextVO formText = null;
    RegFormTextDAO regFormTextDAO = new RegFormTextDAO();
    if ( event.isPublishedOnline() )
    {
      if ( event.getPubl_dat() == null )
      {
        showWrongState( throwExceptionForWrongState, "Event has been published online, but has no publication date: " + event );
      }
      formText = regFormTextDAO.findReportText( ReportTextConstants.PUBLISHED_ONLINE_INTEGER, language, ReportArea.PUBLICATION );
      String formattedDate = new LanguageDateFormatter( language ).longFormat( event.getPubl_dat() );
      HashMap<String, String> values = new HashMap<String, String>();
      values.put( PUBLICATION_DATE_VARIABLE, formattedDate );
      return P2d2.replaceP2d2( formText.getText(), values );
    }
    else
    {
      // The correct text 'not yet published' only exists for designs
      formText = regFormTextDAO.findReportText( ReportTextConstants.NOT_YET_PUBLISHED_INTEGER, language, ReportArea.DATA_STATE, IPType.DESIGN );
      return formText.getText();
    }
  }

  /**
   * Gets the publication text for an event that's published in the magazine.
   * @param ip The IP containing all general data
   * @param currentState The history is built for this date
   * @param event The event which may be published
   * @param language The report language
   * @param reportType The report type
   * @param throwExceptionForWrongState If this is <code>true</code>, a wrong state for an event results in an
   *        <code>IllegalStateException</code>, if it's <code>false</code>, it results in a log message
   * @return The publication text for the event or <code>null</code> if it doesn't have to be published
   * @throws SQLException if the report text can't be fetched
   * @throws Exception if the P2D2 replacement fails
   */
  @SuppressWarnings( "null" )
  private static String getPublicationTextForMagazine( GenericIP ip, PointInTime currentState, Event event, Locale language, ReportType reportType,
      boolean throwExceptionForWrongState ) throws SQLException, Exception
  {
    EreigIDat publicationDate = null;
    if ( event.getPubl_dat() != null )
    {
      publicationDate = BAGISTime.convertDateToMinIDat( event.getPubl_dat() );
    }
    boolean publicationDateExists = ( publicationDate != null );

    // Error checking - depending on <code>throwExceptionForWrongState</code>, this method continues
    if ( !event.isPubl_pflicht() && publicationDateExists )
    {
      showWrongState( throwExceptionForWrongState, "Found event for IP " + ip.getIPKey() + " which must not be published, but which has a publication date: "
          + event );
    }
    if ( event.isPubl_definitiv() && !publicationDateExists )
    {
      showWrongState( throwExceptionForWrongState, "Found event for IP " + ip.getIPKey()
          + " which has definitely been published, but which has no publication date: " + event );
    }
    if ( event.isPubl_definitiv() && !event.isPubl_pflicht() )
    {
      showWrongState( throwExceptionForWrongState, "Publication for IP " + ip.getIPKey() + " is definitive, but not required: " + event );
    }

    // Register view
    if ( reportType.usesRegisterView() )
    {
      /*
       * For design register view: any publication after the current state is ignored
       * For trademark register view - as long as publication in SHAB: any publication which is not definitive or has no publication date yet is ignored
       * For trademark register view - after publication in Swissreg: same rule as design
       */
      if ( ( Types.DESIGN.isInstance( ip ) && publicationDateExists && publicationDate.after( currentState ) )
          || ( Types.TRADEMARK.isInstance( ip ) && ( ( publicationDateExists && !event.isPubl_definitiv() ) || ( event.isPubl_pflicht() && !publicationDateExists ) ) ) )
      {
        String publicationText = createNotYetPublishedText( ip, language );
        LOG.debug( "This is a register view for a trademark or design, but the publication date " + event.getPubl_dat() + " is after the current state "
            + currentState + ": " + publicationText );
        return publicationText;
      }
      else
      {
        String publicationText = createPublicationText( ip, event, language );
        LOG.debug( "This is a register view, the publication date is probably before the current state " + currentState + ": " + publicationText );
        return publicationText;
      }
    }
    // For a dossier view, any valid publication date is shown
    else
    {
      if ( publicationDateExists )
      {
        String publicationText = createPublicationText( ip, event, language );
        LOG.debug( "This is a dossier view, the publication date exists: " + publicationText );
        return publicationText;
      }
      else
      {
        if ( event.isPubl_definitiv() )
        {
          LOG.debug( "This is a dossier view, there's no event date, but the event is definitively published" );
          return null;
        }
        else
        {
          String publicationText = createNotYetPublishedText( ip, language );
          LOG.debug( "This is a dossier view, there's no publication date: " + publicationText );
          return publicationText;
        }
      }
    }
  }

  /**
   * Shows an error message, either by logging it or by throwing an <code>IllegalStateException</code>.
   * This is used to show that the data coming from the database is wrong.
   * @param throwExceptionForWrongState If this is <code>true</code>, a wrong state throws an Exception;
   *        if it's <code>false</code>, a wrong state logs a warning message
   * @param errorMessage The error message to show
   * @throws IllegalStateException if so required
   */
  protected static void showWrongState( boolean throwExceptionForWrongState, String errorMessage ) throws IllegalStateException
  {
    if ( throwExceptionForWrongState )
    {
      throw new IllegalStateException( errorMessage );
    }
    else
    {
      LOG.warn( errorMessage );
    }
  }

  /**
   * Creates the publication text for an event that's not yet published.
   * @param ip The IP containing all general data
   * @param language The report language
   * @return The publication text for an event that's not yet published
   * @throws SQLException if the text can't be fetched
   */
  private static String createNotYetPublishedText( GenericIP ip, Locale language ) throws SQLException
  {
    Integer textNumber = null;
    if ( Types.TRADEMARK.isInstance( ip ) || Types.DESIGN.isInstance( ip ) )
    {
      // Not yet published in ...
      textNumber = ReportTextConstants.NOT_YET_PUBLISHED_INTEGER;
    }
    else
    {
      // Tentative data
      textNumber = ReportTextConstants.TENTATIVE_DATA_INTEGER;
    }
    RegFormTextVO formText = new RegFormTextDAO().findReportText( textNumber, language, ReportArea.DATA_STATE, ip.getBAGISType() );
    // An IP type specific text wasn't found, so try a generic text
    if ( formText == null )
    {
      formText = new RegFormTextDAO().findReportText( textNumber, language, ReportArea.DATA_STATE, IPType.ALL );
    }
    return formText.getText();
  }

  /**
   * Creates the publication text for an event that's published.
   * @param ip The IP containing all general data
   * @param event The event which may be published
   * @param language The report language
   * @return The formatted publication text, which may be <code>null</code> if there's no publication date
   * @throws IllegalArgumentException if the IP type is invalid
   * @throws SQLException if the text can't be fetched
   * @throws Exception if the text substitution fails
   */
  private static String createPublicationText( GenericIP ip, Event event, Locale language ) throws IllegalArgumentException, SQLException, Exception
  {
    if ( event.getPubl_dat() == null )
    {
      return null;
    }

    String publicationText = null;
    Integer publicationIssue = getPublicationIssue( ip, event );
    Integer publicationYear = getPublicationYear( ip, event );

    if ( publicationIssue == null || publicationYear == null )
    {
      LOG.warn( "Found no publication issue or year for IP " + ip.getIPKey() + " and event " + event );
      return null;
    }

    RegFormTextVO formText = null;
    RegFormTextDAO regFormTextDAO = new RegFormTextDAO();

    boolean showFullDate = false;

    if ( Types.TRADEMARK.isInstance( ip ) )
    {
      formText = regFormTextDAO.findReportText( ReportTextConstants.TEXT_2_INTEGER, language, ReportArea.PUBLICATION );
      showFullDate = true;
    }
    else if ( Types.DESIGN.isInstance( ip ) )
    {
      // The publication name may have changed
      if ( hasPublicationChanged( publicationYear, publicationIssue ) )
      {
        formText = regFormTextDAO.findReportText( ReportTextConstants.TEXT_5_INTEGER, language, ReportArea.PUBLICATION );
      }
      else
      {
        formText = regFormTextDAO.findReportText( ReportTextConstants.TEXT_1_INTEGER, language, ReportArea.PUBLICATION );
      }
    }
    else if ( Types.PATENT_GENERIC.isInstance( ip ) || Types.SPC.isInstance( ip ) )
    {
      // The publication name may have changed
      if ( !Types.PATENT_PCT.isInstance( ip ) && hasPublicationChanged( publicationYear, publicationIssue ) )
      {
        formText = regFormTextDAO.findReportText( ReportTextConstants.TEXT_4_INTEGER, language, ReportArea.PUBLICATION );
      }
      else
      {
        formText = regFormTextDAO.findReportText( ReportTextConstants.TEXT_1_INTEGER, language, ReportArea.PUBLICATION );
      }
    }
    else
    {
      throw new IllegalArgumentException( "Invalid IP type " + ip.getType() );
    }
    publicationText = formText.getText();

    HashMap<String, String> values = new HashMap<String, String>();
    values.put( PUBLICATION_NUMBER_VARIABLE, publicationIssue.toString() );
    if ( showFullDate )
    {
      String formattedDate = new LanguageDateFormatter( language ).longFormat( event.getPubl_dat() );
      values.put( PUBLICATION_DATE_VARIABLE, formattedDate );
    }
    else
    {
      values.put( PUBLICATION_YEAR_VARIABLE, publicationYear.toString() );
    }
    return P2d2.replaceP2d2( publicationText, values );
  }

  /**
   * Shows whether the publication has changed, because in 2002, the name of
   * the publication changed.
   * @param publicationYear The year of publication
   * @param publicationIssue The issue of publication, which may be <code>null</code>
   *        (in which case, this always returns <code>false</code>)
   * @return <code>true</code> if the publication name has changed, otherwise <code>false</code>
   */
  private static boolean hasPublicationChanged( Integer publicationYear, Integer publicationIssue )
  {
    if ( publicationIssue == null )
    {
      return false;
    }
    else
    {
      return publicationYear.intValue() > YEAR_OF_PUBLICATION_CHANGE
          || ( publicationYear.intValue() == YEAR_OF_PUBLICATION_CHANGE && publicationIssue.intValue() > ISSUE_OF_PUBLICATION_CHANGE );
    }
  }

  /**
   * Gets the publication issue for the event.
   * @param ip The IP containing all general data
   * @param event The event which may be published
   * @return The publication issue, which may be <code>null</code>
   */
  private static Integer getPublicationIssue( GenericIP ip, Event event )
  {
    if ( Types.PATENT_GENERIC.isInstance( ip ) || Types.SPC.isInstance( ip ) )
    {
      return event.getPmmbl_seite();
    }
    else if ( Types.TRADEMARK.isInstance( ip ) )
    {
      return event.getPubl_nr();
    }
    else if ( Types.DESIGN.isInstance( ip ) )
    {
      return event.getPmmbl_seite();
    }
    else
    {
      throw new IllegalArgumentException( "Unknown IP type " + ip.getType() );
    }
  }

  /**
   * Gets the publication year for the event.
   * @param ip The IP containing all general data
   * @param event The event which may be published
   * @return The publication year, which may be <code>null</code>
   */
  private static Integer getPublicationYear( GenericIP ip, Event event )
  {
    if ( Types.PATENT_GENERIC.isInstance( ip ) || Types.SPC.isInstance( ip ) || Types.DESIGN.isInstance( ip ) )
    {
      return event.getPmmbl_jahr();
    }
    else
    {
      if ( event.getPubl_dat() == null )
      {
        return null;
      }
      else
      {
        Calendar publication = Calendar.getInstance();
        publication.setTime( event.getPubl_dat() );
        return Integer.valueOf( publication.get( Calendar.YEAR ) );
      }
    }
  }
}
