package ch.ige.jipps.history.detail;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.address.Address;
import ch.ige.ipobjects.address.comparator.AddressRoleComparator;
import ch.ige.ipobjects.address.filter.AddressRoleCollator;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.country.dao.CountryDAO;
import ch.ige.ipobjects.country.vo.CountryVO;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.dao.RegInhaltDAO;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.util.enums.address.AddressRole;
import ch.ige.util.format.AddressFormatter;

/**
 * Creates the detail lines for an address, where only invalid addresses of one role are shown.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: InvalidAddressDetail.java,v 1.16 2007/12/19 08:57:02 lewis_lee Exp $
 */
public class InvalidAddressDetail extends BaseHistoryItemDetail<Address>
{
  /**
   * The IP whose language is used for formatting
   */
  protected GenericIP ip;

  /**
   * Collates the address roles
   */
  protected AddressRoleCollator addressRoleCollator;

  /**
   * Creates this with its detail data.
   * @param ip The IP containing the addresses
   * @param addresses The addresses to format
   * @param ipType The IP type
   * @param language The report language
   */
  public InvalidAddressDetail( GenericIP ip, TimePeriodCollection<Address> addresses, IPType ipType, Locale language )
  {
    super( addresses, ipType, language );
    this.ip = ip;
    this.addressRoleCollator = new AddressRoleCollator( addresses );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<Address, Detail> createBuilder()
  {
    return new AddressDetailBuilder();
  }

  /**
   * The class that actually formats the address.
   */
  public class AddressDetailBuilder extends AbstractHistoryItemDetailBuilder<Address, Detail>
  {
    /** TODO This should be a Service! */
    private CountryDAO countryDAO = new CountryDAO();

    /** TODO This should be a Service! */
    private RegInhaltDAO regInhaltDAO = new RegInhaltDAO();

    /**
     * Creates the builder.
     */
    public AddressDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<Address> detailData )
    {
      Collections.sort( detailData, new AddressRoleComparator<Address>() );
      final int size = detailData.size();
      List<List<Address>> items = new ArrayList<List<Address>>( size );
      List<Detail> formattedAddresses = new ArrayList<Detail>( size );
      AddressRole previousAddressRole = null;
      Integer previousAddressRoleTextNumber = new Integer( -1 );
      for ( Address address: detailData )
      {
        items.add( Collections.singletonList( address ) );
        Detail addressDetail = createFormattedAddress( address, InvalidAddressDetail.this.ipType, previousAddressRole, previousAddressRoleTextNumber );
        formattedAddresses.add( addressDetail );
        previousAddressRole = address.getAddressRole();
        previousAddressRoleTextNumber = addressDetail.reportTextNumber;
      }
      return createListManyDataItemsManyRepresentations( items, formattedAddresses );
    }

    /**
     * Creates the formatted address.
     * @param address The address to format
     * @param addressIpType The IP type
     * @param previousAddressRole The address role of the previous address
     * @param previousAddressRoleTextNumber The text number of the previous address
     * @return A <code>List</code> of address lines, each a <code>String</code>
     */
    private Detail createFormattedAddress( Address address, IPType addressIpType, AddressRole previousAddressRole, Integer previousAddressRoleTextNumber )
    {
      try
      {
        Detail detailWithReportText = new Detail( address, InvalidAddressDetail.this );

        // For designs, the country code is not mandatory (in the database, this is the 'country' XX)
        if ( !addressIpType.equals( Types.DESIGN ) && address.getBland_cd() == null )
        {
          throw new IllegalArgumentException( "Country code must not be null for IP type " + addressIpType );
        }

        String countryName = null;
        if ( address.getBland_cd() != null )
        {
          CountryVO country = this.countryDAO.findCountry( address.getBland_cd() );
          Locale ipLanguage = InvalidAddressDetail.this.ip.getLocale();
          countryName = country.getLand_bez( ipLanguage );
        }

        detailWithReportText.textLines = AddressFormatter.format( address.getReg_name(), address.getReg_strasse(), address.getReg_ort(), address.getBland_cd(),
            countryName, address.getLizenz_txt(), false );

        // If several addresses have the same address role, only the first address role is printed.
        // Also, don't print several texts which are the same, even if the address roles differ.
        if ( !address.getAddressRole().equals( previousAddressRole ) )
        {
          Integer reportTextNumber = this.regInhaltDAO.findReportTextNumber( addressIpType.getBAGISType(), address.getAddressRole() );
          if ( !reportTextNumber.equals( previousAddressRoleTextNumber ) )
          {
            detailWithReportText.reportTextNumber = reportTextNumber;
          }
        }

        return detailWithReportText;
      }
      catch ( SQLException ex )
      {
        throw new RuntimeException( "Couldn't fetch country name for <" + address.getBland_cd() + ">", ex );
      }
    }
  }
}
