package ch.ige.jipps.history.detail.trademark;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.trademark.model.GoodsAndServicesHistory.GoodsAndServicesItem;
import ch.ige.ipobjects.trademark.vo.MarWarklVO;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.constants.ReportTextNumberHandler;
import ch.ige.util.format.WdlClassFormatter;
import ch.ige.wdlutil.exception.WdlSyntaxException;
import ch.ige.wdlutil.format.WdlFormatter;

/**
 * Creates the detail lines for a trademark's goods and services (WDL). This includes the WDL text and the class
 * numbers.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: GoodsAndServicesDetail.java,v 1.14 2008/01/18 08:22:48 lewis_lee Exp $
 */
public class GoodsAndServicesDetail extends BaseHistoryItemDetail<GoodsAndServicesItem>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( GoodsAndServicesDetail.class );

  /**
   * Creates this with its detail data.
   * @param goodsAndServices The goods and services to format
   * @param ipType The IP type
   * @param language The report language
   */
  public GoodsAndServicesDetail( TimePeriodCollection<GoodsAndServicesItem> goodsAndServices, IPType ipType, Locale language )
  {
    super( goodsAndServices, ipType, language );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<GoodsAndServicesItem, Detail> createBuilder()
  {
    return new GoodsAndServicesDetailBuilder();
  }

  /**
   * The class that actually formats the goods and services (WDL).
   */
  public class GoodsAndServicesDetailBuilder extends AbstractHistoryItemDetailBuilder<GoodsAndServicesItem, Detail>
  {
    /**
     * Creates the builder.
     */
    public GoodsAndServicesDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<GoodsAndServicesItem> detailData )
    {
      final GoodsAndServicesItem goodsAndServicesItem = detailData.get( 0 );
      List<Detail> goodsAndServicesDetails = new ArrayList<Detail>();

      goodsAndServicesDetails.add( createFormattedGoodsAndServicesText( goodsAndServicesItem ) );
      goodsAndServicesDetails.add( createFormattedGoodsAndServicesNumbers( goodsAndServicesItem ) );

      if ( goodsAndServicesDetails.isEmpty() )
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
      else
      {
        return createListSingleDataItemManyRepresentations( goodsAndServicesItem, goodsAndServicesDetails );
      }
    }

    /**
     * Creates the formatted goods and services classes.
     * @param goodsAndServicesItem The goods and services item containing class numbers
     * @return The formatted goods and services classes
     */
    private Detail createFormattedGoodsAndServicesNumbers( GoodsAndServicesItem goodsAndServicesItem )
    {
      List<Integer> classNumbers = getClassNumbers( goodsAndServicesItem );
      if ( !classNumbers.isEmpty() )
      {
        Detail detail = new Detail( goodsAndServicesItem, GoodsAndServicesDetail.this );
        detail.reportTextNumber = ReportTextNumberHandler.getReportTextNumberForClassification( GoodsAndServicesDetail.this.getIpType().getBAGISType() );
        detail.textLines = new ArrayList<String>();
        detail.textLines.add( WdlClassFormatter.format( classNumbers, false ) );
        return detail;
      }
      else
      {
        return null;
      }
    }

    /**
     * Gets the goods and services class numbers.
     * @param goodsAndServicesItem The goods and services item containing class numbers
     * @return A <code>List</code> of the goods and services class numbers
     */
    private List<Integer> getClassNumbers( GoodsAndServicesItem goodsAndServicesItem )
    {
      List<Integer> wdlClassNumbers = new ArrayList<Integer>();
      if ( !goodsAndServicesItem.getClasses().isEmpty() )
      {
        // The WDL/Nice class numbers
        for ( MarWarklVO niceClass: goodsAndServicesItem.getClasses() )
        {
          wdlClassNumbers.add( niceClass.getWarkl_nr() );
        }
      }
      return wdlClassNumbers;
    }

    /**
     * Creates the formatted goods and services text.
     * @param goodsAndServicesItem The goods and services item to format
     * @return The formatted goods and services text
     */
    private Detail createFormattedGoodsAndServicesText( GoodsAndServicesItem goodsAndServicesItem )
    {
      String originalWdlText = goodsAndServicesItem.getText();
      if ( StringUtils.isNotEmpty( originalWdlText ) )
      {
        try
        {
          Detail detail = new Detail( null, GoodsAndServicesDetail.this );
          // Since the class numbers are always written, the potential 'valid to...' date is always
          // created there.
          detail.reportTextNumber = ReportTextConstants.WDL_INTEGER;
          detail.textLines = new ArrayList<String>();
          // Note that < and > are escaped to &lt; and &gt; when the XML document is created
          detail.textLines.addAll( WdlFormatter.toHtmlListWithH1( originalWdlText, false, false ) );
          return detail;
        }
        catch ( WdlSyntaxException e )
        {
          LOG.error( "Can't format WDL '" + originalWdlText + "'", e );
          return null;
        }
      }
      else
      {
        return null;
      }
    }
  }
}
