package ch.ige.jipps.imageconv;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.text.MessageFormat;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.dao.PropertyJippsDAO;

/**
 * Converts an image using ImageMagick.
 * This is used from the image converter servlet as well as locally for testing.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ImageMagickConverter.java,v 1.3 2009/05/05 15:48:28 lewis_lee Exp $
 */
public class ImageMagickConverter
{
  /**
   * A dot, used in file extensions.
   */
  private static final String DOT = ".";

  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( ImageMagickConverter.class );

  /**
   * The local Windows installation of ImageMagick.
   */
  public static final String LOCAL_WINDOWS_INSTALLATION = "C:\\Programme\\ImageMagick-6.2.9-Q16\\convert.exe";

  /**
   * Contains the parameter to use to convert the image.
   */
  private static final String IMAGEMAGICK_COMMANDLINE = " {0} {1}";

  /**
   * Calls ImageMagick to convert an image.
   * If the call to the external ImageMagick converter program fails,
   * this throws an Exception.
   * 
   * @param sourceImagePath The full path to the image to convert
   * @param extension The file extension to use, which defines the target image format, normally {@link FileConstants#EXTENSION_JPG}
   * @throws SQLException if the ImageMagick path can't be read
   * @throws IOException if the conversion fails; note that the error output is in {@link java.io.IOException#getMessage()}
   * @throws InterruptedException if the external program is interrupted
   */
  public static void convert( String sourceImagePath, String extension ) throws SQLException, IOException, InterruptedException
  {
    String validExtension = extension;
    if ( !extension.startsWith( DOT ) )
    {
      validExtension = DOT + validExtension;
    }
    String targetPath = StringUtils.substringBeforeLast( sourceImagePath, DOT ) + validExtension;
    String imageMagickPath = new PropertyJippsDAO().select( PropertyConstants.IMAGEMAGICK_PATH );
    String command = MessageFormat.format( imageMagickPath + IMAGEMAGICK_COMMANDLINE, new Object[] { sourceImagePath, targetPath } );
    LOG.debug( "Executing " + command );

    Process process = Runtime.getRuntime().exec( command );
    int returnCode = process.waitFor();
    if ( returnCode != 0 )
    {
      InputStream errorStream = process.getErrorStream();
      StringBuffer errorMessage = new StringBuffer();
      byte[] buffer = new byte[4096];
      for ( int size; ( size = errorStream.read( buffer ) ) != -1; )
      {
        errorMessage.append( new String( buffer, 0, size ) );
      }
      throw new IOException( errorMessage.toString() );
    }
  }

  /**
   * Configures the System Property {@link PropertyConstants#IMAGEMAGICK_PATH}
   * to {@link #LOCAL_WINDOWS_INSTALLATION}; this is used for local testing <strong>only</strong>!
   */
  public static void configureLocalImageMagick()
  {
    // Adapt this to your system!
    System.setProperty( PropertyConstants.IMAGEMAGICK_PATH, LOCAL_WINDOWS_INSTALLATION );
  }
}
