package ch.ige.jipps.snapshot.test;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.IOUtils;
import org.custommonkey.xmlunit.SimpleXpathEngine;
import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;

import ch.ige.ipobjects.common.vo.SnapshotVO;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.history.test.FacadeXmlHistoryTest;
import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.enums.IPType;

/**
 * Gets the reference XML documents and compares the registration snapshot to what is generated on the fly.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: FacadeXmlRegistrationSnapshotTest.java,v 1.16 2008/09/22 08:31:39 lewis_lee Exp $
 */
public class FacadeXmlRegistrationSnapshotTest extends AbstractTestFacadeXml
{
  /**
   * This contains a snapshot
   */
  protected static final int TRADEMARK_WITH_SNAPSHOT = 1201320971;

  /**
   * The event date when the snapshot was taken, in this case the registration event.
   */
  protected static final EreigIDat TRADEMARK_EVENT_DATE = EreigIDat.from( 2005080305 );

  /**
   * This contains no snapshot
   */
  protected static final int TRADEMARK_WITHOUT_SNAPSHOT = 1201649793;

  /**
   * Used for XPath queries
   */
  protected SimpleXpathEngine xpathEngine = new SimpleXpathEngine();

  /**
   * The default constructor
   * @param name The test's name
   */
  public FacadeXmlRegistrationSnapshotTest( String name )
  {
    super( name );

    // Force the images to be written.
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.PROCESS_IMAGES );

    this.createSnapshot = true;
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.ipTypesToCheck = new IPType[] { IPType.TRADEMARK_CH };
    this.ipKeysToTest = new int[] { TRADEMARK_WITH_SNAPSHOT, TRADEMARK_WITHOUT_SNAPSHOT };
  }

  // @see ch.ige.jipps.test.AbstractTestFacadeXml#createSnapshot()
  @Override
  protected SnapshotVO createSnapshot() throws Exception
  {
    String snapshotXml = IOUtils.toString( FacadeXmlRegistrationSnapshotTest.class.getResourceAsStream( "/res/test/snapshot_" + TRADEMARK_WITH_SNAPSHOT
        + ".xml" ), "UTF-8" );
    List<Integer> images = new ArrayList<Integer>();
    images.add( 9 );
    return new SnapshotVO( IPKeyVO.create( TRADEMARK_WITH_SNAPSHOT ), TRADEMARK_EVENT_DATE, snapshotXml, images );
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  /**
   * Compares the expected registration snapshot with the actual registration snapshot.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was created
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception If anything goes wrong during the process
   */
  @Override
  protected void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    if ( referenceXmlVo.getIpKey() == TRADEMARK_WITHOUT_SNAPSHOT )
    {
      assertXpathNotExists( "/auszug/registration", actualXml );
    }
    else if ( referenceXmlVo.getIpKey() == TRADEMARK_WITH_SNAPSHOT )
    {
      String convertedXml = ConvertText.escapeHtmlTags( actualXml );
      Document document = XMLUnit.buildTestDocument( convertedXml );

      checkNodeExistenceAndValue( document, null, "/auszug/registration/snapshotTitle", "Stand zum Zeitpunkt der Eintragung" );

      assertXpathExists( "/auszug/registration", document );
      checkSnapshotXmlNode( document, "RFT20", "Marke Nr.", "536303", null );
      checkSnapshotXmlNode( document, "RFT70", "Hinterlegungsdatum / Beginn Schutzfrist", "04. Juli 2005", null );
      checkSnapshotXmlNode( document, "RFT40", "Ablauf Schutzdauer (Jahre)", "10", null );
      checkSnapshotXmlNode( document, "RFT73", "Markeneintragungsgesuch", "50261/2005", null );
      checkSnapshotXmlNode( document, "RFT99", "Marke", null, "9" );
      checkSnapshotXmlNode( document, "RFT420", "Internationale Klassifikation", "29,42", null );
      // For some reason, the escaped < and > are unescaped again...
      checkSnapshotXmlNode( document, "RFT400", "Inhaber/in", "Linus Silvestri AG<br/>Rorschacherstrasse 126<br/>9450 Lüchingen", null );
      checkSnapshotXmlNode(
          document,
          "RFT415",
          "Verzeichnis der Waren und/oder Dienstleistungen",
          "<strong>29</strong><br/>Rindfleisch von Tieren aus biologischer Aufzucht mit Alpung.<p/><strong>42</strong><br/>Qualitätsprüfung gemäss Alp Rind Programm.<br/>",
          null );

      // Check the referenced image
      checkNodeExistenceAndValue( document, null, "/auszug/registration/snapshotReference/bild/@height", "6.45cm" );
      checkNodeExistenceAndValue( document, null, "/auszug/registration/snapshotReference/bild/@width", "7.92cm" );
      checkNodeExistenceAndValue( document, null, "/auszug/registration/snapshotReference/bild/@src", "C:\\temp\\bild_1201320971_9.jpg" );

      // Note: It used to be that having a snapshot changed the look of the history (the first history item, to be exact).
      // But this has since been undone. Just check that the histories are identical.
      new FacadeXmlHistoryTest( getName() ).compareXml( expectedXml, actualXml, referenceXmlVo );
    }
  }

  /**
   * Checks that a snapshot node with a certain ID exists.
   * @param actualXml The XML document as it was created
   * @param id The ID of the key
   * @param germanText The German text to expect
   * @param value The value to expect; is ignored if <code>null</code>
   * @param reference The reference to expect; is ignored if <code>null</code>
   * @throws Exception if the XPath fails
   */
  private void checkSnapshotXmlNode( Document actualXml, String id, String germanText, String value, String reference ) throws Exception
  {
    String idPath = "/auszug/registration/snapshotXml/snapshot/entry/key[@id='" + id + "']";
    checkNodeExistenceAndValue( actualXml, id, idPath, null );

    String germanTranslationPath = idPath + "/de";
    checkNodeExistenceAndValue( actualXml, id, germanTranslationPath, germanText );

    if ( value != null )
    {
      String valuePath = idPath + "/parent::*/value";
      checkNodeExistenceAndValue( actualXml, id, valuePath, value );
    }

    if ( reference != null )
    {
      String referencePath = idPath + "/parent::*/reference[@type='image']";
      checkNodeExistenceAndValue( actualXml, id, referencePath, reference );
    }
  }

  /**
   * Checks one XPath expression and its value.
   * @param actualXml The XML document as it was created
   * @param id The ID of the key
   * @param xpath The XPath expression to apply
   * @param value The value expected from the XPath; is ignored if <code>null</code>
   * @throws Exception if the XPath fails
   */
  protected void checkNodeExistenceAndValue( Document actualXml, String id, String xpath, String value ) throws Exception
  {
    assertXpathExists( xpath, actualXml );
    if ( value != null )
    {
      String xpathValue = this.xpathEngine.evaluate( xpath, actualXml );
      assertEquals( "Checking node with ID " + id, value, xpathValue );
    }
  }
}
