package ch.ige.jipps.test;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import junit.framework.Assert;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.time.StopWatch;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;
import org.apache.xpath.XPathAPI;
import org.custommonkey.xmlunit.SimpleXpathEngine;
import org.custommonkey.xmlunit.XMLTestCase;
import org.custommonkey.xmlunit.XMLUnit;
import org.incava.util.diff.Diff;
import org.incava.util.diff.Difference;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.ipobjects.common.dao.SnapshotDAO;
import ch.ige.ipobjects.common.vo.SnapshotVO;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.jipps.AuszugControl;
import ch.ige.jipps.DesignBilderControl;
import ch.ige.jipps.JIPPSFacadeServicesImpl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.dao.ReferenceXMLDAO;
import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.RegistrationState;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.text.Replacer;

/**
 * Gets the reference XML documents and compares them to what is generated on the fly.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version $Id: AbstractTestFacadeXml.java,v 1.39 2009/01/22 08:05:53 lewis_lee Exp $
 */
public abstract class AbstractTestFacadeXml extends XMLTestCase
{
  /** TODO When everything works, don't throw Exceptions anymore, just use fail() */

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( AbstractTestFacadeXml.class );

  /**
   * Formats {@link ReferenceXMLVO#getRefXmlId()}.
   */
  private static final DecimalFormat ID_NUMBER_FORMAT = new DecimalFormat( "000" );

  /**
   * Gets the XML reference documents
   */
  private ReferenceXMLDAO referenceDAO;

  /**
   * Used for setting up a snapshot during tests
   */
  private SnapshotDAO snapshotDAO;

  /**
   * Used for prettier titles within the test. The key is the report type, the value is the report name per IP type.
   */
  private Map<ReportType, Map<IPType, String>> reportTypes;

  /**
   * The IP types, for prettier formatting. The index corresponds to the IP type. There's no specific DAO with this
   * information - but it's not likely to change!
   */
  private static String[] IP_TYPES = { null, "CH-Patent", "EP-Patent", "PCT-Anmeldung", "Marke CH", "Marke IR", "Design", "Topographien", "Widerspruch", null,
      null, "ESZ" };

  /**
   * The unit under test
   */
  private JIPPSFacadeServicesImpl jippsFacade;

  /**
   * Defines whether the XML is tested for BAGIS.
   */
  protected boolean checkForBagis = true;

  /**
   * Defines for what IP types the checking is done.
   */
  protected IPType[] ipTypesToCheck = null;

  /**
   * If this is <code>true</code>, all IP types are checked, and {@link #ipTypesToCheck} is ignored.
   */
  protected boolean checkAllIpTypes = false;

  /**
   * Defines for what report types no checking is done.
   */
  protected ReportType[] reportTypesToIgnore = new ReportType[] { ReportType.DESIGN_IMAGE_LIST };

  /**
   * Defines at what number (<code>ref_xml_id</code>) the test should start. If it's <code>null</code> it starts
   * at the beginning.
   */
  protected Integer startNumber = null;

  /**
   * Defines at what number (<code>ref_xml_id</code>) the test should end, <strong>including this</strong>. If it's
   * <code>null</code> it runs till the last test case.
   */
  protected Integer endNumber = null;

  /**
   * Defines what IP keys should be tested. If it's empty it's ignored, otherwise it overrides {@link #startNumber} and
   * {@link #endNumber}.
   */
  protected int[] ipKeysToTest = {};

  /**
   * Defines what IP keys should not be tested.
   */
  protected int[] ipKeysNotToTest = {};

  /**
   * If this is <code>true</code>, every time a <code>fail(...)</code> is called, the message is written into the
   * error log instead of directly failling; this enables the test to continue. If it's <code>false</code>, the test
   * fails as usual. <strong>Note: This must never be checked in with the value <code>true</code>!</strong>
   */
  protected final static boolean logErrorInsteadOfFailing = true;

  /**
   * If this is <code>true</code>, the original XML is logged, before it's corrected. If it's <code>false</code>,
   * nothing is logged.
   */
  protected boolean logOriginalExpectedXml = false;

  /**
   * If failures are only logged (see {@link #logErrorInsteadOfFailing}), then this is set to
   * the first failure error message and can thus define whether the whole test failed.
   */
  private static String firstFailure = null;

  /**
   * If <code>true</code>, the execution time for the whole test is measured and written into the log.
   */
  protected boolean useStopwatch = false;

  /**
   * If <code>true</code>, the execution time of every single test case is measured and
   * writetn into the log.
   */
  protected boolean useTestCaseStopwatch = false;

  /**
   * If this is <code>true</code>, a snapshot taken from {@link #createSnapshot()} is inserted during
   * {@link #setUp()} and is used during the test. It only makes sense if the IP type and the report type actually
   * expect a snapshot. After the test finishes, the snapshot is deleted during {@link #tearDown()}.
   */
  protected boolean createSnapshot = false;

  /**
   * A snapshot to use during this test.
   * @see #createSnapshot
   */
  private SnapshotVO snapshot;

  /**
   * Some user...
   */
  private static final int USER_HW = 3;

  /**
   * The regular expression for finding the <code>bild</code> tag
   */
  private static final String BILD_TAG = "<bild [^>]+>";

  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );

    // By default, don't process images, since they require a call to the image processing program/servlet
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.NO_PROCESSING );

    // All files which are read during the tests must be UTF-8
    System.setProperty( "file.encoding", "UTF-8" );

    // Enable log4j logging.
    DOMConfigurator.configure( "C:\\jp\\projects\\jipps\\console_log_config.xml" );
  }

  /**
   * The normal constructor
   * @param args Any arguments
   */
  public AbstractTestFacadeXml( String args )
  {
    super( args );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.referenceDAO = new ReferenceXMLDAO();
    this.snapshotDAO = new SnapshotDAO();
    this.jippsFacade = new JIPPSFacadeServicesImpl();

    this.reportTypes = new HashMap<ReportType, Map<IPType, String>>();
    List<RegAuszTypSTypVO> ipTypeReportTypes = new RegAuszTypSTypDAO().findIpTypeReportTypes();
    for ( RegAuszTypSTypVO ipTypeReportType: ipTypeReportTypes )
    {
      String title = ipTypeReportType.getReportTitle( Locale.GERMAN, RegistrationState.REGISTERED );
      if ( title == null )
      {
        title = ipTypeReportType.getReportTitle( Locale.GERMAN, RegistrationState.APPLICATION );
      }

      if ( !this.reportTypes.containsKey( ipTypeReportType.getReportType() ) )
      {
        this.reportTypes.put( ipTypeReportType.getReportType(), new HashMap<IPType, String>() );
      }
      Map<IPType, String> reportTypesPerIpType = this.reportTypes.get( ipTypeReportType.getReportType() );
      reportTypesPerIpType.put( ipTypeReportType.getIpType(), title );
    }

    if ( this.createSnapshot )
    {
      this.snapshot = createSnapshot();
      Integer snapshotId = this.snapshotDAO.insert( this.snapshot );
      this.snapshot.setSnapshotId( snapshotId );
    }

    XMLUnit.setIgnoreWhitespace( true );

    PathConfiguration.setPathsForLocalTest();
  }

  /**
   * Creates a snapshot which is inserted during {@link #setUp()} and deleted during {@link #tearDown()}.
   * @return A snapshot to insert for testing
   * @throws Exception if the creation goes wrong (a file is not found etc.)
   */
  protected SnapshotVO createSnapshot() throws Exception
  {
    return null;
  }

  /**
   * Gets the XML and tests part of it in a subclass.
   * @param generateXslFo If this is <code>true</code>, the XSL-FO document is generated, otherwise the base XML
   * document with the data
   * @throws Exception if the test fails
   */
  protected void testGetXml( boolean generateXslFo ) throws Exception
  {
    int DEFAULT_START = 0;
    int DEFAUL_END = 99999999; // Should be enough tests ;-)

    List<ReferenceXMLVO> referenceXmlVos = this.referenceDAO.readReferenceXMLs();
    assertTrue( "Checking that there is XML reference data", referenceXmlVos.size() > 0 );

    String imageProcessingMode = System.getProperty( ImageProcessor.PROCESSING_MODE );
    int messageWidth = 43;
    if ( imageProcessingMode.equals( ImageProcessor.PROCESS_IMAGES ) )
    {
      LOG.warn( "|" + StringUtils.center( "-", messageWidth, "-" ) + "|" );
      LOG.warn( "|" + StringUtils.center( "IMAGE PROCESSING IS SWITCHED ON!", messageWidth, " " ) + "|" );
      LOG.warn( "|" + StringUtils.center( "-", messageWidth, "-" ) + "|" );
    }

    if ( this.checkAllIpTypes )
    {
      LOG.warn( StringUtils.center( "#", messageWidth + 2, "#" ) );
      LOG.warn( "#" + StringUtils.center( "Checking all IP types", messageWidth, " " ) + "#" );
      LOG.warn( StringUtils.center( "#", messageWidth + 2, "#" ) );
    }

    int start = DEFAULT_START;
    int end = DEFAUL_END;
    if ( this.startNumber != null )
    {
      start = this.startNumber.intValue();
    }
    if ( this.endNumber != null )
    {
      end = this.endNumber.intValue();
    }

    StopWatch testStopWatch = null;
    if ( this.useStopwatch )
    {
      testStopWatch = new StopWatch();
      testStopWatch.start();
    }
    for ( ReferenceXMLVO xmlVo: referenceXmlVos )
    {
      // Check if this test is specifically allowed or within the start/end range
      if ( this.ipKeysToTest.length > 0 )
      {
        if ( !ArrayUtils.contains( this.ipKeysToTest, xmlVo.getIpKey().intValue() ) )
        {
          LOG.info( "**** IP KEY " + xmlVo.getIpKey() + " IS NOT TESTED ****" );
          continue;
        }
      }
      else if ( this.ipKeysNotToTest.length > 0 )
      {
        if ( ArrayUtils.contains( this.ipKeysNotToTest, xmlVo.getIpKey().intValue() ) )
        {
          LOG.info( "**** IP KEY " + xmlVo.getIpKey() + " IS NOT TESTED ****" );
          continue;
        }
      }
      else if ( xmlVo.getRefXmlId() < start || xmlVo.getRefXmlId() > end )
      {
        continue;
      }

      LOG.info( getInformation( xmlVo ) );

      StopWatch testCaseStopWatch = null;
      if ( this.useTestCaseStopwatch )
      {
        testCaseStopWatch = new StopWatch();
        testCaseStopWatch.start();
      }

      String actualXml = generateXml( xmlVo, generateXslFo );
      if ( actualXml != null )
      {
        if ( generateXslFo )
        {
          compareXml( correctValidTo( xmlVo.getXslFoData() ), correctValidTo( actualXml ), xmlVo );
        }
        else
        {
          compareXml( correctValidTo( xmlVo.getXmlData() ), correctValidTo( actualXml ), xmlVo );
        }
        if ( firstFailure != null && this.logOriginalExpectedXml )
        {
          LOG.info( "Original expected XML before correction:\n" + flatten( xmlVo.getXmlData() ) );
        }

        if ( this.useTestCaseStopwatch && testCaseStopWatch != null )
        {
          testCaseStopWatch.stop();
          LOG.info( "This test case took " + testCaseStopWatch.toString() );
        }
      }
    }
    if ( this.useStopwatch && testStopWatch != null )
    {
      testStopWatch.stop();
      LOG.info( "This test took " + testStopWatch.toString() );
    }

    // Even if failures are only logged, make sure that at the end,
    // the whole test fails!
    if ( logErrorInsteadOfFailing && firstFailure != null )
    {
      Assert.fail( "Test failed - see error log for all failures.\nFirst failure:\n" + firstFailure );
    }
  }

  /**
   * Fails, but may continue the test depending on {@link #logErrorInsteadOfFailing}.
   * @param header The header containing the information about the IP and the report
   */
  public static void fail( String header )
  {
    if ( logErrorInsteadOfFailing )
    {
      LOG.error( header );
      LOG.error( "Test failed" );
      if ( firstFailure == null )
      {
        firstFailure = header;
      }
    }
    else
    {
      Assert.fail( header );
    }
  }

  /**
   * Fails, but may continue the test depending on {@link #logErrorInsteadOfFailing}.
   * @param message The error message to show
   * @param header The header containing the information about the IP and the report
   */
  public static void fail( String message, String header )
  {
    if ( logErrorInsteadOfFailing )
    {
      LOG.error( header );
      LOG.error( message );
      if ( firstFailure == null )
      {
        firstFailure = header + message;
      }
    }
    else
    {
      Assert.fail( header + message );
    }
  }

  /**
   * Fails, but may continue the test depending on {@link #logErrorInsteadOfFailing}.
   * @param message The error message to show
   * @param expectedXml The expected XML snippet
   * @param actualXml The received XML snippet
   * @param header The header containing the information about the IP and the report
   */
  public static void fail( String message, String expectedXml, String actualXml, String header )
  {
    StringBuilder xmlMessage = new StringBuilder();
    xmlMessage.append( "\nExpected:\n---------\n" );
    xmlMessage.append( expectedXml );
    xmlMessage.append( "\n\nReceived:\n---------\n" );
    xmlMessage.append( actualXml );
    xmlMessage.append( "\n\n" );

    if ( logErrorInsteadOfFailing )
    {
      if ( StringUtils.isNotEmpty( header ) )
      {
        LOG.error( header );
      }
      LOG.error( message );
      LOG.error( xmlMessage );
      if ( firstFailure == null )
      {
        firstFailure = header + message + xmlMessage;
      }
    }
    else
    {
      Assert.fail( header + message + xmlMessage );
    }
  }

  /**
   * Compares two pieces of XML, which must exist.
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document as it's created
   * @param xPath The XPath defining what part of the XML document is compared
   * @exception Exception If anything goes wrong
   */
  protected void compareXPath( String expectedXml, String actualXml, String xPath ) throws Exception
  {
    assertXpathsEqual( xPath, expectedXml, xPath, actualXml );
  }

  /**
   * Compares two pieces of XML, which must exist. Note that unlike {@link #compareXPath(String, String, String)}, this
   * gets the values from the documents and then compares with normal JUnit asserts. This means that
   * {@link #logErrorInsteadOfFailing} works.
   * @param expectedDocument The XML document as it's expected
   * @param actualDocument The XML document as it's created
   * @param xPath The XPath defining what part of the XML document is compared
   * @exception Exception If anything goes wrong
   */
  protected void compareXPath( Document expectedDocument, Document actualDocument, String xPath ) throws Exception
  {
    SimpleXpathEngine xpathEngine = new SimpleXpathEngine();
    String expectedValue = xpathEngine.evaluate( xPath, expectedDocument );
    String actualValue = xpathEngine.evaluate( xPath, actualDocument );
    if ( !expectedValue.equals( actualValue ) )
    {
      fail( "Values for XPath '" + xPath + "' differ", expectedValue, actualValue, null );
    }
  }

  /**
   * Compares two pieces of XML; if the nodes referenced by <code>xPath</code> exist in <code>expectedXml</code>,
   * then they must also exist in <code>actualXml</code>.
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document as it's created
   * @param xPath The XPath defining what part of the XML document is compared
   * @exception Exception If anything goes wrong
   */
  protected void compareOptionalXPath( String expectedXml, String actualXml, String xPath ) throws Exception
  {
    SimpleXpathEngine xpathEngine = new SimpleXpathEngine();
    NodeList existingNodes = xpathEngine.getMatchingNodes( xPath, XMLUnit.buildTestDocument( expectedXml ) );
    if ( existingNodes.getLength() > 0 )
    {
      compareXPath( expectedXml, actualXml, xPath );
    }
  }

  /**
   * Compares two pieces of XML; if the nodes referenced by <code>xPath</code> exist in <code>expectedDocument</code>,
   * then they must also exist in <code>actualDocument</code>.
   * @param expectedDocument The XML document as it's expected
   * @param actualDocument The XML document as it's created
   * @param xPath The XPath defining what part of the XML document is compared
   * @exception Exception If anything goes wrong
   */
  protected void compareOptionalXPath( Document expectedDocument, Document actualDocument, String xPath ) throws Exception
  {
    SimpleXpathEngine xpathEngine = new SimpleXpathEngine();
    NodeList existingNodes = xpathEngine.getMatchingNodes( xPath, expectedDocument );
    if ( existingNodes.getLength() > 0 )
    {
      compareXPath( expectedDocument, actualDocument, xPath );
    }
  }

  /**
   * Compares two images. Note that first of all, image processing must be switched on to be able to convert images and
   * compare them. Then, the path may vary, so in the end, only the pure filename is compared.
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document as it's created
   * @param xPath The XPath defining the path to the image
   * @param compareHeightAndWidth If this is <code>true</code>, the image's height and width are also compared
   * @exception Exception If anything goes wrong
   */
  protected void compareImages( String expectedXml, String actualXml, String xPath, boolean compareHeightAndWidth ) throws Exception
  {
    compareImages( XMLUnit.buildTestDocument( expectedXml ), XMLUnit.buildTestDocument( actualXml ), xPath, compareHeightAndWidth );
  }

  /**
   * Compares two images. Note that first of all, image processing must be switched on to be able to convert images and
   * compare them. Then, the path may vary, so in the end, only the pure filename is compared.
   * @param expectedDocument The XML document as it's expected
   * @param actualDocument The XML document as it's created
   * @param xPath The XPath defining the path to the image
   * @param compareHeightAndWidth If this is <code>true</code>, the image's height and width are also compared
   * @exception Exception If anything goes wrong
   */
  protected void compareImages( Document expectedDocument, Document actualDocument, String xPath, boolean compareHeightAndWidth ) throws Exception
  {
    if ( ImageProcessor.processImages() )
    {
      SimpleXpathEngine xpathEngine = new SimpleXpathEngine();
      NodeList expectedNodes = xpathEngine.getMatchingNodes( xPath, expectedDocument );
      NodeList actualNodes = xpathEngine.getMatchingNodes( xPath, actualDocument );
      if ( expectedNodes.getLength() > 1 || actualNodes.getLength() > 1 )
      {
        fail( "Found more than one image node" );
      }
      compareImages( expectedNodes.item( 0 ), actualNodes.item( 0 ), compareHeightAndWidth );
    }
  }

  /**
   * Compares two images. Note that first of all, image processing must be switched on to be able to convert images and
   * compare them. Then, the path may vary, so in the end, only the pure filename is compared.
   * @param expectedNode The expected image node
   * @param actualNode The image node as actually created
   * @param compareHeightAndWidth If this is <code>true</code>, the image's height and width are also compared
   * @exception Exception If anything goes wrong
   */
  protected void compareImages( Node expectedNode, Node actualNode, boolean compareHeightAndWidth ) throws Exception
  {
    if ( ImageProcessor.processImages() )
    {
      if ( expectedNode == null && actualNode != null )
      {
        fail( "Expected no image, but found one" );
      }
      if ( expectedNode != null )
      {
        NamedNodeMap expectedNodeMap = expectedNode.getAttributes();
        NamedNodeMap actualNodeMap = actualNode.getAttributes();
        if ( compareHeightAndWidth )
        {
          compareAttributeValues( expectedNodeMap, actualNodeMap, "height", "auto" );
          compareAttributeValues( expectedNodeMap, actualNodeMap, "width", "auto" );
        }
        compareAttributeValues( expectedNodeMap, actualNodeMap, "src", null );
      }
    }
  }

  /**
   * Compares two node attribute values.
   * @param expectedNodeMap The expected attributes of a node
   * @param actualNodeMap The actual attributes of a node
   * @param attributeName The attribute name to compare
   * @param acceptableDefaultValue If the expected node doesn't exist, but the actual node does, then this defines what
   * value may be in the actual node in order to be still valid
   */
  protected void compareAttributeValues( NamedNodeMap expectedNodeMap, NamedNodeMap actualNodeMap, String attributeName, String acceptableDefaultValue )
  {
    Node expectedNode = expectedNodeMap.getNamedItem( attributeName );
    Node actualNode = actualNodeMap.getNamedItem( attributeName );
    if ( expectedNode == null )
    {
      if ( actualNode != null )
      {
        String actualValue = getFirstText( actualNode );
        if ( !actualValue.equals( acceptableDefaultValue ) )
        {
          fail( "There should be no attribue '" + attributeName + "', but one was found with the value '" + getFirstText( actualNode ) + "'" );
        }
      }
    }
    else
    {
      String expectedValue = getFirstText( expectedNode );
      String actualValue = getFirstText( actualNode );
      if ( !expectedValue.equals( actualValue ) )
      {
        fail( "Different values found for node '" + attributeName + "'", expectedValue, actualValue, null );
      }
    }
  }

  /**
   * Compares the expected XML document with the actual (generated) XML document.
   * @param expectedXml The expected XML document
   * @param actualXml The actual XML document
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception If the comparison fails
   */
  protected abstract void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception;

  /**
   * Generates the XML.
   * @param xmlVo The expected result
   * @param generateXslFo If this is <code>true</code>, the XSL-FO document is generated, otherwise the base XML
   * document with the data
   * @throws Exception if the XML generation fails
   * @return A <code>String</code> with the generated XML
   */
  private String generateXml( ReferenceXMLVO xmlVo, boolean generateXslFo ) throws Exception
  {
    if ( !isReportTypeTested( xmlVo.getReportType() ) )
    {
      LOG.info( "**** REPORT TYPE " + xmlVo.getReportType() + " IS NOT TESTED ****" );
      return null;
    }
    else if ( !isIpTypeTested( xmlVo.getIpType() ) )
    {
      LOG.info( "**** IP TYPE " + xmlVo.getIpType() + " IS NOT TESTED ****" );
      return null;
    }

    PrintJobVO printJob = toPrintJob( xmlVo );
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    printJobs.add( printJob );

    String[] xmlDocuments;
    if ( generateXslFo )
    {
      if ( printJob.formTyp == PrintJobVO.DESIGN_BILDER )
      {
        xmlDocuments = new DesignBilderControl().makeXslFo( printJobs );
      }
      else
      {
        xmlDocuments = new AuszugControl().makeXslFo( printJobs );
      }
    }
    else
    {
      xmlDocuments = this.jippsFacade.getXml( printJobs );
    }
    assertNotNull( "Checking generated XML document", xmlDocuments );
    assertEquals( "Checking generated XML document", 1, xmlDocuments.length );
    return xmlDocuments[0];
  }

  /**
   * Checks whether a report type is tested.
   * @param reportType The report type
   * @return <code>true</code> if the report type must be tested
   */
  private boolean isReportTypeTested( ReportType reportType )
  {
    boolean isReportTypeTested = true;
    for ( ReportType reportTypeToIgnore: this.reportTypesToIgnore )
    {
      if ( reportTypeToIgnore.equals( reportType ) )
      {
        isReportTypeTested = false;
        break;
      }
    }
    return isReportTypeTested;
  }

  /**
   * Checks whether an IP type is tested.
   * @param ipType The IP type
   * @return <code>true</code> if the IP type is tested
   */
  private boolean isIpTypeTested( IPType ipType )
  {
    if ( this.checkAllIpTypes )
    {
      return true;
    }
    else
    {
      // If specific IP keys are configured, there's no use in checking whether an IP type is tested,
      // just these IP keys are used.
      if ( this.ipKeysToTest.length > 0 )
      {
        return true;
      }
      else
      {
        if ( this.ipTypesToCheck == null || this.ipTypesToCheck.length == 0 )
        {
          throw new IllegalStateException( "Found no information about what IP types to check" );
        }
        boolean isIpTypeTested = false;
        for ( IPType ipTypeToCheck: this.ipTypesToCheck )
        {
          if ( ipType.equals( ipTypeToCheck ) )
          {
            isIpTypeTested = true;
            break;
          }
        }
        return isIpTypeTested;
      }
    }
  }

  /**
   * Gets the basic information about a reference XML document.
   * @param referenceXmlVo The reference XML document
   * @return The basic information about a reference XML document
   */
  protected String getInformation( ReferenceXMLVO referenceXmlVo )
  {
    StringBuilder information = new StringBuilder();
    information.append( "--- " );
    information.append( ID_NUMBER_FORMAT.format( referenceXmlVo.getRefXmlId() ) );
    information.append( " -------- " );
    information.append( "Key: " );
    information.append( StringUtils.leftPad( referenceXmlVo.getIpKey().toString(), 10 ) );
    information.append( ", Date: " );
    information.append( DateFormatter.shortFormat( referenceXmlVo.getReportDate() ) );
    information.append( ", Type: " );
    information.append( IP_TYPES[referenceXmlVo.getIpType().toDB()] );
    information.append( ", Report: " );
    Map<IPType, String> reportTypePerIpType = this.reportTypes.get( referenceXmlVo.getReportType() );
    information.append( reportTypePerIpType.get( referenceXmlVo.getIpType() ) );
    return information.toString();
  }

  /**
   * Converts a reference XML document to its corresponding print job.
   * @param referenceXmlVo The reference XML document
   * @return The print job that should create <code>referenceXmlVo</code>
   */
  private PrintJobVO toPrintJob( ReferenceXMLVO referenceXmlVo )
  {
    PrintJobVO printJob = new PrintJobVO();
    if ( referenceXmlVo.getReportType().equals( ReportType.DESIGN_IMAGE_LIST ) )
    {
      printJob.formTyp = PrintJobVO.DESIGN_BILDER;
    }
    else
    {
      printJob.formTyp = PrintJobVO.AUSZUG;
    }
    printJob.schutitTyp = referenceXmlVo.getIpType().toDB();
    printJob.printIgekopf = false;
    printJob.reportId = referenceXmlVo.getIpKey().intValue();
    printJob.maNr = USER_HW;
    printJob.regauszTyp = referenceXmlVo.getReportType().toDB();
    printJob.sprache = BAGISLanguageDomain.parse( referenceXmlVo.getLocale() ).toDB();
    printJob.standDatum = referenceXmlVo.getReportDate();

    // Designs have their current state only in application state
    if ( referenceXmlVo.getIpType().equals( IPType.DESIGN ) && referenceXmlVo.getReportType().equals( ReportType.CURRENT_STATE ) )
    {
      printJob.registerKz = IpRegisterStateConstants.APPLICATION_STATE;
    }
    else
    {
      printJob.registerKz = IpRegisterStateConstants.REGISTRATION_STATE;
    }

    return printJob;
  }

  /**
   * The <code>valid to</code> text may have changed from having a space to having no space, so this removes all
   * spaces from that label.
   * @param xmlDocument The XML document to check
   * @return The same XML with corrected <code>valid to</code> labels
   */
  private String correctValidTo( String xmlDocument )
  {
    return xmlDocument.replaceAll( "(gültig bis:)[^<]", "$1" );
  }

  /**
   * Checks that the value for the chunk of XML between the <strong>first</strong> opening node and the <strong>last</strong>
   * closing node is as expected. For instance, if the XML contains
   * 
   * <pre>
   *        &lt;a&gt;&lt;b&gt;&lt;a&gt;&lt;c&gt;bla&lt;/c&gt;&lt;/a&gt;&lt;/b&gt;&lt;/a&gt;
   * </pre>
   * 
   * then calling this with a <code>nodeName</code> of <code>a</code> will check
   * 
   * <pre>
   *        &lt;a&gt;&lt;b&gt;&lt;a&gt;&lt;c&gt;bla&lt;/c&gt;&lt;/a&gt;&lt;/b&gt;&lt;/a&gt;
   * </pre>
   * 
   * Calling this with a <code>nodeName</code> of <code>b</code> will check
   * 
   * <pre>
   *        &lt;b&gt;&lt;a&gt;&lt;c&gt;bla&lt;/c&gt;&lt;/a&gt;&lt;/b&gt;
   * </pre>
   * 
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document from the test result
   * @param nodeName The node name to check
   * @param referenceXmlVo The VO containing the data for the test case
   */
  protected void checkNode( String expectedXml, String actualXml, String nodeName, ReferenceXMLVO referenceXmlVo )
  {
    String startNode = "<" + nodeName + ">";
    String endNode = "</" + nodeName + ">";

    String expectedValue = flatten( substringBetween( expectedXml, startNode, endNode ) );
    String actualValue = flatten( substringBetween( actualXml, startNode, endNode ) );

    if ( StringUtils.isEmpty( expectedValue ) && StringUtils.isEmpty( actualValue ) )
    {
      return;
    }
    else
    {
      // If no images are processed, the image tag will be empty, so ignore it
      if ( !ImageProcessor.processImages() )
      {
        expectedValue = Replacer.replaceFirst( expectedValue, BILD_TAG, "" );
        actualValue = Replacer.replaceFirst( actualValue, BILD_TAG, "" );
      }

      if ( expectedValue != null && expectedValue.equals( actualValue ) )
      {
        return;
      }
      else
      {
        String differences = getDifferences( expectedValue, actualValue );
        fail( differences, expectedXml, actualXml, getInformation( referenceXmlVo ) );
      }
    }
  }

  /**
   * Gets the differences between two Strings.
   * @param expected The expected String
   * @param actual The actual String
   * @return The differences as formatted result on several lines
   */
  @SuppressWarnings( "unchecked" )
  protected static String getDifferences( String expected, String actual )
  {
    StringBuilder output = new StringBuilder();

    Character[] expectedCharacters = toCharArray( expected );
    Character[] actualCharacters = toCharArray( actual );
    Diff diff = new Diff( expectedCharacters, actualCharacters );
    List<Difference> differences = diff.diff();
    if ( differences.size() > 0 )
    {
      output.append( "\nGot " );
      output.append( differences.size() );
      output.append( " difference" );
      output.append( differences.size() > 1 ? "s" : "" );
      for ( Difference difference: differences )
      {
        int start = NumberUtils.min( difference.getAddedStart(), difference.getDeletedStart(), Integer.MAX_VALUE ) - 10;
        int end = NumberUtils.max( difference.getAddedEnd(), difference.getDeletedEnd(), Integer.MIN_VALUE ) + 10;
        output.append( "\nExpected: ..." );
        output.append( expected.subSequence( start, end ) );
        output.append( "..." );
        output.append( "\nReceived: ..." );
        output.append( actual.subSequence( start, end ) );
        output.append( "..." );
      }
    }
    output.append( "\n" );
    return output.toString();
  }

  /**
   * Gets the String that is nested in between two Strings, taking the last found instance of <code>close</code>.
   * @param string The String to check
   * @param open the String before the substring, may be null
   * @param close the String after the substring, may be null
   * @return the substring, <code>null</code> if no match
   */
  private static String substringBetween( String string, String open, String close )
  {
    if ( string == null || open == null || close == null )
    {
      return null;
    }

    int start = string.indexOf( open );
    if ( start == -1 )
    {
      return null;
    }
    int end = string.lastIndexOf( close );
    if ( end == -1 )
    {
      return null;
    }

    end += close.length();
    return string.substring( start, end );
  }

  /**
   * Flattens the XML document, removing all carriage returns, tabulators etc. and then compares a node using
   * {@link #checkNode(String, String, String, ReferenceXMLVO)}.
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document from the test result
   * @param nodeName The node name to check
   * @param referenceXmlVo The VO containing the data for the test case
   */
  protected void flattenAndCheckNode( String expectedXml, String actualXml, String nodeName, ReferenceXMLVO referenceXmlVo )
  {
    String flattenedExpectedXml = flatten( expectedXml );
    String flattenedActualXml = flatten( actualXml );
    checkNode( flattenedExpectedXml, flattenedActualXml, nodeName, referenceXmlVo );
  }

  /**
   * Flattens an XML document, removing all carriage returns, tabulators etc.
   * @param xml The XML document to flatten
   * @return The flattened XML document without any carriage returns, tabulators etc. or <code>null</code> if
   * <code>xml</code> is <code>null</code>
   */
  protected String flatten( String xml )
  {
    if ( xml == null )
    {
      return null;
    }
    else
    {
      String flattenedXml = xml.replaceAll( "\\r?\\n\\s*", "" );
      return flattenedXml.replaceAll( "\\t", "" );
    }
  }

  /**
   * Gets the nodes that correspond to the given XPath statement.
   * @param xml The XML document
   * @param xPath The XPath string defining nodes
   * @return A <code>List</code> of Nodes
   * @throws TransformerException if the nodes can't be created
   * @throws SAXException if the nodes can't be created
   * @throws IOException if the nodes can't be created
   * @throws ParserConfigurationException if the nodes can't be created
   */
  protected List<Node> getNodes( String xml, String xPath ) throws TransformerException, SAXException, IOException, ParserConfigurationException
  {
    Document xmlDocument = XMLUnit.buildTestDocument( xml );
    List<Node> nodes = new ArrayList<Node>();
    SimpleXpathEngine xpathEngine = new SimpleXpathEngine();
    NodeList selectedNodes = xpathEngine.getMatchingNodes( xPath, xmlDocument );
    int matchingNodes = selectedNodes.getLength();
    for ( int i = 0; i < matchingNodes; i++ )
    {
      nodes.add( selectedNodes.item( i ) );
    }
    return nodes;
  }

  /**
   * Gets the <code>node</code>'s child node called <code>childNodeName</code>.
   * @param node The node containing child nodes
   * @param childNodeName The name of the child node to select
   * @return The child node, which may be <code>null</code>
   */
  protected Node getChildNode( Node node, String childNodeName )
  {
    for ( int i = 0; i < node.getChildNodes().getLength(); i++ )
    {
      Node childNode = node.getChildNodes().item( i );
      if ( childNode.getNodeName().equals( childNodeName ) )
      {
        return childNode;
      }
    }
    return null;
  }

  /**
   * Gets the text of <code>node</code>'s child node called <code>childNodeName</code>.
   * @param node The node containing child nodes
   * @param childNodeName The name of the child node to select
   * @return The value of the child node, which may be <code>null</code>
   */
  protected String getChildNodeFirstText( Node node, String childNodeName )
  {
    Node childNode = getChildNode( node, childNodeName );
    if ( childNode == null )
    {
      return null;
    }
    else
    {
      return getFirstText( childNode );
    }
  }

  /**
   * Checks the texts of <code>node</code>'s child node called <code>childNodeName</code> and fails if they're not
   * the same.
   * @param expectedNode The node as it's expected
   * @param actualNode The node as it's created
   * @param childNodeName The name of the child node to select
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document from the test result
   * @param referenceXmlVo
   */
  protected void compareChildNodeTexts( Node expectedNode, Node actualNode, String childNodeName, String expectedXml, String actualXml,
      ReferenceXMLVO referenceXmlVo )
  {
    String expectedValue = getChildNodeFirstText( expectedNode, childNodeName );
    String actualValue = getChildNodeFirstText( actualNode, childNodeName );
    if ( expectedValue == null && actualValue != null )
    {
      fail( "Checking node " + childNodeName + ": Expected no value, but found " + actualValue, expectedXml, actualXml, getInformation( referenceXmlVo ) );
    }
    if ( expectedValue != null && !expectedValue.equals( actualValue ) )
    {
      fail( "Checking node " + childNodeName + ": Expected value " + expectedValue + ", but found " + actualValue, expectedXml, actualXml,
          getInformation( referenceXmlVo ) );
    }
  }

  /**
   * Checks all texts of <code>node</code>'s child node called <code>childNodeName</code>, checking the texts of
   * the child's children, and fails if they're not the same.
   * @param expectedNode The node as it's expected
   * @param actualNode The node as it's created
   * @param childNodeName The name of the child node to select
   * @param grandchildNodeName The namd of the child's child nodes to select
   * @param expectedXml The XML document as it's expected
   * @param actualXml The XML document from the test result
   * @param referenceXmlVo
   */
  protected void compareAllChildNodeTexts( Node expectedNode, Node actualNode, String childNodeName, String grandchildNodeName, String expectedXml,
      String actualXml, ReferenceXMLVO referenceXmlVo )
  {
    Element expectedChild = (Element) getChildNode( expectedNode, childNodeName );
    Element actualChild = (Element) getChildNode( actualNode, childNodeName );
    if ( expectedChild == null )
    {
      if ( actualChild == null )
      {
        return;
      }
      else
      {
        fail( "Expected no child " + childNodeName + ", but found " + actualChild, expectedXml, actualXml, getInformation( referenceXmlVo ) );
      }
    }
    else
    {
      if ( actualChild == null )
      {
        fail( "Expected child " + childNodeName + ", but found none", expectedXml, actualXml, getInformation( referenceXmlVo ) );
      }
      else
      {
        NodeList expectedGrandchildren = expectedChild.getChildNodes();
        NodeList actualGrandchildren = actualChild.getChildNodes();
        if ( expectedGrandchildren.getLength() != actualGrandchildren.getLength() )
        {
          fail( "Expected " + expectedGrandchildren.getLength() + " grandchild nodes, but found " + actualGrandchildren.getLength(), expectedXml, actualXml,
              getInformation( referenceXmlVo ) );
        }
        else
        {
          for ( int i = 0; i < expectedGrandchildren.getLength(); i++ )
          {
            Node expectedGrandchild = expectedGrandchildren.item( i );
            Node actualGrandchild = actualGrandchildren.item( i );
            if ( expectedGrandchild.getNodeName().equals( grandchildNodeName ) )
            {
              String expectedText = getFirstText( expectedGrandchild );
              String actualText = getFirstText( actualGrandchild );
              if ( expectedText == null )
              {
                if ( actualText != null )
                {
                  fail( "Expected no text, but found " + actualText, expectedXml, actualXml, getInformation( referenceXmlVo ) );
                }
              }
              else
              {
                if ( !expectedText.equals( actualText ) )
                {
                  fail( "On grandchild " + i + ": Expected text " + expectedText + ", but found " + actualText, expectedXml, actualXml,
                      getInformation( referenceXmlVo ) );
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * Gets the text value of the node's first child node.
   * @param node The node whose child text is taken
   * @return the text value of the node's first child node
   */
  protected String getFirstText( Node node )
  {
    Node firstChild = node.getFirstChild();
    if ( firstChild == null )
    {
      return null;
    }
    else
    {
      return firstChild.getNodeValue();
    }
  }

  /**
   * Gets the first line in a <code>List</code> of lines.
   * @param lines The lines of text
   * @return The first line or <code>null</code> if there isn't any
   */
  protected String getFirstLine( List<String> lines )
  {
    if ( lines != null && lines.size() > 0 )
    {
      return lines.get( 0 ).trim();
    }
    else
    {
      return null;
    }
  }

  /**
   * Gets the text lines from a node's subnodes.
   * @param node The node whose subnodes contain text lines
   * @param subNodeName The name of the subnode
   * @return A <code>List</code> of text lines
   * @throws Exception if the text lines can't be fetched
   */
  protected List<String> getTextLines( Node node, String subNodeName ) throws Exception
  {
    List<String> texts = new ArrayList<String>();
    NodeList textNodes = XPathAPI.selectNodeList( node, subNodeName );
    for ( int i = 0; i < textNodes.getLength(); i++ )
    {
      Node stateTextNode = textNodes.item( i );
      texts.add( getFirstText( stateTextNode ) );
    }
    return texts;
  }

  /**
   * Compares two lists of lines.
   * @param expectedLines The expected lines
   * @param actualLines The actual (generated) lines
   */
  protected void compareLines( List<String> expectedLines, List<String> actualLines )
  {
    for ( int i = 0; i < expectedLines.size(); i++ )
    {
      String expectedLine = expectedLines.get( i ).trim();
      if ( i < actualLines.size() )
      {
        String actualLine = actualLines.get( i ).trim();
        if ( !linesMatch( expectedLine, actualLine ) )
        {
          fail( "Wrong line, expected <" + expectedLine + ">, but found <" + actualLine + ">" );
        }
      }
      else
      {
        fail( "Missing the following line in generated XML: <" + expectedLine + ">" );
      }
    }
    if ( actualLines.size() > expectedLines.size() )
    {
      fail( "Found extra lines in generated XML: " + actualLines );
    }
  }

  /**
   * Compares two lines, including all special cases and special old and new formats.
   * @param expectedLine The line as it's expected
   * @param actualLine The line as it's created
   * @return <code>true</code> if the lines match, including any special cases
   */
  protected boolean linesMatch( String expectedLine, String actualLine )
  {
    if ( expectedLine.equals( actualLine ) )
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  /**
   * Converts a String to its Characters.
   * @param string The String to convert
   * @return The Characters as array
   */
  private static Character[] toCharArray( String string )
  {
    char[] characters = string.toCharArray();
    Character[] characterObjects = new Character[characters.length];
    for ( int i = 0; i < characters.length; i++ )
    {
      characterObjects[i] = Character.valueOf( characters[i] );
    }
    return characterObjects;
  }

  // @see junit.framework.TestCase#tearDown()
  @Override
  protected void tearDown() throws Exception
  {
    if ( this.snapshot != null )
    {
      this.snapshotDAO.delete( this.snapshot.getSnapshotId() );
    }

    super.tearDown();
  }

}
