package ch.ige.jipps.test;

import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.io.FileUtils;

import ch.ige.jipps.JIPPSFacadeServicesImpl;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.xml.test.BaseDaoXmlTestTemplate;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Tests {@link ch.ige.jipps.JIPPSFacade} for PDF functionality.
 *
 * <p>Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author    Eric Lewis
 * @version   $Id: JIPPSFacadePdfTest.java,v 1.61 2008/07/18 12:56:53 lewis_lee Exp $
 */
public class JIPPSFacadePdfTest extends BaseDaoXmlTestTemplate
{
  /**
   * Possible IP keys for Swiss patents:
   * 1200050699, 1200630142, 1200600069, 1200628415, 1201061163 (Sonderzeichen)
   *
   * Possible IP keys for European patents:
   * 1201190936, 2000546047, 2000591597, 31323674, 2000927023, 2001116836, 2001153179, 31018353
   *
   * Possible IP keys for PCT registrations:
   * 1201126638
   *
   * Possible IP keys for SPCs:
   * 1200112846 (and its base IP 31018353)
   *
   * Possible IP keys for trademarks:
   * 100031995, 1200883245, 1201640203
   * 1100408333 (Omega, extra-long WDL), 1201866552 (one extra-long WDL class text), 1201531251, 1200729024, 1201114193 (WDL history)
   * 1201320971, 1201585829, 1201633900, 1200637252 (all image history)
   * 1201607794, 1201412573, 1201360129, 1200015755, 1200627910 (all big image history)
   * 1200832708, 1201675540, 191321994 (all image history with register relevant image changes)
   * 1201649793 (image history after registration, which is NOT register relevant)
   * 1201844361 (carriage returns in publication remarks)
   * 116311991, 1200103440 (trademark name contains < and >)
   *
   * Possible IP keys for designs:
   * 1200745229 (Alien-Gestell), 1201678532 (jpeg images), 1200106335 (Schnuefeli), 1200794842 (Postkarten-Galerie),
   * 1200783223 (Plueschtier), 1200765992 (Wunderschoenes Kleid...), 1201495566(Stiefelknecht),
   * 1201427165 (Monsterteil, 44 Ordnungsnummern), 1201193460 (Bedienungstableau mit Beschreibung)
   * 1200768734 (changes on 04.09.2007!), 1201115687, 1201706036 (all image history),
   * 1200750821 (Schraube), 1200751303 (Golfer-Teller), 1200751423 (Sattel-Regenschutz),
   * 1200753159 (Vogelhaus), 1200758319 (Copa do mundo), 1200783165 (Kifferkerze),
   * 1200786128 (Uhren), 1200790261 (Futternapf), 1200832809 (farbiger Schraubengriff),
   * 1200835860 (Biegsame Hände), 1201145111 (Zahnbürste), 1201510463 (Holzspielzeug)
   */

  /**
   * Type of the IP
   */
  private static final IPType IP_TYPE = IPType.TRADEMARK_CH;

  /**
   * A bunch of well-known IP keys.
   */
  private static final int[] IP_KEYS = { 1201531251 }; 

  /**
   * The possible report types for Swiss patents.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_PATENT_CH = {
    ReportType.REGISTER_REPORT_AFTER_REGISTRATION /* 1 */,
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
    ReportType.CURRENT_STATE /* 7 */,
    ReportType.SEARCH_REPORT_AFTER_APPLICATION /* 9 */
  };

  /**
   * The possible report types for European patents.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_PATENT_EP = {
    ReportType.REGISTER_REPORT_AFTER_REGISTRATION /* 1 */,
//    ReportType.SEARCH_REPORT_AFTER_REGISTRATION /* 3 */,
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
    ReportType.CURRENT_STATE /* 7 */,
    ReportType.SEARCH_REPORT_AFTER_APPLICATION /* 9 */
  };

  /**
   * The possible report types for PCTs.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_PATENT_REGISTRATION_PCT = {
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
    ReportType.SEARCH_REPORT_AFTER_APPLICATION /* 9 */
  };

  /**
   * The possible report types for trademarks.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_TRADEMARK = {
    ReportType.REGISTER_REPORT_AFTER_REGISTRATION /* 1 */,
    ReportType.REGISTRATION_CONFIRMATION /* 2 */,
    ReportType.PRIORITY_REPORT /* 4 */,
    ReportType.TRADEMARK_SEARCH /* 5 */,
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
//    ReportType.CURRENT_STATE /* 7 */,
    ReportType.TRADEMARK_RENEWAL /* 8 */,
    ReportType.SEARCH_REPORT_AFTER_APPLICATION /* 9 */,
    ReportType.CORRECTION_CONFIRMATION /* 12 */
  };

  /**
   * The possible report types for designs.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_DESIGN = {
    ReportType.REGISTER_REPORT_AFTER_REGISTRATION /* 1 */,
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
    ReportType.CURRENT_STATE /* 7 */,
    ReportType.REGISTER_REPORT_FOR_OWNER /* 11 */,
//    ReportType.DESIGN_IMAGE_LIST /* 31 - note that this must run on its own! */
  };

  /**
   * The possible report types for SPCs.
   * Comment the ones you don't need.
   */
  private static final ReportType[] REPORT_TYPES_SPC = {
    ReportType.REGISTER_REPORT_AFTER_REGISTRATION /* 1 */,
    ReportType.DOSSIER_REPORT_AFTER_APPLICATION /* 6 */,
    ReportType.CURRENT_STATE /* 7 */,
    ReportType.SEARCH_REPORT_AFTER_APPLICATION /* 9 */
  };
  
  /**
   * Values can be 0 or 1, depending on the report type
   */
//  private static final int REGISTER_KZ = IpRegisterStateConstants.APPLICATION_STATE;
  private static final int REGISTER_KZ = IpRegisterStateConstants.REGISTRATION_STATE;

  /**
   * The reports are created in these languages.
   */
  private static final Locale[] LANGUAGES = new Locale[] {
    Locale.GERMAN,
    Locale.FRENCH,
    Locale.ITALIAN
  };

  /**
   * Date of the requested report
   */
  private static final String CURRENT_DATE = "06.07.2007";

  /**
   * Some user...
   */
  private static final int USER_HW = 3;

  /**
   * The unit under test
   */
  private JIPPSFacadeServicesImpl jippsFacade = new JIPPSFacadeServicesImpl();

  /**
   * The default constructor
   * @param args Any arguments
   */
  public JIPPSFacadePdfTest( String args )
  {
    super( args );
  }

  /**
   * Tests {@link ch.ige.jipps.JIPPSFacadeServices#getMergePdf(List)}.
   * @throws Exception if the test fails
   */
  public void testGetMergePdf() throws Exception
  {
    List<PrintJobVO> printJobs = createPrintJobs();

    byte[] pdfContent = this.jippsFacade.getMergePdf( printJobs );
    assertNotNull( "Checking a PDF was generated", pdfContent );
    
    File pdfFile = File.createTempFile( getFilename(), FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, pdfContent );
    System.out.print( "PDF written to " + pdfFile );
  }

  /**
   * Creates the print jobs to execute.
   * @return A <code>List</code> of print jobs
   * @throws ParseException if the report date can't be parsed
   */
  private List<PrintJobVO> createPrintJobs() throws ParseException
  {
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    for ( int ipKey: IP_KEYS )
    {
      for ( ReportType reportType: getReportTypes() )
      {
        for ( Locale language: LANGUAGES )
        {
          PrintJobVO printJob = new PrintJobVO();
          if ( reportType == ReportType.DESIGN_IMAGE_LIST )
          {
            printJob.formTyp = PrintJobVO.DESIGN_BILDER;
          }
          else
          {
            printJob.formTyp = PrintJobVO.AUSZUG;
          }
          printJob.schutitTyp = IP_TYPE.toDB();
          printJob.printIgekopf = false;
          printJob.reportId = ipKey;
          printJob.maNr = USER_HW;
          printJob.regauszTyp = reportType.toDB();
          printJob.sprache = BAGISLanguageDomain.parse( language ).toDB();
          printJob.standDatum = DateConverter.stringToDate( CURRENT_DATE );
          printJob.registerKz = REGISTER_KZ;

          printJobs.add( printJob );
        }
      }
    }
    return printJobs;
  }

  /**
   * Returns the <code>String</code> with the filename (without path)
   * @return A <code>String</code> with the path to save the file
   */
  private String getFilename()
  {
    StringBuilder filename = new StringBuilder();
    for ( int ipKey: IP_KEYS )
    {
      filename.append( ipKey );
      filename.append( "_" );
    }
    filename.append( "reports_" );
    for ( ReportType reportType: getReportTypes() )
    {
      filename.append( reportType.toDB() );
      filename.append( "_" );
    }

    return filename.toString();
  }

  /**
   * Gets the report types to show depending on {@link #IP_TYPE}.
   * This is a static version of what can be found in the table
   * <code>REGAUSZTYP_STYP</code>.
   * @return The report types to show
   */
  private ReportType[] getReportTypes()
  {
    switch( IP_TYPE )
    {
      case PATENT_CH:
        return REPORT_TYPES_PATENT_CH;
      case PATENT_EP:
        return REPORT_TYPES_PATENT_EP;
      case PATENT_PCT:
        return REPORT_TYPES_PATENT_REGISTRATION_PCT;
      case TRADEMARK_CH:
        return REPORT_TYPES_TRADEMARK;
      case DESIGN:
        return REPORT_TYPES_DESIGN;
      case SPC:
        return REPORT_TYPES_SPC;
      default:
        throw new IllegalStateException( "No valid IP type found" );
    }
  }
}
