package ch.ige.jipps.test.imagelist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.w3c.dom.Node;

import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.enums.report.ReportType;

/**
 * Gets the reference XML documents of the design image list and compares their state
 * to what is generated on the fly.
 * Note that some data contains spaces at the end, so <code>String</code>s are always trimmed.
 *
 * <p>Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author    Esther Bandschapp
 * @version   $Id: FacadeXmlDesignImageListTestTemplate.java,v 1.3 2009/05/06 15:44:13 lewis_lee Exp $
 */
public class FacadeXmlDesignImageListTestTemplate extends AbstractTestFacadeXml
{
  /**
   * The default constructor.
   * @param name The test's name
   */
  public FacadeXmlDesignImageListTestTemplate( String name )
  {
    super( name );
  }

  /**
   * These IP keys were all temporarily converted to PNG images.
   * After this release, they will be changed back to TIFF, so revert the changes made.
   */
  private static final int[] IP_KEYS_WITH_TEMPORARY_PNG_IMAGES = { 1200745229, 1201678532, 1200106335, 1200794842, 1200783223, 1200765992, 1201495566,
      1201427165, 1201193460, 1200768734, 1201115687, 1201706036, 1200750821, 1200751303, 1200751423, 1200753159, 1200758319, 1200783165, 1200786128,
      1200790261, 1200832809, 1200835860, 1201145111, 1201510463, 1201597095, 1200771388, 1202120388 };

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    // Just use design image lists.
    List<ReportType> reportTypes = new ArrayList<ReportType>( Arrays.asList( ReportType.values() ) );
    reportTypes.remove( ReportType.DESIGN_IMAGE_LIST );
    super.reportTypesToIgnore = reportTypes.toArray( this.reportTypesToIgnore );
    this.checkAllIpTypes = true;

    this.useStopwatch = true;
    this.useTestCaseStopwatch = true;

    // Force the images to be written
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.PROCESS_IMAGES );
  }

  /**
   * Shows whether the design temporarily contains (converted) PNG images.
   * @param referenceXmlVo The reference XML
   * @return <code>true</code> if the design contains PNG images, otherwise <code>false</code>
   */
  private boolean containsPNG( ReferenceXMLVO referenceXmlVo )
  {
    int testIpKey = referenceXmlVo.getIpKey().intValue();
    for ( int ipKey: IP_KEYS_WITH_TEMPORARY_PNG_IMAGES )
    {
      if ( ipKey == testIpKey )
      {
        return true;
      }
    }
    return false;
  }

  /**
   * Compares the expected image list with the actual image list.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception if anything goes wrong
   */
  @Override
  protected void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    if ( containsPNG( referenceXmlVo ) )
    {
      expectedXml = expectedXml.replace( FileConstants.EXTENSION_JPG, FileConstants.EXTENSION_PNG );
    }

    compareXPath( expectedXml, actualXml, "/bilderListe/igekopf" );
    compareXPath( expectedXml, actualXml, "/bilderListe/listeHeader/eintrNr" );
    compareXPath( expectedXml, actualXml, "/bilderListe/listeHeader/sprache" );
    compareXPath( expectedXml, actualXml, "/bilderListe/listeHeader/auszugsTitel" );
    compareXPath( expectedXml, actualXml, "/bilderListe/listeHeader/standDat" );
    compareXPath( expectedXml, actualXml, "/bilderListe/listeFooter/page" );

    compareImageLists( expectedXml, actualXml, referenceXmlVo, "/bilderListe/bilder" );
    compareImageLists( expectedXml, actualXml, referenceXmlVo, "/bilderListe/history/bilder" );

    // Just to be 100% sure...
    checkNode( expectedXml, actualXml, "igekopf", referenceXmlVo );
    checkNode( expectedXml, actualXml, "listeHeader", referenceXmlVo );
    checkNode( expectedXml, actualXml, "history", referenceXmlVo );
    checkNode( expectedXml, actualXml, "bilder", referenceXmlVo );
    checkNode( expectedXml, actualXml, "listeFooter", referenceXmlVo );
  }

  /**
   * Compares two lists of images.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   * @param xPath The XPath expression used to get the two lists from the XML documents
   * @throws Exception if the comparison fails
   */
  private void compareImageLists( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo, final String xPath ) throws Exception
  {
    List<Node> expectedNodes = getNodes( expectedXml, xPath );
    List<Node> actualNodes = getNodes( actualXml, xPath );
    for ( int i = 0; i < expectedNodes.size(); i++ )
    {
      Node expectedNode = expectedNodes.get( i );
      Node actualNode = null;
      try
      {
        actualNode = actualNodes.get( i );
        compareChildNodeTexts( expectedNode, actualNode, "modOrdNr", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "bildNr", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "bildNrVon", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "vonEreigDat", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "artLbl", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "gueltigTxt", expectedXml, actualXml, referenceXmlVo );
        compareChildNodeTexts( expectedNode, actualNode, "bisEreigDat", expectedXml, actualXml, referenceXmlVo );

        Node expectedBildNode = getChildNode( expectedNode, "bild" );
        Node actualBildNode = getChildNode( actualNode, "bild" );
        compareImages( expectedBildNode, actualBildNode, true );

        if ( expectedNode.equals( actualNode ) )
        {
          fail( "Expected node <" + expectedNode + ">, but found <" + actualNode + ">", expectedXml, actualXml, getInformation( referenceXmlVo ) );
          break;
        }
      }
      catch ( IndexOutOfBoundsException ex )
      {
        fail( "Expected, but could not find history node " + expectedNode, expectedXml, actualXml, getInformation( referenceXmlVo ) );
        break;
      }
    }
    if ( actualNodes.size() > expectedNodes.size() )
    {
      fail( "Generated XML contains extra nodes: " + actualNodes.subList( expectedNodes.size(), actualNodes.size() ), getInformation( referenceXmlVo ) );
    }
  }
}
