package ch.ige.jipps.text;

import java.sql.SQLException;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.event.dao.ArTextDAO;
import ch.ige.ipobjects.event.vo.ArTextVO;
import ch.ige.util.constants.EventNumberConstants;
import ch.ige.util.enums.event.EventTypeTextType;
import ch.ige.util.validation.Check;

/**
 * Fetches event texts based on the event text numbers in objects and fills
 * the corresponding text into the object.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: EventTextLoader.java,v 1.13 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class EventTextLoader
{
  /**
   * Loads the event texts and sets them in the event text containers.
   * @param eventTextContainers The <code>List</code> of event text containers to fill with event texts
   * @param language The language to use for the event texts
   * @param setUniqueEventText If this is <code>true</code>, only the first event text with the same
   *        number is set - this implies that <code>eventTextConstainers</code> are ordered by
   *        event text numbers; otherwise all event texts are set.
   * @throws SQLException if the event texts can't be loaded
   */
  public static void insertEventText( List<? extends EventTextContainer> eventTextContainers, Locale language, boolean setUniqueEventText ) throws SQLException
  {
    insertEventText( eventTextContainers, language, setUniqueEventText, true );
  }

  /**
   * Loads the event texts, if they exist, and sets them in the event text containers.
   * @param eventTextContainers The <code>List</code> of event text containers to fill with event texts
   * @param language The language to use for the event texts
   * @param setUniqueEventText If this is <code>true</code>, only the first event text with the same
   *        number is set - this implies that <code>eventTextConstainers</code> are ordered by
   *        event text numbers; otherwise all event texts are set.
   * @throws SQLException if the event texts can't be loaded
   */
  public static void insertOptionalEventText( List<? extends EventTextContainer> eventTextContainers, Locale language, boolean setUniqueEventText )
      throws SQLException
  {
    insertEventText( eventTextContainers, language, setUniqueEventText, false );
  }

  /**
   * Loads the event texts and sets them in the event text containers.
   * @param eventTextContainers The <code>List</code> of event text containers to fill with event texts
   * @param language The language to use for the event texts
   * @param setUniqueEventText If this is <code>true</code>, only the first event text with the same
   *        number is set - this implies that <code>eventTextConstainers</code> are ordered by
   *        event text numbers; otherwise all event texts are set.
   * @param checkForEventTextNumber If this is <code>true</code>, the event text number must exist in the event text container
   * @throws SQLException if the event texts can't be loaded
   */
  private static void insertEventText( List<? extends EventTextContainer> eventTextContainers, Locale language, boolean setUniqueEventText,
      boolean checkForEventTextNumber ) throws SQLException
  {
    Check.notNull( eventTextContainers, "event text containers" );
    Check.notNull( language, "language" );

    if ( eventTextContainers.isEmpty() )
    {
      return;
    }

    int lowestEventNumber = 999999999;
    int highestEventNumber = -999999999;
    for ( EventTextContainer container: eventTextContainers )
    {
      if ( checkForEventTextNumber && container.getEventNumber() == null )
      {
        throw new IllegalArgumentException( "Found no event number in <" + container + ">" );
      }
      if ( container.getEventNumber() != null )
      {
        int eventNumber = container.getEventNumber().intValue();
        if ( eventNumber < lowestEventNumber )
        {
          lowestEventNumber = eventNumber;
        }
        if ( eventNumber > highestEventNumber )
        {
          highestEventNumber = eventNumber;
        }
      }
    }

    Map<Integer, ArTextVO> eventTexts = new ArTextDAO().findEventTextsByEventNumber( EventTypeTextType.REGISTER_REPORT, language, lowestEventNumber,
        highestEventNumber );

    Integer previousEventTextNumber = null;
    for ( EventTextContainer container: eventTextContainers )
    {
      // This specific event has no report text
      if ( container.getEventNumber() != null && !container.getEventNumber().equals( EventNumberConstants.DESIGNER_ENTRY_INTEGER ) )
      {
        ArTextVO eventText = eventTexts.get( container.getEventNumber() );
        if ( eventText == null )
        {
          throw new IllegalArgumentException( "Found unknown event number <" + container.getEventNumber() + "> in <" + container + ">" );
        }
        if ( setUniqueEventText )
        {
          if ( !container.getEventNumber().equals( previousEventTextNumber ) )
          {
            container.setEventText( eventText.getText() );
          }
        }
        else
        {
          container.setEventText( eventText.getText() );
        }
        previousEventTextNumber = container.getEventNumber();
      }
    }
  }
}
