package ch.ige.jipps.text;

import java.sql.SQLException;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.validation.Check;

/**
 * Fetches report texts based on the report text numbers in objects and fills
 * the corresponding text into the object.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: ReportTextLoader.java,v 1.10 2008/05/06 07:37:43 lewis_lee Exp $
 */
public class ReportTextLoader
{
  /**
   * Loads the report texts and sets them in the report text containers.
   * Note that this expectes the report text number to exist.
   * @param reportTextContainers The <code>List</code> of report text containers to fill with report texts
   * @param ipType The IP type
   * @param language The language to use for the report texts
   * @param reportType The report type
   * @param reportArea The report area
   * @param setUniqueReportText If this is <code>true</code>, only the first report text with the same
   *        number is set - this implies that <code>reportTextConstainers</code> are ordered by
   *        report text numbers; otherwise all report texts are set.
   * @throws SQLException if the report texts can't be loaded
   */
  public static void insertReportText( List<? extends ReportTextContainer> reportTextContainers, IPType ipType, Locale language, ReportType reportType,
      ReportArea reportArea, boolean setUniqueReportText ) throws SQLException
  {
    insertReportText( reportTextContainers, ipType, language, reportType, reportArea, setUniqueReportText, true );
  }

  /**
   * Loads the report texts, if they exist, and sets them in the report text containers.
   * @param reportTextContainers The <code>List</code> of report text containers to fill with report texts
   * @param ipType The IP type
   * @param language The language to use for the report texts
   * @param reportType The report type
   * @param reportArea The report area
   * @param setUniqueReportText If this is <code>true</code>, only the first report text with the same
   *        number is set - this implies that <code>reportTextConstainers</code> are ordered by
   *        report text numbers; otherwise all report texts are set.
   * @throws SQLException if the report texts can't be loaded
   */
  public static void insertOptionalReportText( List<? extends ReportTextContainer> reportTextContainers, IPType ipType, Locale language, ReportType reportType,
      ReportArea reportArea, boolean setUniqueReportText ) throws SQLException
  {
    insertReportText( reportTextContainers, ipType, language, reportType, reportArea, setUniqueReportText, false );
  }

  /**
   * Loads the report texts and sets them in the report text containers.
   * @param reportTextContainers The <code>List</code> of report text containers to fill with report texts
   * @param ipType The IP type
   * @param language The language to use for the report texts
   * @param reportType The report type
   * @param reportArea The report area
   * @param setUniqueReportText If this is <code>true</code>, only the first report text with the same
   *        number is set - this implies that <code>reportTextConstainers</code> are ordered by
   *        report text numbers; otherwise all report texts are set.
   * @param checkForReportTextNumber If this is <code>true</code>, the report text number must exist. Missing
   *        report text numbers result in an <code>IllegalArgumentException</code>.
   *        If this is <code>false</code>, the report text is inserted if a report text number exists, otherwise
   *        the <code>ReportTextContainer</code> is left untouched.
   * @throws SQLException if the report texts can't be loaded
   */
  private static void insertReportText( List<? extends ReportTextContainer> reportTextContainers, IPType ipType, Locale language, ReportType reportType,
      ReportArea reportArea, boolean setUniqueReportText, boolean checkForReportTextNumber ) throws SQLException
  {
    Check.notNull( reportTextContainers, "report text containers" );
    Check.notNull( ipType, "IP type" );
    Check.notNull( language, "language" );
    Check.notNull( reportType, "report type" );
    Check.notNull( reportArea, "report area" );

    Map<Integer, RegFormTextVO> reportTexts = new RegFormTextDAO().findReportTextsAsMap( language, reportType, ipType.getBAGISType(), reportArea );
    Integer previousReportTextNumber = null;
    for ( ReportTextContainer container: reportTextContainers )
    {
      if ( checkForReportTextNumber )
      {
        if ( container.getReportTextNumber() == null )
        {
          throw new IllegalArgumentException( "Found no report text number in <" + container + ">" );
        }
      }
      else
      {
        if ( container.getReportTextNumber() == null )
        {
          continue;
        }
      }
      RegFormTextVO reportText = reportTexts.get( container.getReportTextNumber() );
      if ( reportText == null )
      {
        throw new IllegalArgumentException( "Found unknown report text number <" + container.getReportTextNumber() + "> in <" + container + ">" );
      }
      if ( setUniqueReportText )
      {
        if ( !container.getReportTextNumber().equals( previousReportTextNumber ) )
        {
          container.setReportText( reportText.getText() );
        }
      }
      else
      {
        container.setReportText( reportText.getText() );
      }
      previousReportTextNumber = container.getReportTextNumber();
    }
  }
}
