package ch.ige.jipps.text;

import java.sql.SQLException;
import java.util.Date;
import java.util.Locale;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.patent.PatentEP;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.constants.ReportTextConstants;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;

/**
 * Handles the watermark for reports.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: Watermark.java,v 1.1 2008/10/23 13:37:03 lewis_lee Exp $
 */
public class Watermark
{
  /**
   * Gets a possible watermark for a given report.
   * @param reportType The report type
   * @param ip The IP
   * @param language The report's language
   * @param effectiveStateDate The date for which the report is created
   * @return The watermark to print or {@code null} if there's none to print
   * @throws SQLException if fetching the text fails
   */
  public static String getWatermark( ReportType reportType, GenericIP ip, Locale language, Date effectiveStateDate ) throws SQLException
  {
    String watermark = null;
    IPType ipType = ip.getBAGISType();
    // The current state for designs contains in fact tentative data, so show it as a watermark.
    if ( ipType.equals( IPType.DESIGN ) && reportType.equals( ReportType.CURRENT_STATE ) )
    {
      RegFormTextVO formText = new RegFormTextDAO().findReportText( ReportTextConstants.TENTATIVE_DATA_INTEGER, language, ReportArea.DATA_STATE );
      if ( formText != null )
      {
        watermark = formText.getText();
      }
      else
      {
        throw new IllegalStateException( "Didn't find watermark text with no. " + ReportTextConstants.TENTATIVE_DATA_INTEGER + " for language " + language
            + " in area " + ReportArea.DATA_STATE );
      }
    }

    // If a patent isn't published, it gets a corresponding watermark.
    if ( ipType.isPatent() && !isPatentPublished( ip, effectiveStateDate ) )
    {
      RegFormTextVO formText = new RegFormTextDAO().findReportText( ReportTextConstants.NOT_YET_PUBLISHED, language, ReportArea.DATA_STATE, IPType.PATENT_CH );
      if ( formText != null )
      {
        watermark = formText.getText();
      }
      else
      {
        throw new IllegalStateException( "Didn't find watermark text with no. " + ReportTextConstants.NOT_YET_PUBLISHED + " for language " + language
            + " in area " + ReportArea.DATA_STATE );
      }
    }
    return watermark;
  }

  /**
   * Shows whether a patent has been published <strong>regarding the watermark</strong>.
   * @param patent The patent to check for publication
   * @param effectiveStateDate The date for which the report is created
   * @return {@code true} if the patent has been published, otherwise {@code false}
   */
  private static boolean isPatentPublished( GenericIP patent, Date effectiveStateDate )
  {
    boolean isPublished = false;
    IPType patentType = patent.getBAGISType();
    // PCTs and SPCs never get any watermark, so they are always considered published.
    if ( patentType.equals( IPType.PATENT_PCT ) || patentType.equals( IPType.SPC ) )
    {
      isPublished = true;
    }
    // A Swiss patent is published if any of its events has been published.
    else if ( patentType.equals( IPType.PATENT_CH ) )
    {
      isPublished = patent.getFirstPublication( EreigIDat.max( effectiveStateDate ) ) != null;
    }
    // If the A publication has been reached, an EP patent is published.
    else if ( patentType.equals( IPType.PATENT_EP ) )
    {
      PatentEP patentEP = (PatentEP) patent;
      isPublished = !effectiveStateDate.before( patentEP.getPublicationDateA() );
    }
    return isPublished;
  }
}
