package ch.ige.jipps.xml;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

import org.apache.commons.lang.StringUtils;

import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.BFormTxtDruckVO;
import ch.ige.jipps.vo.BfakTxtVO;
import ch.ige.jipps.vo.FormTextVO;
import ch.ige.jipps.vo.JippsFaktPosVO;
import ch.ige.jipps.vo.JippsFakturVO;
import ch.ige.jipps.vo.LetterVO;
import ch.ige.jipps.xml.letter.Adresse;
import ch.ige.jipps.xml.letter.Bfaktpos;
import ch.ige.jipps.xml.letter.Igekopf;
import ch.ige.jipps.xml.letter.Letter;
import ch.ige.jipps.xml.letter.ObjectFactory;
import ch.ige.util.address.IpiAddress;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.BAGISLanguage;

/**
 * Creates the XML document for a letter.
 * <p>
 * Copyright (c) 2003-2006 Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author Priska Steiner
 * @author Marc Bouquet
 * @author Esther Bandschapp
 * @version $Id: LetterWriteXml.java,v 1.8 2008/07/23 15:19:37 lewis_lee Exp $
 */
public class LetterWriteXml
{
  private ObjectFactory objectFactory;

  private static JAXBContext jaxbContext;

  private static Marshaller marshaller;

  private Letter letter;

  /**
   * The signature part <code>Best regards</code> in different languages.
   */
  private static final Map<BAGISLanguage, List<String>> BEST_REGARDS;

  static
  {
    //TODO One day, this should be standardized and saved in the database
    BEST_REGARDS = new HashMap<BAGISLanguage, List<String>>();
    BEST_REGARDS.put( BAGISLanguage.DE, Arrays.asList( "Mit freundlichen Grüssen" ) );
    BEST_REGARDS.put( BAGISLanguage.FR, Arrays.asList( "Avec nos meilleures salutations", "Avec nos salutations les meilleures",
        "Avec nos salutations distinguées", "Veuillez agréer, Madame, Monsieur, l'expression de nos salutations distinguées",
        "Dans l'intervalle, veuillez agréer, Madame, Monsieur, l'assurance de notre considération distinguée",
        "En vous remerciant de votre commande, nous vous prions d'agréer, Madame, Monsieur, l'expression de nos salutations distinguées" ) );
    BEST_REGARDS.put( BAGISLanguage.IT, Arrays.asList( "Distinti saluti", "Con la massima stima" ) );
  }

  /**
   * Creates an XML document from the letter.
   * 
   * @param letterVO A value object containing the data for the letter
   * @return A <code>String</code> containing the XML document
   * @throws JAXBException
   */
  public String toXml( LetterVO letterVO ) throws JAXBException
  {
    if ( this.objectFactory == null )
    {
      jaxbContext = JAXBContext.newInstance( "ch.ige.jipps.xml.letter" );
      this.objectFactory = new ObjectFactory();
      marshaller = jaxbContext.createMarshaller();
    }

    setDefaults( letterVO.jippsFakturVO );

    this.letter = this.objectFactory.createLetter();

    // Igekopf füllen
    Igekopf igekopf = writeIgekopf( letterVO );
    this.letter.setIgekopf( igekopf );

    // Adresse füllen
    Adresse adresse = writeAdresse( letterVO );
    this.letter.setAdresse( adresse );

    // Bfaktpositionen zum XML hinzufügen
    writeBfaktPositionen( letterVO );

    // Einzelfelder fürs XML holen
    Locale language = BAGISLanguage.parse( letterVO.jippsFakturVO.sprachCd.trim() ).toLocale();
    String formattedInvoiceDate = DateFormatter.longFormat( letterVO.jippsFakturVO.bfaktDat, language );
    this.letter.setBfaktDat( formattedInvoiceDate );

    if ( letterVO.jippsFakturVO.txtNrBeilage > 0 )
    {
      for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
      {
        if ( bfaktxt.textNr == letterVO.jippsFakturVO.txtNrBeilage )
        {
          this.letter.setTxtBeilage( ConvertText.replaceHtmlTags( bfaktxt.text ) );
        }
      }
    }

    // Formtexte holen und abfüllen; Werte aus Bfaktur auch dazu holen
    writeFormTexte( letterVO );

    // BformTexte holen und abfüllen
    writeBformTexte( letterVO );

    return XmlMarshaller.toString( jaxbContext, marshaller, "letter.xsd", this.letter );
  }

  /**
   * Sets defaults for the XML document, changing <code>null</code> values to an empty String.
   * 
   * @param jippsFakturVO The invoice to default
   */
  private void setDefaults( JippsFakturVO jippsFakturVO )
  {
    // TODO If some of these are optional, then they should be optional in the XML schema
    jippsFakturVO.untName = StringUtils.defaultString( jippsFakturVO.untName );
    jippsFakturVO.ihrZeichen = StringUtils.defaultString( jippsFakturVO.ihrZeichen );
    jippsFakturVO.unsereRef = StringUtils.defaultString( jippsFakturVO.unsereRef );
    jippsFakturVO.telNr = StringUtils.defaultString( jippsFakturVO.telNr );
    jippsFakturVO.kzEinschr = StringUtils.defaultString( jippsFakturVO.kzEinschr );
    jippsFakturVO.kzKorrektur = StringUtils.defaultString( jippsFakturVO.kzKorrektur );
  }

  /**
   * Creates the IGE header
   * 
   * @param letterVO Containing the data which is needed for the job
   * @return The IGE header as a <code>Igekopf object</code>
   * @throws JAXBException
   */
  private Igekopf writeIgekopf( LetterVO letterVO ) throws JAXBException
  {
    Igekopf igekopf = this.objectFactory.createIgekopf();

    if ( !letterVO.printIgekopf )
    {
      igekopf.setIgekopfDe( "" );
      igekopf.setIgekopfFr( "" );
      igekopf.setIgekopfIt( "" );
      igekopf.setIgekopfEn( "" );
      igekopf.setIgekopfAdr( "" );
      igekopf.setIgekopfKto( "" );
    }
    else
    {
      igekopf.setIgekopfDe( IpiAddress.getGermanName() );
      igekopf.setIgekopfFr( IpiAddress.getFrenchName() );
      igekopf.setIgekopfIt( IpiAddress.getItalianName() );
      igekopf.setIgekopfEn( IpiAddress.getEnglishName() );
      igekopf.setIgekopfAdr( IpiAddress.getFullGermanAddress() );
      igekopf.setIgekopfKto( "" );
    }
    return igekopf;
  }

  /**
   * Creates the address
   * 
   * @param letterVO Containing the data which is needed for the job
   * @return The address in an <code>Adresse object</code>
   * @throws JAXBException
   */
  private Adresse writeAdresse( LetterVO letterVO ) throws JAXBException
  {
    Adresse adresse = this.objectFactory.createAdresse();
    adresse.setName( ConvertText.replaceHtmlTags( letterVO.versadrVO.getNameOrEmptyStr() ) );
    adresse.setZeile1( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile1OrEmptyStr() ) );
    adresse.setZeile2( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile2OrEmptyStr() ) );
    adresse.setZeile3( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile3OrEmptyStr() ) );
    adresse.setZeile4( ConvertText.replaceHtmlTags( letterVO.versadrVO.getZeile4OrEmptyStr() ) );
    adresse.setPlzOrt( getPlzOrt( letterVO.versadrVO.getLandCdOrEmptyStr(), letterVO.versadrVO.getPlzOrEmptyStr(), letterVO.versadrVO.getOrtOrEmptyStr() ) );

    if ( letterVO.jippsFakturVO.txtAdrZusatz > 0 )
    {
      for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
      {
        if ( bfaktxt.textNr == letterVO.jippsFakturVO.txtAdrZusatz )
        {
          adresse.setTxtAdrZusatz( ConvertText.replaceHtmlTags( bfaktxt.text ) );
        }
      }
    }
    return adresse;
  }

  /**
   * Matches the post code (PLZ) and the place (Ort) with the country code <strong>NOTE: Changings of this method
   * require also changings in the class HinterlegungWriteXml</strong>
   * 
   * @param landCd The country code (CH = Switzerland, AUS = Australia)
   * @param plz The post code
   * @param ort The place
   * @return A <code>String</code> with the combination of country, postcode and place
   */
  private String getPlzOrt( String landCd, String plz, String ort )
  {
    if ( landCd.trim().equalsIgnoreCase( "CH" ) )
    {
      return plz + " " + ort;
    }
    else
    {
      return landCd + " - " + plz + " " + ort;
    }
  }

  /**
   * Writes the <code>BfakturaPositionen</code> into a list for the XML
   * 
   * @param letterVO Containing the data which is needed for the job
   * @throws JAXBException
   */
  @SuppressWarnings( "unchecked" )
  private void writeBfaktPositionen( LetterVO letterVO ) throws JAXBException
  {
    for ( JippsFaktPosVO posVO: letterVO.jippsFaktPositionen )
    {
      Bfaktpos pos = this.objectFactory.createBfaktpos();
      if ( posVO.invoiceLineItem.textNumber > 0 )
      {
        for ( BfakTxtVO bfaktxt: letterVO.bfakTexte )
        {
          if ( bfaktxt.textNr == posVO.invoiceLineItem.textNumber )
          {
            pos.setTxtEreignisVar( ConvertText.replaceHtmlTags( bfaktxt.text ) );
          }
        }
      }
      this.letter.getBfaktpos().add( pos );
    }
  }

  /**
   * Schreibt die Formtexte ins XML.
   * 
   * @param letterVO enthält alle für ein Schreiben benötigten VO's
   */
  private void writeFormTexte( LetterVO letterVO )
  {
    for ( FormTextVO formtext: letterVO.formTexte )
    {
      // TODO Use enums
      int zeile = 0;
      if ( formtext.lineNumber != null )
      {
        zeile = formtext.lineNumber.intValue();
      }
      switch( zeile )
      {
        case 10:
        {
          if ( letterVO.jippsFakturVO.kzKorrektur.equals( "1" ) )
          {
            this.letter.setTxtKorrektur( ConvertText.replaceHtmlTags( formtext.text ) );
          }
          break;
        }
        case 30:
        {
          if ( letterVO.jippsFakturVO.kzKorrektur.equals( "2" ) )
          {
            this.letter.setTxtKorrektur( ConvertText.replaceHtmlTags( formtext.text ) );
          }
          break;
        }
        case 35:
        {
          this.letter.setLblUnsereRef( ConvertText.replaceHtmlTags( formtext.text ) );
          this.letter.setTxtUnsereRef( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.unsereRef ) );
          break;
        }
        case 40:
        {
          if ( letterVO.jippsFakturVO.kzEinschr.equals( "1" ) )
          {
            this.letter.setTxtEinschreiben( ConvertText.replaceHtmlTags( formtext.text ) );
          }
          break;
        }
        case 55:
        {
          this.letter.setLblTelNr( ConvertText.replaceHtmlTags( formtext.text + " " ) );
          this.letter.setTxtTelNr( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.telNr ) );
          break;
        }
        case 80:
        {
          this.letter.setLblIhrZeichen( ConvertText.replaceHtmlTags( formtext.text ) );
          this.letter.setTxtIhrZeichen( ConvertText.replaceHtmlTags( letterVO.jippsFakturVO.ihrZeichen ) );
          break;
        }
        case 160:
        {
          this.letter.setTxtSeite( ConvertText.replaceHtmlTags( formtext.text ) );
          break;
        }
        default:
          break;
      }
    }
  }

  /**
   * Schreibt die BFormTexte ins XML
   * 
   * @param letterVO enthält alle für ein Schreiben benötigten VO's
   */
  @SuppressWarnings( "unchecked" )
  private void writeBformTexte( LetterVO letterVO )
  {
    for ( BFormTxtDruckVO bformText: letterVO.bformTexte )
    {
      switch( bformText.textType )
      {
        case 0:
        {
          this.letter.setTxtAbteilung( ConvertText.replaceHtmlTags( bformText.text ) );
          break;
        }
        case 1:
        {
          this.letter.setTxtBetreff( ConvertText.replaceHtmlTags( bformText.text ) );
          break;
        }
        case 2:
        {
          this.letter.setTxtKopf( ConvertText.replaceHtmlTags( bformText.text ) );
          break;
        }
        case 3:
        {
          // There was a problem with the signature: It should be printed together on a page and not be
          // separated by the pagination. The problem was in the length of the signature, since it mostly
          // doesn't only contain "Best regards, ... <name>", but also a lot of other text.
          // That's why in the XSL-FO the 'keep-together' is not set to 'always' if the text is longer than a
          // certain length.
          // So the only way to solve the problem is to introduce another signature node which only contains
          // the signature itself. Thus, every txtSchluss node can be kept together separately by XSL-FO.
          List<String> bestRegards = BEST_REGARDS.get( bformText.language );
          if ( bestRegards == null )
          {
            throw new IllegalStateException( "Can't find language " + bformText.language + " for signature" );
          }
          int signaturePosition = -1;
          for ( String bestRegardsCase: bestRegards )
          {
            signaturePosition = bformText.text.indexOf( bestRegardsCase );
            if ( signaturePosition > -1 )
            {
              break;
            }
          }

          if ( signaturePosition == -1 )
          {
            this.letter.getTxtSchluss().add( ConvertText.replaceHtmlTags( bformText.text ) );
          }
          else
          {
            this.letter.getTxtSchluss().add( ConvertText.replaceHtmlTags( bformText.text.substring( 0, signaturePosition ) ) );
            this.letter.getTxtSchluss().add( ConvertText.replaceHtmlTags( bformText.text.substring( signaturePosition ) ) );
          }
          break;
        }
        default:
          break;
      }
    }
  }
}
