package ch.ige.pctedi.client;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

/**
 * Zipfile einer Verarbeitungswoche welches Pct Daten beinhaltet
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class PctZipFile extends File
{
  /**
   * Default id.
   */
  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger( PctZipFile.class );

  private File unzipDirectory = null;

  
  /**
   * constructor
   * @param pathname
   */
  public PctZipFile( String pathname ) 
  {
    super( pathname );
    init();
  }

  /**
   * constructor
   * @param parent
   * @param child
   */
  public PctZipFile( String parent, String child ) 
  {
    super( parent, child );
    init();    
  }  
  
  /**
   * initialization   
   * reads path of unzip directory out of properties and clean up it
   * writes information about processfile to logfile
   */ 
  private void init() 
  {
    setUnzipDirectory();
    logFileInformation();
  }
  
  
  /**
   * writes size and modified datetime of the processfile to logfile  
   */
  public void logFileInformation()
  {
    LOG.debug( "Size in bytes: " +  length() );
    
    Calendar cal = Calendar.getInstance();
    cal.setTimeInMillis( lastModified() ) ;
    DateFormat df = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
    String lastModifiedTime = df.format(cal.getTime());
      
    LOG.debug( "last modified time: " + lastModifiedTime);   
  }
  
  
  /**   
   * reads path of unzip directory out of properties and make or clean up it
   */  
  private void setUnzipDirectory() 
  {
    unzipDirectory = new File( PctProperties.getInstance().getProperty( PctProperties.UNZIP_DIRECTORY ) );
    deleteUnzipDirecotry();
    unzipDirectory.mkdirs();
  }

  
  /**
   * @return path of unzip directory
   */
  public String getUnzipDirectory()
  {
    return unzipDirectory.getPath();
  }
  
  
  /**
   * Ermittelt die Verarbeitungswoche anhand des Filename (6-stellige Zahl)
   * @return String Verarbeitungswoche
   */
  public String extractProcessWeek()
  {
    Pattern p = Pattern.compile( "[0-9]{6}" );
    Matcher m = p.matcher( this.getName() );

    m.find();
    return m.group();
  }  
  
 
 /**
  * unpack content of processing file to unzip directory with external tool PKZIP
  * @return path of unzip directory
  * @throws Exception unexpected code in unpacking process
  */
  public String unzip() throws Exception
  {
    LOG.debug( "Extracting file: " + getPath() + " to " + getUnzipDirectory());
      
    StringBuffer command  = new StringBuffer();
    command.append( "c:\\programme\\pkware\\pkzipc\\pkzipc -ext -dir -silent ");
    command.append( getPath() );
    command.append( " " );
    command.append( getUnzipDirectory() );
    command.append( " *.xml " );
    
    LOG.debug( "execute command : " + command.toString() );
  
    // Entpackt alle xml-dateien in subdirectory decomp 
    Process process = Runtime.getRuntime().exec(command.toString());   
   
    // wait until process is finished 
    int code = process.waitFor();

    if (code != 0 )
    {
      throw new Exception("unpacking process terminated with unexpected code: " + code );
    }    
    
    return getUnzipDirectory();
  }


  /**
   * pack xml files to tar.gz with external tool PKZIP
   * @throws Exception unexpected code in packing process
   */
  public void packAndMoveToDone() throws Exception
  {
    String line;   

    StringBuffer command  = new StringBuffer();
    command.append( "c:\\programme\\pkware\\pkzipc\\pkzipc -add -dir -silent ");
    command.append( createDoneFile().getPath() );
    command.append( " *.xml " );
    
    LOG.debug( "execute command : " + command.toString() );
    
    //  packt alle xml-dateien in subdirectory done
    Runtime runtime = Runtime.getRuntime();
    Process process = runtime.exec(command.toString(), null, unzipDirectory);   
    int code = process.waitFor();
    
    if (code != 0 )
    {
      throw new Exception("packing process terminated with unexpected code: " + code );
    }
    
    BufferedReader input = new BufferedReader( new InputStreamReader( process.getInputStream()));
    while ((line = input.readLine()) != null) 
    {
      LOG.info( line );
    }
    
    input.close();    
   
    LOG.debug( "pack done " );
  }
  
  /**
   * clean up unzip directory (recursive)
   */
  public void deleteUnzipDirecotry()
  {
    StringBuffer command = new StringBuffer();
    command.append( "cmd.exe /c rd /S/Q " );
    command.append( getUnzipDirectory() );

    LOG.debug( "execute command : " + command.toString() );

    try
    {
      Runtime.getRuntime().exec( command.toString() );
    }
    catch ( IOException e )
    {
      LOG.error( "Exception in deleteUnzipDirecotry: " + e.getMessage(), e );
    }
    
  }
  
   /**
   * Verschiebt Zipfile in Unterverzeichnis done
   */
  private File createDoneFile()
  {
    File doneDirectory = new File( this.getParent(), "done\\" );
    doneDirectory.mkdirs();

    File doneFile = new File( doneDirectory.getPath(), this.getName() );
    if (doneFile.exists())
    {
      doneFile.delete();
    }

    return doneFile;
  }
}
