package ch.ige.pctedi.server;

import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.ejb.CreateException;

import ch.ige.pctedi.vo.IpcVO;
import ch.ige.pctedi.vo.PctVO;
import ch.ige.util.ServiceLocator;
import ch.ige.util.exception.ServiceLocatorException;

/**
 * BusinessDelegate für PCT
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Martin Trachsel, Daniel Haller Strahm
 * @version $Id: PctDelegate.java,v 1.17 2008/07/01 07:43:35 kuenzli Exp $
 */

public class PctDelegate
{
  private static PctDelegate pctDelegate;

  private PctFacade facade;

  @SuppressWarnings( "unchecked" )
  private static final Class HOME_CLASS = PctFacadeHome.class;

  // Singleton
  @SuppressWarnings( "unchecked" )
  private PctDelegate() throws PctException
  {
    try
    {
      PctFacadeHome home = (PctFacadeHome) ServiceLocator.getInstance().getHome( PctFacadeHome.JNDI_NAME, HOME_CLASS );
      facade = home.create();
    }
    catch ( ServiceLocatorException ex )
    {
      // ServiceLocaterException in Applikations-Exception umwandeln
      throw new PctException( ex.toString() );
    }
    catch ( CreateException ex )
    {
      // CreateException in Applikations-Exception umwandeln
      throw new PctException( ex.toString() );
    }
    catch ( RemoteException ex )
    {
      // RemoteException in Applikations-Exception umwandeln
      throw new PctException( ex.toString() );
    }

  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * @return PctDelegate
   *  @throws PctException
   */
  public static PctDelegate getInstance() throws PctException
  {
    if ( pctDelegate == null )
    {
      pctDelegate = new PctDelegate();
    }
    return pctDelegate;
  }

  /**
   * Kreiert ein Pct PCT.
   *
   * @param pctVO Value Object der Pct
   * @throws PctException
   */
  public void createPct( final PctVO pctVO ) throws PctException
  {
    try
    {
      facade.createPct( pctVO );
    }
    catch ( Exception ex )
    {
      // TODO Method toString auf Verschachtelungen überprüfen
      throw new PctException( ex.toString() );
    }
  }

  /**
   * Testet, ob das PCT bereits vorhanden ist.
   * @param schutitNr welche getestet wird
   * @return int SchutitKey
   * @throws PctException
   */
  public int selectSchutitKey( final String schutitNr ) throws PctException
  {
    try
    {
      return facade.selectSchutitKey( schutitNr );
    }
    catch ( Exception ex )
    {
      throw new PctException( ex.toString() );
    }
  }

  /**
   * Sagt aus, ob der Titel auf der Datenbank gefunden wurde. 
   * Die Suche geht über Schutztitelnummer und Gesuch_nummer. Je nachdem, wie der Titel gefunden wurde, 
   * gebe ich ein anderes Resultat zurück. 
   * 
   * @param pctVO Werte des pct. Mindestens die Felder schutitNr und gesuchNr müssen gefüllt sein
   * @return  int 0: kein Schutztitel vorhanden
   *              1: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben das selbe Resultat.
   *              2: Schutztitel vorhanden; nur Suche über schutitNr ergab ein Resultat.
   *              3: Schutztitel vorhanden; nur Suche über GesuchNr ergab ein Resultat.
   *              4: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben unterschiedliche Resultate.
   * @exception PctException
   */
  public int findPct( PctVO pctVO ) throws PctException
  {
    try
    {
      return facade.findPct( pctVO );
    }
    catch ( Exception ex )
    {
      throw new PctException( ex.toString() );
    }
  }

  /**
   * Holt alle BLänder.
   *
   * @return blandWork  ArrayList
   * @throws PctException
   */
  @SuppressWarnings( "unchecked" )
  public List<String> getAllBland() throws PctException
  {
    try
    {
      return facade.getAllBland();
    }
    catch ( Exception ex )
    {
      throw new PctException( ex.toString() );
    }
  }

  /**
   * Holt alle Mappings uniode iso8859
   * @return Map
   * @throws Exception
   */
  @SuppressWarnings( "unchecked" )
  public Map<String, String> getAllMappingUnicode() throws Exception
  {
    return facade.getAllMappingUnicode();
  }

  /**
   * Holt einen IPC-Eintrag
   * @param ipcSymb String IPC-Symbol
   * @param gueltigVon Date
   * @throws Exception
   * @return IpcVO IPC-Eintrag
   */
  public IpcVO getIpc( final String ipcSymb, final Date gueltigVon ) throws Exception
  {
    return facade.getIpc( ipcSymb, gueltigVon );
  }

  /**
   * Holt den jüngsten IPC-Eintrag
   * @param ipcSymb String IPC-Symbol
   * @throws Exception
   * @return IpcVO IPC-Eintrag
   */
  public IpcVO getYoungestIpc( final String ipcSymb ) throws Exception
  {
    return facade.getYoungestIpc( ipcSymb );
  }

  /**
   * erstellt Ereignis 30040 Verlängerung und gibt die Meldung zurück, was gemacht wurde
   * @param schutitKey
   * @param publDat
   * @throws Exception
   * @return String Meldung der Aktion
   * 
   */
  public String createExtension( final int schutitKey, final java.util.Date publDat ) throws Exception
  {
    return facade.createExtension( schutitKey, publDat );
  }

}
