package ch.ige.st36xmlgenerator.format;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.WordUtils;

/**
 * Contains utility methods for converting element names from the ch-st.36 XML
 * to class class names.
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Patrizia Fabrizioli
 * @version   $Id: XMLElementConverter.java,v 1.4 2008/09/25 07:29:27 fabrizioli Exp $
 */
public class XMLElementConverter
{
  /**
   * This usually separates words in database column names, such as
   * <code>customer_name</code>
   */
  private static final char[] ELEMENT_NAME_SEPARATOR = new char[] { '-' };

  /**
   * The prefix for a getter method
   */
  private static final String GETTER_PREFIX = "get";

  /**
   * The prefix for a setter method
   */
  private static final String SETTER_PREFIX = "set";

  /**
   * Converts a element name to its corresponding class name, if a class name mapping to a type occurs.
   * For example, <code>designation-code</code> is converted to <code>DesignationCodeType</code>.
   * @param elementName The element name
   * @return The class name
   * or an empty String if <code>elementName</code> is <code>null</code> or empty
   */
  public static String toClassName( String elementName )
  {
    if ( StringUtils.isEmpty( elementName ) )
    {
      return "";
    }
    return toCamelCase( elementName ) + "Type";
  }

  /**
   * Converts a element name to its corresponding class name, if a class name mapping to a reference type occurs.
   * For example, <code>designation-code</code> is converted to <code>DesignationCodeTypeRef</code>.
   * @param elementName The element name
   * @return The class name
   * or an empty String if <code>elementName</code> is <code>null</code> or empty
   */
  public static String toRefClassName( String elementName )
  {
    if ( StringUtils.isEmpty( elementName ) )
    {
      return "";
    }
    return toCamelCase( elementName ) + "TypeRef";
  }

  /**
   * Converts an element name to its corresponding class name, if a one-to-one class name mapping occurs
   * For example, <code>designation-code</code> is converted to <code>DesignationCode</code>.
   * @param elementName The element name
   * @return The class name
   * or an empty String if <code>elementName</code> is <code>null</code> or empty
   */
  public static String toSimpleClassName( String elementName )
  {
    if ( StringUtils.isEmpty( elementName ) )
    {
      return "";
    }
    return toCamelCase( elementName );
  }

  /**
   * Converts an element name  to its corresponding getter name.
   * For example, <code>customer-name</code> is converted to <code>getCustomerName</code>.
   * @param elementName The element name, which is normally separated by <code>-</code>.
   * @return the getter name
   * or an empty String if <code>elementName</code> is <code>null</code> or empty
   */
  public static String toGetter( String elementName )
  {
    if ( StringUtils.isEmpty( elementName ) )
    {
      return "";
    }
    return GETTER_PREFIX + toCamelCase( elementName );
  }

  /**
   * Converts an element name  to its corresponding setter name.
   * For example, <code>customer-name</code> is converted to <code>getCustomerName</code>.
   * @param elementName The element name, which is normally separated by <code>-</code>.
   * @return the setter name
   * or an empty String if <code>elementName</code> is <code>null</code> or empty
   */
  public static String toSetter( String elementName )
  {
    if ( StringUtils.isEmpty( elementName ) )
    {
      return "";
    }
    return SETTER_PREFIX + toCamelCase( elementName );
  }

  /**
   * Converts the column name to camel case, starting with an uppercase character.
   * For example, <code>customer_name</code> is converted to <code>CustomerName</code>.
   * @param columnName The database column name, which is normally separated by <code>_</code>.
   * @return The database column name as camel case String
   * or an empty String if <code>columnName</code> is <code>null</code> or empty
   */
  private static String toCamelCase( String columnName )
  {
    String memberVariable = WordUtils.capitalizeFully( columnName, ELEMENT_NAME_SEPARATOR );
    return StringUtils.remove( memberVariable, ELEMENT_NAME_SEPARATOR[0] );
  }

 
  /**
   * Get the Root Class name, which maps differently in JAXB from the other elements
   * @param rootElement name
   * @return root element class name
   */
  public static String toRootClassName( String rootElement )
  {
    if ( StringUtils.isEmpty( rootElement ) )
    {
      return "";
    }
    return toCamelCase( rootElement );
  }

}
