package ch.ige.st36xmlgenerator.metadata;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.iso_relax.verifier.Schema;

import ch.ige.ipobjects.patent.vo.PatentDocumentSchemaVO;
import ch.ige.st36xmlgenerator.metadata.XPathXMLAccessor.AttributeDefinition;
import ch.ige.xmlutil.doc.DocumentType;
import ch.ige.xmlutil.doc.DocumentTypes;
import ch.ige.xmlutil.doc.Validator;
import ch.ige.xmlutil.doc.XMLDocument;

/**
 * The extractor of the metadata for the generation of the XML in a specified schema version
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author fabrizioli
 * @version $Id: JAXBXMLGeneratorMetadataExtractor.java,v 1.11 2008/09/25 07:29:27 fabrizioli Exp $
 */
public class JAXBXMLGeneratorMetadataExtractor implements XMLGeneratorMetadataExtractor
{

  private static final Logger LOG = Logger.getLogger( JAXBXMLGeneratorMetadataExtractor.class );

  private PatentDocumentSchemaVO schema;

  private Schema verifierSchema;

  private Map<AccessorDescription, XMLAccessorContainer> cachedAccessors = new HashMap<AccessorDescription, XMLAccessorContainer>( 32 );

  private Document parsedMetadataDescription;

  private Map<Integer, String> accessorDescriptorKeys;

  /**
   * @param schema
   */
  public JAXBXMLGeneratorMetadataExtractor( PatentDocumentSchemaVO schema )
  {
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "[TRACE] create new extractor for " + schema.getId_string() );
    }
    this.schema = schema;
    this.ensureCorrectMetadataVersion();
    this.registerAccessorDescriptorKeys();
  }


  public Integer getSchemaIdentifier()
  {
    return this.schema.getPatdocschema_id();
  }


  public String getStringSchemaIdentifier()
  {
    return this.schema.getId_string();
  }


  public Schema getVerifierSchema()
  {
    if ( this.verifierSchema == null )
    {
      try
      {
        DocumentType schemaFromCatalog = DocumentTypes.getPatdocTypesBySchemaName().get( this.getStringSchemaIdentifier() );
        if ( schemaFromCatalog == null )
        {
          throw new RuntimeException( "XSD Schema does not exist in catalog " + this.getStringSchemaIdentifier() );
        }
        this.verifierSchema = Validator.compileSchema( schemaFromCatalog );
      }
      catch ( Exception e )
      {
        throw new RuntimeException( "XSD Schema can not be compiled", e );
      }
    }
    return this.verifierSchema;
  }

  public XMLAccessorContainer getXMLAccessors( Integer inidCode, String kindcode ) throws JAXBMetadataException
  {

    AccessorDescription accessorDescription = getAccessorForInidCode( inidCode, kindcode );
    XMLAccessorContainer xmlaccessors = cachedAccessors.get( accessorDescription );
    if ( xmlaccessors == null )
    {
      xmlaccessors = accessorDescription.getXMLAccessors( this.getParsedMetadataDescription() );
      if ( xmlaccessors != null )
      {
        this.cachedAccessors.put( accessorDescription, xmlaccessors );
      }

    }
    return xmlaccessors;
  }

  public Set<Integer> getAllInidCodes()
  {
    return accessorDescriptorKeys.keySet();
  }

  private AccessorDescription getAccessorForInidCode( int inidCode, String kindcode )
  {
    String accessorKey = accessorDescriptorKeys.get( inidCode );
    if ( accessorKey == null )
    {
      throw new IllegalArgumentException( "Inid Code " + inidCode + " is not valid for the generation of the bibliographic data with " + schema.getId_string() );
    }
    return deriveAccessorDescription( accessorKey, kindcode );
  }

  private void registerAccessorDescriptorKeys()
  {
    accessorDescriptorKeys = new HashMap<Integer, String>( 20 );
    accessorDescriptorKeys.put( 11, AccessorDescription.INFO_KEY_INID11 );
    accessorDescriptorKeys.put( 12, AccessorDescription.INFO_KEY_INID12 );
    accessorDescriptorKeys.put( 21, AccessorDescription.INFO_KEY_INID21 );
    accessorDescriptorKeys.put( 22, AccessorDescription.INFO_KEY_INID22 );
    accessorDescriptorKeys.put( 24, AccessorDescription.INFO_KEY_INID24 );
    accessorDescriptorKeys.put( 30, AccessorDescription.INFO_KEY_INID30 );
    accessorDescriptorKeys.put( 43, AccessorDescription.INFO_KEY_INID43 );
    accessorDescriptorKeys.put( 45, AccessorDescription.INFO_KEY_INID45 );
    accessorDescriptorKeys.put( 51, AccessorDescription.INFO_KEY_INID51 );
    accessorDescriptorKeys.put( 54, AccessorDescription.INFO_KEY_INID54 );
    accessorDescriptorKeys.put( 60, AccessorDescription.INFO_KEY_INID60 );
    accessorDescriptorKeys.put( 62, AccessorDescription.INFO_KEY_INID62 );
    accessorDescriptorKeys.put( 71, AccessorDescription.INFO_KEY_INID71 );
    accessorDescriptorKeys.put( 172, AccessorDescription.INFO_KEY_INID72W );
    accessorDescriptorKeys.put( 72, AccessorDescription.INFO_KEY_INID72 );
    accessorDescriptorKeys.put( 73, AccessorDescription.INFO_KEY_INID73 );
    accessorDescriptorKeys.put( 74, AccessorDescription.INFO_KEY_INID74 );
    accessorDescriptorKeys.put( 86, AccessorDescription.INFO_KEY_INID86 );
    accessorDescriptorKeys.put( 87, AccessorDescription.INFO_KEY_INID87 );
    accessorDescriptorKeys.put( 88, AccessorDescription.INFO_KEY_INID88 );
  }

  private Document getParsedMetadataDescription()
  {
    if ( this.parsedMetadataDescription == null )
    {
      try
      {
        LOG.debug( "[TRACE] parse xml to DOM" );
        String schemaId = schema.getId_string().substring( 0, schema.getId_string().length() - 4 );
        InputStream xmlStream = this.getClass().getResourceAsStream( "/res/xmlgenerator_metadata_" + schemaId + ".xml" );
        XMLDocument xmlgenMetadata = XMLDocument.fromUTF8InputStream( DocumentTypes.GENERIC, xmlStream );
        {
          this.parsedMetadataDescription = DocumentHelper.parseText( xmlgenMetadata.getContent().toString() );
        }
      }
      catch ( Exception e )
      {
        String msg = "Failed to parse metadata description for " + this.schema.getId_string();
        LOG.error( msg, e );
        throw new JAXBMetadataException( msg, e );
      }
    }
    return this.parsedMetadataDescription;
  }

  private void ensureCorrectMetadataVersion()
  {
    Document metadata = this.getParsedMetadataDescription();
    if ( metadata != null )
    {
      Node metadataAbout = metadata.selectSingleNode( "/metadata/@dtd-version" );
      if ( metadataAbout == null )
      {
        throw new JAXBMetadataException( "no dtd-version found in metatdata description." );
      }
      else if ( !this.schema.getId_string().equals( metadataAbout.getText() ) )
      {
        throw new JAXBMetadataException( "metatdata description is about " + metadataAbout.getText() + " but the schema is " + this.schema.getId_string() );
      }
    }
  }

  // ---------------------------------------------------------------------------
  // Access strategies

  private AccessorDescription deriveAccessorDescription( String key, String kindCode )
  {
    return new KindDiscriminatingAccessorDescription( key, kindCode );
  }

/**
 * The description of an accessor in the metadata file
 * @author fabrizioli
 *
 */
  private static interface AccessorDescription
  {

    public static final String INFO_KEY_INID11 = "inid11";

    public static final String INFO_KEY_INID12 = "inid12";

    public static final String INFO_KEY_INID88 = "inid88";

    public static final String INFO_KEY_INID87 = "inid87";

    public static final String INFO_KEY_INID86 = "inid86";

    public static final String INFO_KEY_INID74 = "inid74";

    public static final String INFO_KEY_INID73 = "inid73";

    public static final String INFO_KEY_INID72 = "inid72";

    public static final String INFO_KEY_INID71 = "inid71";

    public static final String INFO_KEY_INID72W = "inid72W";

    public static final String INFO_KEY_INID62 = "inid62";

    public static final String INFO_KEY_INID60 = "inid60";

    public static final String INFO_KEY_INID21 = "inid21";

    public static final String INFO_KEY_INID22 = "inid22";

    public static final String INFO_KEY_INID24 = "inid24";

    public static final String INFO_KEY_INID30 = "inid30";

    public static final String INFO_KEY_INID43 = "inid43";

    public static final String INFO_KEY_INID45 = "inid45";

    public static final String INFO_KEY_INID51 = "inid51";

    public static final String INFO_KEY_INID54 = "inid54";

    public XMLAccessorContainer getXMLAccessors( Document metadataDescription );

  } // inner-interface

  private static final class KindDiscriminatingAccessorDescription implements AccessorDescription
  {

    /**
     * This is the registry of all accessors
     */
    private static final List<XMLAccessorContainer> CACHED_XPATHS = new ArrayList<XMLAccessorContainer>( 52 );

    private String key;

    private String kindcode;

    public KindDiscriminatingAccessorDescription( String key, String kindcode )
    {
      this.key = key.toLowerCase();
      this.kindcode = ( kindcode == null ) ? null : kindcode.toUpperCase();
    }

    // @see ch.ige.patdocutil.SchemaBasedMetadataExtractor.AccessorDescription#getXPaths(org.dom4j.Document)
    @SuppressWarnings( "unchecked" )
    public XMLAccessorContainer getXMLAccessors( Document metadataDescription )
    {

      List<? extends Node> selectedPaths = metadataDescription.selectNodes( this.getQueryForSpecificXPaths() ); // unchecked
      // from
      // dom4j

      List<? extends Node> paths = ( this.kindcode == null ) ? Collections.<Node>emptyList() : selectedPaths;
      if ( paths.isEmpty() )
      {
        paths = metadataDescription.selectNodes( this.getQueryForGenericXPaths() ); // unchecked from dom4j
      }
      if ( paths.isEmpty() )
      {
        return null;
      }
      Node mandatoryNode = (Node) metadataDescription.selectNodes( this.getQueryForMandatory() ).get( 0 );
      List<Node> listElementNodes = metadataDescription.selectNodes( this.getQueryForListElement() );
      XMLAccessorContainer result = new XMLAccessorContainer( paths.size() );
      result.setMandatory( mandatoryNode.getText().equals( "mandatory" ) );
      boolean first = true;
      Map<String, List<AttributeDefinition>> attributeListMap = createAttributes( metadataDescription );
      for ( Iterator<? extends Node> it = paths.iterator(); it.hasNext(); )
      {

        String xpath = ( (Node) it.next() ).getText().trim();
        // JXPathXMLAccessor Container for more information if necessary

        result.add( XPathXMLAccessor.create( xpath, !listElementNodes.isEmpty() ? ( xpath.contains( listElementNodes.get( 0 ).getText() ) ? Collections
            .<String>singletonList( listElementNodes.get( 0 ).getText() ) : Collections.<String>emptyList() ) : Collections.<String>emptyList(), first,
            attributeListMap ) );

        first = false;
      }

      return result;
    }

    public List<Integer> getAllInidCodes()
    {
      return null;
    }

    private Map<String, List<AttributeDefinition>> createAttributes( Document metadataDescription )
    {

      List<Node> attributeTargetNodes = metadataDescription.selectNodes( this.getQueryForAttributeTarget() );
      if ( !( attributeTargetNodes == null || attributeTargetNodes.isEmpty() ) )
      {
        Map<String, List<AttributeDefinition>> attributeListMap = new HashMap<String, List<AttributeDefinition>>( attributeTargetNodes.size() );
        for ( Node elementName: attributeTargetNodes )
        {
          String element = elementName.getText();
          List<Node> attributeNameNodes = metadataDescription.selectNodes( this.getQueryForAttributeNames( element ) );
          List<Node> attributeValueeNodes = metadataDescription.selectNodes( this.getQueryForAttributeValues( element ) );
          List<Node> attributeValueTypeNodes = metadataDescription.selectNodes( this.getQueryForAttributeValueType( element ) );

          List<AttributeDefinition> attributeDefintions = new ArrayList<AttributeDefinition>( attributeNameNodes.size() );
          for ( int i = 0; i < attributeNameNodes.size(); i++ )
          {
            String name = attributeNameNodes.get( i ).getStringValue();
            String value = attributeValueeNodes.get( i ).getStringValue();
            String valueType = attributeValueTypeNodes.get( i ).getText();
            attributeDefintions.add( new AttributeDefinition( name, value, valueType ) );
          }

          attributeListMap.put( element, attributeDefintions );
        }
        return attributeListMap;
      }
      return Collections.emptyMap();

    }

    private String getQueryForAttributeTarget()
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]/attributelist/@element";
    }

    private String getQueryForAttributeNames( String elementName )
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]/attributelist[@element='" + elementName
          + "']/attribute/name";
    }

    private String getQueryForAttributeValues( String elementName )
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]//attributelist[@element='" + elementName
          + "']/attribute/value";
    }

    private String getQueryForAttributeValueType( String elementName )
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]//attributelist[@element='" + elementName
          + "']/attribute/value/@type";
    }

    private String getQueryForGenericXPaths()
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[not(@kind)]/xpath";
    }

    private String getQueryForSpecificXPaths()
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]/xpath";
    }

    private String getQueryForMandatory()
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]/@type";
    }

    private String getQueryForListElement()
    {
      return "/metadata/info[@name='" + this.key + "']/accessor[contains(@kind, '[" + this.kindcode + "]')]/@listElement";
    }

    public String toString()
    {
      return "[Accessor key=" + key + " kind=" + kindcode + "]";
    }

    // @see java.lang.Object#hashCode()
    public int hashCode()
    {
      final int PRIME = 31;
      int result = 1;
      result = PRIME * result + ( ( this.key == null ) ? 0 : this.key.hashCode() );
      result = PRIME * result + ( ( this.kindcode == null ) ? 0 : this.kindcode.hashCode() );
      return result;
    }

    // @see java.lang.Object#equals(java.lang.Object)
    public boolean equals( Object obj )
    {
      if ( this == obj )
        return true;
      if ( obj == null )
        return false;
      if ( getClass() != obj.getClass() )
        return false;
      final KindDiscriminatingAccessorDescription other = (KindDiscriminatingAccessorDescription) obj;
      if ( this.key == null )
      {
        if ( other.key != null )
          return false;
      }
      else if ( !this.key.equals( other.key ) )
        return false;
      if ( this.kindcode == null )
      {
        if ( other.kindcode != null )
          return false;
      }
      else if ( !this.kindcode.equals( other.kindcode ) )
        return false;
      return true;
    }

  } // inner-class

}
