package ch.ige.ses.client.batch;

import org.apache.log4j.*;
import java.io.*;
import java.util.*;

import ch.ige.ses.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.server.batchctrl.BatchDelegate;
import ch.ige.ses.server.sapctrl.SapDelegateInbound;
import ch.ige.util.client.*;

/**
 * SES Batch Programm zur Starten der AMV-Verarbeitung.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Thomas Iten, Stefan Nydegger
 * @version $Id: SesAmvStart.java,v 1.33 2007/11/28 14:11:53 iten Exp $
 */
public class SesAmvStart extends BatchBasis
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger("ch.ige.ses.client.batch");

  private static final String JOB_NAME            = "SesAmvStart";
  private static final int    JOB_CTRL_INTERVAL   = 60;     // controll intervall [sec]
  private static final int    JOB_TIMEOUT         = 3600;   // timeout [sec]

  private static final int    MODUS_HELP          = 0;    // program modes
  private static final int    MODUS_AUTO          = 1;
  private static final int    MODUS_MAN           = 2;

  private static final String OK_NUM_SUCCESS      = "0";
  private static final String OK_MSG_SUCCESS      = "Batch Job erfolgreich beendet";

  private static final String OK_NUM_RESTART      = "1";
  private static final String OK_MSG_RESTART      = "Restart Batch Job erfolgreich beendet";

  private static final String OK_NUM_FINISH       = "2";
  private static final String OK_MSG_FINISH       = "Finish Batch Job erfolgreich beendet";

  private static final String ERR_NUM_EXCEPTION   = "-1";
  private static final String ERR_MSG_EXCEPTION   = "Eine unerwartete Exception ist aufgetreten";

  private static final String ERR_NUM_WRONG_ARG   = "-2";
  private static final String ERR_MSG_WRONG_ARG   = "Aufruf Program mit ungültigen Parametern";

  private static final String ERR_NUM_INIT        = "-3";
  private static final String ERR_MSG_INIT        = "Fehler bei der Initialisierung des Batch Job.";

  private static final String ERR_NUM_DISABLED    = "-4";
  private static final String ERR_MSG_DISABLED    = "Der AMV Job ist nicht Enabled. " +
                                                    "Bitte auf Tabelle SES.Property den Eintrag <batch.job." + JOB_NAME + ".enabled> überprüfen.";

  private static final String ERR_NUM_LAST_LAUF   = "-5";
  private static final String ERR_MSG_LAST_LAUF   = "Die letzte Verarbeitung ist noch nicht beendet, bitte beenden sie diese vor dem Neustart oder schalten Sie die Option <batch.SesAmvStart.checkLastState.enabled> aus.";

  private static final String ERR_NUM_NO_DATA     = "-6";
  private static final String ERR_MSG_NO_DATA     = "Der AMV Job hat auf BAGIS keine Datensätze gefunden zum verarbeiten.";

  private static final String ERR_NUM_TIMEOUT     = "-7";
  private static final String ERR_MSG_TIMEOUT     = "Der AMV Job hat die Timeout Zeit überschritten und wurde beendet";

  private static final String ERR_NUM_JOB_STOPPED = "-8";
  private static final String ERR_MSG_JOB_STOPPED = "Der AMV Job wurde mit fehlerhaften Einträgen beendet";

  private static final String ERR_NUM_JOB_RESTART = "-9";
  private static final String ERR_MSG_JOB_RESTART = "Restart AMV Job nur möglich, wenn der Lauf gestoppet (LAUF_STAT=3) ist.";

  private static final String ERR_NUM_MAX_ERRORS  = "-10";
  private static final String ERR_MSG_MAX_ERRORS  = "Die maximale Anzahl Fehler wurde überschritten. Der AMV Job wird abgebrochen.";

  //
  // fields
  //
  private int             fModus        = MODUS_HELP;   // set by loadParams()

  private boolean         fInitialized  = false;        // set by init()
  private BatchDelegate   fDelegate     = null;
  private boolean         fE2Enabled    = false;
  private int             fMaxErrors    = 1;
  private int             fProcessDelay = 1000;


  private AmvLaufVO       fAmvLaufVO    = null;

  private boolean         fAppendMsgBodyOnBatchException = false;

  private String          fNextStep     = "";

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SesAmvStart()
  {
    super(JOB_NAME);
  }

  //---------------------------------------------------------------------------
  // methods - initialization
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Programmhilfe
   * Zeigt mit welchen Parametern das Program gestartet werden kann.
   */
  private void printUsage()
  {
    System.out.println("Usage: " + BuildInfo.CVS_TAG);
    System.out.println("SesAmvStart [-h] [-?] [help] [?] modus");
    System.out.println("  -h, -?, help, ?   Anzeige Hilfe (usage)");
    System.out.println("  modus             Programm Modus [auto,man]" );
  }


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args             programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams(String[] args) throws BatchException
  {
    fNextStep = "loadParams - begin";
    LOG.debug(fNextStep);

    if ( (args.length == 1) &&
         args[0].equalsIgnoreCase("auto") )
    {
        fModus = MODUS_AUTO;
    }
    else if ( (args.length == 1) &&
              args[0].equalsIgnoreCase("man") )
    {
      fModus = MODUS_MAN;
    }
    else if ( (args.length == 1) &&
              (args[0].equalsIgnoreCase("-h")   || args[0].equalsIgnoreCase("-?") ||
               args[0].equalsIgnoreCase("help") || args[0].equalsIgnoreCase("?")) )
    {
      fModus = MODUS_HELP;
    }
    else
    {
      setNotifyConsole(true);
      throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
    }

    fNextStep = "loadParams - done";
    LOG.debug(fNextStep);
  }


  /**
   * initialisierung batch job einstellungen.
   *
   * da das batch job programm drei verschiedene einstiegspunkte hat (main,
   * restart und finish) werden alle zentralen intialisierungen an dieser
   * stelle vorgenommen.
   *
   * @throws BatchException
   */
  private void init() throws BatchException
  {
    try
    {
      // check initialization
      if (fInitialized)
        return;

      // get delegate
      fNextStep = "init - BatchDelegate.getInstance()";
      LOG.debug(fNextStep);
      fDelegate = BatchDelegate.getInstance();

      // set email subject title
      setSubject("AMV-Erinnerung Ses");

      // get option maxErrors
      fNextStep = "init - fDelegate.getProperty(batch.SesAmvStart.maxErrors)";
      LOG.debug(fNextStep);
      fMaxErrors = Integer.parseInt(fDelegate.getProperty("batch.SesAmvStart.maxErrors"));
      addSystemMessage("Option <Max Errors>: " + fMaxErrors);

      // get option processDelay
      fNextStep = "init - fDelegate.getProperty(batch.SesAmvStart.processDelay)";
      LOG.debug(fNextStep);
      fProcessDelay = Integer.parseInt(fDelegate.getProperty("batch.SesAmvStart.processDelay"));
      addSystemMessage("Option <Process Delay>: " + fProcessDelay + " ms");

      // get option e2Enabled
      fNextStep = "init - fDelegate.amvSesIsErinnerung2Enabled()";
      LOG.debug(fNextStep);
      fE2Enabled = fDelegate.amvSesIsErinnerung2Enabled();
      addSystemMessage("Option <Print Erinnerung 2> enabled: " + fE2Enabled);

      // initalisation done
      fInitialized = true;
    }
    catch (Exception ex)
    {
      throw new BatchException ( ERR_NUM_INIT, ERR_MSG_INIT +
                                 " Step: " + fNextStep + ", Exception: " + ex.toString() );
    }
  }


  //---------------------------------------------------------------------------
  // methods - processing
  //---------------------------------------------------------------------------

  /**
   * Start der automatischen Markenverlängerung. Die Methode steuert die ganze
   * Verarbeitung von der Erstellung eines neuen Laufes, bis zum Ende.
   *
   * @throws  BatchException  Ein bekannter Fehler ist aufgetreten
   * @throws  Exception       Ein unerwarteter Fehler ist aufgetreten
   */
  private void autoStart() throws BatchException, Exception
  {
    ArrayList e1List  = null;
    ArrayList e2List  = null;


    //
    // check if last lauf is finished (only if this option is enabled)
    //
    if ( fDelegate.amvSesIsCheckLastStateEnabled() )
    {
      fNextStep = "autoStart - check last state is enabled";
      LOG.debug(fNextStep);
      if ( ! fDelegate.amvSesIsLastLaufFinished() )
      {
        fNextStep = "autoStart - last state is not finished !";
        LOG.debug(fNextStep);
        throw new BatchException (ERR_NUM_LAST_LAUF, ERR_MSG_LAST_LAUF);
      }
    }
    else
    {
      fNextStep = "autoStart - check last state is disabled";
      LOG.debug(fNextStep);
    }

    //
    // read bfakt-id's (erinnerung 1 & 2) from bagis
    //
    fNextStep = "autoStart - read erinnerung 1 from bagis";
    LOG.debug(fNextStep);
    e1List = fDelegate.amvBagisGetRechnungNrList(AmvRechnungVO.RE_CD_ERINNERUNG_1);
    LOG.debug(fNextStep + ", " + e1List.size() + " records found");

    fNextStep = "autoStart - read erinnerung 2 from bagis";
    LOG.debug(fNextStep);
    e2List = fDelegate.amvBagisGetRechnungNrList(AmvRechnungVO.RE_CD_ERINNERUNG_2);
    LOG.debug(fNextStep + ", " + e2List.size() + " records found");


    //
    // check count (erinnerung 1 & 2) to process
    //
    if ( (e1List.size() == 0) &&  (e2List.size() == 0) )
    {
      fNextStep = "check count (erinnerung 1 & 2) to process, no records found at all !";
      LOG.warn(fNextStep);
      throw new BatchException (ERR_NUM_NO_DATA, ERR_MSG_NO_DATA);
    }


    //
    // create new lauf
    //
    fNextStep = "autoStart - create new lauf";
    LOG.debug(fNextStep);
    fAmvLaufVO = fDelegate.amvSesCreateLauf();


    //
    // insert bfakt-id's to ses and update lauf state
    //
    fNextStep = "autoStart - insert erinnerung 1 and 2 to ses db";
    LOG.debug(fNextStep);
    fDelegate.amvSesInsertRechnung(fAmvLaufVO.amvLaufId,e1List,e2List);


    //
    // update bfaktur.status_druck on bagis
    //
    fNextStep = "autoStart - update status druck on bagis";
    LOG.debug(fNextStep);

    setStatusDruck(e1List,"1");
    setStatusDruck(e2List,fE2Enabled ? "1" : "9");

    //
    // get & send rechnung to SapInbound Service
    //
    fNextStep = "autoStart - send erinnerung 1 to sap inbound service";
    LOG.debug(fNextStep);
    sendRechnung(AmvRechnungVO.RE_CD_ERINNERUNG_1, e1List);

    if (fE2Enabled)
    {
      fNextStep = "autoStart - send erinnerung 2 to sap inbound service";
      LOG.debug(fNextStep);
      sendRechnung(AmvRechnungVO.RE_CD_ERINNERUNG_2, e2List);
    }
    else
    {
      fNextStep = "autoStart - send erinnerung 2 to sap inbound service is DISABLED";
      LOG.debug(fNextStep);
    }

    fNextStep = "autoStart - update lauf status on SES";
    LOG.debug(fNextStep);
    fDelegate.amvSesSetLaufStatus(fAmvLaufVO.amvLaufId, AmvLaufVO.LAUF_STAT_SENT);


    //
    // wait for end of process or timeout
    //
    fNextStep = "autoStart - wait for end of process or timeout";
    LOG.debug(fNextStep);
    waitProcessEnd(JOB_CTRL_INTERVAL,JOB_TIMEOUT);


    //
    // check for errors
    //
    fNextStep = "autoStart - process finished, check for errors";
    LOG.debug(fNextStep);

    if (fDelegate.amvSesHasErrors(fAmvLaufVO.amvLaufId))
    {
      fNextStep = "autoStart - process finished, errors found";
      LOG.debug(fNextStep);

      // update lauf_stat to STOPPED
      fDelegate.amvSesSetLaufStatus(fAmvLaufVO.amvLaufId, AmvLaufVO.LAUF_STAT_STOPPED);

      // end processing with error message
      fAppendMsgBodyOnBatchException = true;
      throw new BatchException (ERR_NUM_JOB_STOPPED, ERR_MSG_JOB_STOPPED);
    }
    else
    {
      fNextStep = "autoStart - process finished, no errors found";
      LOG.debug(fNextStep);
    }


    //
    // send finish lauf
    //
    fNextStep = "autoStart - send finish lauf";
    LOG.debug(fNextStep);
    SapDelegateInbound.getInstance().sendCtrlAmvFinish(fAmvLaufVO);

  } // end method


  /**
   * Aktualisiert auf der BAGIS Tabelle bfaktur den status_druck auf den
   * angegenen Wert für die Liste mit Rechnungsnummern (bfakt_id's).
   *
   * @param   list    Integer Liste mit Rechnungsnummern (bfakt_id)
   * @param   status  Gewünschter Druckstatus
   */
  private void setStatusDruck(ArrayList list, String status)
  {
    try
    {
      LOG.debug("setStatusDruck - status=" + status + "...");
      fDelegate.amvBagisSetRechnungStatusDruck(list,status);
      LOG.debug("setStatusDruck - done");
    }
    catch (Exception ex)
    {
      StringBuffer buf = new StringBuffer();
      int          cnt = 0;
      ErrorVO      err = null;


      // assemble bfakt_id list
      for (Iterator iter = list.iterator(); iter.hasNext(); )
      {
        Integer item = (Integer)iter.next();

        buf.append(item);
        cnt++;

        if (cnt < 10)
        {
          buf.append("  ");
        }
        else
        {
          buf.append("\n");
          cnt = 0;
        }
      }

      // create and save error message
      err = new ErrorVO ( "SesAMVStart",
                          "setStatusDruck - Exception: " + ex.getMessage(),
                          ErrorVO.DATA_TYPE_CD_STRING,
                          "Failed to update status-druck to status " + status +
                          " for bfakt_id's: \n" + buf.toString() );
      saveError(err);

      // log error message
      LOG.error(err.description);
      LOG.error((String)err.data);
    }
  }


  /**
   * Selektiert die Rechnungsdaten auf Bagis und sendet diese zur weiteren
   * Verarbeitung an den SapInbound Service.
   *
   * @param   rechnungCd    Rechnungstyp (Erinnerung 1 oder 2)
   * @param   list          Integer Liste mit Rechnungsnummern (bfakt_id)
   * @throws  Exception     Unerwarteter Fehler
   */
  private void sendRechnung (int rechnungCd, ArrayList list ) throws Exception
  {
    AmvRechnungVO vo    = new AmvRechnungVO();
    int           cnt   = 0;
    int           errors= 0;          // error count for stopping the process


    //
    // process list with bfakt_id's
    //
    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      // init
      cnt++;
      vo.clear();
      vo.amvLaufId  = fAmvLaufVO.amvLaufId;
      vo.rechnungCd = rechnungCd;
      vo.bfaktId    = ( (Integer)iter.next() ).intValue();

      // get rechnung from bagis and send to sapinbound
      try
      {
        LOG.debug("sendRechnung - #" + cnt + " with bfakt_id " + vo.bfaktId + "...");

        fDelegate.amvBagisCreateRechnung
          (fAmvLaufVO.amvLaufId, fAmvLaufVO.laufNr, rechnungCd, vo.bfaktId );

        vo.rechnungStat = AmvRechnungVO.RE_STAT_SENT;
      }
      catch (Exception ex)
      {
        errors++;

        vo.rechnungStat = AmvRechnungVO.RE_STAT_SAP_ERROR;
        vo.errorMsg     = "sendRechnung - get rechnung from bagis and send to sapinbound failed !" +
                          " exception is: " + ex.toString();
        LOG.error("sendRechnung - " + vo.errorMsg);
      }

      // update amv_rechnung
      fDelegate.amvSesSetRechnungStatus(vo);

      // check error count
      if (errors >= fMaxErrors)
      {
        fNextStep = "sendRechnung - maxErrors reached, stopping process !";
        LOG.debug(fNextStep);
        throw new BatchException (ERR_NUM_MAX_ERRORS,ERR_MSG_MAX_ERRORS);
      }

      // wait for memory recovery on app server [ms]
      sleepMs(fProcessDelay);
    }
  }


  /**
   * Wartet bis alle Datensätze vom SapInbound Service an SAP übermittelt
   * wurden oder die angegebenen Timout Zeit überschritten wurde.
   * Da der SapInbound Service asynchron arbeitet, werden die Stati der Einträge
   * der SES Tabelle AMV_RECHNUNG kontrolliert.
   *
   * @param   ctrlInterval    Abfrage Intervall [sec]
   * @param   timeout         Maximale Abfragezeit [sec]
   * @throws  BatchException  Timeout wurde überschritten
   * @throws  Exception       Unerwarteter Fehler ist aufgetreten
   */
  private void waitProcessEnd(int ctrlInterval, int timeout)
      throws BatchException, Exception
  {
    boolean   isRunning   = true;
    int       elapsedTime = 0;

    while (isRunning)
    {
      sleep(ctrlInterval);

      elapsedTime = elapsedTime + ctrlInterval;
      if (elapsedTime >= timeout)
      {
        throw new BatchException
          (ERR_NUM_TIMEOUT, ERR_MSG_TIMEOUT + " (Timeout=" + timeout + "sec)");
      }

      isRunning = fDelegate.amvSesIsRunning(fAmvLaufVO.amvLaufId);
    }
  }


  /**
   * Die Verarbeitung wird um die angegebenen Sekunden unterbrochen.
   *
   * @param seconds   Anzahl Sekunden, die gewartet werden soll.
   */
  private void sleep(int seconds)
  {
    try
    {
      Thread.sleep(seconds * 1000);
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
  } // end method


  /**
   * Die Verarbeitung wird um die angegebenen Sekunden unterbrochen.
   *
   * @param ms   Anzahl Milisekunden, die gewartet werden soll.
   */
  private void sleepMs(int ms)
  {
    try
    {
      Thread.sleep(ms);
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
  } // end method

  /**
   * Aufbereitung der Batch Job Message. Diese wird sowohl für die Erstellung
   * des ok/nok Files verwendet, als auch für die EMail Benachrichtigung.
   * Das Format ist im Konzept definiert, damit Ses, Jipps und Bagis wenn
   * möglich (verfügbar) die gleichen Angaben Rückmelden.
   *
   * Beachte:
   * Tritt bei der Aufbereitung der Daten ein Fehler auf, so wird dieser
   * ebenfalls in der Meldung angezeigt. Die Methode gibt in diesem Fall
   * auch einen String zurück und wird nicht mit einer Exception beendet.
   *
   * @param   header  optionaler header, wird am anfang des message bodies
   *                  eingefügt.
   * @return          Message Body
   */
  private String getMessageBody(String header)
  {
    StringBuffer  buf     = new StringBuffer();   // message
    int[]         cnt     = null;                 // e1/e2 counts and total
    int           total   = 0;
    ArrayList     errList       = null;           // error list and titel
    boolean       errTitleE1Set = false;
    boolean       errTitleE2Set = false;


    try
    {
      //
      // Header
      //
      if (header != null)
        buf.append(header + "\n\n");

      //
      // Lauf
      //
      buf.append("Laufnummer: " + fAmvLaufVO.laufNr + "\n\n");

      //
      // Rechnungen (Fakturaanforderungen)
      //
      cnt = fDelegate.amvSesGetRechnungCount(fAmvLaufVO.amvLaufId);

      total = 0;
      for (int i = 0; i < cnt.length; i++)
        total = total + cnt[i];

      buf.append("Anzahl Rechnungen (Fakturaanforderungen): " + total + "\n");
      if (cnt[AmvRechnungVO.RE_STAT_SELECTED] > 0)
        buf.append("  - davon selektiert: "   + cnt[AmvRechnungVO.RE_STAT_SELECTED]     + "\n");
      if (cnt[AmvRechnungVO.RE_STAT_SENT] > 0)
        buf.append("  - davon gesendet: "     + cnt[AmvRechnungVO.RE_STAT_SENT]         + "\n");
      buf.append("  - davon an SAP OK: "    + cnt[AmvRechnungVO.RE_STAT_SAP_OK]       + "\n");
      buf.append("  - davon an SAP NOK: "   + cnt[AmvRechnungVO.RE_STAT_SAP_ERROR]    + "\n");
      if (cnt[AmvRechnungVO.RE_STAT_SUPPRESS_E2] > 0)
        buf.append("  - davon Rechnungsempfaenger 2 nicht gesendet: " + cnt[AmvRechnungVO.RE_STAT_SUPPRESS_E2]  + "\n");
      buf.append("\n");

      //
      // Erinnerungen 1 (Rechnungsempfänger)
      //
      cnt = fDelegate.amvSesGetRechnungEmpfCount
              ( fAmvLaufVO.amvLaufId, AmvRechnungVO.RE_CD_ERINNERUNG_1 );

      total = 0;
      for (int i = 0; i < cnt.length; i++)
        total = total + cnt[i];

      buf.append("Anzahl Rechnungsempfaenger 1: " + total + "\n");
      if (cnt[AmvRechnungVO.RE_STAT_SELECTED] > 0)
        buf.append("  - davon selektiert: "  + cnt[AmvRechnungVO.RE_STAT_SELECTED] + "\n");
      if (cnt[AmvRechnungVO.RE_STAT_SENT] > 0)
        buf.append("  - davon gesendet: " + cnt[AmvRechnungVO.RE_STAT_SENT] + "\n");
      buf.append("  - davon OK: "  + cnt[AmvRechnungVO.RE_STAT_SAP_OK] + "\n");
      buf.append("  - davon NOK: " + cnt[AmvRechnungVO.RE_STAT_SAP_ERROR] + "\n\n");


      //
      // Erinnerungen 2 (Rechnungsempfänger)
      //
      cnt = fDelegate.amvSesGetRechnungEmpfCount
              ( fAmvLaufVO.amvLaufId, AmvRechnungVO.RE_CD_ERINNERUNG_2 );

      total = 0;
      for (int i = 0; i < cnt.length; i++)
        total = total + cnt[i];

      if ( total > 0 )
      {
        buf.append("Anzahl Rechnungsempfaenger 2: " + total + "\n");
        if (cnt[AmvRechnungVO.RE_STAT_SELECTED] > 0)
          buf.append("  - davon selektiert: "  + cnt[AmvRechnungVO.RE_STAT_SELECTED] + "\n");
        if (cnt[AmvRechnungVO.RE_STAT_SENT] > 0)
          buf.append("  - davon gesendet: " + cnt[AmvRechnungVO.RE_STAT_SENT] + "\n");
        if (fE2Enabled)
        {
          buf.append("  - davon OK: "  + cnt[AmvRechnungVO.RE_STAT_SAP_OK] + "\n");
        }
        else
        {
          buf.append("  - davon OK: "  + cnt[AmvRechnungVO.RE_STAT_SUPPRESS_E2] + " (ohne Ausgabe auf SAP) \n");
        }
        buf.append("  - davon NOK: " + cnt[AmvRechnungVO.RE_STAT_SAP_ERROR] + "\n");
      }
      
      
      //
      // Fehlerliste
      //
      errList = fDelegate.amvSesGetRechnungListByStat
                  ( fAmvLaufVO.amvLaufId, AmvRechnungVO.RE_STAT_SAP_ERROR );

      for (Iterator iter = errList.iterator(); iter.hasNext(); )
      {
        AmvRechnungVO item = (AmvRechnungVO)iter.next();

        // set title e1
        if ( (! errTitleE1Set) &&
             (item.rechnungCd == AmvRechnungVO.RE_CD_ERINNERUNG_1) )
        {
          buf.append("\nFehlerliste Erinnerung 1:\n");
          errTitleE1Set = true;
        }

        // set title e2
        if ( (! errTitleE2Set) &&
             (item.rechnungCd == AmvRechnungVO.RE_CD_ERINNERUNG_2) )
        {
          buf.append("\nFehlerliste Erinnerung 2:\n");
          errTitleE2Set = true;
        }

        // append error data
        buf.append("bfakt_id " + item.bfaktId + ":\n" + item.errorMsg + "\n");
      }

    }
    catch (Exception ex)
    {
      LOG.error("getMessageBody exception occured: " + ex.getMessage());
      buf.append("getMessageBody exception occured: " + ex.getMessage());
    }

    return buf.toString();
  }

  /**
   * Speichert eine Fehlermeldung auf der Ses Tabelle Error. Diese kann
   * anschliessend mit Hilfe des Ses Monitors bearbeitet werden.
   *
   * @param errorVO Fehlermeldung
   */
  private void saveError (ErrorVO errorVO)
  {
    try
    {
      SapDelegateInbound.getInstance().receiveError(errorVO);
    }
    catch (Exception ex)
    {
      LOG.fatal ("saveError - Ein unerwarteter Fehler ist aufgetreten " +
                 "ErrorVO   : \n" + errorVO.toString() + "\n" +
                 "Exception : \n" + ex.getMessage() );
    }
  }


  //---------------------------------------------------------------------------
  // methods - manual mode mode
  //---------------------------------------------------------------------------

  /**
   * Start manuellen Modus (nur für Testzwecke)
   *
   * @throws Exception  unerwarteter Fehler
   */
  private void manStart() throws Exception
  {
    int cmd = 0;

    while (cmd < 99)
    {
      cmd = manPrintCmd();

      try
      {
        manExecuteCmd(cmd);
      }
      catch (BatchException ex)
      {
        System.out.println("Es ist eine BatchException aufgetreten:" );
        System.out.println("Error Number : " + ex.getErrorNr());
        System.out.println("Error Message: " + ex.getErrorMsg());
      }
      catch (Exception ex)
      {
        System.out.println("Es ist ein Fehler unerwarteter Fehler aufgetreten:" );
        System.out.println("Exception: \n" + ex.getMessage());
        System.out.println("Stack Trace: ");
        ex.printStackTrace();
      }
    }
  }


  private int manPrintCmd()
  {
    String      buf;
    int         cmd   = 0;
    boolean     valid = false;


    System.out.println();
    System.out.println("SesAmvStart - Menu manueller Modus");
    System.out.println("-----------------------------------------------");
    System.out.println("    1 : restart                                ");
    System.out.println("    2 : finish                                 ");
    System.out.println();
    System.out.println("   10 : set field LAUF_STAT from table AMV_LAUF");
    System.out.println("   11 : reset errors from table AMV_RECHNUNG   ");
    System.out.println("                                               ");
    System.out.println("   TEST ONLY                                   ");
    System.out.println("   50 : Select RechnungNrListe Erinnerung 1    ");
    System.out.println("   51 : Select RechnungNrListe Erinnerung 2    ");
    System.out.println("   52 : Set RechnungStatusGedruckt             ");
    System.out.println("   53 : Create Rechnung                        ");
    System.out.println("   60 : Create Lauf                            ");
    System.out.println();
    System.out.println("   99 : EXIT                                   ");
    System.out.println();

    while (! valid)
    {
      try
      {
        buf = manReadLine("Eingabe = ");
        cmd = Integer.parseInt(buf);
        valid = true;
      }
      catch (Exception ex)
      {
        System.out.println("Falsche Eingabe, bitte geben Sie eine Zahl ein.");
      }
    }

    return cmd;
  }


  private void manExecuteCmd (int cmd) throws Exception
  {
    switch (cmd)
    {
      case 1:
      {
        int     amvLaufId   = 0;
        int     sendMailFlg = 0;
        boolean sendMail    = false;
        String  ret         = null;

        // get input data
        System.out.println("Eingabe Restart Parameter: ");
        amvLaufId   = Integer.parseInt( manReadLine("  amvLaufId              = ") );
        sendMailFlg = Integer.parseInt( manReadLine("  sendMail (1=ja/0=nein) = ") );

        // execute action
        if (sendMailFlg == 1)
          sendMail = true;

        ret = restart(amvLaufId,sendMail);

        // show return
        System.out.println("Rueckgabewert Restart: \n" + ret);
        break;
      }

      case 2:
      {
        int     amvLaufId   = 0;
        int     sendMailFlg = 0;
        boolean sendMail    = false;
        String  ret         = null;

        // get input data
        System.out.println("Eingabe Finish Parameter: ");
        amvLaufId   = Integer.parseInt( manReadLine("  amvLaufId              = ") );
        sendMailFlg = Integer.parseInt( manReadLine("  sendMail (1=ja/0=nein) = ") );

        // execute action
        if (sendMailFlg == 1)
          sendMail = true;

        ret = finish(amvLaufId,sendMail);

        // show return
        System.out.println("Rueckgabewert Finish: \n" + ret);
        break;
      }

      case 10:
      {
        int     amvLaufId   = 0;
        int     laufStat    = 0;

        // get input data
        System.out.println("Eingabe LAUF_STAT: ");
        amvLaufId   = Integer.parseInt( manReadLine("  amvLaufId  = ") );
        laufStat    = Integer.parseInt( manReadLine("  laufStat   = ") );

        // execute action
        fDelegate.amvSesSetLaufStatus(amvLaufId,laufStat);

        // all done
        break;
      }

      case 11:
      {
        int     amvLaufId   = 0;
        int     rows        = 0;

        // get input data
        System.out.println("Eingabe Reset Errors: ");
        amvLaufId   = Integer.parseInt( manReadLine("  amvLaufId  = ") );

        // execute action
        rows = fDelegate.amvSesResetErrors(amvLaufId);

        // all done
        System.out.println("reset done for " + rows + " error entries");
        break;
      }

      //
      // TEST ONLY
      //
      case 50:
      {
        int       cnt  = 0;
        ArrayList list = fDelegate.amvBagisGetRechnungNrList(AmvRechnungVO.RE_CD_ERINNERUNG_1);

        System.out.println("Liste mit bfakt_id's Erinnerung 1:");

        if (list != null)
        {
          for (Iterator iter = list.iterator(); iter.hasNext(); )
          {
            Integer item = (Integer)iter.next();
            System.out.println("  " + item.intValue());
            cnt++;
          }
        }
        System.out.println(cnt + " Eintraege gefunden");
        break;
      }


      case 51:
      {
        int       cnt  = 0;
        ArrayList list = fDelegate.amvBagisGetRechnungNrList(AmvRechnungVO.RE_CD_ERINNERUNG_2);

        System.out.println("Liste mit bfakt_id's Erinnerung 2:");
        if (list != null)
        {
          for (Iterator iter = list.iterator(); iter.hasNext(); )
          {
            Integer item = (Integer)iter.next();
            System.out.println("  " + item.intValue());
            cnt++;
          }
        }
        System.out.println(cnt + " Eintraege gefunden");
        break;
      }


      case 52:
      {
        ArrayList   list    = new ArrayList();
        int         bfaktId = 0;

        System.out.println("Geben Sie die gewuenschten Rechnungsnummern (bfakt_id's) ein.");
        System.out.println("(Abbruch mit Eingabe=0) :");
        do {
          bfaktId = Integer.parseInt( manReadLine("  Enter bfakt_id = ") );
          if (bfaktId > 0)
            list.add( new Integer(bfaktId) );
        } while(bfaktId > 0);

        fDelegate.amvBagisSetRechnungStatusDruck(list,"9");
        System.out.println("Der Rechnungsstatus wurde auf gedruckt gesetzt.");
        break;
      }


      case 53:
      {
        System.out.println("Create Rechnung:");
        int     amvLaufId     = Integer.parseInt( manReadLine("  amvLaufId           = ") );
        String  amvLaufNr     =                   manReadLine("  amvLaufNr           = ");
        int     amvRechnungCd = Integer.parseInt( manReadLine("  amvRechnungCd [1,2] = ") );
        int     bfaktId       = Integer.parseInt( manReadLine("  bfaktId             = ") );

        fDelegate.amvBagisCreateRechnung
            ( amvLaufId, amvLaufNr, amvRechnungCd, bfaktId );

        break;
      }

      case 60:
      {
        AmvLaufVO vo = fDelegate.amvSesCreateLauf();
        System.out.println(vo.toString());
        break;
      }

      //
      // exit application
      //
      case 99:
      {
        break;
      }
    } // end switch

  } // end method


  private String manReadLine(String prompt)
  {
    try
    {
      BufferedReader stdin  = new BufferedReader(new InputStreamReader(System.in));

      System.out.print (prompt);
      return(stdin.readLine());
    }
    catch (Exception ex)
    {
      return ("");
    }
  }


  //***************************************************************************
  // schnittstelle zum ses monitor
  //***************************************************************************

  /**
   * Führt einen Restart der aktuellen AMV Verarbeitung durch. Ein Restart ist
   * nur dann möglich, wenn der LAUF_STAT der SES Tabelle AMV_LAUF auf dem
   * Status 3 "Verarbeiung mit Fehler gestoppt" ist.
   *
   * Beachte:
   * Ist der Status <3 so wurde die Verarbeitung vorher mit einem Fehler
   * abgebrochen. In diesem Fall muss zuerst manuell die Fehlerursache behoben
   * werden.
   *
   * @param   amvLaufId   Lauf Identifikation
   * @param   sendEmail   Flag, ob nach der Ausführung eine Email Nachricht
   *                      versendet werden soll oder nicht.
   *
   * @return  String      Verarbeitungsmeldung (entspricht dem Email Text)
   *
   * @throws Exception    Bei der Verarbeitung ist ein Fehler aufgetreten
   */
  public String restart (int amvLaufId, boolean sendEmail) throws Exception
  {
    ArrayList e1List  = null;
    ArrayList e2List  = null;
    int       rows    = 0;

    try
    {
      //
      // init
      //
      LOG.debug("### SesAmvStart - restart... ###");

      fNextStep = "restart - init";
      LOG.debug(fNextStep);
      setNotifyAll(false, sendEmail, false);    // disable file and console output
      init();


      //
      // get lauf data and check actual state
      //
      fNextStep = "restart - get lauf data and check actual state";
      LOG.debug(fNextStep);

      fAmvLaufVO = fDelegate.amvSesGetLauf(amvLaufId);

      if (fAmvLaufVO.laufStat != AmvLaufVO.LAUF_STAT_STOPPED)
      {
        throw new BatchException(ERR_NUM_JOB_RESTART, ERR_MSG_JOB_RESTART);
      }


      //
      // reset actual lauf state to "started"
      //
      fNextStep = "restart - reset actual lauf state to started";
      LOG.debug(fNextStep);

      fAmvLaufVO.laufStat = AmvLaufVO.LAUF_STAT_STARTED;
      fDelegate.amvSesSetLaufStatus(fAmvLaufVO.amvLaufId, fAmvLaufVO.laufStat);


      //
      // reset error entries on ses table amv_rechnung
      //
      fNextStep = "restart - reset error entries on ses table amv_rechnung";
      LOG.debug(fNextStep);

      rows = fDelegate.amvSesResetErrors(fAmvLaufVO.amvLaufId);
      LOG.debug("restart - reset done for " + rows + " error entries");


      //
      // get erinnerung 1 list and erinnerung 2 list
      //
      fNextStep = "restart - get erinnerung 1 list";
      LOG.debug(fNextStep);

      e1List = fDelegate.amvSesGetRechnungNrList
          (fAmvLaufVO.amvLaufId, AmvRechnungVO.RE_CD_ERINNERUNG_1);

      LOG.debug(fNextStep + ", " + e1List.size() + " records found");

      if (fE2Enabled)
      {
        fNextStep = "restart - get erinnerung 2 list";
        LOG.debug(fNextStep);

        e2List = fDelegate.amvSesGetRechnungNrList
            (fAmvLaufVO.amvLaufId, AmvRechnungVO.RE_CD_ERINNERUNG_2);

        LOG.debug(fNextStep + ", " + e2List.size() + " records found");
      }
      else
      {
        fNextStep = "restart - get erinnerung 2 list SKIPPED";
        LOG.debug(fNextStep);
      }


      //
      // send rechnung to SapInbound Service
      //
      fNextStep = "restart - send erinnerung 1 to sap inbound service";
      LOG.debug(fNextStep);

      sendRechnung(AmvRechnungVO.RE_CD_ERINNERUNG_1, e1List);

      if (fE2Enabled)
      {
        fNextStep = "restart - send erinnerung 2 to sap inbound service";
        LOG.debug(fNextStep);
        sendRechnung(AmvRechnungVO.RE_CD_ERINNERUNG_2, e2List);
      }
      else
      {
        fNextStep = "restart - send erinnerung 2 to sap inbound service SKIPPED";
        LOG.debug(fNextStep);
      }

      fNextStep = "restart - update lauf status";
      LOG.debug(fNextStep);
      fDelegate.amvSesSetLaufStatus(fAmvLaufVO.amvLaufId, AmvLaufVO.LAUF_STAT_SENT);


      //
      // wait for end of process or timeout
      //
      fNextStep = "restart - wait for end of process or timeout";
      LOG.debug(fNextStep);
      waitProcessEnd(JOB_CTRL_INTERVAL,JOB_TIMEOUT);


      //
      // check for errors
      //
      fNextStep = "restart - process finished, check for errors";
      LOG.debug(fNextStep);

      if (fDelegate.amvSesHasErrors(fAmvLaufVO.amvLaufId))
      {
        fNextStep = "restart - process finished, errors found";
        LOG.debug(fNextStep);

        // update lauf_stat to STOPPED
        fDelegate.amvSesSetLaufStatus(fAmvLaufVO.amvLaufId, AmvLaufVO.LAUF_STAT_STOPPED);

        // end processing with error message
        fAppendMsgBodyOnBatchException = true;
        throw new BatchException (ERR_NUM_JOB_STOPPED, ERR_MSG_JOB_STOPPED);
      }
      else
      {
        fNextStep = "restart - process finished, no errors found";
        LOG.debug(fNextStep);
      }


      //
      // send finish lauf
      //
      fNextStep = "restart - send finish lauf";
      LOG.debug(fNextStep);
      SapDelegateInbound.getInstance().sendCtrlAmvFinish(fAmvLaufVO);


      //
      // notify that job successfully finished and return message
      //
      fNextStep = "restart - notify that job successfully finished and return message";
      LOG.debug(fNextStep);

      setMessage ( MSG_TYPE_OK, OK_NUM_RESTART,
                   getMessageBody(OK_MSG_RESTART) );
      notifyMessage();

      LOG.debug("### SesAmvStart - restart successfully done ###");
      return getMessage();

    }
    catch (BatchException bex)
    {
      LOG.error("restart - BatchException  : " + bex.getErrorMsg());
      LOG.error("restart - Hint (next step): " + fNextStep);
      if (fAppendMsgBodyOnBatchException)
      {
        setMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                     getMessageBody(bex.getErrorMsg()) );
      }
      else
      {
        setMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                     bex.getErrorMsg() );
      }
      notifyMessage();
      LOG.debug("### SesAmvStart - restart failed ###");
      throw bex;
    }
    catch (Exception ex)
    {
      LOG.error("restart - Exception       : " + ex.getMessage());
      LOG.error("restart - Hint (next step): " + fNextStep);
      setMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION +
                   " Hint: " + fNextStep + " Exception: " + ex.toString() );
      notifyMessage();
      LOG.debug("### SesAmvStart - restart failed ###");
      throw ex;
    }

  } // end restart


  /**
   * Beendet die aktuelle AMV Verarbeitung ohne zu prüfen, ob alle Daten
   * erfolgreich verarbeitet wurden.
   *
   * @param   amvLaufId   Lauf Identifikation
   * @param   sendEmail   Flag, ob nach der Ausführung eine Email Nachricht
   *                      versendet werden soll oder nicht.
   *
   * @return  String      Verarbeitungsmeldung (entspricht dem Email Text)
   *
   * @throws Exception    Bei der Verarbeitung ist ein Fehler aufgetreten
   */
  public String finish (int amvLaufId, boolean sendEmail) throws Exception
  {
    try
    {
      //
      // init
      //
      LOG.debug("### SesAmvStart - finish... ###");

      fNextStep = "finish - init";
      LOG.debug(fNextStep);

      setNotifyAll(false, sendEmail, false);    // disable file and console output
      init();


      //
      // get complete lauf data
      //
      fNextStep = "finish - get complete lauf data";
      LOG.debug(fNextStep);

      fAmvLaufVO = fDelegate.amvSesGetLauf(amvLaufId);


      //
      // send finish lauf
      //
      fNextStep = "finish - send finish lauf";
      LOG.debug(fNextStep);

      SapDelegateInbound.getInstance().sendCtrlAmvFinish(fAmvLaufVO);


      //
      // set message that job successfully finished and return message
      //
      fNextStep = "finish - set message that job successfully finished and return message";
      LOG.debug(fNextStep);

      setMessage ( MSG_TYPE_OK, OK_NUM_FINISH,
                   getMessageBody(OK_MSG_FINISH) );
      notifyMessage();

      LOG.debug("### SesAmvStart - finish successfully done ###");
      return getMessage();

    }
    catch (BatchException bex)
    {
      LOG.error("finish - BatchException  : " + bex.getErrorMsg());
      LOG.error("finish - Hint (next step): " + fNextStep);
      if (fAppendMsgBodyOnBatchException)
      {
        setMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                     getMessageBody(bex.getErrorMsg()) );
      }
      else
      {
        setMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                     bex.getErrorMsg() );
      }
      notifyMessage();
      LOG.debug("### SesAmvStart - finish failed ###");
      throw bex;
    }
    catch (Exception ex)
    {
      LOG.error("finish - Exception       : " + ex.getMessage());
      LOG.error("finish - Hint (next step): " + fNextStep);
      setMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION +
                   " Hint: " + fNextStep + " Exception: " + ex.toString() );
      notifyMessage();
      LOG.debug("### SesAmvStart - finish failed ###");
      throw ex;
    }

  } // end finish

  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    SesAmvStart job   = new SesAmvStart();

    try
    {
      //
      // init
      //
      ClientLogInitalizer.initalizeLogging(ClientLogInitalizer.BATCH_CONFIG);
      LOG.debug("### SesAmvStart - main... ###");

      job.loadParams(args);
      job.init();


      //
      // run job
      //
      switch (job.fModus)
      {
        case MODUS_AUTO:
        {
          // check if job is enabled
          if ( ! job.enabled() )
            throw new BatchException(ERR_NUM_DISABLED,ERR_MSG_DISABLED);

          // run job
          job.autoStart();

          // notify that job successfully finished
          job.setMessage ( MSG_TYPE_OK, OK_NUM_SUCCESS,
                           job.getMessageBody(OK_MSG_SUCCESS) );
          job.notifyMessage();

          // end
          break;
        }
        case MODUS_MAN:
        {
          job.setNotifyAll(false,false,true);         // console notify only
          job.manStart();
          break;
        }
        case MODUS_HELP:
        {
          job.printUsage();
          break;
        }
      } // end switch


      //
      // End
      //
      LOG.debug("### SesAmvStart - main successfully done ###");
      System.exit(0);
    }
    catch (BatchException bex)
    {
      LOG.error("main - BatchException  : " + bex.getErrorMsg());
      LOG.error("main - Hint (next step): " + job.fNextStep);
      if (job.fAppendMsgBodyOnBatchException)
      {
        job.setMessage ( MSG_TYPE_NOK, bex.getErrorNr(),
                         job.getMessageBody(bex.getErrorMsg()) );
        job.notifyMessage();
      }
      else
      {
        job.notifyMessage ( MSG_TYPE_NOK, bex.getErrorNr(), bex.getErrorMsg() );
      }
      LOG.debug("### SesAmvStart - main failed ###");
      System.exit( Integer.parseInt(bex.getErrorNr()) );
    }
    catch (Exception ex)
    {
      LOG.error("main - Exception       : " + ex.getMessage());
      LOG.error("main - Hint (next step): " + job.fNextStep);
      job.notifyMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION +
                          " Hint: " + job.fNextStep + " Exception: " + ex.toString() );
      LOG.debug("### SesAmvStart - main failed ###");
      System.exit( Integer.parseInt(ERR_NUM_EXCEPTION) );
    }

  } // end main


} // end class


