package ch.ige.ses.client.sapinbound;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.TreeSet;

import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;

import org.apache.log4j.Logger;

import ch.ige.ses.BuildInfo;
import ch.ige.ses.client.saputil.Convert;
import ch.ige.ses.client.saputil.ErrorFile;
import ch.ige.ses.client.saputil.SapProperties;
import ch.ige.ses.server.sapctrl.SapDelegateInbound;
import ch.ige.ses.util.Jms;
import ch.ige.ses.util.JmsQueue;
import ch.ige.ses.vo.ses.ErrorVO;

/**
 * SAP Inbound Client - Automatic Mode
 * Diese Klasse steuert den SAP Inbound Client im normalen Betrieb. Sie liest
 * Meldungen aus der JMS Queue und gibt diese via BAPI Call an SAP weiter.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: AutoMode.java,v 1.84 2007/01/09 16:48:29 iten Exp $
 */
public class AutoMode implements
    javax.jms.MessageListener,            // handle message subscriptions
    javax.jms.ExceptionListener           // trap failed connections
{
  //
  // const
  //
  private static final Logger   LOG                           = Logger.getLogger(AutoMode.class);

  private static final int      DEF_SAP_TRACE                 = 0;
  private static final int      DEF_CONNECTION_RETRY_PERIOD   = 60;
  private static final int      DEF_CONNECTION_PING_INTERVAL  = 60;

  private static final boolean  JMS_SESSION_TRANSACTED        = false;
  private static final int      JMS_SESSION_ACKNOWLEDGE_MODE  = Session.AUTO_ACKNOWLEDGE;

  //
  // fields
  //
  private AutoRFC           fRfc      = null;     // sap rfc calls
  private AutoObjectMessage fObjMsg   = null;     // auto object message
  private JmsQueue          fJmsQueue = null;     // jms queue

  private int         fSapConnRetryPeriod;        // sap & jms connection:
  private int         fSapConnPingInterval;       // retry period & ping interval
  private int         fJmsConnRetryPeriod;        // in seconds

  private boolean     fStopFlag         = false;  // stop application
  private boolean     fSapAliveFlag     = false;  // true if sap is connected
  private boolean     fJmsAliveFlag     = false;  // true if jms is connected

  private Thread      fWaitThread       = null;   // used to interrupt waiting thread

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public AutoMode()
  {
    SapProperties sp = new SapProperties();

    try
    {
      // init params & properties
      logBuildInfo();
      logParams();
      logProperties("SAP Inbound Client - Properties",SapInbound.getProperties());
      logProperties("System - Properties", System.getProperties());

      // init sap java connection & repositiory
      LOG.info("Initialise SAP Connetion & Repository...");
      fRfc    = new AutoRFC();
      fObjMsg = new AutoObjectMessage();
      LOG.info("  >> Sucessfully done");

      // init jms queue
      LOG.info("Initialise JMS Queue...");
      fJmsQueue = new JmsQueue ( JmsQueue.RUN_ON_CLIENT,
                                 Jms.JNDI_JMS_QUEUE_FACTORY_NAME,
                                 Jms.JMS_INBOUND_QUEUE_NAME,
                                 JMS_SESSION_TRANSACTED,
                                 JMS_SESSION_ACKNOWLEDGE_MODE );
      LOG.info("  >> Sucessfully done");

      // init options
      LOG.info("Initialise SapInbound Trace Options...");
      fObjMsg.setSapTrace( sp.getInt (SapInbound.getProperties(),
                                      "sapinbound.trace",DEF_SAP_TRACE) );
      LOG.info("  >> Sucessfully done");

      // init connection ping intervall & retry period
      LOG.info("Initialise JMS & SAP Connection Ping Intervall & Retry Period ...");
      fSapConnPingInterval  = sp.getInt(SapInbound.getProperties(),"sap.connection.ping.intervall",DEF_CONNECTION_PING_INTERVAL);
      fSapConnRetryPeriod   = sp.getInt(SapInbound.getProperties(),"sap.connection.retry.period",DEF_CONNECTION_RETRY_PERIOD);
      fJmsConnRetryPeriod   = sp.getInt(SapInbound.getProperties(),"jms.connection.retry.period",DEF_CONNECTION_RETRY_PERIOD);
      LOG.info("  >> Sucessfully done");

      // end initialize
      LOG.info(">> Initialize SAP Inbound Client successfully done !");
    }
    catch (Exception ex)
    {
      LOG.fatal("Initialize SAP Inbound Client failed, Exit Application !");
      LOG.fatal("  Exception: " + ex.toString());
      System.exit(-1);
    }
  }

  //---------------------------------------------------------------------------
  // interface methods
  //---------------------------------------------------------------------------

  public void onMessage(Message message)
  {
    boolean rollbackMsg = false;

    try
    {
      if (message instanceof javax.jms.TextMessage)
      {
        LOG.warn( "onMessage - received & SKIPPED unhandled TextMessage: " +
                  ((javax.jms.TextMessage)message).getText() + " !" );
      }
      else if (message instanceof javax.jms.ObjectMessage)
      {
        LOG.debug("onMessage - received ObjectMessage, start processing...");
        rollbackMsg = fObjMsg.process((ObjectMessage)message, fRfc, fJmsQueue);
      }
      else
      {
        LOG.warn("onMessage - received & SKIPPED unknown message type !");
      }

    }

    catch(Exception ex)
    {
      ex.printStackTrace();
      LOG.error("onMessage - exception occured: " + ex.toString());
      try
      {
        ErrorVO err         = new ErrorVO();
        err.source          = "SapInbound" + SapInbound.getProgramId();
        err.employeeId      = null;
        err.employeeNotify  = ErrorVO.EMPL_NOTIFY_KEINE;
        err.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_DEFAULT;
        err.errorStat       = ErrorVO.ERROR_STAT_PENDENT;
        err.description     = "onMessage - exception occured: " + ex.toString();
        err.dataTypeCd      = ErrorVO.DATA_TYPE_CD_NO_DATA;
        err.data            = null;

        fObjMsg.saveError(err);
      }
      catch (Exception ex2)
      {
        // ignore
      }
    }

    finally
    {
      if ( JMS_SESSION_TRANSACTED )
      {
        try
        {
          if (rollbackMsg)
            this.fJmsQueue.getSession().rollback();
          else
            this.fJmsQueue.getSession().commit();
        }
        catch (Exception ex)
        {
          LOG.error("onMessage - exception during commit/rollback occured: " + ex.toString());
        }
      }
    }

  } // end method


  public void onException (javax.jms.JMSException ex)
  {
    LOG.warn  ("### onException - " + ex.toString());
    this.stopJms();

  } // end method

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void run()
  {
    try
    {
      //
      // init
      //
      setupSAPConnection();       // setup sap connections

      setupJMSConnection();       // setup sap connections

      cleanUpFailedErrorVOs();    // clean up error vo(s) (do this call
                                  // after jms setup is done successfully,
                                  // since chance to reach app server is greater)

      LOG.info("### SAP Inbound Client - Setup finished, start listening ###");


      //
      // check connections
      //
      while ( ! this.getStopFlag() )
      {
        // check connection to sap
        // (check connection to jms is done by the onException handler)
        this.wait(this.fSapConnRetryPeriod);
        if (! getStopFlag())
          this.pingSAPConnection();

        // establish sap connection
        if ( ! getStopFlag() && ! this.getSapAliveFlag() )
        {
          LOG.info("run - Shutdown JMS connection & restart establishing connections.");
          this.stopJms();
          this.setupSAPConnection();
        }

        // establish jms connection, do clean up check
        if ( ! getStopFlag() && ! this.getJmsAliveFlag() )
        {
          this.wait(this.fJmsConnRetryPeriod);      // wait befor retry
          SapDelegateInbound.removeInstance();      // important: remove old objects for lookup
          this.setupJMSConnection();                // start retry
          this.cleanUpFailedErrorVOs();             // clean up error vo(s)
          LOG.info("### SAP Inbound Client - Connections reestablished, start listening ###");
        }

      } // while

    }
    catch (Exception ex)
    {
      LOG.fatal("run - service terminates with exception: " + ex.toString());
    }
    finally
    {
      //
      // exit application gracefully
      //
      stopJms();
      fRfc.reset();
      LOG.info("### SAP Inbound Client - Shutdown application im progress ...");
    }

  } // end run

  //
  // public getter & setter (synchronized because possible call's
  // from main thread, shutdown thread and onException / onMessage handlers)
  //
  public synchronized void setStopFlag(boolean state)
  {
    // set flag
    LOG.info("setStopFlag - called");
    this.fStopFlag = state;

    // interrupt waiting thread
    if ( (state) && (fWaitThread != null) )
    {
      LOG.info("setStopFlag - interrupt waiting thread");
      this.fWaitThread.interrupt();
    }
  }

  public synchronized boolean getStopFlag()
  {
    return this.fStopFlag;
  }

  //---------------------------------------------------------------------------
  // private methods - connection handling
  //---------------------------------------------------------------------------

  private void setupJMSConnection ()
  {
    boolean  setup    = false;
    String   clientId = "SapInbound" + SapInbound.getProgramId();


    // init
    LOG.info("Setup JMS Connection (Receiver)...");

    // setup connection
    while ( ! getStopFlag() && ! setup )
    {
      try
      {
        // setup reliable receiver
        fJmsQueue.connectReliableReceiver(clientId);

        // setup listener & start listening
        this.fJmsQueue.getConnection().setExceptionListener(this);
        this.fJmsQueue.getReceiver().setMessageListener(this);
        this.fJmsQueue.getConnection().start();

        // setup done
        setup = true;
      }
      catch (Exception ex)
      {
        LOG.error("  setupJMSConnection - failed (" + ex.toString() + "), " +
                  "retry setup in " + this.fJmsConnRetryPeriod + " seconds...");

        this.wait(this.fJmsConnRetryPeriod);
      }
    } // end while

    // all done
    this.setJmsAliveFlag(true);
    if ( LOG.isInfoEnabled() )
      LOG.info("  >> JMS Connection successfully established to " +  Jms.JMS_INBOUND_QUEUE_NAME);

  } // end method


  // synchronize because possible call's from main thread and onException handler
  private synchronized void stopJms()
  {
    this.fJmsQueue.disconnect();
    this.setJmsAliveFlag(false);
  }


  private void setupSAPConnection()
  {
    boolean      setup    = false;
    String       echoText = "SapInbound Client - ProgramId: " + SapInbound.getProgramId();
    StringBuffer retMsg   = new StringBuffer();
    int          retCode  = 0;


    // init
    LOG.info("Setup SAP Connection ...");

    // setup (check) connection
    while ( ! getStopFlag() && ! setup )
    {
      if ( ! fRfc.isIntialized() )
      {
        fRfc.initialize();
      }

      retMsg.delete(0,retMsg.length());
      retCode = fRfc.testConnection(echoText,retMsg);

      if (retCode == RFC.RET_CODE_SUCCESS)
      {
        setup = true;
      }
      else
      {
        LOG.error("  setupSAPConnection - failed (" + retMsg.toString() + "), " +
                  "retry setup in " + this.fSapConnRetryPeriod + " seconds...");

        fRfc.reset();

        this.wait(this.fSapConnRetryPeriod);
      }

    } // while

    // all done
    this.setSapAliveFlag(true);
    if ( LOG.isInfoEnabled() )
      LOG.info("  >> SAP Connection successfully established to " +  retMsg.toString());

  } // end method


  private void pingSAPConnection()
  {
    String       echoText = "Ping from SapInbound Client - ProgramId: " + SapInbound.getProgramId();
    StringBuffer dummy    = new StringBuffer();

    try
    {
      if ( fRfc.testConnection(echoText,dummy) != RFC.RET_CODE_SUCCESS )
      {
        LOG.warn("### pingSAPConnection - lost connection to SAP !");
        this.setSapAliveFlag(false);
      }
      else
      {
        LOG.debug("pingSAPConnection - connection to SAP is alive.");
      }
    }
    catch (Exception ex)
    {
      LOG.warn("### pingSAPConnection - lost connection to SAP !");
      this.setSapAliveFlag(false);
    }

  } // end method


  private void wait(int seconds)
  {
    try
    {
      fWaitThread = Thread.currentThread();
      Thread.sleep(seconds * 1000);
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
    finally
    {
      fWaitThread = null;
    }
  } // end method


  //
  // private methods - getter & setter (synchronize because possible call's
  // from main thread and onException / onMessage handlers)
  //

  private synchronized void setSapAliveFlag(boolean state)
  {
    this.fSapAliveFlag = state;
  }
  private synchronized boolean getSapAliveFlag()
  {
    return this.fSapAliveFlag;
  }

  private synchronized void setJmsAliveFlag(boolean state)
  {
    this.fJmsAliveFlag = state;
  }
  private synchronized boolean getJmsAliveFlag()
  {
    return this.fJmsAliveFlag;
  }

  //---------------------------------------------------------------------------
  // private methods - misc
  //---------------------------------------------------------------------------

  /**
   * Diese Methode sucht im lokalen Verzeichnis nach nicht verarbeiteten
   * ErrorVO Meldungen und versucht, diese nochmals zu verarbeiten.
   *
   * Dieser Fall kann auftreten, falls beim Aufruf von saveError() der
   * AppServer nicht verfügbar ist und das ErrorVO nur noch lokal gesichert
   * wird.
   */
  private void cleanUpFailedErrorVOs()
  {
    ErrorFile   ef    = null;
    List        list  = null;
    Iterator    it    = null;
    String      fn    = null;
    ErrorVO     err   = null;

    try
    {
      // start clean up
      LOG.info ("Clean up error vo(s) ....");

      ef = new ErrorFile ( SapInbound.getProperties().getProperty("sapinbound.data.path"),
                           SapInbound.getProgramIdentifier() );

      list = ef.getList();
      it   = list.iterator();
      LOG.info ("  " + list.size() + " pending error vo(s) found to clean up");

      while ( it.hasNext() )
      {
        try
        {
          fn  = (String) it.next();
          err = ef.readObjectFile(fn);
          SapDelegateInbound.getInstance().receiveError(err);
          ef.rename(fn,ef.EXT_CLEANUP_OK);
          LOG.info("  process file " + fn + " ... ok");
        }
        catch (Exception ex)
        {
          ef.rename(fn,ef.EXT_CLEANUP_NOK);
          LOG.info("  process file " + fn + " ... nok");
        }
      } // end while

      LOG.info ("  >> Clean up error vo(s) successfully done");

    }
    catch (Exception ex)
    {
      LOG.error ( "cleanUpFailedErrorVOs - clean up failed with exception: "
                  + ex.getMessage() );
    }
  }


  private void logProperties(String title, Properties prop)
  {
    Enumeration eNames = prop.propertyNames();
    TreeSet     set  = new TreeSet();
    String      key;

    if ( LOG.isInfoEnabled() )
    {
      LOG.info(title);
      while ( eNames.hasMoreElements() )
      {
        key = eNames.nextElement().toString();
        set.add(key);
      }

      for (Iterator i = set.iterator(); i.hasNext(); )
      {
        key = (String) i.next();
        LOG.info("  " + Convert.toSapStr(key,40) + ": " + prop.getProperty(key));
      }

    }
  } // end method


  private void logParams()
  {
    if ( LOG.isInfoEnabled() )
    {
      LOG.info("Params");
      LOG.info("   Program ID          : " + SapInbound.getProgramId()  );
      LOG.info("   Program Action      : " + SapInbound.getProgramAction() );
      LOG.info("   Program Environment : " + SapInbound.getProgramEnv() );
      LOG.info("   Properties File     : " + SapInbound.getPropertiesFile() );
    }
  } // end method


  private void logBuildInfo()
  {
    if ( LOG.isInfoEnabled() )
    {
      LOG.info("Build Info");
      LOG.info("   CVS Tag      : " + BuildInfo.CVS_TAG  );
      LOG.info("   Build Id     : " + BuildInfo.BUILD_ID );      
      LOG.info("   Default Provider Url used, when java.naming.provider.url is set to <default>:" );
      LOG.info("   Provider Url : " + ch.ige.util.BuildInfo.PROVIDER_URL );
    }
  } // end method

} // end class


