package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import org.apache.log4j.*;
import org.apache.log4j.xml.DOMConfigurator;
import com.sap.mw.jco.*;

import ch.ige.ses.client.saputil.*;

/**
 * SAP Inbound Client für die Kommunikation zwischen J2EE und SAP.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @author  T. Iten
 * @version $Id: SapInbound.java,v 1.25 2007/05/24 15:18:24 iten Exp $
 */
public class SapInbound
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger("ch.ige.ses");

  public static final String[] PROGRAM_ACTION_TYPES = {"auto","man","help"};
  public static final String[] PROGRAM_ENV_TYPES    = {"dev","test","acc","prod"};

  private static final String  DEF_PROGRAM_NAME      = "SapInbound";
  private static final String  DEF_PROGRAM_ID        = "00";
  private static final String  DEF_PROGRAM_ACTION    = PROGRAM_ACTION_TYPES[0];
  private static final String  DEF_PROGRAM_ENV       = PROGRAM_ENV_TYPES[0];
  private static final String  DEF_PROPERTIES_FILE   = "C:/jp/projects/ses/install/programme/ige/ses/config/SapInbound.properties";

  //
  // fields
  //
  private static String      fProgramId      = DEF_PROGRAM_ID;         // params
  private static String      fProgramAction  = DEF_PROGRAM_ACTION;
  private static String      fProgramEnv     = DEF_PROGRAM_ENV;
  private static String      fPropertiesFile = DEF_PROPERTIES_FILE;

  private static Properties  fProperties;                              // properties

  private static  ManualMode fMan  = null;    // application modes
  private static  AutoMode   fAuto = null;    // (outside main because of shutdown method)


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public SapInbound()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Gibt den Program Namen zurück. Dieser ist Konstant und kann nicht von
   * aussen (via Parameter) gesetzt werden.
   *
   * @return program name
   */
  public static String getProgramName()
  {
    return  DEF_PROGRAM_NAME;
  }

  /**
   * Gibt den Programparameter ProgramId zurück.
   *
   * @return program id
   */
  public static String getProgramId()
  {
    return fProgramId;
  }

  /**
   * Gibt den Programparameter ProgramAction zurück.
   *
   * @return program action
   */
  public static String getProgramAction()
  {
    return fProgramAction;
  }

  /**
   * Gibt den Programparameter ProgramEnv zurück.
   *
   * @return program environment
   */
  public static String getProgramEnv()
  {
    return fProgramEnv;
  }

  /**
   * Gibt den Programparameter ProgramFile zurück.
   *
   * @return properties file name
   */
  public static String getPropertiesFile()
  {
    return fPropertiesFile;
  }

  /**
   * Gibt die Properties zurück.
   *
   * @return properties
   */
  public static Properties getProperties()
  {
    return fProperties;
  }

  /**
   * Gibt den eindeutigen Program Identifier zurück. Dieser besteht aus dem
   * Namen und der Id. Beispiel: SapInbound01
   *
   * @return program identifier
   */
  public static String getProgramIdentifier()
  {
    return  DEF_PROGRAM_NAME + fProgramId;
  }

  /**
   * Zeigt an, ob das Programm in der produktiven Umgebung läuft.
   *
   * @return    Produktion (true) oder Entwicklung/Integration/Abnahme (false)
   */
  public static boolean isProduction()
  {
    if (fProgramEnv.equalsIgnoreCase(PROGRAM_ENV_TYPES[3]))
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Programmhilfe - zeigt in erster Linie, mit welchen Parametern
   * das Program gestartet werden kann
   */
  private void printUsage()
  {
    System.out.println("SAP Inbound Client - Usage:");
    System.out.println();

    System.out.println("sapinbound [-p ProgramId] [-e Environment] [-a Action]");
    System.out.println("           [-f File] ");
    System.out.println("           [-h] [-?] [help] [?]");
    System.out.println();

    System.out.println("  -p     ProgramId welche die Applikation eindeutig identifiziert ");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ID);

    System.out.println("  -e     Environment in der das Programm ablaeuft [dev,test,acc,prod]");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ENV);

    System.out.println("  -a     Action welche das Programm ausfuehrt [auto,man,help]");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ACTION);

    System.out.println("  -f     Pfad- und Filename der Property Datei ");
    System.out.println("         Defaultwert = " + DEF_PROPERTIES_FILE);

    System.out.println();
    System.out.println("  -h, -?, help, ? ");
    System.out.println("         SAP Inbound Client - Help wird angezeigt. ");

  } // end method


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args   Programm Argumente
   * @throws Exception if load fails
   */
  private void loadParams(String[] args) throws Exception
  {
    int i = 0;

    try
    {
      while (i < args.length)
      {
        if  ( args[i].equals("-p") )
        {
          fProgramId = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-e") ) {
          fProgramEnv = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-a") ) {
          fProgramAction = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-f") ) {
          fPropertiesFile = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-h")   || args[i].equals("-?") ||
                    args[i].equals("help") || args[i].equals("?") ) {
          fProgramAction = "help";
          i++;
        } else {
          throw new Exception();
        }
      } // end while

    }
    catch (Exception ex)
    {
      throw new Exception ("getParams - Das Argument (" + args[i] + ") ist ungültig oder unvollständig !");
    }

  } // end method


  /**
   * Einlesen der Programmeigenschaften aus den Property Datei.
   * @throws Exception if load fails
   */
  private void loadProperties() throws Exception
  {
    SapProperties sp = new SapProperties();

    fProperties = sp.load(fPropertiesFile,fProgramEnv,PROGRAM_ENV_TYPES);
    fProperties = sp.setPlaceholderValue(fProperties,sp.PLACEHOLDER_PID,fProgramId);

    sp.setSystemProperties(fProperties);

  } // end method


  //***************************************************************************
  // main
  //***************************************************************************
  public static void main(String[] args)
  {
    SapInbound    si   = null;

    try
    {
      //
      // init
      //
      si = new SapInbound();

      // load params
      si.loadParams(args);

      // show usage and exit application
      if ( si.fProgramAction.equals("help") )
      {
        si.printUsage();
        System.exit(0);
      }

      // load properties and set java system params
      si.loadProperties();

      // start logging
      DOMConfigurator.configure
        ( new URL(si.fProperties.getProperty("log.configfile.url")) );

      // set jco trace options
      JCO.setTracePath ( fProperties.getProperty("jco.trace.path") );
      JCO.setTraceLevel( Integer.parseInt(fProperties.getProperty("jco.trace.level")) );

      //
      // run application
      //
      if ( si.fProgramAction.equals("auto") )         // start automatic
      {                                               // deamon process
        LOG.info("###");
        LOG.info("### SAP Inbound Client - Startup Application ...");
        LOG.info("###");

        fAuto = new AutoMode();
        fAuto.run();

        LOG.info("### SAP Inbound Client - Shutdown Application successfully done. Good bye, have a nice day.");
      }

      else if ( si.fProgramAction.equals("man") )     // start manual mode
      {                                               // for test purpose
        fMan = new ManualMode();
        fMan.run();
      }


      //
      // end of application
      //
      System.exit(0);

    }
    catch (Exception ex)
    {
      if ( fAuto != null )
      {
        LOG.fatal("### SAP Inbound Client failed, Exit Application !");
        LOG.fatal("### Exception: " + ex.toString());
      }
      else
      {
        System.err.println("### SAP Inbound Client failed, Exit Application !");
        System.err.println("### Exception: " + ex.toString());
        System.err.println("### Stack Trace:");
        ex.printStackTrace();
      }

      System.exit(-1);
    }

  } // end main

  //***************************************************************************
  // shutdown (wird von JWinSvc aufgerufen wird, um den Service zu stoppen)
  //***************************************************************************

  public static void shutdown()
  {
    if (fAuto != null)
    {
      LOG.info("shutdown - stop windows service");
      fAuto.setStopFlag(true);
      fAuto = null;
    }
  }


} // end SapInbound


