package ch.ige.ses.client.sapoutbound;

import java.util.*;
import java.io.*;
import java.net.URL;
import org.apache.log4j.*;
import org.apache.log4j.xml.DOMConfigurator;
import com.sap.mw.jco.*;

import ch.ige.ses.client.saputil.*;

/**
 * SAP Outbound Server für die Kommunikation zwischen SAP und J2EE.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapOutbound.java,v 1.19 2007/11/08 13:41:51 iten Exp $
 */
public class SapOutbound
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger("ch.ige.ses");

  public static final String[] PROGRAM_ACTION_TYPES = {"auto","man","help"};
  public static final String[] PROGRAM_ENV_TYPES    = {"dev","test","acc","prod"};

  private static final String  DEF_PROGRAM_NAME      = "SapOutbound";
  private static final String  DEF_PROGRAM_ID        = "00";
  private static final String  DEF_PROGRAM_ACTION    = PROGRAM_ACTION_TYPES[0];
  private static final String  DEF_PROGRAM_ENV       = PROGRAM_ENV_TYPES[0];
  private static final String  DEF_PROPERTIES_FILE   = "C:/jp/projects/ses/install/programme/ige/ses/config/SapOutbound.properties"; 

  //
  // fields
  //
  private static String     fProgramId      = DEF_PROGRAM_ID;         // params
  private static String     fProgramAction  = DEF_PROGRAM_ACTION;
  private static String     fProgramEnv     = DEF_PROGRAM_ENV;
  private static String     fPropertiesFile = DEF_PROPERTIES_FILE;

  private static Properties fProperties;                              // properties

  private static  ManualMode fMan  = null;    // application modes
  private static  AutoMode   fAuto = null;    // (outside main because of shutdown method)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public SapOutbound()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Gibt den Program Namen zurück. Dieser ist Konstant und kann nicht von
   * aussen (via Parameter) gesetzt werden.
   *
   * @return program name
   */
  public static String getProgramName()
  {
    return  DEF_PROGRAM_NAME;
  }


  /**
   * Gibt den Programparameter ProgramId zurück.
   *
   * @return program id
   */
  public synchronized static String getProgramId()
  {
    return fProgramId;
  }

  /**
   * Gibt den Programparameter ProgramAction zurück.
   *
   * @return action
   */
  public static String getProgramAction()
  {
    return fProgramAction;
  }

  /**
   * Gibt den Programparameter ProgramEnv zurück.
   *
   * @return environment
   */
  public static String getProgramEnv()
  {
    return fProgramEnv;
  }

  /**
   * Gibt den Programparameter PropertiesFile zurück.
   *
   * @return prperties file name
   */
  public static String getPropertiesFile()
  {
    return fPropertiesFile;
  }

  /**
   * Gibt die Properties zurück.
   *
   * @return properties
   */
  public static Properties getProperties()
  {
    return fProperties;
  }

  /**
   * Gibt den eindeutigen Program Identifier zurück. Dieser besteht aus dem
   * Namen und der Id. Beispiel: SapOutbound01
   *
   * @return program identifier
   */
  public static String getProgramIdentifier()
  {
    return  DEF_PROGRAM_NAME + fProgramId;
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Programmhilfe - zeigt in erster Linie, mit welchen Parametern
   * das Program gestartet werden kann
   */
  private void printUsage()
  {
    System.out.println("SAP Outbound Server - Usage:");
    System.out.println();

    System.out.println("sapoutbound [-p ProgramId] [-e Environment] [-a Action]");
    System.out.println("            [-f File] ");
    System.out.println("            [-h] [-?] [help] [?]");
    System.out.println();

    System.out.println("  -p     ProgramId welche die Applikation eindeutig identifiziert ");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ID);

    System.out.println("  -e     Environment in der das Programm ablaeuft [dev,test,acc,prod]");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ENV);

    System.out.println("  -a     Action welche das Programm ausfuehrt [auto,man,help]");
    System.out.println("         Defaultwert = " + DEF_PROGRAM_ACTION);

    System.out.println("  -f     Pfad- und Filename der Property Datei ");
    System.out.println("         Defaultwert = " + DEF_PROPERTIES_FILE);

    System.out.println();
    System.out.println("  -h, -?, help, ? ");
    System.out.println("         SAP Outbound Server - Help wird angezeigt. ");

  } // end method


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args   Programm Argumente
   * @throws Exception if load fails
   */
  private void loadParams(String[] args) throws Exception
  {
    int i = 0;

    try
    {
      while (i < args.length)
      {
        if  ( args[i].equals("-p") )
        {
          fProgramId = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-e") ) {
          fProgramEnv = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-a") ) {
          fProgramAction = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-f") ) {
          fPropertiesFile = args[i+1];
          i = i+2;
        } else if ( args[i].equals("-h")   || args[i].equals("-?") ||
                    args[i].equals("help") || args[i].equals("?") ) {
          fProgramAction = "help";
          i++;
        } else {
          throw new Exception();
        }
      } // end while

    }
    catch (Exception ex)
    {
      throw new Exception ("getParams - Das Argument (" + args[i] + ") ist ungültig oder unvollständig !");
    }

  } // end loadParams


  /**
   * Einlesen der Programmeigenschaften aus den Property Datei.
   * @throws Exception if load fails
   */
  private void loadProperties() throws Exception
  {
    SapProperties sp = new SapProperties();

    fProperties = sp.load(fPropertiesFile,fProgramEnv,PROGRAM_ENV_TYPES);
    fProperties = sp.setPlaceholderValue(fProperties,sp.PLACEHOLDER_PID,fProgramId);

    sp.setSystemProperties(fProperties);

  } // loadProperties


  //***************************************************************************
  // main
  //***************************************************************************
  public static void main(String[] args)
  {
    SapOutbound so   = null;


    try
    {
      //
      // init
      //
      so = new SapOutbound();

      // load params
      so.loadParams(args);

      // show usage and exit application
      if ( so.fProgramAction.equals("help") )
      {
        so.printUsage();
        System.exit(0);
      }

      // load properties and set java system params
      so.loadProperties();

      // start logging
      DOMConfigurator.configure
        ( new URL(so.fProperties.getProperty("log.configfile.url")) );


      //
      // run application
      //
      if ( so.fProgramAction.equals("auto") )         // start automatic
      {                                               // deamon process
        LOG.info("###");
        LOG.info("### SAP Outbound Server - Startup Application ...");
        LOG.info("###");

        fAuto = new AutoMode();
        fAuto.run();

        LOG.info("### SAP Outbound Server - Shutdown Application successfully done. Good bye, have a nice day.");

      }

      else if ( so.fProgramAction.equals("man") )     // start manual mode
      {                                               // for test purpose
        fMan = new ManualMode();
        fMan.run();
      }

      //
      // end of application
      //
      System.exit(0);

    }

    catch (Exception ex)
    {
      if ( fAuto != null )
      {
        LOG.fatal("### SAP Outbound Server failed, Exit Application !");
        LOG.fatal("### Exception: " + ex.toString());
      }
      else
      {
        System.err.println("### SAP Outbound Server failed, Exit Application !");
        System.err.println("### Exception: " + ex.toString());
        System.err.println("### Stack Trace:");
        ex.printStackTrace();
      }

      System.exit(-1);
    }

  } // end main

  //***************************************************************************
  // shutdown (wird von JWinSvc aufgerufen wird, um den Service zu stoppen)
  //***************************************************************************

  public static void shutdown()
  {
    if (fAuto != null)
    {
      LOG.info("shutdown - stop windows service");
      fAuto.setStopFlag(true);
    }
  }

} // end SapOutbound


