package ch.ige.ses.client.sapoutbound;

import java.util.*;
import com.sap.mw.jco.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.SapProperties;

/**
 * Initialisiert, startet und stoppt die SAP Outbount Server.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapServerController.java,v 1.14 2006/12/11 12:43:36 iten Exp $
 */
public class SapServerController implements
    JCO.ServerExceptionListener,                // handle server exceptions
    JCO.ServerStateChangedListener              // inform about state changes
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapServerController.class);

  //
  // fields
  //
  private int           fMaxServers;            // maximal number of servers
  private String        fJCoSrvGwHost;          // jco server - gateway host
  private String        fJCoSrvGwServ;          // jco server - gateway service number
  private String        fJCoSrvProgId;          // jco server - program id

  private boolean       fJCoTrace    = false;   // jco trace enabled / disabled

  private IRepository   fJCoRepository;         // jco repositiory

  private SapServer     fServerList[];          // list of sap servers
  private boolean       fServerAlive = false;   // true if sap server's are connected


  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public SapServerController() throws JCO.Exception
  {
    Properties prop = SapOutbound.getProperties();

    LOG.info("Initialise Sap Server Controller");

    // get server properties
    LOG.info("  get properties");
    this.fMaxServers   = Integer.parseInt(prop.getProperty("max_servers"));

    this.fJCoSrvGwHost = prop.getProperty("jco.server.gwhost");
    this.fJCoSrvGwServ = prop.getProperty("jco.server.gwserv");
    this.fJCoSrvProgId = prop.getProperty("jco.server.progid");


    // add a connection pool to the specified system
    LOG.info("  add a connection pool to the specified system");
    JCO.addClientPool
      ( prop.getProperty("jco.clientpool.key"),               // alias for this pool
                                                              // max. number of connections
        Integer.parseInt(prop.getProperty("jco.clientpool.max_connections")),
        prop.getProperty("jco.clientpool.client"),            // sap client
        prop.getProperty("jco.clientpool.user"),              // userid
        SapProperties.decodeProperty                          // password
          (prop.getProperty("jco.clientpool.passwd")),
        prop.getProperty("jco.clientpool.lang"),              // language
        prop.getProperty("jco.clientpool.r3name"),            // host name
        prop.getProperty("jco.clientpool.group") );           // system number


    // create a new repository
    LOG.info("  create a new repository");
    this.fJCoRepository = JCO.createRepository
      ( prop.getProperty("jco.repository.name"),
        prop.getProperty("jco.clientpool.key") );


    // create list of sap servers
    LOG.info("  create list of sap servers");
    this.fServerList = new SapServer[this.fMaxServers];


    // set jco trace options
    LOG.info("  set trace options");
    JCO.setTraceLevel( Integer.parseInt(prop.getProperty("jco.trace.level")) );
    JCO.setTracePath ( prop.getProperty("jco.trace.path") );

    if (JCO.getTraceLevel() > 0)              // jco trace is enabled
      this.fJCoTrace = true;


    // add listener
    if (prop.getProperty("jco.server.add.exceptionlistener").equals("1"))
    {
      LOG.info("  addServerExceptionListener");
      JCO.addServerExceptionListener(this);
    }

    if (prop.getProperty("jco.server.add.statechangedlistener").equals("1"))
    {
      LOG.info("  addServerStateChangedListener");
      JCO.addServerStateChangedListener(this);
    }

  } // end constructor


  //---------------------------------------------------------------------------
  // interface methods
  //---------------------------------------------------------------------------

  public void serverExceptionOccurred(JCO.Server server, Exception ex)
  {
    SapServer  sapsrv = (SapServer) server;

    // log exception
    LOG.warn
      ( sapsrv.fSapServerName +
        "serverExceptionOccurred - exception in server " + server.getProgID() + ":" +
        ex.getMessage().substring(0,53) + "..." );

    // stop server connections to sap
    stopServers();
  }


  public void serverStateChangeOccurred(JCO.Server server, int old_state, int new_state)
  {
    SapServer  sapsrv = (SapServer) server;
    String     msg    = sapsrv.fSapServerName +
                        "serverStateChangeOccurred - changed state from [";


    if ((old_state & JCO.STATE_STOPPED    ) != 0) msg = msg + " STOPPED ";
    if ((old_state & JCO.STATE_STARTED    ) != 0) msg = msg + " STARTED ";
    if ((old_state & JCO.STATE_LISTENING  ) != 0) msg = msg + " LISTENING ";
    if ((old_state & JCO.STATE_TRANSACTION) != 0) msg = msg + " TRANSACTION ";
    if ((old_state & JCO.STATE_BUSY       ) != 0) msg = msg + " BUSY ";

    msg = msg + "] to [";

    if ((new_state & JCO.STATE_STOPPED    ) != 0) msg = msg + " STOPPED ";
    if ((new_state & JCO.STATE_STARTED    ) != 0) msg = msg + " STARTED ";
    if ((new_state & JCO.STATE_LISTENING  ) != 0) msg = msg + " LISTENING ";
    if ((new_state & JCO.STATE_TRANSACTION) != 0) msg = msg + " TRANSACTION ";
    if ((new_state & JCO.STATE_BUSY       ) != 0) msg = msg + " BUSY ";
    msg = msg + "]";

    if ( LOG.isDebugEnabled() )
    {
      LOG.debug(msg);
    }
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  // synchronize because possible call's caller and serverExceptionOccurred handler

  public synchronized void startServers()
  {
    int   i = 0;

    try
    {
      // start server's
      for (i = 0; i < this.fMaxServers; i++)
      {
        // create server
        this.fServerList[i] = new SapServer
          ( this.fJCoSrvGwHost, this.fJCoSrvGwServ,
            this.fJCoSrvProgId, this.fJCoRepository, i );

        // set jco trace
        this.fServerList[i].setTrace(this.fJCoTrace);

        // start listening
        this.fServerList[i].start();

        // log info
        if ( LOG.isDebugEnabled() )
          LOG.debug("startServers - SapServer #" + i + " started successfully.");
      } // end for

      // server's successfully connected to sap
      setServerAliveFlag(true);

    }
    catch (Exception ex)
    {
      LOG.error("startServers - SapServer #" + i + " could NOT be started !");
      LOG.error("  Exception: " + ex.getMessage());
      stopServers();
    }

  } // end method


  public synchronized void stopServers()
  {

    // stop servers
    for (int i = 0; i < this.fMaxServers; i++)
    {
      if ( fServerList[i] != null )
      {
        try
        {
          // note: after a stop, a server can't be restarted (see jco documentation)
          this.fServerList[i].stop();
          if ( LOG.isDebugEnabled() )
            LOG.debug("stopServers - SapServer #" + i + " stopped.");
        }
        catch (Exception ex)
        {
          if ( LOG.isDebugEnabled() )
            LOG.debug("stopServers - exception during stop SapServer #" + i + " occured.");
        }
        finally
        {
          fServerList[i] = null;
        }
      } // end if
    } // end for

    // server's are disconnected from sap
    setServerAliveFlag(false);

  } // end method


  public synchronized boolean getServerAliveFlag()
  {
    return this.fServerAlive;
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private synchronized void setServerAliveFlag(boolean state)
  {
    this.fServerAlive = state;
  }


} // end class


