package ch.ige.ses.client.saprequest;

import java.io.Serializable;
import javax.jms.*;
import javax.naming.*;

import ch.ige.ses.util.Jms;
import ch.ige.ses.util.JmsQueue;
import ch.ige.ses.exception.SapCallException;

/**
 * Diese Klasse kappselt Client Request's an SAP.
 *
 * Dabei wird via JMS eine Nachricht an den SapInbound Service gesendet.
 * Dieser führt den entsprechenden SAP Aufruf aus und sendet seine Antwort
 * über eine temporäre Queue zurück (Request / Reply Pattern).
 *
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                           ACHTUNG                           !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                                                             !
 *    !  Diese Klasse verwendet J2EE Restrictions und darf nur      !
 *    !  von Java Client Programmen oder Servlets verwendet werden. !
 *    !                                                             !
 *    ! Da diese Klasse auch von einem Servlet verwendet wird, darf !
 *    ! nicht geloggt werden (Serverlogging oder lokales Logging...)!
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapRequest.java,v 1.11 2006/12/29 07:56:54 iten Exp $
 */
public class SapRequest
{
  //
  // const
  //
  private   static final boolean  JMS_SESSION_TRANSACTED        = false;
  private   static final int      JMS_SESSION_ACKNOWLEDGE_MODE  = Session.AUTO_ACKNOWLEDGE;
  protected static final int      JMS_TIME_TO_LIVE              = 20;

  //
  // fields
  //
  private JmsQueue  fJmsQueue = null;
  private boolean   fTimeout  = false;    // last request ended with a timeout

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Erzeugt die Klasse SapRequest mit der Aufrufe zum SAP System im
   * Reply / Request Modus abgesetzt werden können.
   */
  public SapRequest()
  {
    fJmsQueue = new JmsQueue ( 
        JmsQueue.RUN_ON_CLIENT,
        Jms.JNDI_JMS_QUEUE_FACTORY_NAME,
        Jms.JMS_INBOUND_QUEUE_NAME,
        this.JMS_SESSION_TRANSACTED,
        this.JMS_SESSION_ACKNOWLEDGE_MODE );
  }

  //---------------------------------------------------------------------------
  // protected methods
  //---------------------------------------------------------------------------

  /**
   * Erstellt Verbindung zur JMS Queue welche zur Kommunikation zum SAP
   * System verwendet wird. Muss vor dem ersten Aufruf von request()
   * ausgeführt werden.
   *
   * @throws SapCallException Fehler beim Verbindungsaufbau zum SAP System
   */
  protected void connect() throws SapCallException
  {
    connect(JMS_TIME_TO_LIVE);
  }

  /**
   * Erstellt Verbindung zur JMS Queue welche zur Kommunikation zum SAP
   * System verwendet wird. Muss vor dem ersten Aufruf von request()
   * ausgeführt werden.
   *
   * @param   timeToLive        Gültigkeit der Meldung und temp. Queue [sec]
   * @throws  SapCallException  Fehler beim Verbindungsaufbau zum SAP System
   */
  protected void connect(int timeToLive) throws SapCallException
  {
    try
    {
      fJmsQueue.connectRequestor(timeToLive);

    }
    catch (Exception ex)
    {
      throw new SapCallException ( "Fehler beim Verbindungsaufbau zum SAP System",
                                   ex.getMessage() );
    }
  }

  /**
   * Beendet Verbindung zur JMS Queue. Muss nach dem letzten Aufruf von
   * request() ausgeführt werden.
   */
  protected void disconnect()
  {
    fJmsQueue.disconnect();
  }


  /**
   * Ausführen eines Request's an das SAP System.
   *
   * @param request   Objekt Message welche an SAP (SapInbound Service) übertragen werden soll
   * @param type      Typ der Message gemäss Definition in ch.ige.ses.util.JMS
   * @param desc      Beschreibung der Message gemäss Definition in ch.ige.ses.util.JMS
   * @return          Antwort vom SapInboundService
   * @throws SapCallException Kommuniktionsfehler mit dem SAP System
   */
  protected SapReturn request (Serializable request, int type, String desc)
      throws SapCallException

  {
    ObjectMessage om;
    Object        reply;

    try
    {
      fTimeout = false;

      om = fJmsQueue.getSession().createObjectMessage(request);
      setMessageProperties(om,type,desc);

      reply = fJmsQueue.request(om);

      if (reply == null)
      {
        fTimeout = true;
        throw new Exception();
      }

      return (SapReturn) reply;

    }
    catch (Exception ex)
    {
      if (fTimeout)
      {
        throw new SapCallException ( "Fehler bei der Kommunikation mit dem SAP System",
                                     "Timeout (" + fJmsQueue.getTimeToLive() + " sec) wurde überschritten");
      }
      else
      {
        throw new SapCallException ( "Fehler bei der Kommunikation mit dem SAP System",
                                     ex.getMessage() );
      }
    }
  }


  /**
   * Gibt an ob beim letzten request das Timeout überschritten wurde
   * @return true, falls Timeout überschritten wurde
   */
  public boolean timeoutOccured()
  {
    return fTimeout;
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void setMessageProperties (Message m, int type, String description)
      throws JMSException
  {
    m.setIntProperty    ( Jms.JMS_PROP_NAME_MSG_TYPE, type );
    m.setStringProperty ( Jms.JMS_PROP_NAME_MSG_DESC, description );
  }

} // end class

