package ch.ige.ses.client.saputil;

import java.io.*;
import java.util.*;
import java.text.*;
import org.apache.log4j.*;

import ch.ige.ses.vo.ses.*;

/**
 * Error File
 * Diese Klasse speichert die Nutzdaten eines ErrorVO als Textdatei und wird im
 * Falle das die Daten nicht in der SES Error Tabelle gespeichert werden können
 * verwendet.
 * Im weiteren wird das ErrorVo auch als Objekt gespeichert, so dass bei einem
 * wiederaufbau der Verbindung die nicht verarbeiteten daten wieder eingelesen
 * und weiterverarbeitet werden können.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: ErrorFile.java,v 1.9 2006/12/13 14:54:56 iten Exp $
 */
public class ErrorFile
{
  //
  // const
  //
  private static final    Logger  LOG             = Logger.getLogger(ErrorFile.class);
  private static final    String  DEF_IDENT       = "SesErrorFile";
  private static final    String  DEF_PATH        = "c:/data";
  private static final    int     MAX_CNT         = 10;

  private static final    String  EXT_TEXT_FILE   = ".txt";   // text and opbject
  private static final    String  EXT_OBJECT_FILE = ".obj";   // file extension

  public static final     String  EXT_CLEANUP_OK  = ".ok";    // additional extension
  public static final     String  EXT_CLEANUP_NOK = ".nok";   // after clean up


  //
  // fields
  //
  private String  fPath       = DEF_PATH;

  private String  fTextName   = null;         // name of text file
  private String  fObjectName = null;         // name of object file

  private String  fIdent      = DEF_IDENT;    // unique file name identifier
                                              // used to generate uniqe file
                                              // names for each service


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public ErrorFile()
  {
    // lazy bone
  }

  public ErrorFile(String path, String identifier)
  {
    File file = new File(path);

    if (file.exists())
    {
      fPath = path;
    }
    else
    {
      LOG.error( "constructor - the given path (" + path + ") doesn't exist, " +
                 "application is using the default path (" + fPath + ")" );
    }

    this.fIdent = identifier;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Save ErrorVO to local file system. Two Files will be generated, a text
   * file and an object file. the have the same name but different extensions.
   *
   * @param err ErrorVO to save
   */
  public void save(ErrorVO err)
  {
    int step = 0;

    try
    {
      // init
      step++;
      setFilename();

      // save text file
      step++;
      writeTextFile(err);
      if ( LOG.isDebugEnabled() )
        LOG.debug("save - ErrorVO saved to local file " + getTextFilename() );

      // save object
      step++;
      writeObjectFile(err);
      if ( LOG.isDebugEnabled() )
        LOG.debug("save - ErrorVO saved to local file " + getObjectFilename() );
    }
    catch (Exception ex)
    {
      if (step < 3)
      {
        LOG.fatal ( "save - could not save ErrorVO to local file " + getTextFilename() + ", " +
                    "exception is " + ex.getMessage() );
      }
      else
      {
        LOG.fatal ( "save - could not save ErrorVO to local object file " + getObjectFilename() + ", " +
                    "exception is " + ex.getMessage() );
      }
    } // end catch
  } // end method


  /**
   * Return list of filenames (incl. path) of unprocessed object files.
   *
   * @return  ArrayList with filenames
   */
  public List getList()
  {
    ArrayList list = new ArrayList();   // list with unprocessed object files
                                        // (path- and name of file)
    File[]    fa   = null;              // file array with all files and directories

    // get list
    fa = new File(fPath).listFiles();

    // exit if list is empty
    if (fa == null)
      return list;

    // filter object files
    for (int i = 0; i < fa.length; i++)
    {
      if ( ( fa[i].isFile() ) &&
           ( fa[i].getName().indexOf(EXT_CLEANUP_OK)  == -1) &&
           ( fa[i].getName().indexOf(EXT_CLEANUP_NOK) == -1) &&
           ( fa[i].getName().indexOf(fIdent)          != -1) &&
           ( fa[i].getName().indexOf(EXT_OBJECT_FILE) != -1) )
      {
        list.add(fa[i].getPath());
      }
    }

    // return found object file names
    return list;

  } // end method


  /**
   * Read ErrorVO object from local file system and return original object.
   *
   * @param   objectFilename  path- and filename of object file
   * @return                  error vo object
   * @throws Exception        unexpected error occured
   */
  public ErrorVO readObjectFile(String objectFilename) throws Exception
  {
    FileInputStream   fis  = new FileInputStream (objectFilename);
    ObjectInputStream in    = new ObjectInputStream(fis);
    ErrorVO           err   = null;

    err = (ErrorVO) in.readObject();
    in.close();

    return err;
  }


  /**
   * Rename object & text file. The rename will append the cleanupExt
   * to the existing filename to mark, thaht the file was already touched
   * by the cleanup process.
   *
   *    implementation detail: get text filename from object filename
   *                            0 1 2 3 4 5 6 7 8
   *      objectFilename     ..\H E L L O . O B J     len = 9
   *      textFilename       ..\H E L L O . T X T     len = 9
   *
   * @param     objectFilename    path- and filename of object file
   * @param     cleanupExt        additional extension
   *                              [EXT_CLEANUP_OK or EXT_CLEANUP_NOK]
   * @throws Exception            unexpected error occured
   */
  public void rename(String objectFilename, String cleanupExt) throws Exception
  {
    String textFilename = objectFilename.substring (0,objectFilename.length()-
                            EXT_OBJECT_FILE.length()) + EXT_TEXT_FILE;

    File objSrc = new File(objectFilename);
    File txtSrc = new File(textFilename);

    File objDest = new File(objectFilename + cleanupExt);
    File txtDest = new File(textFilename   + cleanupExt);

    objSrc.renameTo(objDest);
    txtSrc.renameTo(txtDest);
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void writeTextFile(ErrorVO err) throws IOException
  {
    File        file  = new File (getTextFilename());
    FileWriter  out   = new FileWriter(file);

    out.write(err.toString());
    out.close();
  }


  private void writeObjectFile(ErrorVO err) throws IOException
  {
    FileOutputStream    file  = new FileOutputStream (getObjectFilename());
    ObjectOutputStream  out   = new ObjectOutputStream(file);

    out.writeObject(err);
    out.close();
  }


  private boolean existTextFile()
  {
    File file = new File (getTextFilename());
    return (file.exists());
  }


  private String getTextFilename()
  {
    return (fPath + "/" + fTextName);
  }


  private String getObjectFilename()
  {
    return (fPath + "/" + fObjectName);
  }


  private void setFilename() throws Exception
  {
    GregorianCalendar cal   = new GregorianCalendar();
    int               cnt   = 0;

    formatFilename(cal,cnt++);

    while (existTextFile() && cnt < MAX_CNT)
    {
      formatFilename(cal,cnt++);
    }

    if (existTextFile())
    {
      throw new Exception ( "setFilename - couln't generate a unique filename," +
                            "last name was: " + getTextFilename() );
    }
  }


  private void formatFilename(GregorianCalendar calendar, int cnt)
  {
    String name = null;

    name  = fIdent + "-ErrorVO-" +
            calendar.get(Calendar.YEAR) +
            formatLeadingBlank(calendar.get(Calendar.MONTH)+1,2)      +
            formatLeadingBlank(calendar.get(Calendar.DAY_OF_MONTH),2) + "-" +
            formatLeadingBlank(calendar.get(Calendar.HOUR),2)         +
            formatLeadingBlank(calendar.get(Calendar.MINUTE),2)       +
            formatLeadingBlank(calendar.get(Calendar.SECOND),2)       + "-" +
            formatLeadingBlank(calendar.get(Calendar.MILLISECOND),3);

    if (cnt > 0)
    {
      name = name + "(" + Integer.toString(cnt) +")";
    }

    fTextName   = name + EXT_TEXT_FILE;
    fObjectName = name + EXT_OBJECT_FILE;
  }


  private String formatLeadingBlank (int value, int len)
  {
    StringBuffer buf = new StringBuffer(len);

    buf.append(value);
    while (buf.length() < len)
    {
      buf.insert(0,'0');
    }
    return buf.toString();
  }

} // end class


