package ch.ige.ses.client.saputil;

import java.io.*;
import java.util.*;

/**
 * Convertiert BAGIS Texte in formatierten SAP Text.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapText.java,v 1.18 2006/12/13 14:54:56 iten Exp $
 */
public class SapText
{
  //
  // const
  //
  private static final int    FIRST_LINE_MAX_LEN    = 40;

  private static final int    SAP_TEXT_LEN          = 70;
  private static final char   SAP_FORMAT_BEGIN_TAG  = '<';
  private static final String SAP_FORMAT_ABSATZ     = "/";    // "*" = neuer Absatz (NOT USED)
                                                              // "/" = neue Zeile
  private static final String SAP_FORMAT_FLIESSTEXT = "=";

  public  static final String BAGIS_BREAK           = "<br>";

  private static final String[][] BAGIS_SAP_FORMATS =
    { {"<b>", "<B>"},     {"</b>","</>"},
      {"<u>", "<U>"},     {"</u>","</>"},
      {"<i>", "<I>"},     {"</i>","</>"},
      {"<br/>", "<br>"},  {"<br />","<br>"},
      {",,","\""} };      // Doppelkoma wird in SAP als Tab interpretiert !

  //
  // fields
  //
  private ArrayList fTextList         = new ArrayList();  // line 1...n (SapTextVO list)

  private String    fSplitTextHeader  = null;             // line 1
  private ArrayList fSplitTextList    = new ArrayList();  // line 2...n (SapTextVO list)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Constructs a SapText
   */
  public SapText()
  {
    // lazy bone
  }

  /**
   * Constructs a SapText
   *
   * @param bagisText initial bagis text to load and convert
   */
  public SapText(String bagisText)
  {
    this.add(bagisText);
  }


  /**
   * Constructs a SapText
   *
   * @param bagisTextA initial bagis text #1 to load and convert
   * @param bagisTextB initial bagis text #2 to load, convert and append
   */
  public SapText(String bagisTextA, String bagisTextB)
  {
    this.add(bagisTextA);
    this.add(bagisTextB);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * add new bagis text. the following steps will be processed:<br>
   *  1: map formats from bagis to sap
   *  2: create separate lines for each bagis break sign
   *  3: format lines into sap text lines with the given sap text len
   *     for the full sap text
   *  4: prepare the split text
   *
   * @param bagisText bagis text to add
   */
  public void add(String bagisText)
  {
    String      format = this.mapFormats(bagisText);
    ArrayList   list   = this.breakLines(format);

    // full text list
    formSapText(list,fTextList);

    // split text preparation
    if (fSplitTextHeader == null)
    {
      Iterator  it    = list.iterator();
      ArrayList split = new ArrayList();

      if ( it.hasNext() )
      {
        String line = (String)it.next();
        int    len  = line.length();

        if (len > FIRST_LINE_MAX_LEN)
        {
          len = FIRST_LINE_MAX_LEN;
        }
        fSplitTextHeader = line.substring(0,len);

        while ( it.hasNext() )
        {
          split.add(it.next());
        }

        formSapText(split,fSplitTextList);
      }
    }
    else
    {
      formSapText(list,fSplitTextList);
    }
  }


  public ArrayList getTextList()
  {
    return fTextList;
  }


  public String getSplitTextHeader()
  {
    return fSplitTextHeader;
  }


  public ArrayList getSplitTextList()
  {
    return fSplitTextList;
  }


  public String toStringText()
  {
    Iterator it      = this.fTextList.iterator();
    StringBuffer buf = new StringBuffer();

    while ( it.hasNext() )
      buf.append(((SapTextVO)it.next()).toString() + "\n");

    return buf.toString();
  }


  public String toStringSplitText()
  {
    Iterator it      = this.fSplitTextList.iterator();
    StringBuffer buf = new StringBuffer();

    buf.append("HEADER (Line 1): \n");
    buf.append("\t[" + fSplitTextHeader + "]\n");

    buf.append("BODY (Line 2...n): \n");
    while ( it.hasNext() )
      buf.append(((SapTextVO)it.next()).toString() + "\n");

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------
  private String mapFormats(String text)
  {
    StringBuffer buf = new StringBuffer(text);
    int start;
    int end;

    for (int i = 0; i <  BAGIS_SAP_FORMATS.length; i++)
    {
      start = buf.toString().indexOf(BAGIS_SAP_FORMATS[i][0]);

      while (start > -1)
      {
        end   = start + BAGIS_SAP_FORMATS[i][0].length();
        buf.replace(start,end,BAGIS_SAP_FORMATS[i][1]);

        start = buf.toString().indexOf(BAGIS_SAP_FORMATS[i][0]);
      }
    }

    return buf.toString();
  }


  private ArrayList breakLines(String text)
  {
    ArrayList lineList = new ArrayList();        // list of string objects
    int start          = 0;
    int end            = text.indexOf(this.BAGIS_BREAK);

    while (end > -1)
    {
      lineList.add( text.substring(start,end) );

      start = end + this.BAGIS_BREAK.length();
      end   = text.indexOf(this.BAGIS_BREAK,start);
    }
    lineList.add( text.substring(start) );

    return lineList;
  }


  private void formSapText(ArrayList lineList, ArrayList voList)
  {
    Iterator  it      = lineList.iterator();
    String    line    = null;
    int       start   = 0;
    int       end     = 0;


    while ( it.hasNext() )
    {
      //
      // get line
      //
      line  = (String)it.next();

      //
      // write first sap text - line
      //
      start = 0;
      end   = ((line.length() > this.SAP_TEXT_LEN) ? this.SAP_TEXT_LEN : line.length());

      // move start of format tag to next row if it doesn't fit to actual row
      if ( ( (end-start) > (SAP_TEXT_LEN-2) ) &&
           (line.charAt(end-1) == SAP_FORMAT_BEGIN_TAG) )
      {
        end--;
      }
      else if ( ( (end-start) > (SAP_TEXT_LEN-2) ) &&
                (line.charAt(end-2) == SAP_FORMAT_BEGIN_TAG) )
      {
        end = end - 2;
      }

      // move trailing blanks to next row since sap can't handle them
      while ((end > (start+1)) && (line.charAt(end - 1) == ' '))
      {
        end--;
      }

      // add line
      voList.add
       ( new SapTextVO(this.SAP_FORMAT_ABSATZ,line.substring(start,end)) );

      //
      // write next sap text - lines
      //
      while (end < line.length())
      {
        start = end;
        end   = ((line.length()-start > this.SAP_TEXT_LEN) ? start+this.SAP_TEXT_LEN : line.length());

        // move start of format tag to next row if it doesn't fit to actual row
        if ( ( (end-start) > (SAP_TEXT_LEN-2) ) &&
             (line.charAt(end-1) == SAP_FORMAT_BEGIN_TAG) )
        {
          end--;
        }
        else if ( ( (end-start) > (SAP_TEXT_LEN-2) ) &&
                  (line.charAt(end-2) == SAP_FORMAT_BEGIN_TAG) )
        {
          end = end - 2;
        }

        // move trailing blanks to next row since sap can't handle them
        while ((end > (start+1)) && (line.charAt(end - 1) == ' '))
        {
          end--;
        }

        // add line
        voList.add
         ( new SapTextVO(this.SAP_FORMAT_FLIESSTEXT,line.substring(start,end)) );
      }
    } // end it.hasNext()

  } // end method


  private String toStringLineList (ArrayList lineList)
  {
    Iterator it      = lineList.iterator();
    StringBuffer buf = new StringBuffer();

    while ( it.hasNext() )
      buf.append("\t [" + (String)it.next() + "] \n");

    return buf.toString();
  }

} // end class


