package ch.ige.ses.server.bagisoutbound;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.bagis.BfakturDAO;
import ch.ige.ses.server.dao.bagis.JoinJgbStatistikDAO;
import ch.ige.ses.server.dao.bagis.JoinSAPRechnungDAO;
import ch.ige.ses.server.error.SESErrorWorkLocal;
import ch.ige.ses.server.error.SESErrorWorkLocalHome;
import ch.ige.ses.util.ReplaceTexte;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.ses.ErrorVO;

/**
 * SES: Bean zur Verarbeitung der Jahresgebühren BAGIS-Rechnungen<br>
 * Liest die für die Jahresgebühren benötigten Bfakturen von Bagis ein
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger
 * @version  $Id: SESJgbBagisReadBean.java,v 1.43 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESJgbBagisRead"  
 *  display-name="SESJgbBagisRead"
 *  view-type="local"
 *  jndi-name="SESJgbBagisRead"
 *  local-jndi-name="SESJgbBagisReadLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="RequiresNew"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorWork"
 *   view-type="local"
 */
public class SESJgbBagisReadBean implements SessionBean
{
  //
  // const
  //
  private static final Logger LOG         = Logger.getLogger(SESJgbBagisReadBean.class);
  private static final String SOURCE_NAME = "SESJgbBagisReadBean";

  //
  // fields
  //
  SessionContext              sessionContext;

  SESErrorWorkLocalHome       sesErrorWorkLocalHome = null;
  SESErrorWorkLocal           sesErrorWorkLocal     = null;

  private JoinSAPRechnungDAO  joinSAPRechnungDAO;
  private ReplaceTexte        replaceTexte;

  private ArrayList           al;               // ArrayList mit SAPRechnungsVO



  //---------------------------------------------------------------------------
  // ejb methods
  //---------------------------------------------------------------------------

  /**
   * Holt mittels Lookup das HomeInterface und instanziert das RemoteInterface
   * des Beans SESErrorWork
   */
  public void ejbCreate()
  {
    try
    {
      //Naming-Kontext ermitteln
      Context ctx = new InitialContext();
      Object ref = ctx.lookup(sesErrorWorkLocalHome.COMP_NAME);
      sesErrorWorkLocalHome = (SESErrorWorkLocalHome)ref;
      sesErrorWorkLocal = sesErrorWorkLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("SESJgbBagisReadBean Methode ejbCreate: CreateException: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("SESJgbBagisReadBean Methode ejbCreate: NamingException: " + ex.getMessage());
    }
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }


  //---------------------------------------------------------------------------
  // public methods - jgb lauf
  //---------------------------------------------------------------------------

  /**
   * Methode readJGBFromBagis liest die Jahresgebühren von BAGIS ein
   *
   * @param   bFaktIDVon  Beginn der zu selektierenden Bfakturen
   * @param   bFaktIDBis  Ende der zu selektierenden Bfakturen
   * @return  ArrayList   ArrayList von SAPRechnungVO
   * 
   * @ejb.interface-method
   */
  public ArrayList readJGBFromBagis(int bFaktIDVon, int bFaktIDBis)
  {
		try
    {
      // init
      if (joinSAPRechnungDAO == null)
      {
        joinSAPRechnungDAO = new  JoinSAPRechnungDAO();
      }

      if (replaceTexte == null)
      {
        replaceTexte = new ReplaceTexte();
      }

      // ArrayList mit SaprechnungsVO holen
      al = joinSAPRechnungDAO.readJGB(bFaktIDVon, bFaktIDBis);

      // Texte umwandeln
      replaceTexte.PrepareTexte(al);

      // end
      return al;

    }
    catch (Exception ex)
    {
      // log error
      LOG.error( "readJGBFromBagis - Ein unerwarteter Fehler ist aufgetreten \n" +
                 "bFaktIDVon = " + bFaktIDVon + "\n" +
                 "bFaktIDBis = " + bFaktIDBis + "\n" +
                 "Exception  = " + ex.getMessage() );


      // assemble error vo
      ErrorVO errorVO = new ErrorVO();
      errorVO.source          = "SESJgbBagisReadBean Method: readJGBFromBagis";
      errorVO.employeeNotify  = ErrorVO.EMPL_NOTIFY_PENDENT;
      errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_JGB;
      errorVO.description     = "Ein unerwarteter Fehler ist aufgetreten. \n" +
                                "Exception : " + ex.getMessage();
      errorVO.dataTypeCd      = ErrorVO.DATA_TYPE_CD_STRING;
      errorVO.errorStat       = ErrorVO.ERROR_STAT_PENDENT;
      errorVO.data            = "bFaktIDVon = " + bFaktIDVon + "\n" +
                                "bFaktIDBis = " + bFaktIDBis;

      // send error vo
      try
      {
        sesErrorWorkLocal.sendError(errorVO);
      }
      catch (Exception sendex)
      {
        LOG.fatal ( "readJGBFromBagis - sesErrorWorkLocal.sendError(errorVO) failed \n" +
                    "ErrorVO Data: \n" + errorVO.toString() );
      }

      // end exception handling
      sessionContext.setRollbackOnly();
      throw new EJBException(ex);
    }
	}


  /**
   * Update Druckstatus auf 1 damit beim nächsten JGB Lauf diese
   * Einträge nicht mehr selektiert werden. Dies ist wegen den
   * Adressen mit Ländercode ungleich CH und FL. Diese werden von SAP nicht
   * gedruckt. daher wird ihr Status auch nicht aktualisiert.
   *
   * @param  vo  ValueObject mit Rechnungsdaten
   * @return     ErrorVO im Fehlerfall sonst null
   * 
   * @ejb.interface-method
   */
  public ErrorVO updateDruckstatus (SAPRechnungVO vo)
  {
    BfakturDAO  bfakturDAO = new BfakturDAO();

    try
    {
      bfakturDAO.updateBfakturDruckstatus(vo.bfakt_id,"1");

      return null;

    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();

      ErrorVO errorVO         = new ErrorVO();
      errorVO.source          = SOURCE_NAME;
      errorVO.employeeNotify  = ErrorVO.EMPL_NOTIFY_KEINE;    // keine Benachrichtigung
      errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_JGB;
      errorVO.description     = "StatusDruck mit bfakt_id " + vo.bfakt_id + " konnte auf BAGIS nicht auf 1 gesetzt werden \n" +
                                "Exception : \n" + ex.getMessage() ;
      errorVO.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_VO;
      errorVO.errorStat       = ErrorVO.ERROR_STAT_PENDENT;
      errorVO.data            = vo;

      LOG.error(errorVO.description);
      return errorVO;
    }
  }

  //---------------------------------------------------------------------------
  // public methods - jgb statistik
  //---------------------------------------------------------------------------

  /**
   * Gibt die Anzahl Fakturaanforderungen zurück.
   * @see JoinJgbStatistikDAO
   * 
   * @param minBfaktId 
   * @param maxBfaktId 
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int getCntFakturen(int minBfaktId, int maxBfaktId) throws Exception
  {
    try
    {
      return ( new JoinJgbStatistikDAO().getCntFakturen(minBfaktId,maxBfaktId) );
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getCntFakturen - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  }


  /**
   * Gibt die Anzahl Rechnungsempfänger zurück.
   * @see JoinJgbStatistikDAO
   * 
   * @param minBfaktId 
   * @param maxBfaktId 
   * @return result 
   * @throws Exception 
   * 
   * @ejb.interface-method
   */
  public int getCntRechEmpf(int minBfaktId, int maxBfaktId) throws Exception
  {
    try
    {
      return ( new JoinJgbStatistikDAO().getCntRechEmpf(minBfaktId,maxBfaktId) );
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getCntRechEmpf - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  }


  /**
   * Holt den Ländercode zu den Adressen der Map und ergänzt damit die
   * "values" der entsprechenden key Einträge.
   *
   * @param   map          Map mit Adressnummern
   * @throws  Exception    Unerwarteter Fehler
   * @return  HashMap      Map mit Adressnummer/Ländercode Paaren
   * 
   * @ejb.interface-method
   */
  public HashMap getLandCd(HashMap map) throws Exception
  {
    JoinJgbStatistikDAO   dao = new JoinJgbStatistikDAO();
    Integer               key = null;
    String                val = null;

    try
    {
      for (Iterator iter = map.keySet().iterator(); iter.hasNext(); )
      {
        key = (Integer)iter.next();
        val = dao.getLandCd(key.intValue());
        map.put(key,val);
      }

      return map;
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getLandCd - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  }

} // end class





