package ch.ige.ses.server.dao.ses;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.util.Date;
import ch.ige.ses.vo.bagis.*;
import ch.ige.util.server.*;

/**
 * DataAccessObject für die Tabelle bfaktpos_ses
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BfaktposSESDAO.java,v 1.7 2006/12/13 14:54:56 iten Exp $
 */

public class BfaktposSESDAO extends SesDAO
{

  public BfaktposSESDAO()
  {
    // lazy bone
  }

  /**
   * Selektiert einen Datensatz aus der Tabelle bfaktpos_ses
   *
   * @param  bfaktId                  BFaktur ID
   * @param  bfaktposNr               BFaktur PositionsNr
   * @return SAPRechnungsPositionVO   Value Object
   * @exception SQLException             Unerwarteter SQL Fehler
   * @exception IOException              Unerwarteter I/O Fehler
   */
  public SAPRechnungsPositionVO select(int bfaktId, int bfaktposNr)
    throws SQLException, IOException
  {
    Connection              con = null;
    PreparedStatement       ps  = null;
    ResultSet               rs  = null;
    SAPRechnungsPositionVO  vo  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE bfakt_id = ? AND bfakt_posnr = ? " );

      ps.setInt(1, bfaktId);
      ps.setInt(2, bfaktposNr);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillSAPSAPRechnungsPositionVO(rs);
      }

      return vo;                                // return valid vo or null

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektiert alle Positionen einer Rechnung aus der Tabelle bfaktpos_ses
   *
   * @param  bfaktId       bfaktur ID
   * @return ArrayList     ArrayList mit SAPRechnungsPositionVO
   * @exception SQLException  Unerwarteter SQL Fehler
	 * @exception IOException   Unerwarteter I/O Fehler
   */
  public ArrayList select(int bfaktId) throws SQLException, IOException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
				      "WHERE bfakt_id = ? " );

      ps.setInt(1, bfaktId);

      rs = ps.executeQuery();

      while( rs.next() )
      {
        al.add( fillSAPSAPRechnungsPositionVO(rs) );
      }

      return al;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Fügt einen Datensatz in die Tabelle bfaktpos_ses ein
   *
   * @param vo  ValueObject      SAPRechnungsPositionVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert(SAPRechnungsPositionVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
					"INSERT INTO bfaktpos_ses(" +
							"bfakt_id, " +
							"bfakt_posnr, " +
							"art_nr, " +
							"lageinh, " +
							"anzahl, " +
							"preis, " +
							"artkurztext, " +
							"arttext, " +
							"artzusatztext) " +
					"VALUES( ?,?,?,?,?,?,?,?,?)"
      );

      ps.setInt (1, vo.bfakt_id);
			ps.setInt(2, vo.bfakt_posnr);
      ps.setString (3, vo.art_nr);
      ps.setString (4, vo.lageinh);
			ps.setInt(5, vo.anzahl);
			if(vo.preis == null)
			{
				ps.setNull(6, java.sql.Types.NUMERIC);
			}
			else
			{
			  ps.setBigDecimal(6, vo.preis);
			}

			ps.setString(7, vo.artKurztext);
			ps.setString(8, vo.artText);
			ps.setString(9, vo.artZusatzText);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Aktualisiert einen Datensatz in der Tabelle bfaktpos_ses
   *
   * @param     vo            SAPRechnungsPositionVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update(SAPRechnungsPositionVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
					"UPDATE bfaktpos_ses SET " +
							"art_nr = ?, " +
							"lageinh = ?, " +
							"anzahl = ?, " +
							"preis = ?, " +
							"artkurztext = ?, " +
							"arttext = ?, " +
							"artzusatztext = ?, " +
			        "changed_dat = SYSDATE " +
					"WHERE 	bfakt_id = ? AND " +
							"bfakt_posnr = ? "
          );

			ps.setString(1, vo.art_nr);
      ps.setString(2, vo.lageinh);
			ps.setInt(3, vo.anzahl);
      if(vo.preis == null)
			{
				ps.setNull(4, java.sql.Types.NUMERIC);
			}
			else
		  {
				ps.setBigDecimal(4, vo.preis);
		  }
			ps.setString(5, vo.artKurztext);
			ps.setString(6, vo.artText);
			ps.setString(7, vo.artZusatzText);

			ps.setInt(8, vo.bfakt_id);
			ps.setInt(9, vo.bfakt_posnr);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Löscht einen Datensatz aus der Tabelle bfaktpos_ses
   *
   * @param     bfaktId       Primary Key
   * @param     bfaktPosNr    Primary Key
   * @return    int           Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int bfaktId, int bfaktPosNr) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "DELETE FROM bfaktpos_ses " +
          "WHERE bfakt_id = ? " +
          "AND   bfakt_posnr = ? ");
      ps.setInt(1, bfaktId);
      ps.setInt(2, bfaktPosNr);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
    * delivers sql statement for the select methods WITHOUT the where part
    * note:
    * the field must correspondent with the fields used in the method
    * fillSAPSAPRechnungsPositionVO !
    *
    * @return statement
    */
  private String getSelectStatement()
  {
    return
      ( "SELECT bfakt_id, bfakt_posnr, art_nr, lageinh, anzahl, preis, " +
        "artkurztext, arttext, artzusatztext, created_dat, changed_dat " +
        "FROM bfaktpos_ses ");
  }


  /**
    * fill SAPRechnungsPositionVO with data from result set
    *
    * @param     rs                     valid result set
    * @return    SAPRechnungsPositionVO value object
    * @exception SQLException  unexpected sql error
    * @exception IOException   unexpected io error
    */
  private SAPRechnungsPositionVO fillSAPSAPRechnungsPositionVO (ResultSet rs)
    throws SQLException, IOException
  {
    SAPRechnungsPositionVO  vo    = new SAPRechnungsPositionVO();

    vo.bfakt_id       = rs.getInt("bfakt_id");
    vo.bfakt_posnr    = rs.getInt("bfakt_posnr");
    vo.art_nr         = rs.getString("art_nr");
    vo.lageinh        = rs.getString("lageinh");
    vo.anzahl         = rs.getInt("anzahl");
    vo.preis          = rs.getBigDecimal("preis");
    vo.artKurztext    = rs.getString("artkurztext");

    vo.artText        = getClobAsString(rs,"arttext");
    vo.artZusatzText  = getClobAsString(rs,"artzusatztext");

    vo.createdDat     = rs.getTimestamp("created_dat") == null ? null : new Date(rs.getTimestamp("created_dat").getTime());
    vo.changedDat     = rs.getTimestamp("created_dat") == null ? null : new Date(rs.getTimestamp("changed_dat").getTime());

    return vo;

  } // end method

} // end class

