package ch.ige.ses.server.dao.ses;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.util.Date;
import ch.ige.ses.vo.bagis.*;
import ch.ige.util.server.*;


/**
 * DataAccessObject für Tabelle bfaktur_ses
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BfakturSESDAO.java,v 1.7 2007/11/13 12:35:02 nydegger Exp $
 */
public class BfakturSESDAO extends SesDAO
{

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public BfakturSESDAO()
  {
    super();
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

	/**
	 * Selektiert einen Datensatz aus der Tabelle bfaktur_ses
	 *
	 * @param  bfaktId          BFaktur ID
	 * @param  jgbLaufId        Lauf ID
	 * @return SAPRechnungVO    Value Object
	 * @exception SQLException  Unerwarteter SQL Fehler
	 * @exception IOException   Unerwarteter IO Fehler
	 */
	public SAPRechnungVO select(int bfaktId, int jgbLaufId)
      throws SQLException, IOException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    SAPRechnungVO       vo  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps  = con.prepareStatement
              ( getSelectStatement() +
                "WHERE bfakt_id = ? AND jgb_lauf_id = ? " );

      ps.setInt(1, bfaktId);
      ps.setInt(2, jgbLaufId);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillSAPRechnungVO(rs);
      }

      return vo;                                // return valid vo or null
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * select list of SAPRechnungVO for a given bfaktId range, a given jgbLaufId
   * and a choosen state or all states
   * note:
   *  - the vo's content is restricted to the data from table bfaktur_ses !
   *  - to get a SAPRechnungVO with complete billing data use the methods
   *    from the JoinSESRechnungDAO.<br>
   *  - if the param statusVerarbeitung is set to NULL then all vo in the
   *    choosen range are selected otherwise only the vo for the desired state.
   *
   * @param  bfaktIdFrom          start bfakt id range
   * @param  bfaktIdTo            end bfakt id range
   * @param  jgbLaufId            lauf id
   * @param  statusVerarbeitung   state (see SAPRechnungVO.STATUS_VERARBEITUNG_*) or null
   * @return ArrayList of SAPRechnungVO or empty list
   * @exception SQLException  unexpected sql error
   * @exception IOException   unexpected io error
   */
  public ArrayList select ( int bfaktIdFrom, int bfaktIdTo, int jgbLaufId,
                            String statusVerarbeitung )
      throws SQLException, IOException
  {
    Connection          con   = null;
    PreparedStatement   ps    = null;
    ResultSet           rs    = null;
    ArrayList           al    = new ArrayList();
    String              where = "WHERE bfakt_id >= ? AND bfakt_id <= ? AND jgb_lauf_id = ? ";

    try
    {
      // set optional status_veraebeitung clause
      if ( statusVerarbeitung != null )
        where = where + "AND status_verarbeitung = ? ";

      // prepare stmt
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement
              ( getSelectStatement() + where + "ORDER BY bfakt_id" );

      ps.setInt(1, bfaktIdFrom);
      ps.setInt(2, bfaktIdTo);
      ps.setInt(3, jgbLaufId);
      if(statusVerarbeitung != null)
        ps.setString(4, statusVerarbeitung);

      // get data
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        al.add( fillSAPRechnungVO(rs) );
      }

      // return list
      return al;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Gibt eine ArrayList von Bfakturen mit einem bestimmten Verarbeitungsstatus
   * eines Jahresgebühren - Laufes zurück
   * @param jgbLaufId Jahresgebühren - Lauf
   * @param statusVerarbeitung Verarbeitungsstatus
   * @return ArrayList mit SAPRechnungVO's
   * @throws SQLException
   * @throws IOException
   * @author Patrick Bolzli
   */
  public ArrayList select(int jgbLaufId, String statusVerarbeitung)
                          throws SQLException, IOException
  {
    Connection          con   = null;
    PreparedStatement   ps    = null;
    ResultSet           rs    = null;
    ArrayList           al    = new ArrayList();
    String sql       = "SELECT bfakt_id, hauptrechempf, " +
                       "error_source, error_description " +
                       "FROM ses.bfaktur_ses " +
                       "WHERE jgb_lauf_id = ? " +
                       "AND status_verarbeitung = ? " +
                       "ORDER BY error_description";
    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(sql);
      ps.setInt(1, jgbLaufId);
      ps.setString(2, statusVerarbeitung);

      // get data
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        SAPRechnungVO vo     = new SAPRechnungVO();
        vo.bfakt_id          = rs.getInt("bfakt_id");
        vo.hauptRechEmpf     = rs.getInt("hauptrechempf");
        vo.error_source      = rs.getString("error_source");
        vo.error_description = rs.getString("error_description");

        al.add(vo);
      }

      return al;
    }

    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method


  /**
   * Selektiert alle Datensatze eines JGB-Laufes aus der Tabelle bfaktur_ses
   * mit einem bestimmten Verarbeitungsstatus
   *
   * @param jgbLaufId Lauf-Id des JGB-Laufes
   * @param statusVerarbeitung Verarbeitungsstatus
   * @return Liste von SapRechnungV0s
   * @throws Exception Unerwarteter Fehler
   */
  public ArrayList selectStatusVerarbList(int jgbLaufId, String statusVerarbeitung) throws Exception
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    ArrayList           al  = new ArrayList();
    SAPRechnungVO       vo  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps  = con.prepareStatement
      ( "SELECT  bfakt_id, " +
                "jgb_lauf_id, " +
                "status_verarbeitung, " +
                "error_source, " +
                "error_description " +
        "FROM bfaktur_ses  " +
        "WHERE jgb_lauf_id = ? AND status_verarbeitung = ? " +
        "ORDER BY error_description");

      ps.setInt(1, jgbLaufId);
      ps.setString(2, statusVerarbeitung);

      rs = ps.executeQuery();

      while ( rs.next() )
      {
        vo    = new SAPRechnungVO();

        vo.bfakt_id         = rs.getInt("bfakt_id");
        vo.jgbLaufId        = rs.getInt("jgb_lauf_id");
        vo.status_verarbeitung = rs.getString("status_verarbeitung");
        vo.error_source = rs.getString("error_source");
        vo.error_description = rs.getString("error_description");

        al.add(vo);
      }
      // return list
      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

   /**
    * Gibt eine ArrayList von BfaktIds mit einem bestimmten Verarbeitungsstatus
    * eines Jahresgebühren - Laufes zurück
    *
    * @param jgbLaufId  JGB-Lauf
    * @param statusVerarbeitung Verarbeitungsstatus
    * @return ArrayList von bfaktIds
    * @throws Exception Unerwartete Fehlersituation
    */
   public ArrayList selectBfaktIds(int jgbLaufId, String statusVerarbeitung) throws Exception
   {
     Connection          con = null;
     PreparedStatement   ps  = null;
     ResultSet           rs  = null;
     ArrayList           al  = new ArrayList();

     try
     {
       con = DBHelper.getConnection(DBHelper.SES);

       ps  = con.prepareStatement
       ( "SELECT  bfakt_id " +
         "FROM bfaktur_ses  " +
         "WHERE jgb_lauf_id = ? AND status_verarbeitung = ? " );

       ps.setInt(1, jgbLaufId);
       ps.setString(2, statusVerarbeitung);

       rs = ps.executeQuery();

       while ( rs.next() )
       {
         al.add(new Integer(rs.getInt("bfakt_id")));
       }
       // return list
       return al;
     }
     finally
     {
       DBHelper.close(ps, con);
     }
  }

  /**
   * Gibt eine die Anzahl Bfakturen eines JGB - Laufes zurück
   * @param jgbLaufId Jahresgebühren - Lauf
   * @return Anzahl Datensätze
   * @throws SQLException
   * @throws IOException
   * @author Patrick Bolzli
   */
  public int count(int jgbLaufId) throws SQLException
  {
    Connection          con   = null;
    PreparedStatement   ps    = null;
    ResultSet           rs    = null;
    int                 anz   = 0;
    String sql       = "SELECT count(*) as Anzahl from ses.bfaktur_ses " +
                       "WHERE jgb_lauf_id = ?";
    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(sql);
      ps.setInt(1, jgbLaufId);

      // get data
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        anz = rs.getInt("Anzahl");
      }

      return anz;
    }

    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method


  /**
   * Gibt die Anzahl Bfakturen pro Verarbeitungsstatus eines JGB - Laufes zurück
   * @param jgbLaufId Jahresgebühren - Lauf
   * @return Anzahl Datensätze pro Verarbeitungsstatus
   * @throws SQLException
   */
  public HashMap countStatusVerarbeitung(int jgbLaufId) throws SQLException
  {
    Connection          con   = null;
    PreparedStatement   ps    = null;
    ResultSet           rs    = null;

    HashMap hashMap = new HashMap();

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement("SELECT count(status_verarbeitung) as anzahl, status_verarbeitung " +
                                "FROM ses.bfaktur_ses " +
                                "WHERE jgb_lauf_id = ? group by status_verarbeitung");
      ps.setInt(1, jgbLaufId);

      rs = ps.executeQuery();

      while ( rs.next() )
      {
        String status = rs.getString("status_verarbeitung");
        String anz = rs.getString("anzahl");

        hashMap.put(status, anz);
      }

      return hashMap;
    }

    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method


  /**
   * Selektiert folgende Attribute der Tabelle BFAKTUR_SES
   *  bfakt_id
   *  jgb_lauf_id
   *  hauptrechempf
   *  regulierer
   *  auftraggeber
   *  schutit_nr
   *  schutit_typ
   *  status_druck
   *  created_dat
   *  changed_dat
   * @param sapRechnungVO Enthält die Selektionskriterien
   * @return ArrayList von SAPRechnungVO die den Selektionskriteren entsprechen
   * @throws SQLException unerwartete Fehlersituation
   */
  public ArrayList select(SAPRechnungVO sapRechnungVO) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    ArrayList al  = new ArrayList();

    StringBuffer select = new StringBuffer();
    StringBuffer where  = new StringBuffer();
    StringBuffer order  = new StringBuffer();
    String and = "";

    select.append("SELECT bfakt_id, jgb_lauf_id, hauptrechempf, regulierer, " +
                    "auftraggeber, schutit_nr, schutit_typ, status_verarbeitung, " +
                    "created_dat, changed_dat " +
                  "FROM ses.bfaktur_ses ");
    where.append("WHERE ");
    order.append(" ORDER BY bfakt_id ");

    if (sapRechnungVO.bfakt_id != 0)
    {
      where.append("bfakt_id = " + sapRechnungVO.bfakt_id);
      and = " AND ";
    }

    if (sapRechnungVO.jgbLaufId != 0)
    {
      where.append(and);
      where.append("jgb_lauf_id = " + sapRechnungVO.jgbLaufId);
      and = " AND ";
    }

    if (sapRechnungVO.hauptRechEmpf != 0)
    {
      where.append(and);
      where.append("hauptrechempf = " + sapRechnungVO.hauptRechEmpf);
      and = " AND ";
    }

    if (sapRechnungVO.regulierer != 0)
    {
      where.append(and);
      where.append("regulierer = " + sapRechnungVO.regulierer);
      and = " AND ";
    }

    if (sapRechnungVO.auftraggeber != 0)
    {
      where.append(and);
      where.append("auftraggeber = " + sapRechnungVO.auftraggeber);
      and = " AND ";
    }

    if (!(sapRechnungVO.schutit_nr.equals("")))
    {
      where.append(and);
      where.append("schutit_nr = '" + sapRechnungVO.schutit_nr + "'");
      and = " AND ";
    }

    if (sapRechnungVO.schutit_typ != 0)
    {
      where.append(and);
      where.append("schutit_typ = " + sapRechnungVO.schutit_typ);
      and = " AND ";
    }

    if (!(sapRechnungVO.status_verarbeitung.equals("")))
    {
      where.append(and);
      where.append("status_verarbeitung = '" + sapRechnungVO.status_verarbeitung + "'");
      and = " AND ";
    }

    String sql =  select.toString() +
                  ( where.toString().equals("WHERE ") ? "" : where.toString() ) +
                  order.toString();

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      {
        SAPRechnungVO vo = new SAPRechnungVO();

        vo.bfakt_id       = rs.getInt("bfakt_id");
        vo.jgbLaufId      = rs.getInt("jgb_lauf_id");
        vo.hauptRechEmpf  = rs.getInt("hauptrechempf");
        vo.regulierer     = rs.getInt("regulierer");
        vo.auftraggeber   = rs.getInt("auftraggeber");
        vo.schutit_nr     = rs.getString("schutit_nr");
        vo.schutit_typ    = rs.getInt("schutit_typ");
        vo.status_verarbeitung  = rs.getString("status_verarbeitung");
        vo.created_dat    = rs.getTimestamp("created_dat");
        vo.changed_dat    = rs.getTimestamp("changed_dat");

        al.add(vo);
      }

      return al;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }



  /**
   * Selektiert die grösste bfakt_id des gewünschtes Laufes aus der Tabelle bfaktur_ses
   * @param     jgbLaufID                lauf_id eines Jahresgebühren Laufes
   * @return    grösste bfakt_id des ensprechenden Laufes
   * @exception SQLException             Unerwarteter SQL Fehler
   */
  public int selectMaxBfaktID(int jgbLaufID) throws SQLException
  {
    Connection              con = null;
    PreparedStatement       ps  = null;
    ResultSet               rs  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
         "SELECT MAX(bfakt_id) AS bfakt_id " +
         "FROM bfaktur_ses " +
         "WHERE jgb_lauf_id = ? ");

      ps.setInt(1, jgbLaufID);

      rs = ps.executeQuery();
      if( !rs.next())
      {
        return 0;
      }
      return rs.getInt("bfakt_id");
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert die kleinste bfakt_id des gewünschten Laufes aus der Tabelle bfaktur_ses
   * @param     jgbLaufID                lauf_id eines Jahresgebühren Laufes
   * @return    kleinste bfakt_id des entsprechenden Laufes
   * @exception SQLException             Unerwarteter SQL Fehler
   */
  public int selectMinBfaktID(int jgbLaufID) throws SQLException
  {
    Connection              con = null;
    PreparedStatement       ps  = null;
    ResultSet               rs  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
         "SELECT MIN(bfakt_id) AS bfakt_id " +
         "FROM bfaktur_ses " +
         "WHERE jgb_lauf_id = ? ");

      ps.setInt(1, jgbLaufID);

      rs = ps.executeQuery();
      if( !rs.next())
      {
        return 0;
      }
      return rs.getInt("bfakt_id");
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Fügt einen Datensatz in die Tabelle bfaktur_ses ein
   *
   * @param vo  ValueObject      SAPRechnungsPositionVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert(SAPRechnungVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
					"INSERT INTO bfaktur_ses(" +
		      		"bfakt_id, " +
					    "jgb_lauf_id, " +
				      "hauptrechempf, " +
				      "regulierer, " +
							"auftraggeber, " +
			        "bfakt_dat, " +
					    "schutit_nr, " +
				      "schutit_typ, " +
					    "schutit_typ_bez, " +
				      "inhaber, " +
				      "ihr_zeichen, " +
						  "tel_nr, " +
			        "sprach_cd, " +
					    "druck_art, " +
				      "faellig_dat, " +
				      "anz_kopie, " +
					    "fakt_betrag, " +
				      "kz_einschr, " +
				      "status_verarbeitung, " +
				      "unsere_ref, " +
				      "unt_name, " +
				      "var_text, " +
				      "zahl_dat, " +
					    "waehr_cd, " +
				      "employee_id, " +
				      "kontaktperson, " +
				      "abteilung, " +
				      "kopftext, " +
				      "beilagentext, " +
              "belehrungstext, " +
              "error_source, " +
              "error_description, " +
              "verfall_dat ) " +
					"VALUES( ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
      );

      ps.setInt (1, vo.bfakt_id);
			ps.setInt(2, vo.jgbLaufId);
      ps.setInt (3, vo.hauptRechEmpf);
      ps.setInt (4, vo.regulierer);
			ps.setInt(5, vo.auftraggeber);
			if(vo.bfakt_dat == null)
			{
				ps.setNull(6, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(6, new java.sql.Date(vo.bfakt_dat.getTime()));
			}
			ps.setString(7, vo.schutit_nr);
			ps.setInt(8, vo.schutit_typ);
			ps.setString(9, vo.schutit_typ_bez);
			ps.setString(10, vo.inhaber);
			ps.setString(11, vo.ihr_zeichen);
			ps.setString(12, vo.tel_nr);
			ps.setString(13, vo.sprach_cd);
			ps.setString(14, vo.druck_art);
			if(vo.faellig_dat == null)
			{
				ps.setNull(15, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(15, new java.sql.Date(vo.faellig_dat.getTime()));
			}
			ps.setInt(16, vo.anz_kopie);
			if(vo.fakt_betrag == null)
			{
				ps.setNull(17, java.sql.Types.NUMERIC);
			}
			else
			{
			  ps.setBigDecimal(17, vo.fakt_betrag);
			}
			ps.setString(18, vo.kz_einschr);
			ps.setString(19, vo.status_verarbeitung);
			ps.setString(20, vo.unsere_ref);
			ps.setString(21, vo.unt_name);
			ps.setString(22, vo.var_text);
			if(vo.zahl_dat == null)
			{
				ps.setNull(23, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(23, new java.sql.Date(vo.zahl_dat.getTime()));
			}
			ps.setString(24, vo.waehrung);
			ps.setString(25, vo.employeeId);
      ps.setString(26, vo.kontaktPerson);
			ps.setString(27, vo.abteilung);
			ps.setString(28, vo.kopfText);
			ps.setString(29, vo.beilagenText);
			ps.setString(30, vo.belehrungText);
      ps.setString(31, vo.error_source);
      ps.setString(32, vo.error_description);
      if(vo.verfall_dat == null)
      {
        ps.setNull(33, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(33, new java.sql.Date(vo.verfall_dat.getTime()));
      }

			ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Aktualisiert einen Datensatz in der Tabelle bfaktur_ses
   *
   * @param     vo            SAPRechnungsPositionVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update(SAPRechnungVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
					"UPDATE bfaktur_ses SET " +
					    "hauptrechempf = ?, " +
				      "regulierer = ?, " +
							"auftraggeber = ?, " +
			        "bfakt_dat = ?, " +
					    "schutit_nr = ?, " +
				      "schutit_typ = ?, " +
					    "schutit_typ_bez = ?, " +
				      "inhaber = ?, " +
				      "ihr_zeichen = ?, " +
						  "tel_nr = ?, " +
			        "sprach_cd = ?, " +
					    "druck_art = ?, " +
				      "faellig_dat = ?, " +
				      "anz_kopie = ?, " +
					    "fakt_betrag = ?, " +
				      "kz_einschr = ?, " +
				      "status_verarbeitung = ?, " +
				      "unsere_ref = ?, " +
				      "unt_name = ?, " +
				      "var_text = ?, " +
				      "zahl_dat = ?, " +
					    "waehr_cd = ?, " +
				      "employee_id = ?, " +
				      "kontaktperson = ?, " +
				      "abteilung = ?, " +
				      "kopftext = ?, " +
				      "beilagentext = ?, " +
				      "belehrungstext = ?, " +
							"changed_dat = SYSDATE, " +
              "error_source = ?, " +
              "error_description = ?, " +
              "verfall_dat = ? " +
					"WHERE 	bfakt_id = ? AND " +
							"jgb_lauf_id = ? "
			);

      ps.setInt (1, vo.hauptRechEmpf);
      ps.setInt (2, vo.regulierer);
			ps.setInt(3, vo.auftraggeber);
			if(vo.bfakt_dat == null)
			{
				ps.setNull(4, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(4, new java.sql.Date(vo.bfakt_dat.getTime()));
			}
			ps.setString(5, vo.schutit_nr);
			ps.setInt(6, vo.schutit_typ);
			ps.setString(7, vo.schutit_typ_bez);
			ps.setString(8, vo.inhaber);
			ps.setString(9, vo.ihr_zeichen);
			ps.setString(10, vo.tel_nr);
			ps.setString(11, vo.sprach_cd);
			ps.setString(12, vo.druck_art);
			if(vo.faellig_dat == null)
			{
				ps.setNull(13, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(13, new java.sql.Date(vo.faellig_dat.getTime()));
			}
			ps.setInt(14, vo.anz_kopie);
			if(vo.fakt_betrag == null)
			{
				ps.setNull(15, java.sql.Types.NUMERIC);
			}
			else
			{
			  ps.setBigDecimal(15, vo.fakt_betrag);
			}
			ps.setString(16, vo.kz_einschr);
			ps.setString(17, vo.status_verarbeitung);
			ps.setString(18, vo.unsere_ref);
			ps.setString(19, vo.unt_name);
			ps.setString(20, vo.var_text);
			if(vo.zahl_dat == null)
			{
				ps.setNull(21, java.sql.Types.DATE);
			}
			else
			{
			  ps.setDate(21, new java.sql.Date(vo.zahl_dat.getTime()));
			}
			ps.setString(22, vo.waehrung);
			ps.setString(23, vo.employeeId);
      ps.setString(24, vo.kontaktPerson);
			ps.setString(25, vo.abteilung);
			ps.setString(26, vo.kopfText);
			ps.setString(27, vo.beilagenText);
			ps.setString(28, vo.belehrungText);
		  ps.setString(29, vo.error_source);
      ps.setString(30, vo.error_description);
      if(vo.verfall_dat == null)
      {
        ps.setNull(31, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(31, new java.sql.Date(vo.verfall_dat.getTime()));
      }
      ps.setInt(32, vo.bfakt_id);
			ps.setInt(33, vo.jgbLaufId);


      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * update the field statusVerarbeitung and the error fields in case of
   * statusVerarbeitung == SAPRechnungVO.STATUS_VERARBEITUNG_ERROR
   * otherwise the error fields may be null.
   *
   * @param bfakt_id            primary key
   * @param jgbLaufId           primary key
   * @param statusVerarbeitung  field to update
   * @param errorSource         field to update (only in case of error state)
   * @param errorDescription    field to update (only in case of error state)
   * @return                    number of rows affected
   * @throws SQLException       unexpected error occured
   */
  public int update ( int bfakt_id, int jgbLaufId, String statusVerarbeitung,
                      String errorSource, String errorDescription ) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    String            set = "SET changed_dat = SYSDATE, status_verarbeitung = ? ";
    int               idx = 1;

    try
    {
      if ( statusVerarbeitung.equals(SAPRechnungVO.STATUS_VERARBEITUNG_ERROR) )
      {
        set = set + ", error_source = ?, error_description = ? ";
      }

      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement ( "UPDATE bfaktur_ses " + set +
                                   "WHERE 	bfakt_id = ? AND jgb_lauf_id = ?" );

      ps.setString (idx++,statusVerarbeitung);
      if ( statusVerarbeitung.equals(SAPRechnungVO.STATUS_VERARBEITUNG_ERROR) )
      {
        ps.setString (idx++,errorSource);
        ps.setString (idx++,errorDescription);
      }
      ps.setInt (idx++,bfakt_id);
      ps.setInt (idx++,jgbLaufId);

      return ps.executeUpdate();

    }
    finally
    {
      DBHelper.close(ps, con);
    }

  } // end method


  /**
   * Methode updateStatusVerarbeitung dient zum Update des Attribut STATUS_VERARBEITUNG
   * der Tabelle BFAKTUR_SES mit Status = '9' (Bedingung: Rechnung bereits an SAP übertragen)
   *
   * @param   vo  SAPRechnungStatusVO
   * @return  Status ausgeführtes Statement
   * @throws  SQLException
   */
  public int updateStatusVerarbeitung(SAPRechnungStatusVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE bfaktur_ses SET " +
          "       status_verarbeitung = ?, " +
          "       changed_dat = SYSDATE " +
          "WHERE 	bfakt_id = ? AND status_verarbeitung = '1' "
      );

      ps.setString(1, vo.status_druck);
      ps.setInt(2, vo.bfakt_id);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Methode updateStatusVerarbeitung dient zum Update des Attribut STATUS_VERARBEITUNG
   * der Tabelle BFAKTUR_SES mit Status = 'E' (Fehler)
   *
   * @param   bfaktId
   * @return  Status ausgeführtes Statement
   * @throws  SQLException
   */
  public int updateStatusVerarbeitung(int jgbLaufId, int bfaktId, String status) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE bfaktur_ses SET " +
          "       status_verarbeitung = ?, " +
          "       changed_dat = SYSDATE " +
          "WHERE 	jgb_lauf_id = ? AND bfakt_id = ? AND status_verarbeitung = 'E' "
      );

      ps.setString(1, status);
      ps.setInt(2, jgbLaufId);
      ps.setInt(3, bfaktId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Löscht einen Datensatz aus der Tabelle bfaktur_ses
   *
   * @param     bfaktId       Primary Key
   * @param     jgbLaufId     Primary Key
   * @return    int           Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int bfaktId, int jgbLaufId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "DELETE FROM bfaktur_ses " +
          "WHERE bfakt_id = ? " +
          "AND   jgb_lauf_id = ? ");
      ps.setInt(1, bfaktId);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  //----------------------------------------------------------------------------
  // private methods
  //----------------------------------------------------------------------------

  /**
    * delivers sql statement for the select methods WITHOUT the where part
    * note:
    * the field must correspondent with the fields used in the method
    * fillSAPRechnungVO !
    *
    * @return select statement
    */
  private String getSelectStatement()
  {
    return
      ( "SELECT " +
        " bfakt_id, jgb_lauf_id, hauptrechempf, regulierer, auftraggeber, " +
        "bfakt_dat, schutit_nr, schutit_typ, schutit_typ_bez, inhaber, ihr_zeichen, " +
        "tel_nr, sprach_cd, druck_art, faellig_dat, anz_kopie, fakt_betrag, " +
        "kz_einschr, status_verarbeitung, unsere_ref, unt_name, var_text, zahl_dat, " +
        "waehr_cd, employee_id, kontaktperson, abteilung, " +
        "kopftext, beilagentext, belehrungstext, created_dat, changed_dat, " +
        "error_source, error_description, verfall_dat " +
        "FROM bfaktur_ses  " );

  } // end method


  /**
    * fill SAPRechnungVO with data from result set
    *
    * @param     rs            valid result set
    * @return    SAPRechnungVO value object
    * @exception SQLException  unexpected sql error
    * @exception IOException   unexpected io error
    */
  private SAPRechnungVO fillSAPRechnungVO (ResultSet rs)
    throws SQLException, IOException
  {
    SAPRechnungVO vo    = new SAPRechnungVO();

    vo.bfakt_id         = rs.getInt("bfakt_id");
    vo.jgbLaufId        = rs.getInt("jgb_lauf_id");
    vo.hauptRechEmpf    = rs.getInt("hauptrechempf");
    vo.regulierer       = rs.getInt("regulierer");
    vo.auftraggeber     = rs.getInt("auftraggeber");
    vo.bfakt_dat        = rs.getDate("bfakt_dat");
    vo.schutit_nr       = rs.getString("schutit_nr");
    vo.schutit_typ      = rs.getInt("schutit_typ");
    vo.schutit_typ_bez  = rs.getString("schutit_typ_bez");
    vo.inhaber          = rs.getString("inhaber");
    vo.ihr_zeichen      = rs.getString("ihr_zeichen");
    vo.tel_nr           = rs.getString("tel_nr");
    vo.sprach_cd        = rs.getString("sprach_cd");
    vo.druck_art        = rs.getString("druck_art");
    vo.faellig_dat      = rs.getDate("faellig_dat");
    vo.anz_kopie        = rs.getInt("anz_kopie");
    vo.fakt_betrag      = rs.getBigDecimal("fakt_betrag");
    vo.kz_einschr       = rs.getString("kz_einschr");
    vo.status_verarbeitung = rs.getString("status_verarbeitung");
    vo.unsere_ref       = rs.getString("unsere_ref");
    vo.unt_name         = rs.getString("unt_name");
    vo.var_text         = rs.getString("var_text");
    vo.zahl_dat         = rs.getDate("zahl_dat");
    vo.waehrung         = rs.getString("waehr_cd");
    vo.employeeId       = rs.getString("employee_id");
    vo.kontaktPerson    = rs.getString("kontaktperson");
    vo.abteilung        = rs.getString("abteilung");

    vo.kopfText         = getClobAsString(rs,"kopftext");
    vo.beilagenText     = getClobAsString(rs,"beilagentext");
    vo.belehrungText    = getClobAsString(rs,"belehrungstext");

    vo.created_dat      = rs.getTimestamp("created_dat") == null ? null : new Date(rs.getTimestamp("created_dat").getTime());
    vo.changed_dat      = rs.getTimestamp("changed_dat") == null ? null : new Date(rs.getTimestamp("changed_dat").getTime());
    vo.error_source     = rs.getString("error_source");
    vo.error_description= rs.getString("error_description");
    vo.verfall_dat      = rs.getDate("verfall_dat");

    return vo;

  } // end method


} // end class


