package ch.ige.ses.server.error;

import java.sql.SQLException;

import ch.ige.ses.vo.bagis.*;
import org.apache.log4j.*;

import ch.ige.ses.vo.ses.*;
import ch.ige.ses.util.*;
import ch.ige.ses.util.SESConstants;
import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.util.vo.VersadrVO;


/**
 * <p>Überschrift: SESErrorPrepareMail</p>
 * <p>Beschreibung: Bereitet Emails aufr</p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author  Stefan Nydegger
 * @version $Id: SESErrorPrepareMail.java,v 1.18 2009/03/12 16:55:44 locher Exp $
 */
public class SESErrorPrepareMail
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESErrorPrepareMail.class);

  private static final String DEFAULT_TO      = "Userdesk@ipi.ch";
  private static final String DEFAULT_CC      = "Team85@ipi.ch";
  private static final String DEFAULT_FROM    = DEFAULT_TO;

  private static final String DEFAULT_SUBJECT = "Fehler SES Schnittstelle";
  private static final String DEFAULT_BODY    = "Bitte mit dem Userdesk Kontakt aufnehmen";


  //
  // fields
  //
  private String smptServer = null;

  private String to         = null;               // Im Fehlerfall werden die
  private String cc         = null;               // Defaultwerte verwendet
  private String from       = null;

  private String subject    = null;               // Im Fehlerfall werden die
  private String body       = null;               // Defaultwerte verwendet

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Constructor
   */
  public SESErrorPrepareMail()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
  * Die Methode prepareMail bereitet eine Emailnachricht auf und sendet diese.
  * <p>
  * Je nach Art des Fehlers (Adresse, Rechnung, Jahresgebühren...) wird eine
  * andere Emailnachricht aufbereitet
  *
  * @param  isAdmin   true = Email wird an Admin gesendet, false = Email wird an Benutzer gesendet
  * @param  errorVO   ValueObject mit allen Fehlerinformationen
  * @return           true: Mail erfolgreich verschickt, false: Mail konnte nicht verschickt werden
  */
  public boolean prepareMail(boolean isAdmin, ErrorVO errorVO)
  {
    try
    {
      getEmailServer();
      getEmailAdress(isAdmin,errorVO.employeeId);
      getEmailMessage(errorVO);

      return MailSender.sendMail(smptServer, to, cc, from, subject, body);

    }
    catch (Exception ex)
    {
      LOG.error ( "prepareMail - Ein unerwarteter Fehler ist aufgetreten. " +
                  "Exception : " + ex.getMessage() );

      return false;
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private String getProperty (String key) throws Exception
  {
    try
    {
      return new PropertyDAO().select(key).value;
    }
    catch (Exception ex)
    {
      LOG.error ( "getProperty - Error reading key <" + key + "> " +
                  "Exception : " + ex.getMessage() );

      throw new Exception(ex.getMessage());
    }
  }


  private String getProperty (String key, String defaultValue)
  {
    try
    {
      return getProperty(key);
    }
    catch (Exception ex)
    {
      return defaultValue;
    }
  }


  private void getEmailServer() throws Exception
  {
    smptServer = getProperty("mail.smtpserver");
  }


  private void getEmailAdress (boolean isAdmin, String employeeId)
  {
    SESADSHelper adsHelper = SESADSHelper.createSESADSHelper();

    if ( isAdmin )
    {
      to = getProperty("error.mail.userdesk",DEFAULT_TO);
    }
    else
    {
      to = adsHelper.getEmailFromEmployeeID(employeeId);
      cc = getProperty("error.mail.userdesk",DEFAULT_CC);

      if ( (to == null) || to.equals("") )
      {
        to = cc;
        cc = null;
      }
    }

    from = getProperty("error.mail.from",DEFAULT_FROM);
  }


  private void getEmailMessage(ErrorVO vo)
  {
    String kopfText = null;
    String fussText = null;

    try
    {

      // ---------------------------------------------------------------------
      // Generelle Texte
      // ---------------------------------------------------------------------

      if ( vo.employeeMsgType == ErrorVO.EMPL_MSG_TYPE_DEFAULT )
      {
        LOG.debug("getEmailMessage - Default Message");

        subject  = getProperty("error.mail.subject");
        kopfText = getProperty("error.mail.kopftext");
        fussText = getProperty("error.mail.fusstext");
        fussText = SESErrorParseMail.parseFussText(fussText, vo);

        body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
      }
      else
      {
        LOG.debug("getEmailMessage - Spezial Message");

        subject  = getProperty("error.mail.subject_msg_type_"  + vo.employeeMsgType);
        kopfText = getProperty("error.mail.kopftext_msg_type_" + vo.employeeMsgType);
        fussText = getProperty("error.mail.fusstext_msg_type_" + vo.employeeMsgType);
        fussText = SESErrorParseMail.parseFussText(fussText, vo);

        body = kopfText + "\n" + fussText;
      }


      // ---------------------------------------------------------------------
      // Individuelle Erweiterungen
      // ---------------------------------------------------------------------

      //
      // EMPL_MSG_TYPE_MAX_EMPF
      //
      if ( vo.employeeMsgType == vo.EMPL_MSG_TYPE_MAX_EMPF )
      {
        SAPRechnungVO sapRechnungsVO = null;

        LOG.debug("getEmailMessage - EMPL_MSG_TYPE_MAX_EMPF");

        sapRechnungsVO = (SAPRechnungVO)vo.data;
        subject   = SESErrorParseMail.parseMaxEmpfSubject(subject, sapRechnungsVO);
        kopfText  = SESErrorParseMail.parseMaxEmpfKopfText(kopfText, sapRechnungsVO);

        body = kopfText + "\n" + fussText;
      }

      //
      // EMPL_MSG_TYPE_RECHNUNG
      //
      else if ( vo.employeeMsgType == ErrorVO.EMPL_MSG_TYPE_RECHNUNG )
      {
        String bfaktId = "";

        LOG.debug("getEmailMessage - EMPL_MSG_TYPE_RECHNUNG");

        if ( vo.dataTypeCd == ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_VO )
        {
          LOG.debug("getEmailMessage - DATA_TYPE_CD_SAP_RECHNUNG_VO");

          bfaktId  = "" + ((SAPRechnungVO)vo.data).bfakt_id;
          kopfText = SESErrorParseMail.parseRechnungKopfText(kopfText, bfaktId);

          body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
        }
        else if ( vo.dataTypeCd == ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_ANNULATION_VO )
        {
          bfaktId  = "" + ((SAPRechnungAnnullationVO)vo.data).bfakt_id;
          kopfText = SESErrorParseMail.parseRechnungKopfText(kopfText, bfaktId);

          body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
        }
      }

      //
      // EMPL_MSG_TYPE_JGB
      //
      else if ( vo.employeeMsgType == ErrorVO.EMPL_MSG_TYPE_JGB )
      {
        LOG.debug("getEmailMessage - EMPL_MSG_TYPE_JGB");

        body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
      }

      //
      // EMPL_MSG_TYPE_ADR
      //
      else if ( vo.employeeMsgType == ErrorVO.EMPL_MSG_TYPE_ADR )
      {
        String adrNr = null;
        LOG.debug("getEmailMessage - EMPL_MSG_TYPE_ADR");

        if ( vo.dataTypeCd == ErrorVO.DATA_TYPE_CD_SAP_VERS_ADR_VO )
        {
          LOG.debug("getEmailMessage - DATA_TYPE_CD_SAP_VERS_ADR_VO");
          adrNr = "" + ((VersadrVO)vo.data).getAdrNr();
        }
        else if( vo.dataTypeCd == ErrorVO.DATA_TYPE_CD_SAP_ADR_STATUS_VO )
        {
          LOG.debug("getEmailMessage - DATA_TYPE_CD_SAP_ADR_STATUS_VO");
          adrNr = "" + ((SAPAdrStatusVO)vo.data).adrNr;
        }

        kopfText = SESErrorParseMail.parseAdrKopfText(kopfText, adrNr);
        body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
      }

      //
      // EMPL_MSG_TYPE_STAKO
      //
      else if(vo.employeeMsgType == ErrorVO.EMPL_MSG_TYPE_STAKO)
      {
        body = kopfText + "\n\n" + vo.description + "\n\n" + fussText;
      }
    }
    catch (Exception ex)
    {
      LOG.error ( "getEmailMessage - Ein unerwarteter Fehler ist aufgetreten. " +
                  "Exception : " + ex.getMessage() );

      subject = DEFAULT_SUBJECT;
      body    = DEFAULT_BODY;
    }
  }

} // end class

