package ch.ige.ses.server.sapctrl;

import java.util.ArrayList;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.ses.AmvLaufDAO;
import ch.ige.ses.server.dao.ses.AmvRechnungDAO;
import ch.ige.ses.server.dao.ses.BfakturSESDAO;
import ch.ige.ses.server.dao.ses.JGBLaufDAO;
import ch.ige.ses.server.dao.ses.JoinSESRechnungDAO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.ses.AmvRechnungVO;
import ch.ige.ses.vo.ses.JGBLaufVO;
import ch.ige.ses.vo.ses.SAPJGBJobVO;

/**
 * Bearbeitung von Requests vom SAP Inbound Service
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapRequestBean.java,v 1.22 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SapRequest"  
 *  display-name="SapRequest"
 *  view-type="local"
 *  jndi-name="SapRequest"
 *  local-jndi-name="SapRequestLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SapRequestBean implements SessionBean
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapRequestBean.class);


  //
  // fields
  //
  SessionContext  sessionContext;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate()
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // public methods - jgb
  //---------------------------------------------------------------------------

  /**
   * first get jgb lauf info and update sap stat, then
   * select list of SAPRechnungVO for a given bfaktId range, a given jgbLaufId
   * and the state == SAPRechnungVO.STATUS_VERARBEITUNG_NEW from the following
   * tables:
   *   - bfaktur_ses
   *   - bfaktjgb_ses
   *   - bfaktpos_ses.
   *   - jgb_lauf
   *
   * @param job           SAPJGBJobVO with bfaktId range and jgbLaufId
   * @return              ArrayList of SAPRechnungVO or empty list if nothing is found
   * @exception Exception unexpected error
   * 
   * @ejb.interface-method
   */
  public ArrayList requestJgbRechnungJob (SAPJGBJobVO job) throws Exception {
    JGBLaufDAO  laufDAO = new JGBLaufDAO(); // jgb lauf data access object
    JGBLaufVO   laufVO  = null;             // jgb lauf record

    ArrayList   al      = null;             // list of SAPRechnungVO or
                                            // empty list if nothing is found

    try
    {
      // get and update jgb lauf
      LOG.debug("requestJgbRechnungJob - select jgb lauf info");
      laufVO = laufDAO.select(job.jgbLaufId);

      if ( job.lastJob )
      {
        if ( (laufVO.sapStat == JGBLaufVO.SAP_STAT_NOT_STARTED) ||
             (laufVO.sapStat == JGBLaufVO.SAP_STAT_STARTED) )
        {
          LOG.debug("requestJgbRechnungJob - updateSapStat to <started last>");
          laufDAO.updateSapStat ( laufVO.jgbLaufId,
                                  JGBLaufVO.SAP_STAT_STARTED_LAST );
        }
      }
      else
      {
        if (laufVO.sapStat == JGBLaufVO.SAP_STAT_NOT_STARTED)
        {
          LOG.debug("requestJgbRechnungJob - updateSapStat to <started>");
          laufDAO.updateSapStat ( laufVO.jgbLaufId,
                                  JGBLaufVO.SAP_STAT_STARTED );
        }
      }


      // select jgb rechnungs list
      LOG.debug("requestJgbRechnungJob - get jgb rechnung list from ses db");
      al = new JoinSESRechnungDAO().selectJGBRechnung
            ( job.bfaktIdFrom, job.bfaktIdTo,
              job.jgbLaufId,   SAPRechnungVO.STATUS_VERARBEITUNG_NEW );

      LOG.debug("requestJgbRechnungJob - successfully done");

      return al;

    }
    catch (Exception ex)
    {
      LOG.error("requestJgbRechnungJob - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }


  /**
   * update jgb rechnung states for a given list of SAPRechnungVO's
   * processed by sap. the following tables will be changed:
   *   - bfaktur_ses
   *     fields status_verarbeitung, error_source, error_description
   *   - jgb_lauf
   *     fields anzahl_bfaktur_an sap, anzahl_bfaktjgb_an_sap
   *
   * @param  reList Processed SAPRechnungVO list
   * @throws Exception unexpected error
   * 
   * @ejb.interface-method
   */
  public void requestJgbRechnungUpdate (ArrayList reList) throws Exception
  {
    BfakturSESDAO  bfakturSESDAO  = new BfakturSESDAO();
    JGBLaufDAO     laufDAO        = new JGBLaufDAO();
    SAPRechnungVO  reVO           = null;

    int            bfaktJgbCnt    = 0;       // anzahl bfaktjgb_ses einträge
                                             // (= haupt + zusatz rechnungs empfänger)

    int            bfakturSapOk   = 0;       // anzahl bfaktur an sap ok
    int            bfakturSapNok  = 0;       // anzahl bfaktur an sap nok

    boolean        debugFirst     = true;
    int            debugIdFrom    = 0;
    int            debugIdTo      = 0;

    try
    {
      //
      // init & check for empty list
      //
      LOG.debug("requestJgbRechnungUpdate - update jgb rechnung list");

      if (reList.size() == 0)
      {
        LOG.warn("requestJgbRechnungUpdate - rechnungList is empty, request will be ignored !");
        throw new Exception("requestJgbRechnungUpdate - rechnungList is empty, request will be ignored !");
      }


      //
      // update bfaktur_ses
      //
      for (Iterator i = reList.iterator(); i.hasNext(); )
      {
        reVO = (SAPRechnungVO) i.next();

        bfakturSESDAO.update ( reVO.bfakt_id, reVO.jgbLaufId, reVO.status_verarbeitung,
                               reVO.error_source, reVO.error_description );

        bfaktJgbCnt = bfaktJgbCnt + 1 + reVO.jgbZusatzRechEmpfParam.size();

        if ( reVO.status_verarbeitung.equals(reVO.STATUS_VERARBEITUNG_SAP) )
        {
          bfakturSapOk++;
        }
        else
        {
          bfakturSapNok++;
        }

        if (debugFirst)
        {
          debugIdFrom = reVO.bfakt_id;
          debugIdTo   = reVO.bfakt_id;
          debugFirst  = false;
        }
        else
        {
          debugIdTo   = reVO.bfakt_id;
        }
      }

      //
      // update jgb_lauf (INCREMENT)
      //
      laufDAO.incrementAnzahlAnSap ( reVO.jgbLaufId, reList.size(),
                                     bfakturSapOk, bfakturSapNok, bfaktJgbCnt );

      //
      // all done
      //
      LOG.debug ( "requestJgbRechnungUpdate - successfully done, " +
                  "bfakt_id " + debugIdFrom + " to " + debugIdTo );

    }
    catch (Exception ex)
    {
      LOG.error("requestJgbRechnungUpdate - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }

  
  /**
   * request jgb lauf select
   * 
   * @param jgbLaufId
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public JGBLaufVO requestJgbLaufSelect (int jgbLaufId) throws Exception
  {
    JGBLaufVO vo = null;

    try
    {
      LOG.debug("requestJgbLaufSelect - get jgb lauf vo from ses db");

      vo = new JGBLaufDAO().select(jgbLaufId);

      LOG.debug("requestJgbLaufSelect - successfully done");

      return vo;

    }
    catch (Exception ex)
    {
      LOG.error("requestJgbLaufSelect - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }

  
  /**
   * request jgb lauf update
   * 
   * @param jgbLaufId
   * @param sapStat
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void requestJgbLaufUpdate(int jgbLaufId, int sapStat) throws Exception
  {
    try
    {
      LOG.debug("requestJgbLaufUpdate - update sap stat from jgb lauf");

      new JGBLaufDAO().updateSapStat(jgbLaufId,sapStat);

      LOG.debug("requestJgbLaufUpdate - successfully done");

    }
    catch (Exception ex)
    {
      LOG.error("requestJgbLaufUpdate - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }

  /**
   * request jgb end condition
   * 
   * @param jgbLaufId
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public boolean requestJgbEndCondition(int jgbLaufId) throws Exception
  {
    return new JoinSESRechnungDAO().checkJgbEndCondition(jgbLaufId);
  }

  /**
   * request jgb stop condition
   * 
   * @param jgbLaufId
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public boolean requestJgbStopCondition(int jgbLaufId) throws Exception
  {
    return new JoinSESRechnungDAO().checkJgbStopCondition(jgbLaufId);
  }

  //---------------------------------------------------------------------------
  // public methods - amv
  //---------------------------------------------------------------------------

  /**
   * @param vo 
   * @throws Exception 
   * 
   * @ejb.interface-method
   */
  public void requestAmvRechnungUpdate(AmvRechnungVO vo) throws Exception
  {
    try
    {
      LOG.debug("requestAmvRechnungUpdate - update amv_rechnung");

      new AmvRechnungDAO().update(vo);

      LOG.debug("requestAmvRechnungUpdate - successfully done");
    }
    catch (Exception ex)
    {
      LOG.error("requestAmvRechnungUpdate - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }

  /**
   * @param amvLaufId
   * @param laufStat
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void requestAmvLaufStatUpdate(int amvLaufId, int laufStat) throws Exception
  {
    try
    {
      LOG.debug("requestAmvLaufStatUpdate - update amv_lauf");

      new AmvLaufDAO().updateLaufStat(amvLaufId,laufStat);

      LOG.debug("requestAmvLaufStatUpdate - successfully done");
    }
    catch (Exception ex)
    {
      LOG.error("requestAmvLaufStatUpdate - Exception occured: " + ex.getMessage());
      sessionContext.setRollbackOnly();
      throw new Exception(ex.getMessage());
    }
  }


} // end class

