package ch.ige.wdltool.dao;

import java.io.Serializable;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.hibernate.validator.InvalidValue;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.dao.CachingDAO;
import ch.ige.dbutils.handlers.TypedResultSetHandler;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.dbutils.sql.SQLWhereStatementCreator;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.util.collection.CollectionSize;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.util.TermUtils;
import ch.ige.wdltool.validator.Validator;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.TermReplicationVO;
import ch.ige.wdlutil.text.SearchText;

/**
 * The abstract superclass for any DAO accessing the table <code>TERM</code>.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @param <V> The VO type
 * @param <I> The identifier type
 * @author   Eric Lewis
 * @version  $Id: BaseTermDAO.java,v 1.59 2008/12/09 16:09:57 bandschapp Exp $
 */
public abstract class BaseTermDAO<V extends AbstractVO & IdentifiableVO<I>, I extends Serializable> extends AbstractWdlDAO<V, I> implements CachingDAO
{
  /**
   * The number of terms that exist in the database
   */
  public static final int ESTIMATED_NUMBER_OF_TERMS = 30000;

  /**
   * The number of bytes in a megabyte
   */
  private static final int MEGABYTE = 1024 * 1024;

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( BaseTermDAO.class );

  /**
   * An <code>OR</code> part of an SQL statement
   */
  protected static final String OR = " OR ";

  /**
   * An <code>AND</code> part of an SQL statement
   */
  protected static final String AND = " AND ";

  /**
   * The date format used in the complex search.
   */
  protected static final DateFormat COMPLEX_SEARCH_DATE_FORMAT = new SimpleDateFormat( "dd.MM.yyyy" );

  /**
   * All the terms are cached in here, ordered by language and then by term type
   */
  private static Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> terms;

  /**
   * For term grouping, all the terms <strong>that are in a term group</strong> are cached in
   * here as well, ordered by term group ID and then by language
   */
  private static Map<Integer, Map<Locale, List<AbstractTermVO>>> termGroupTerms;

  /**
   * The term table's name in the DB.
   */
  private final String tableName = "term";

  /**
   * The name of the DB schema to use.
   */
  private String schemaName = null;

  /**
   * The only constructor
   */
  public BaseTermDAO()
  {
    super();
  }

  /**
   * Sets the schema name to use for the term.
   * @param schemaName The DB schema name to use
   */
  public void setSchemaName( String schemaName )
  {
    this.schemaName = schemaName;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected final String getTableName()
  {
    if ( this.schemaName == null )
    {
      return this.tableName;
    }
    else
    {
      return this.schemaName + "." + this.tableName;
    }
  }

  @Override
  protected String getIdColumnName()
  {
    return "term_id";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected final String getColumnNames()
  {
    return "data_source_id, class_id, term_group_id, term_type, "
        + "language_sn, term, search_term, is_generic_bool, is_public_bool, private_comment, public_comment, "
        + "is_active_bool, renewal_dat, usage_count, latest_occurrence_dat, created_user, changed_user, changed_dat, is_checker_only_bool";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getAliases()
  @Override
  protected final Map<String, String> getAliases()
  {
    return SQLStatementCreator.createAliases( "data_source_id", "data_source", "class_id", "class_number", "language_sn", "language", "is_generic_bool",
        "is_generic", "is_public_bool", "is_public", "is_active_bool", "is_active", "renewal_dat", "renewal", "latest_occurrence_dat", "latest_occurrence",
        "created_user", "creation_user", "changed_user", "change_user", "changed_dat", "change_date", "is_checker_only_bool", "is_checker_only" );
  }

  /**
   * Gets the term's term type.
   * @return The term's term type
   */
  protected abstract TermType getTermType();

  /**
   * Gets the term's type name, like <code>Nice term</code>.
   * @return The term's type name
   */
  protected abstract String getTermTypeName();

  // @see ch.ige.dbutils.dao.CachingDAO#clearCache()
  public synchronized void clearCache()
  {
    terms = null;
    termGroupTerms = null;
  }

  // @see ch.ige.dbutils.dao.CachingDAO#describeCachedContent()
  public CharSequence describeCachedContent()
  {
    StringBuilder description = new StringBuilder();
    if ( terms != null )
    {
      description.append( terms.toString() );
    }
    if ( termGroupTerms != null )
    {
      description.append( termGroupTerms.toString() );
    }
    return description.toString();
  }

  // @see ch.ige.dbutils.dao.CachingDAO#getCacheSize()
  public int getCacheSize()
  {
    int size = 0;
    if ( terms != null )
    {
      for ( Locale language: terms.keySet() )
      {
        size += CollectionSize.size( terms.get( language ) );
      }
    }
    if ( termGroupTerms != null )
    {
      for ( Integer termGroupId: termGroupTerms.keySet() )
      {
        size += CollectionSize.size( termGroupTerms.get( termGroupId ) );
      }
    }
    return size;
  }

  /**
   * Gets the terms, which are cached after the first loading.
   * 
   * @return The terms by language and then by term type (class)
   * @throws SQLException if the query fails
   */
  protected final synchronized Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> getTerms() throws SQLException
  {
    if ( terms == null )
    {
      SQLStatementCreator<AbstractTermVO> localSqlCreator = new SQLStatementCreator<AbstractTermVO>( getTableName(), getIdColumnName(), getColumnNames(),
          getIdSequenceName(), getAliases() );

      // IPI term candidates are never cached
      SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
      whereStatement.appendFragmentWithParameters( " term_type != ?", TermType.IPI_TERM_CANDIDATE.toDB() );

      List<AbstractTermVO> allTerms = selectVOList( ESTIMATED_NUMBER_OF_TERMS, AbstractTermVO.class, localSqlCreator.selectByOrderBy( whereStatement,
          "LENGTH(term) DESC" ), whereStatement.getStatementParameters() );
      LOG.debug( "Got " + allTerms.size() + " terms in total" );

      LOG.info( "Heap size before term caching: " + Runtime.getRuntime().totalMemory() / MEGABYTE + " MB" );
      LOG.info( "Free memory before term caching: " + Runtime.getRuntime().freeMemory() / MEGABYTE + " MB" );
      terms = createTermCache( allTerms );
      termGroupTerms = createTermGroupTermsCache( allTerms );
      LOG.info( "Heap size after term caching: " + Runtime.getRuntime().totalMemory() / MEGABYTE + " MB" );
      LOG.info( "Free memory after term caching: " + Runtime.getRuntime().freeMemory() / MEGABYTE + " MB" );
    }
    return terms;
  }

  /**
   * Gets the term group terms, which are cached after the first loading.
   * 
   * @return The terms by term group ID and then by language
   * @throws SQLException if the query fails
   */
  protected final synchronized Map<Integer, Map<Locale, List<AbstractTermVO>>> getTermGroupTerms() throws SQLException
  {
    if ( termGroupTerms == null )
    {
      // Create the cache
      getTerms();
    }
    return termGroupTerms;
  }

  /**
   * From all terms, creates the structure required for {@link #terms}.
   * @param allTerms All the existing terms
   * @return The structure required for {@link #terms}
   */
  private Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> createTermCache( List<AbstractTermVO> allTerms )
  {
    Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> termsMap = new HashMap<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>>();
    for ( AbstractTermVO term: allTerms )
    {
      if ( !termsMap.containsKey( term.getLanguage() ) )
      {
        //LOG.debug( "Creating terms for language " + term.getLanguage() );
        termsMap.put( term.getLanguage(), new HashMap<Class<? extends AbstractTermVO>, List<AbstractTermVO>>() );
      }
      Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>> typeTerms = termsMap.get( term.getLanguage() );
      if ( !typeTerms.containsKey( term.getClass() ) )
      {
        //LOG.debug( "Creating terms for language " + term.getLanguage() + " and " + term.getClass() );
        typeTerms.put( term.getClass(), new ArrayList<AbstractTermVO>( 1000 ) );
      }
      List<AbstractTermVO> termTypeTerms = typeTerms.get( term.getClass() );
      termTypeTerms.add( term );
    }

    int numberOfAssignedTerms = 0;
    for ( Locale language: termsMap.keySet() )
    {
      for ( Class<? extends AbstractTermVO> termClass: termsMap.get( language ).keySet() )
      {
        int numberOfTerms = termsMap.get( language ).get( termClass ).size();
        numberOfAssignedTerms += numberOfTerms;
        LOG.debug( "Got " + numberOfTerms + " terms for language " + language + " and " + termClass );
      }
    }
    LOG.debug( "This means a total of " + numberOfAssignedTerms + " classes were assigned." );
    if ( allTerms.size() != numberOfAssignedTerms )
    {
      LOG.error( "From a total of " + allTerms.size() + " terms, only " + numberOfAssignedTerms + " were assigned" );
    }
    return termsMap;
  }

  /**
   * From all terms, creates the structure required for {@link #termGroupTerms}.
   * @param allTerms All the existing terms
   * @return The structure required for {@link #termGroupTerms}
   */
  private Map<Integer, Map<Locale, List<AbstractTermVO>>> createTermGroupTermsCache( List<AbstractTermVO> allTerms )
  {
    Map<Integer, Map<Locale, List<AbstractTermVO>>> termsGroupTermsMap = new HashMap<Integer, Map<Locale, List<AbstractTermVO>>>();
    for ( AbstractTermVO term: allTerms )
    {
      Integer termGroupId = TermUtils.getTermGroupId( term );
      if ( termGroupId != null )
      {
        TermUtils.setTermGroupNumber( term );
        if ( !termsGroupTermsMap.containsKey( termGroupId ) )
        {
          //LOG.debug( "Creating terms for term group ID " + termGroupId );
          termsGroupTermsMap.put( termGroupId, new HashMap<Locale, List<AbstractTermVO>>() );
        }
        Map<Locale, List<AbstractTermVO>> termGroupLanguageTerms = termsGroupTermsMap.get( termGroupId );
        if ( !termGroupLanguageTerms.containsKey( term.getLanguage() ) )
        {
          //LOG.debug( "Creating terms for term group ID " + termGroupId + " and language " + term.getLanguage() );
          termGroupLanguageTerms.put( term.getLanguage(), new ArrayList<AbstractTermVO>( 1000 ) );
        }
        List<AbstractTermVO> targetTerms = termGroupLanguageTerms.get( term.getLanguage() );
        targetTerms.add( term );
      }
    }

    return termsGroupTermsMap;
  }

  // @see ch.ige.dbutils.dao.AbstractDAO#selectVOList(int, java.lang.Class, ch.ige.dbutils.sql.SQLStatement, java.lang.Object[])
  @Override
  protected <T extends AbstractVO> List<T> selectVOList( int estimateResultCount, Class<T> type, SQLStatement<T> query, Object... parameters )
      throws SQLException
  {
    return this.getQueryRunner().query( query, VOList( type, estimateResultCount ), parameters );
  }

  // @see ch.ige.dbutils.dao.AbstractDAO#VOList(Class, int)
  public static <T extends AbstractVO> TypedResultSetHandler<List<T>> VOList( Class<T> type, int expectedSize )
  {
    return new TermVOListHandler<T>( type, AbstractVO.determineDefaultNullReplacements( type ), expectedSize );
  }

  /**
   * Gets one specific term.
   * @param termId The unique ID of the term
   * @param termClass The type of term to get
   * @return The term or <code>null</code> if it doesn't exist
   * @throws SQLException if the query fails
   */
  protected AbstractTermVO find( Integer termId, Class<? extends AbstractTermVO> termClass ) throws SQLException
  {
    Check.notNegative( termId, "term ID" );
    Check.notNull( termClass, "term class" );

    Map<Locale, Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>>> cachedTerms = getTerms();
    for ( Locale language: cachedTerms.keySet() )
    {
      Map<Class<? extends AbstractTermVO>, List<AbstractTermVO>> languageTerms = cachedTerms.get( language );
      LOG.debug( "Got keys" + StringUtils.join( languageTerms.keySet().iterator(), ", " ) );
      List<AbstractTermVO> allTerms = languageTerms.get( termClass );
      // Maybe the ID can't be found, because there are no such VOs yet...
      if ( allTerms != null )
      {
        for ( AbstractTermVO term: allTerms )
        {
          if ( term.getIdentifier().equals( termId ) )
          {
            return term;
          }
        }
      }
    }
    return null;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#findAll()
  @Override
  protected List<V> findAll() throws SQLException
  {
    return findAllRestricted( getFindAllWhereStatement() );
  }

  /**
   * Gets nearly all terms, excluding the ones in the where statement .
   * @param whereStatement The restriction of data which should be returned.
   * @return The VOs left after the restriction.
   * @throws SQLException if the query fails
   */
  protected List<V> findAllRestricted( SQLWhereStatementCreator whereStatement ) throws SQLException
  {
    SQLStatementCreator<V> statementCreator = new SQLStatementCreator<V>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
        getAliases() );
    SQLStatement<V> findAllStatement = statementCreator.selectByOrderBy( whereStatement, getIdColumnName() );
    return selectVOList( 200, getVOType(), findAllStatement, whereStatement.getStatementParameters() );
  }

  /**
   * Gets the <code>WHERE</code> statement for {@link #findAll()}.
   * @return The where statement for all terms for a certain type.
   */
  protected SQLWhereStatementCreator getFindAllWhereStatement()
  {
    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    whereStatement.appendFragmentWithParameters( " term_type = ?", getTermType().toDB() );
    return whereStatement;
  }

  /**
   * Inserts a term.
   * @param term The term to insert
   * @return The new identifier
   */
  @Override
  public I insert( V term )
  {
    return insert( term, true );
  }

  /**
   * Inserts a term.
   * <strong>Note: If you're not sure what you're doing, use {@link #insert(AbstractVO)}!</strong>
   * @param term The term to insert
   * @param clearCache If this is <code>true</code>, the cache is cleared, otherwise it's left as is
   * @return The new identifier
   */
  protected final I insert( V term, boolean clearCache )
  {
    Check.notNull( term, getTermTypeName() );
    TermUtils.setChangeTimestamp( (AbstractTermVO) term );
    TermUtils.setSearchTerm( (AbstractTermVO) term );
    I id = super.insert( term );
    if ( clearCache )
    {
      clearCache();
    }
    return id;
  }

  /**
   * Updates a term.
   * @param term The term to update
   */
  @Override
  public void update( V term )
  {
    update( term, true );
  }

  /**
   * Updates a term.
   * <strong>Note: If you're not sure what you're doing, use {@link #update(AbstractVO)}!</strong>
   * @param term The term to update
   * @param clearCache If this is <code>true</code>, the cache is cleared, otherwise it's left as is
   */
  protected final void update( V term, boolean clearCache )
  {
    Check.notNull( term, getTermTypeName() );
    TermUtils.setChangeTimestamp( (AbstractTermVO) term );
    TermUtils.setSearchTerm( (AbstractTermVO) term );
    super.update( term );
    if ( clearCache )
    {
      clearCache();
    }
  }

  /**
   * Deletes a term.
   * @param termId The term's identifier
   */
  @Override
  public void delete( Integer termId )
  {
    delete( termId, getTermType(), true );
  }

  /**
   * Deletes a term.
   * <strong>Note: If you're not sure what you're doing, use {@link #delete(Integer)}!</strong>
   * @param termId The term's identifier
   * @param clearCache If this is <code>true</code>, the cache is cleared, otherwise it's left as is
   */
  protected final void delete( Integer termId, boolean clearCache )
  {
    delete( termId, getTermType(), clearCache );
  }

  /**
   * Deletes a term.
   * @param termId The unique identifier for a term
   * @param termType The term type to delete, just to be sure!
   * @param clearCache If this is <code>true</code>, the cache is cleared, otherwise it's left as is
   */
  @SuppressWarnings( "unchecked" )
  private final void delete( final Integer termId, final TermType termType, final boolean clearCache )
  {
    Check.notNegative( termId, getTermTypeName() + " ID" );
    final SQLStatement<Integer> deleteByTermId = getSqlCreator().deleteBy( getIdColumnName() + " = ? AND term_type = ?" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( deleteByTermId, termId, termType.toDB() );
        verifyUpdateCount( count, 1, "update count must be 1" );
        if ( clearCache )
        {
          clearCache();
        }
      }
    } );
  }

  /**
   * Reads the terms which should be replicated and creates
   * the SQL <code>INSERT</code> statement together with all the
   * data to insert.
   * Note: All public and active terms should be replicated - including 'checker only' terms.
   * @param termSchemaName The schema name used to fully qualify the term table
   * @return The SQL statement and data to replicate all terms
   * @throws SQLException if the query fails
   */
  public abstract TermReplicationVO getReplicationTerms( String termSchemaName ) throws SQLException;

  /**
   * Appends the full <code>LIKE</code> statement used for a search term.
   * For instance
   * <pre>
   * begriff LIKE '%Holz%' OR suchbegriff LIKE '%Holz%' OR kommentar_int LIKE '%Holz%' OR kommentar_ext LIKE '%Holz%' 
   * </pre>
   * @param statement The statement to append to
   * @param searchText The search text
   * @param language The language to use
   * @param searchComment The flag determining whether search should be limited to the comment or to the term only.
   * If this is <code>true</code> search in comment only
   * If this is <code>false</code> search in term only
   * If this is <code>null</code> both comment and term are searched
   * @param exactSearch If this is <code>true</code> then fuzzy-search over search term is disabled otherwise term and search term are used for matching. 
   * @param createLikeStatement If this is <code>true</code>, a <code>LIKE</code> fragment is appended,
   *        otherwise a strict equality fragment with <code>=</code>
   */
  protected static void appendFullEqualityStatement( SQLWhereStatementCreator statement, String searchText, Locale language, Boolean searchComment,
      boolean exactSearch, boolean createLikeStatement )
  {
    final boolean compareUsingLowerCase = true;
    if ( StringUtils.isNotEmpty( searchText ) )
    {
      if ( searchComment == null )
      {
        statement.appendTextEqualityStatement( "term", searchText, createLikeStatement, compareUsingLowerCase );
        if ( !exactSearch )
        {
          statement.appendFragment( OR );
          statement.appendTextEqualityStatement( "search_term", SearchText.toSearchText( searchText, language ), createLikeStatement, compareUsingLowerCase );
        }
        statement.appendFragment( OR );
        statement.appendTextEqualityStatement( "private_comment", searchText, createLikeStatement, compareUsingLowerCase );
        statement.appendFragment( OR );
        statement.appendTextEqualityStatement( "public_comment", searchText, createLikeStatement, compareUsingLowerCase );
      }
      else if ( searchComment == true )
      {
        statement.appendTextEqualityStatement( "private_comment", searchText, createLikeStatement, compareUsingLowerCase );
        statement.appendFragment( OR );
        statement.appendTextEqualityStatement( "public_comment", searchText, createLikeStatement, compareUsingLowerCase );

      }
      else if ( searchComment == false )
      {
        statement.appendTextEqualityStatement( "term", searchText, createLikeStatement, compareUsingLowerCase );
        if ( !exactSearch )
        {
          statement.appendFragment( OR );
          statement.appendTextEqualityStatement( "search_term", SearchText.toSearchText( searchText, language ), createLikeStatement, compareUsingLowerCase );
        }
      }
    }
  }

  /**
   * Appends a <code>NLSSORT</code> fragment (used for ordering) to an SQL statement.
   * For instance
   * <pre>
   * NLSSORT(name, 'NLS_SORT = French')
   * </pre>
   * @param statement The statement to append to
   * @param columnName The name of the column used
   * @param language The language to use for sorting
   */
  protected static void appendLanguageSpecificOrderingColumn( SQLWhereStatementCreator statement, String columnName, Locale language )
  {
    // This doesn't work with '?' placeholders!
    statement.appendFragment( "NLSSORT(" );
    statement.appendFragment( columnName );
    statement.appendFragment( ", 'NLS_SORT = " + language.getDisplayLanguage( Locale.ENGLISH ) + "')" );
  }

  /**
   * Appends the default <code>ORDER BY</code> fragment for terms.
   * For instance
   * <pre>
   * ORDER BY class_id, NLSSORT(term, 'NLS_SORT = French')
   * @param statement The statement to append to
   * @param language The language to use for sorting
   */
  protected static void appendDefaultOrderBy( SQLWhereStatementCreator statement, Locale language )
  {
    statement.appendFragment( " ORDER BY class_id" );
    if ( language != null )
    {
      statement.appendFragment( ", " );
      appendLanguageSpecificOrderingColumn( statement, "term", language );
    }
  }

  /**
   * Escapes single quotes in the search text by repeating them,
   * for instance turning <code>I'm good</code> into <code>I''m good</code>
   * @param searchText The search text whose single quotes are escaped
   * @return The search text with escape single quotes
   */
  protected static String escapeSingleQuotes( final String searchText )
  {
    return StringUtils.replace( searchText, "'", "''" );
  }

  /**
   * Checks a term for length.
   * @param term The term to check
   * @throws IllegalArgumentException if the term is too long
   */
  protected void checkTermLength( String term ) throws IllegalArgumentException
  {
    Check.notEmpty( term, "search text" );
    try
    {
      AbstractTermVO termVO = new IpiTermVO();
      termVO.setTerm( term );
      Validator.validate( termVO, false );
    }
    catch ( WdlValidationException e )
    {
      List<InvalidValue> invalidValues = e.getInvalidValues();
      for ( InvalidValue invalidValue: invalidValues )
      {
        if ( invalidValue.getPropertyName().equals( "term" ) && invalidValue.getMessage().startsWith( "validator.length" ) )
        {
          throw new IllegalArgumentException( "Invalid term length " + term.length(), e );
        }
      }
    }
  }
}
