package ch.ige.wdltool.dao;

import java.sql.SQLException;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.util.Configuration;
import ch.ige.wdltool.vo.ConfigurationVO;

/**
 * <strong>This DAO should normally not be used, use {@link Configuration} instead!</strong>
 * Accesses the table <code>CONFIGURATION</code>.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   hec
 * @version  $Id: ConfigurationDAO.java,v 1.17 2008/03/31 09:58:12 lewis_lee Exp $
 */
public class ConfigurationDAO extends AbstractWdlDAO<ConfigurationVO, Integer>
{
  /**
   * The type of value object to return
   */
  private static final Class<ConfigurationVO> VO_TYPE = ConfigurationVO.class;

  /**
   * The column names for the table
   */
  private static final String COLUMN_NAMES = "key, value";

  /**
   * Used to identify a configuration
   */
  private static final String WHERE_KEY_CLAUSE = "key = ?";

  /**
   * public constructor
   */
  public ConfigurationDAO()
  {
    super();
  }

  /**
   * Finds a single configuration entry.
   * @param key The configuration entry's unique key
   * @return The configuration entry or <code>null</code> if none is found
   * @throws SQLException if the query fails
   */
  public ConfigurationVO find( final String key ) throws SQLException
  {
    Check.notEmpty( key, "key" );
    return selectVO( ConfigurationVO.class, getSqlCreator().selectBy( WHERE_KEY_CLAUSE ), key );
  }

  /**
   * Inserts or updates a configuration entry.
   * If the key already exists, the key/value pair is updated;
   * if the key doesn't exist, the key/value pair is inserted.
   * @param key The configuration key to look for
   * @param value The value to set
   * @return The configuration entry
   * @throws SQLException if the query fails
   */
  public ConfigurationVO save( final String key, final String value ) throws SQLException
  {
    Check.notEmpty( value, "value" );
    ConfigurationVO existingConfiguration = find( key );

    if ( existingConfiguration == null )
    {
      ConfigurationVO newConfiguration = new ConfigurationVO();
      newConfiguration.setKey( key );
      newConfiguration.setValue( value );
      insert( newConfiguration );
      return newConfiguration;
    }
    else
    {
      existingConfiguration.setValue( value );
      update( existingConfiguration );
      return existingConfiguration;
    }
  }

  /**
   * Deletes a single configuration entry.
   * Note that if the key doesn't exist, this doesn't throw any Exception.
   * @param key The configuration entry's unique key
   * @throws SQLException if the query fails
   */
  public void delete( final String key ) throws SQLException
  {
    Check.notEmpty( key, "key" );
    ConfigurationVO existingConfiguration = find( key );
    if ( existingConfiguration != null )
    {
      delete( existingConfiguration.getIdentifier() );
    }
  }

  /**
   * Inserts a configuration entry and sets its ID.
   * @param configurationEntry The configuration entry to insert
   * @return The ID of the inserted configuration
   */
  protected Integer insert( final ConfigurationVO configurationEntry )
  {
    Check.notNull( configurationEntry, "configuration entry" );

    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( ConfigurationDAO.this.getInsert(), getValues( configurationEntry ) );
        verifyUpdateCount( count, 1, "Update count must be 1" );
        Integer id = ConfigurationDAO.this.selectCurrentSequenceValue( runner, getIdSequenceName() );
        configurationEntry.setIdentifier( id );
        this.setResult( id );
      }

      /**
       * Since the configuration has changed in the DB, make it clear to Commons Config that it must reload.
       */
      public void commit() throws SQLException
      {
        super.commit();
        Configuration.readConfig();
      }
    } );
  }

  /**
   * Updates a configuration entry.
   * @param configurationEntry The configuration entry to update
   */
  @SuppressWarnings( "unchecked" )
  protected void update( final ConfigurationVO configurationEntry )
  {
    Check.notNull( configurationEntry, "configuration entry" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( ConfigurationDAO.this.getUpdate(), getValuesWithIdentifier( configurationEntry ) );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }

      /**
       * Since the configuration has changed in the DB, make it clear to Commons Config that it must reload.
       */
      public void commit() throws SQLException
      {
        super.commit();
        Configuration.readConfig();
      }
    } );

  }

  /**
   * Deletes a configuration entry. Also notifies Configuration about required reload.
   * @param configurationId The unique identifier for a configuration entry
   */
  @SuppressWarnings( "unchecked" )
  protected void delete( final Integer configurationId )
  {
    Check.notNegative( configurationId, "configuration ID" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( ConfigurationDAO.this.getDelete(), configurationId );
        verifyUpdateCount( count, 1, "update count must be 1" );
      }

      /**
       * Since the configuration has changed in the DB, make it clear to Commons Config that it must reload.
       */
      public void commit() throws SQLException
      {
        super.commit();
        Configuration.readConfig();
      }
    } );

  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected String getColumnNames()
  {
    return COLUMN_NAMES;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected String getTableName()
  {
    return "configuration";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( ConfigurationVO vo )
  {
    return new Object[] { vo.getKey(), vo.getValue() };
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<ConfigurationVO> getVOType()
  {
    return VO_TYPE;
  }
}
