package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.util.Configuration;
import ch.ige.wdltool.vo.DataSourceVO;

/**
 * Accesses the table <code>DATA_SOURCE</code>.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: DataSourceDAO.java,v 1.7 2008/03/31 09:58:12 lewis_lee Exp $
 */
public class DataSourceDAO extends AbstractWdlDAO<DataSourceVO, DataSource>
{
  /**
   * The only constructor.
   */
  public DataSourceDAO()
  {
    super();
  }

  /**
   * The column names for the table
   */
  private static final String COLUMN_NAMES = "name, term_validity_period, term_confirmation_period, is_cleaner_active_bool, changed_dat";

  /**
   * The column name aliases
   */
  private static final Map<String, String> ALIASES = SQLStatementCreator.createAliases( "is_cleaner_active_bool", "is_cleaner_active", "changed_dat",
      "change_date" );

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<DataSourceVO> getVOType()
  {
    return DataSourceVO.class;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected String getTableName()
  {
    return "data_source";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected String getColumnNames()
  {
    return COLUMN_NAMES;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getAliases()
  @Override
  protected Map<String, String> getAliases()
  {
    return ALIASES;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( DataSourceVO dataSource )
  {
    return new Object[] { dataSource.getName(), dataSource.getTermValidityPeriod(), dataSource.getTermConfirmationPeriod(), dataSource.isCleanerActive(),
        dataSource.getChangeDate() };
  }

  /**
   * Finds all data sources.
   * @return The data sources ordered by ID
   * @throws SQLException if the query fails
   */
  public List<DataSourceVO> findAll() throws SQLException
  {
    return super.findAll();
  }

  /**
   * Finds a data source.
   * @param dataSource The data source to find
   * @return The data source or <code>null</code> if it doesn't exist
   * @throws SQLException if the query fails
   */
  public DataSourceVO find( final DataSource dataSource ) throws SQLException
  {
    Check.notNull( dataSource, "data source" );
    return super.find( dataSource.toDB() );
  }

  /**
   * Updates a data source.
   * @param dataSource The data source to update
   */
  @SuppressWarnings( "unchecked" )
  public void update( final DataSourceVO dataSource )
  {
    // Note: This must be implemented individually because of the commit()
    Check.notNull( dataSource, "data source" );

    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        Timestamp now = new Timestamp( System.currentTimeMillis() );
        dataSource.setChangeDate( now );
        int count = runner.update( DataSourceDAO.this.getUpdate(), getValuesWithIdentifier( dataSource ) );
        verifyUpdateCount( count, 1, "Update count must be 1" );
      }

      /**
       * Since the configuration has changed in the DB, make it clear to Commons Config that it must reload.
       */
      public void commit() throws SQLException
      {
        super.commit();
        Configuration.readConfig();
      }
    } );
  }
}
