package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.vo.IpiTermGroupVO;
import ch.ige.wdltool.vo.TermGroupVO;

/**
 * Accesses the table <code>TERM_GROUP</code> and deals with the IPI term group data.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bandschapp
 * @version  $Id: IpiTermGroupDAO.java,v 1.15 2008/03/31 09:58:12 lewis_lee Exp $
 */
public class IpiTermGroupDAO extends BaseTermGroupDAO<IpiTermGroupVO>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( IpiTermGroupDAO.class );

  /**
   * New IPI term group numbers are fetched from this sequence.
   */
  private static final String IPI_TERM_NUMBER_SEQUENCE = "seq_ipi_term_no";

  /**
   * Checks whether a new IPI term group number coming from {@link #IPI_TERM_NUMBER_SEQUENCE}
   * already exists.
   */
  private static final SQLStatement<Integer> SELECT_EXISTING_IPI_TERM_NUMBER = SQL.select( "SELECT COUNT(*) FROM term_group WHERE ipi_term_no = ?" );

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( IpiTermGroupVO vo )
  {
    return new Object[] { null, null, vo.getIpiTermNumber() };
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<IpiTermGroupVO> getVOType()
  {
    return IpiTermGroupVO.class;
  }

  /**
   * Gets the next free IPI term group number and inserts a
   * IPI term group with it.
   * @return The inserted IPI term group with the next free IPI term group number
   * @throws SQLException if the IPI term group can't be created
   */
  public IpiTermGroupVO createNextFreeIpiTermGroup() throws SQLException
  {
    boolean foundNewTermNumber = false;
    Integer termNumber = null;
    do
    {
      termNumber = selectNextSequenceValue( getQueryRunner(), IPI_TERM_NUMBER_SEQUENCE );
      LOG.debug( "Next IPI term number from sequence is " + termNumber );

      Integer existingTermNumbers = selectInteger( getQueryRunner(), SELECT_EXISTING_IPI_TERM_NUMBER, termNumber );
      String baseMessage = "Found " + existingTermNumbers + " IPI term group(s) with number " + termNumber;
      LOG.debug( baseMessage );
      if ( existingTermNumbers != 0 && existingTermNumbers != 1 )
      {
        throw new SQLException( baseMessage + ", but there should be only none or one" );
      }
      if ( existingTermNumbers == 0 )
      {
        foundNewTermNumber = true;
      }
    } while ( !foundNewTermNumber );

    if ( termNumber == null )
    {
      throw new SQLException( "Sequence for IPI term number must have run out" );
    }

    IpiTermGroupVO ipiTermGroup = new IpiTermGroupVO();
    ipiTermGroup.setIpiTermNumber( termNumber );
    Integer id = insert( ipiTermGroup );
    LOG.debug( "Inserted new IPI term number " + termNumber + " with ID " + id );
    return ipiTermGroup;
  }

  /**
   * Inserts an IPI term group.
   * @param ipiTermGroup The IPI term group to insert
   * @return The ID of the inserted IPI term group
   */
  public Integer insert( final IpiTermGroupVO ipiTermGroup )
  {
    Integer id = super.insert( ipiTermGroup );
    clearCache();
    return id;
  }

  /**
   * Finds one IPI term group.
   * @param ipiTermGroupId The ID of this term group
   * @return The IPI term group or <code>null</code> if none is found
   * @throws SQLException in case the database access is not successful
   */
  public IpiTermGroupVO find( final Integer ipiTermGroupId ) throws SQLException
  {
    Check.notNegative( ipiTermGroupId, "IPI term group ID" );
    return (IpiTermGroupVO) getTermGroups( IpiTermGroupVO.class ).get( ipiTermGroupId );
  }

  /**
   * Finds one IPI term group.
   * @param ipiTermNumber The number of this term group
   * @return The IPI term group or <code>null</code> if none is found
   * @throws SQLException in case the database access is not successful
   */
  public IpiTermGroupVO findByNumber( final Integer ipiTermNumber ) throws SQLException
  {
    Check.notNegative( ipiTermNumber, "IPI term number" );
    Collection<TermGroupVO> termGroups = getTermGroups( IpiTermGroupVO.class ).values();
    IpiTermGroupVO foundIpiTermGroup = null;
    for ( TermGroupVO termGroupVO: termGroups )
    {
      IpiTermGroupVO ipiTermGroup = (IpiTermGroupVO) termGroupVO;
      if ( ipiTermGroup.getIpiTermNumber().equals( ipiTermNumber ) )
      {
        foundIpiTermGroup = ipiTermGroup;
        break;
      }
    }
    return foundIpiTermGroup;
  }

  /**
   * Finds all the IPI term groups.
   * @return The IPI term groups ordered by ID
   * @throws SQLException in case the database access is not successful
   */
  public List<IpiTermGroupVO> findAll() throws SQLException
  {
    Collection<TermGroupVO> rawTermGroups = getTermGroups( IpiTermGroupVO.class ).values();
    List<IpiTermGroupVO> termGroups = new ArrayList<IpiTermGroupVO>( rawTermGroups.size() );
    for ( TermGroupVO rawTermGroup: rawTermGroups )
    {
      termGroups.add( (IpiTermGroupVO) rawTermGroup );
    }
    return termGroups;
  }

  /**
   * Updates an IPI term group.
   * @param ipiTermGroup The IPI term group to be updated
   */
  public void update( final IpiTermGroupVO ipiTermGroup )
  {
    super.update( ipiTermGroup );
    clearCache();
  }

  /**
   * Deletes an IPI term group.
   * @param ipiTermGroupId The ID of the IPI term group to be deleted
   */
  public void delete( final Integer ipiTermGroupId )
  {
    super.delete( ipiTermGroupId );
    clearCache();
  }
}
