package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.StopwordVO;

/**
 * Tests {@link ch.ige.wdltool.dao.StopwordDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: StopwordDAOTest.java,v 1.26 2008/04/29 16:38:12 heinzer Exp $
 */
public class StopwordDAOTest extends BaseDAOWithCRUDTestTemplate<StopwordDAO, StopwordVO, Integer>
{
  /**
   * The minimum number of stopwords in the DB. 
   */
  private static final int NUMBER_OF_STOPWORDS = 110;

  /**
   * The minimum number of German stopwords in the DB. 
   */
  private static final int NUMBER_OF_GERMAN_STOPWORDS = 50;

  /**
   * The only constructor
   * @param name The test's name
   */
  public StopwordDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<StopwordDAO> getDAOClass()
  {
    return StopwordDAO.class;
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    // Find an existing stopword
    StopwordVO existingStopword = getExpectedVO();
    defaultTestReadCollection( existingStopword, NUMBER_OF_STOPWORDS );

    List<StopwordVO> stopwords = getDAO().findAll();
    compareVOsAndValidateLoaded( existingStopword, stopwords );

    int previousLength = 999999999;
    Locale previousLanguage = Locale.CHINESE;
    List<Locale> languagesInStopwords = new ArrayList<Locale>( 3 );
    for ( StopwordVO stopword: stopwords )
    {
      Locale currentLanguage = stopword.getLanguage();
      if ( !currentLanguage.equals( previousLanguage ) )
      {
        assertTrue( "Checking that stopwords are sorted by language", !languagesInStopwords.contains( currentLanguage ) );
        languagesInStopwords.add( currentLanguage );
        previousLength = 999999999;
      }

      assertTrue( "Checking that the stopwords are sorted by descending length", stopword.getWord().length() <= previousLength );
      previousLength = stopword.getWord().length();
      previousLanguage = stopword.getLanguage();
    }

    // There may or may not be Italian stopwords
    assertTrue( "Checking number of languages", languagesInStopwords.size() >= 2 && languagesInStopwords.size() <= 3 );
    assertEquals( "Checking first language", Locale.GERMAN, languagesInStopwords.get( 0 ) );
    assertEquals( "Checking second language", Locale.FRENCH, languagesInStopwords.get( 1 ) );
    if ( languagesInStopwords.size() > 2 )
    {
      assertEquals( "Checking third language", Locale.ITALIAN, languagesInStopwords.get( 2 ) );
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#findAll(Locale)}.
   * @throws Exception if the test fails
   */
  public void testFindAllWithLanguage() throws Exception
  {
    List<StopwordVO> stopwords = getDAO().findAll( Locale.GERMAN );
    assertTrue( "Checking number of German stopwords", stopwords.size() >= NUMBER_OF_GERMAN_STOPWORDS );
    compareVOsAndValidateLoaded( getExpectedVO(), stopwords );

    int previousLength = 999999999;
    for ( StopwordVO stopword: stopwords )
    {
      assertTrue( "Checking that the stopwords are sorted by descending length", stopword.getWord().length() <= previousLength );
      previousLength = stopword.getWord().length();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    StopwordVO stopwordVO = createAndInsert( true );
    defaultTestRead( stopwordVO, -1, 999999999, stopwordVO.getIdentifier() );

    // Find an existing stopword
    StopwordVO existingStopword = getExpectedVO();
    compareVOs( existingStopword, getDAO().find( existingStopword.getIdentifier() ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#find(Locale, String)}.
   * @throws Exception if the test fails
   */
  public void testFindStopwordAndLanguage() throws Exception
  {
    StopwordVO existingStopword = getExpectedVO();

    // Try to find an existing stopword, but with the wrong language
    assertNull( getDAO().find( Locale.FRENCH, existingStopword.getWord() ) );

    // Find an existing stopword
    compareVOs( existingStopword, getDAO().find( Locale.GERMAN, existingStopword.getWord() ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#findAnyWord(String)}.
   * @throws Exception if the test fails
   */
  public void testFindAnyWord() throws Exception
  {
    List<StopwordVO> stopwords = getDAO().findAnyWord( "pharma" );
    assertEquals( "Checking number of stopwords", 2, stopwords.size() );
    checkStopword( stopwords, 0, Locale.FRENCH, "pharmaceutique" );
    checkStopword( stopwords, 1, Locale.GERMAN, "pharmazeutische" );

    // Several words
    stopwords = getDAO().findAnyWord( "haus médical" );
    assertEquals( "Checking number of stopwords", 4, stopwords.size() );
    checkStopword( stopwords, 0, Locale.GERMAN, "haushalt" );
    checkStopword( stopwords, 1, Locale.FRENCH, "médical" );
    checkStopword( stopwords, 2, Locale.FRENCH, "médicale" );
    checkStopword( stopwords, 3, Locale.FRENCH, "médicales" );
  }

  /**
   * Checks the stopword.
   * @param stopwords All found stopwords
   * @param index The index of the stopword to check
   * @param language The expected language
   * @param stopword The expected stopword itself
   */
  private static void checkStopword( List<StopwordVO> stopwords, int index, Locale language, String stopword )
  {
    StopwordVO stopwordVO = stopwords.get( index );
    assertEquals( "Checking stopword language", language, stopwordVO.getLanguage() );
    assertEquals( "Checking stopword itself", stopword, stopwordVO.getWord() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#findAnyWord(Locale, String)}.
   * @throws Exception if the test fails
   */
  public void testFindAnyWordWithLanguage() throws Exception
  {
    List<StopwordVO> stopwords = getDAO().findAnyWord( Locale.GERMAN, "pharma" );
    assertEquals( "Checking number of stopwords", 1, stopwords.size() );
    checkStopword( stopwords, 0, Locale.GERMAN, "pharmazeutische" );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#insert(ch.ige.wdltool.vo.StopwordVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#update(ch.ige.wdltool.vo.StopwordVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO#delete(Collection)}.
   * @throws Exception if the test fails
   */
  public void testDeleteSeveral() throws Exception
  {
    try
    {
      getDAO().delete( (Collection<Integer>) null );
      fail( "Null IDs must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().delete( new ArrayList<Integer>() );
      fail( "Empty IDs must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    StopwordVO stopword1 = createAndInsert();
    StopwordVO stopword2 = createAndInsert();

    List<Integer> stopwordIds = new ArrayList<Integer>();
    stopwordIds.add( stopword1.getIdentifier() );
    stopwordIds.add( stopword2.getIdentifier() );

    getDAO().delete( stopwordIds );

    StopwordVO loadedVO = read( stopword1.getIdentifier() );
    assertNull( "Checking that delete was successful", loadedVO );

    loadedVO = read( stopword2.getIdentifier() );
    assertNull( "Checking that delete was successful", loadedVO );

    clearSavedId();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO} caching.
   * @throws Exception if the test fails
   */
  public void testCaching() throws Exception
  {
    defaultTestCache( getExpectedVO().getIdentifier() );
    defaultTestCacheSize( getExpectedVO().getIdentifier(), NUMBER_OF_STOPWORDS );

    long[] runTimes = defaultTestCachePerformanceForRead( getExpectedVO(), getExpectedVO().getIdentifier() );
    System.out.println( "Reading one stopword without cache took " + runTimes[0] + " milliseconds." );
    System.out.println( "Reading one stopword with cache took " + runTimes[1] + " milliseconds." );

    runTimes = defaultTestCachePerformanceForReadCollection( getExpectedVO(), NUMBER_OF_STOPWORDS );
    System.out.println( "Reading all stopwords without cache took " + runTimes[0] + " milliseconds." );
    System.out.println( "Reading all stopwords with cache took " + runTimes[1] + " milliseconds." );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO} caching for inserting.
   * @throws Exception if the test fails
   */
  public void testCacheConsistencyForInsert() throws Exception
  {
    defaultTestCacheConsistencyForCreate( true, (Object[]) null );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO} caching for updating.
   * @throws Exception if the test fails
   */
  public void testCacheConsistencyForUpdate() throws Exception
  {
    defaultTestCacheConsistencyForUpdate( (Object[]) null );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StopwordDAO} caching for deleting.
   * @throws Exception if the test fails
   */
  public void testCacheConsistencyForDelete() throws Exception
  {
    defaultTestCacheConsistencyForDelete( (Object[]) null );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected StopwordVO read( Integer stopwordId ) throws SQLException
  {
    return getDAO().find( stopwordId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<StopwordVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().findAll();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( StopwordVO originalVO, StopwordVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking language", originalVO.getLanguage(), loadedVO.getLanguage() );
    assertEquals( "Checking stopword", originalVO.getWord(), loadedVO.getWord() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( StopwordVO originalVO, StopwordVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected Integer insert( StopwordVO stopwordVO ) throws SQLException
  {
    return getDAO().insert( stopwordVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void modifyVOBeforeUpdate( StopwordVO stopwordVO )
  {
    stopwordVO.setLanguage( Locale.ITALIAN );
    stopwordVO.setWord( "sono un stopword" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected void update( StopwordVO stopwordVO ) throws SQLException
  {
    getDAO().update( stopwordVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @SuppressWarnings( "unused" )
  @Override
  protected void delete( Integer stopwordId ) throws SQLException
  {
    getDAO().delete( stopwordId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public StopwordVO getSerializableVO()
  {
    StopwordVO stopword = new StopwordVO( Locale.FRENCH, "je suis un stopword" );
    stopword.setIdentifier( 33 );
    return stopword;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  public StopwordVO getExpectedVO()
  {
    StopwordVO stopword = new StopwordVO( Locale.GERMAN, "über" );
    stopword.setIdentifier( 170 ); // This may change!
    return stopword;
  }
}
