package ch.ige.wdltool.dao;

import java.io.ObjectStreamException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import ch.ige.dbutils.handlers.VOListHandler;
import ch.ige.dbutils.handlers.assigners.FieldAssigner;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.wdltool.vo.IpiTermGroupVO;
import ch.ige.wdltool.vo.NiceTermGroupVO;
import ch.ige.wdltool.vo.StandardWdlTermGroupVO;
import ch.ige.wdltool.vo.TermGroupVO;

/**
 * Converts the result of a query to a <code>List</code> of term group value objects.
 * This is required because the results can be different VO types.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @param <T> The VO type
 * @author   Eric Lewis
 * @version  $Id: TermGroupVOListHandler.java,v 1.1 2007/10/11 14:32:56 lewis_lee Exp $
 */
public class TermGroupVOListHandler<T extends AbstractVO> extends VOListHandler<T>
{
  /**
   * Creates the handler.
   * 
   * @param type The class of the returned value objects
   * @param expectedSize The number of value objects to expect from the query
   */
  public TermGroupVOListHandler( Class<T> type, int expectedSize )
  {
    super( type, expectedSize );
  }

  /**
   * Creates the handler.
   * 
   * @param type The class of the returned value objects
   * @param nullReplacementsByFieldName The <code>Map</code> to replace <code>null</code> values in the VO. The key
   *          is the field name (a String) and the value is an object to set instead of <code>null</code>
   * @param expectedSize The number of value objects to expect from the query
   */
  public TermGroupVOListHandler( Class<T> type, Map<String, Object> nullReplacementsByFieldName, int expectedSize )
  {
    super( type, nullReplacementsByFieldName, expectedSize );
  }

  /**
   * Creates the VO and fills it using the result set and the field assigners.
   * 
   * @param resultSet The result of the query
   * @param fieldAssigners A <code>List</code> of {@link ch.ige.dbutils.handlers.assigners.FieldAssigner} objects to
   *          use for transferring column values into VO fields
   * @return The new VO with its fields set from the result set
   * @throws SQLException if the result set can't be read
   */
  @SuppressWarnings( "unchecked" )
  protected T createVO( ResultSet resultSet, List<? extends FieldAssigner<T, ?>> fieldAssigners ) throws SQLException
  {
    TermGroupVO vo = null;
    if ( resultSet.getObject( "nice_term_number" ) != null )
    {
      vo = new NiceTermGroupVO();
    }
    else if ( resultSet.getObject( "ipi_term_number" ) != null )
    {
      vo = new IpiTermGroupVO();
    }
    else if ( resultSet.getObject( "name" ) != null )
    {
      vo = new StandardWdlTermGroupVO();
    }
    else
    {
      throw new IllegalStateException( "Got unknown term group type" );
    }
    for ( FieldAssigner assigner: fieldAssigners )
    {
      assigner.assign( vo, resultSet );
    }
    try
    {
      // this is unchecked. Is this signature "Object readResolve" needed by the Serialization system, or would
      // "AbstractVO readResolve" work too???
      return (T) vo.readResolve();
    }
    catch ( ObjectStreamException e )
    {
      SQLException sex = new SQLException( "exception when calling readResolve of " + vo );
      sex.initCause( e );
      throw sex;
    }
  }
}
