package ch.ige.wdltool.ejb;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.wdltool.ejb.impl.WdlServicesImpl;

/**
 * The factory for getting a WDL service.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServicesFactory.java,v 1.10 2008/04/21 13:03:08 heinzer Exp $
 */
public class WdlServicesFactory
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( WdlServicesFactory.class );

  /**
   * The error message to use in the generic runtime exception when the ejb lookup fails.
   */
  private static String errorMessage = "Can't initialize the WdlServicesFactory";

  /**
   * Shows whether this is running on a J2EE server
   */
  private static boolean runsInJ2EEMode = false;

  /**
   * Contains the remote WDL services within a J2EE server
   */
  private static Wdl j2eeWdlServices = null;

  /**
   * Contains the local WDL services within a J2EE server
   */
  private static WdlLocal j2eeWdlLocalServices = null;

  /**
   * Contains the standalone WDL (remote and local) services for testing etc.
   */
  private static WdlServicesImpl standaloneWdlServices = null;

  /**
   * Hide this from public access, use {@link #getLocalInstance()} or {@link #getRemoteInstance()} .
   *
   */
  private WdlServicesFactory()
  {
    // Nothing to do
  }

  /**
   * Overrides the error message that is used in the generic runtime exception generated by a failure of the ejb lookup.
   * @param errorMessage The error message to use
   */
  public static void setErrorMessage( String errorMessage )
  {
    WdlServicesFactory.errorMessage = errorMessage;
  }

  /**
   * Gets the WDL local properties, either as a bean (on the J2EE server)
   * or a local class (for local tests).
   * @return The WDL local properties
   */
  public static final WdlLocalServices getLocalInstance()
  {
    return getLocalInstance( false );
  }

  /**
   * Gets the WDL local properties, either as a bean (on the J2EE server)
   * or a local class (for local tests). This supports self referencing lookup.
   * @param selfReference When <code>true</code> the lookup is to be performed from inside the WDL bean (<code>JNDI_NAME</code> used), when <code>false</code> the lookup comes from a different bean (<code>COMP_NAME</code> used).
   * @return The WDL local properties
   */
  public static final WdlLocalServices getLocalInstance( boolean selfReference )
  {
    if ( runsInJ2EEMode )
    {
      if ( j2eeWdlLocalServices == null )
      {
        try
        {
          InitialContext context = new InitialContext();
          String lookupKey;
          if ( selfReference )
          {
            lookupKey = WdlLocalHome.JNDI_NAME;
          }
          else
          {
            lookupKey = WdlLocalHome.COMP_NAME;
          }
          WdlServicesFactory.j2eeWdlLocalServices = ( (WdlLocalHome) context.lookup( lookupKey ) ).create();
          LOG.debug( "WDL local services created for J2EE server" );
        }
        catch ( NamingException e )
        {
          LOG.error( "NamingException while creating context: " + e.getMessage(), e );
          throw new RuntimeException( errorMessage, e );
        }
        catch ( CreateException e )
        {
          LOG.error( "CreateException while creating WDL bean: " + e.getMessage(), e );
          throw new RuntimeException( errorMessage, e );
        }
      }
      return j2eeWdlLocalServices;
    }
    else
    {
      return getStandaloneWdlServices();
    }
  }

  /**
   * Gets the WDL properties, either as a bean (on the J2EE server)
   * or a local class (for local tests).
   * @return The WDL properties
   */
  public static final WdlServices getRemoteInstance()
  {
    if ( runsInJ2EEMode )
    {
      if ( j2eeWdlServices == null )
      {
        try
        {
          InitialContext context = new InitialContext();
          WdlServicesFactory.j2eeWdlServices = ( (WdlHome) context.lookup( WdlHome.JNDI_NAME ) ).create();
          LOG.debug( "WDL remote services created for J2EE server" );
        }
        catch ( NamingException e )
        {
          LOG.error( "NamingException while creating context: " + e.getMessage(), e );
          throw new RuntimeException( errorMessage, e );
        }
        catch ( RemoteException e )
        {
          LOG.error( "RemoteException while creating WDL bean: " + e.getMessage(), e );
          throw new RuntimeException( errorMessage, e );
        }
        catch ( CreateException e )
        {
          LOG.error( "CreateException while creating WDL bean: " + e.getMessage(), e );
          throw new RuntimeException( errorMessage, e );
        }
      }
      return j2eeWdlServices;
    }
    else
    {
      return getStandaloneWdlServices();

    }
  }

  /**
   * Gets the WDL services (remote and local) for standalone use.
   * @return The WDL services
   */
  public static final WdlServicesImpl getStandaloneWdlServices()
  {
    if ( standaloneWdlServices == null )
    {
      standaloneWdlServices = new WdlServicesImpl();
    }
    return standaloneWdlServices;
  }

  /**
   * Sets the WDL services (both remote and local) for use within a J2EE server.
   * This must be called by beans that use this, in <code>ejbCreate()</code>.
   * This must be called by clients that use this in in EJB (not stand alone) mode.
   */
  public static final void setEJBWdlServices()
  {
    runsInJ2EEMode = true;
  }
}
