package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.vo.VOUtils;
import ch.ige.wdltool.dao.GeneralTermDAO;
import ch.ige.wdltool.dao.IpiTermCandidateDAO;
import ch.ige.wdltool.dao.StopwordDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlUpdateException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.validator.Validator;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.ExtendedSearchVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.StopwordVO;

/**
 * Implements services for general terms.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Christian Heinzer
 * @version $Id: GeneralTermImpl.java,v 1.25 2008/12/08 16:04:21 heinzer Exp $
 */
public class GeneralTermImpl extends BaseTermServicesImpl<GeneralTermDAO, AbstractTermVO, Integer>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( GeneralTermImpl.class );

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<GeneralTermDAO> getDAOClass()
  {
    return GeneralTermDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "term";
  }

  /**
   * Finds all active terms of a language that correspond to the exact <code>term</code>.
   * Note that an active term is either a {@link NiceTermVO}, a {@link NegativeIpiTermVO} or a
   * {@link IpiTermVO} where {@link IpiTermVO#isActive()} is <code>true</code>.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param term The term to search for
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the term candidates can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#findTermWithinActiveTerms(java.util.Locale, java.lang.Integer, java.lang.String, java.lang.Boolean)
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( Locale language, Integer classNumber, String term, Boolean isCheckerOnly )
      throws WdlObjectNotFoundException, WrappedRuntimeException
  {
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>();
    try
    {
      terms = DAOPool.get( getDAOClass() ).findTermWithinActiveTerms( language, classNumber, term, isCheckerOnly );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search for term in: " + language + " and class " + classNumber + " with search term: " + term,
          e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    LOG.debug( "searched word: " + term + ", found " + terms.size() );
    return terms;
  }

  /**
   * Finds all active terms of a language that correspond to any word within <code>searchTerm</code>.
   * This splits up <code>searchTerm</code> into separate words and tries to find
   * any of those words in the search terms. For this to work, it simplifies the words in <code>searchTerm</code>.
   * Note that an active term is either a {@link NiceTermVO}, a {@link NegativeIpiTermVO} or a
   * {@link IpiTermVO} where {@link IpiTermVO#isActive()} is <code>true</code>.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param language The language in which terms are searched for; if this is <code>null</code>, all languages are taken
   * @param searchTerm The term to search for, which can contain any number of blank-separated words
   * @param isCheckerOnly This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   * 
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the word can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#findAnyWordWithinActiveTerms(java.util.Locale, java.lang.String)
   */
  public List<AbstractTermVO> findAnyWordWithinActiveTerms( Locale language, String searchTerm, Boolean isCheckerOnly ) throws WdlObjectNotFoundException,
      WrappedRuntimeException
  {
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>();
    try
    {
      if ( language == null )
      {
        terms = DAOPool.get( getDAOClass() ).findAnyWordWithinActiveTerms( searchTerm, isCheckerOnly );
      }
      else
      {
        terms = DAOPool.get( getDAOClass() ).findAnyWordWithinActiveTerms( language, searchTerm, isCheckerOnly );
      }
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search for any word in: " + language + " with search term: " + searchTerm, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    LOG.debug( "searched word: " + searchTerm + ", found " + terms.size() );
    return terms;
  }

  /**
   * Finds all active terms that correspond to the search criteria defined in <code>searchDefinition</code>.
   * <br>Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getTermGroupTerms()}</li>
   * <li>{@link IpiTermVO#getIpiTermGroupNumber()}</li>
   * <li>{@link NiceTermVO#getTermGroupTerms()}</li>
   * <li>{@link NiceTermVO#getNiceTermGroupNumber()}</li>
   * </ul>
   * @param searchDefinition The extended search parameters
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the terms can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#findTermWithinActiveTerms(ExtendedSearchVO)
   */
  public List<AbstractTermVO> findTermWithinActiveTerms( ExtendedSearchVO searchDefinition ) throws WdlObjectNotFoundException, WrappedRuntimeException
  {
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>();
    try
    {
      terms = DAOPool.get( getDAOClass() ).findTermWithinActiveTerms( searchDefinition );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search " + searchDefinition, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    LOG.debug( "Extended search, found " + terms.size() );
    return terms;
  }

  /**
   * Finds all active terms in the given language whose search term match exactly the given term
   * (after it has been transformed into a search term).
   * @param term The term to search for (language and term), which is searched exactly as given, after having been
   *        transformed to a search term
   * @return All active terms whose search term matches <code>term</code> or an empty <code>List</code> if none were found. The term used as input is not returned.
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws WdlObjectNotFoundException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findSameSearchTermWithinActiveTerms(ch.ige.wdltool.vo.AbstractTermVO)
   */
  public List<AbstractTermVO> findSameSearchTermWithinActiveTerms( AbstractTermVO term ) throws WrappedRuntimeException, WdlObjectNotFoundException
  {
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>();
    try
    {
      terms = DAOPool.get( getDAOClass() ).findSameSearchTermWithinActiveTerms( term );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search " + term, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return terms;
  }

  /**
   * Updates the class number for a number of terms.
   * @param terms The terms to update
   * @param classNumber The new class number
   * @throws WdlUpdateException if the terms can't be updated
   * @throws WrappedRuntimeException if any argument is wrong
   * @throws WdlValidationException if the new class number doesn't match the term group number
   * @see ch.ige.wdltool.ejb.WdlServices#updateClassNumber(java.util.List, java.lang.Integer)
   */
  public void updateClassNumber( List<AbstractTermVO> terms, Integer classNumber ) throws WdlUpdateException, WrappedRuntimeException, WdlValidationException
  {
    String uniqueIds = StringUtils.join( VOUtils.extractIDs( terms ).iterator(), ", " );
    String errorMessage = "Couldn't update terms with IDs " + uniqueIds + " with class number " + classNumber;

    try
    {
      IpiTermCandidateDAO ipiTermCandidateDAO = DAOPool.get( IpiTermCandidateDAO.class );
      for ( AbstractTermVO term: terms )
      {
        term.setClassNumber( classNumber );
        Validator.validate( term, true );
        List<IpiTermCandidateVO> candidates = ipiTermCandidateDAO.find( term.getLanguage(), classNumber, term.getTerm() );
        if ( !candidates.isEmpty() )
        {
          throw new WdlUpdateException( "Can't update to class number when same term candidate exists: " + candidates.get( 0 ), null, term );
        }
      }

      DAOPool.get( getDAOClass() ).updateClassNumber( terms, classNumber );
    }
    catch ( UnitOfWorkException e )
    {
      ExceptionThrower.throwUpdateException( errorMessage, e, null );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwUpdateException( errorMessage, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.DAO, java.util.Collection)
  @Override
  protected void delete( GeneralTermDAO dao, Collection<Integer> ids )
  {
    dao.delete( ids );
  }

  /**
   * Checks whether a term already exists as stopword. This is used to prevent wrong entries when inserting new terms.
   * 
   * @param term The term to check
   * @throws WdlValidationException if the term already exists as stopword
   */
  static void checkForExistingStopword( AbstractTermVO term ) throws WdlValidationException
  {
    try
    {
      StopwordDAO stopwordDAO = DAOPool.get( StopwordDAO.class );
      StopwordVO existingStopword = stopwordDAO.find( term.getLanguage(), term.getTerm() );
      if ( existingStopword != null )
      {
        Validator.throwWdlValidationException( Validator.MESSAGE_EXISTING_STOPWORD, term, "term", term.getTerm() );
      }
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwSQLException( "Couldn't find stopword", e );
    }
  }
}
