package ch.ige.wdltool.exception;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.Collection;

import javax.ejb.EJBException;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.vo.AbstractVO;

/**
 * Wraps and throws Exceptions.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ExceptionThrower.java,v 1.6 2008/02/05 13:59:42 lewis_lee Exp $
 */
public class ExceptionThrower
{
  /**
   * Creates an Exception to throw to the JEE container,
   * which forces a transaction rollback.
   * <strong>Use this only when you know what you are doing!</strong>
   * Normally, the other application Exceptions in this class should be thrown.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @return An Exception to throw to the JEE container
   */
  public static EJBException createEJBException( String errorMessage, Exception e )
  {
    return new EJBException( errorMessage, e );
  }

  /**
   * Creates an {@link EJBException} from an {@link SQLException},
   * based on the idea that any expected DB problem <strong>does not</strong> throw
   * an SQLException. This is only used for system errors!
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @throws EJBException The system-level exception
   */
  public static void throwSQLException( String errorMessage, SQLException e ) throws EJBException
  {
    throw new EJBException( errorMessage, e );
  }

  /**
   * Throws an exception used whenever an object can't be created.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @param valueObject The value object that should have been inserted
   * @throws WdlCreateException showing that the object can't be inserted
   */
  public static void throwCreateException( String errorMessage, Exception e, AbstractVO valueObject ) throws WdlCreateException
  {
    throw new WdlCreateException( errorMessage, e, valueObject );
  }

  /**
   * Throws an exception used whenever an object can't be updated.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @param valueObject The value object that should have been updated
   * @throws WdlUpdateException showing that the object can't be updated
   */
  public static void throwUpdateException( String errorMessage, Exception e, AbstractVO valueObject ) throws WdlUpdateException
  {
    throw new WdlUpdateException( errorMessage, e, valueObject );
  }

  /**
   * Creates an exception used whenever an object isn't found.
   * @param errorMessage The error message to show
   * @param identifier The identifier used to find the object
   * @throws WdlObjectNotFoundException showing that the object can't be found
   */
  public static void throwObjectNotFoundException( String errorMessage, Serializable identifier ) throws WdlObjectNotFoundException
  {
    throw new WdlObjectNotFoundException( errorMessage, null, identifier );
  }

  /**
   * Throws an exception used whenever an object isn't found.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @param identifier The identifier used to find the object
   * @throws WdlObjectNotFoundException showing that the object can't be found
   */
  public static void throwObjectNotFoundException( String errorMessage, Exception e, Serializable identifier ) throws WdlObjectNotFoundException
  {
    throw new WdlObjectNotFoundException( errorMessage, e, identifier );
  }

  /**
   * Throws an exception used whenever an object can't be deleted.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @param identifier The identifier used to delete the object
   * @throws WdlRemoveException showing that the object can't be removed
   */
  public static void throwRemoveException( String errorMessage, Exception e, Serializable identifier ) throws WdlRemoveException
  {
    throw new WdlRemoveException( errorMessage, e, identifier );
  }

  /**
   * Throws an exception used whenever a collection of objects can't be deleted.
   * @param errorMessage The error message to show
   * @param e The original Exception
   * @param identifiers The identifiers used to delete the object
   * @throws WdlRemoveException showing that the object can't be removed
   */
  public static void throwRemoveException( String errorMessage, Exception e, Collection<? extends Serializable> identifiers ) throws WdlRemoveException
  {
    throw new WdlRemoveException( errorMessage, e, identifiers );
  }

  /**
   * Converts an unchecked Exception into a checked: Some <code>RuntimeException</code>s are expected, but the
   * container logs them as error. So this converts it and lets the caller handle the problem.
   * @param errorMessage The error message to show
   * @param e The causing Exception
   * @throws WrappedRuntimeException if <code>e</code> is of a known type; otherwise this just rethrows <code>e</code>
   */
  public static void throwRuntimeException( String errorMessage, RuntimeException e ) throws WrappedRuntimeException
  {
    if ( e instanceof IllegalArgumentException )
    {
      throw new WrappedIllegalArgumentException( errorMessage, (IllegalArgumentException) e );
    }
    else if ( e instanceof UnitOfWorkException )
    {
      throw new WrappedUnitOfWorkException( errorMessage, (UnitOfWorkException) e );
    }
    else
    {
      throw e;
    }
  }
}
