package ch.ige.wdltool.text;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.lang.StringUtils;

import ch.ige.util.validation.Check;

/**
 * Converts special characters to upper case characters and vice versa.
 * For instance, a French <code>é</code> is written in upper case as <code>E</code>, but
 * can also be accepted as <code>É</code>.
 * On the other hand, an upper case <code>E</code> can be one of the following lower case
 * characters: <code>e, é, è, ê, ë</code>
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: UpperCaseCharacterConverter.java,v 1.5 2008/03/31 09:58:23 lewis_lee Exp $
 */
@SuppressWarnings( "unchecked" )
public class UpperCaseCharacterConverter
{
  /**
   * The language to use
   */
  private Locale language;

  /**
   * Shows the relationship between a lower case special French character and its
   * upper case counterparts. Any lower case character can have several upper case
   * counterparts, so as long as a character in this array is upper case, it belongs
   * to the preceding lower case character.
   * Don't forget to include the regular characters (like <code>a</code>, <code>e</code> etc.)!
   */
  private static final char[] FRENCH_UPPER_CASE_CHARACTERS = { 'a', 'A', 'à', 'À', 'A', 'â', 'Â', 'A', 'c', 'C', 'ç', 'Ç', 'C', 'e', 'E', 'é', 'É', 'E', 'è',
      'È', 'E', 'ê', 'Ê', 'E', 'ë', 'Ë', 'E', 'i', 'I', 'î', 'Î', 'I', 'ï', 'Ï', 'I', 'o', 'O', 'ô', 'Ô', 'O', 'u', 'U', 'ù', 'Ù', 'U', 'û', 'Û', 'U', 'ü',
      'Ü', 'U', 'y', 'Y', 'ÿ', 'Ÿ', 'Y' };

  /**
   * Maps French lower case characters to their upper case counterpart. 
   */
  private static Map<Character, List<Character>> FRENCH_LOWER_CASE_MAP;

  /**
   * Maps French upper case characters to their lower case counterpart. 
   */
  private static Map<Character, List<Character>> FRENCH_UPPER_CASE_MAP;

  static
  {
    Map<Character, List<Character>> frenchLowerCaseMap = new TreeMap<Character, List<Character>>();
    Map<Character, List<Character>> frenchUpperCaseMap = new TreeMap<Character, List<Character>>();

    char previousCharacter = 'X';
    char currentLowerCaseCharacter = 'x';
    for ( char character: FRENCH_UPPER_CASE_CHARACTERS )
    {
      Character theCharacter = Character.valueOf( character );
      boolean isLowerCase = Character.isLowerCase( character );

      if ( isLowerCase )
      {
        currentLowerCaseCharacter = character;
      }

      // Create Map entry for every new upper case or lower case character
      if ( isLowerCase && Character.isUpperCase( previousCharacter ) )
      {
        frenchLowerCaseMap.put( theCharacter, new ArrayList<Character>() );
      }
      if ( !isLowerCase && !frenchUpperCaseMap.containsKey( theCharacter ) )
      {
        frenchUpperCaseMap.put( theCharacter, new ArrayList<Character>() );
      }

      // Create new entries
      if ( !isLowerCase )
      {
        frenchUpperCaseMap.get( theCharacter ).add( Character.valueOf( currentLowerCaseCharacter ) );
        frenchLowerCaseMap.get( currentLowerCaseCharacter ).add( theCharacter );
      }

      previousCharacter = character;
    }

    FRENCH_LOWER_CASE_MAP = Collections.unmodifiableMap( frenchLowerCaseMap );
    FRENCH_UPPER_CASE_MAP = Collections.unmodifiableMap( frenchUpperCaseMap );
  }

  /**
   * Creates this for a language.
   * @param language The language to use
   */
  public UpperCaseCharacterConverter( Locale language )
  {
    Check.notNull( language, "language" );
    this.language = language;
  }

  /**
   * Converts the first character of a String:
   * If the character is upper case, this returns the possible
   * lower case characters; if the character is lower case, this
   * returns the possible upper case characters.
   * @param text The text to check
   * @param includeNormalCaseSwitch If this is <code>true</code>, the normal upper case
   *        character according to Java (for instance <code>É</code> corresponding to <code>é</code>)
   *        is also included; if this is <code>false</code>, only the special upper case
   *        character is included (for instance <code>E</code> corresponding to <code>é</code>)
   * @return <code>null</code> if the first character doesn't have a special character
   *         logic behind it; otherwise all the characters that correspond to the first character
   */
  public List<Character> firstCharacterConverter( String text, boolean includeNormalCaseSwitch )
  {
    if ( StringUtils.isEmpty( text ) || !this.language.equals( Locale.FRENCH ) )
    {
      return null;
    }
    else
    {
      Character firstCharacter = Character.valueOf( text.charAt( 0 ) );
      Character normalSwitchedCaseCharacter = null;
      List<Character> characters = null;
      if ( Character.isUpperCase( firstCharacter ) )
      {
        normalSwitchedCaseCharacter = new Character( Character.toLowerCase( firstCharacter ) );
        characters = FRENCH_UPPER_CASE_MAP.get( firstCharacter );
      }
      else
      {
        normalSwitchedCaseCharacter = new Character( Character.toUpperCase( firstCharacter ) );
        characters = FRENCH_LOWER_CASE_MAP.get( firstCharacter );
      }
      if ( characters == null )
      {
        return null;
      }
      else
      {
        List<Character> usableCharacters = new ArrayList<Character>( characters );
        if ( !includeNormalCaseSwitch && usableCharacters.contains( normalSwitchedCaseCharacter ) )
        {
          usableCharacters.remove( normalSwitchedCaseCharacter );
        }
        if ( usableCharacters.isEmpty() )
        {
          return null;
        }
        else
        {
          return usableCharacters;
        }
      }
    }
  }
}
