package ch.ige.wdltool.util;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.wdltool.dao.ConfigurationDAO;
import ch.ige.wdltool.dao.DataSourceDAO;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.vo.DataSourceVO;

/**
 * Tests {@link ch.ige.wdltool.util.Configuration}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author heinzer
 * @author Eric Lewis
 * @version  $Id: ConfigurationTest.java,v 1.10 2008/07/15 07:29:38 lewis_lee Exp $
 */
public class ConfigurationTest extends BaseDAOTestTemplate<DataSourceDAO, DataSourceVO>
{
  /**
   * The attribute for the term validity period
   */
  private static final String TERM_VALIDITY_PERIOD = "termValidityPeriod";

  /**
   * The attribute for the term confirmation period
   */
  private static final String TERM_CONFIRMATION_PERIOD = "termConfirmationPeriod";

  /**
   * The attribute for the checker log directory
   */
  private static final String CHECKER_LOG_DIRECTORY = "checkerLogDirectory";

  /**
   * The attribute for the cleaner log directory
   */
  private static final String CLEANER_LOG_DIRECTORY = "cleanerLogDirectory";

  /**
   * The attribute for the analyzer log directory
   */
  private static final String ANALYZER_LOG_DIRECTORY = "analyzerLogDirectory";

  /**
   * Contains all configuration keys, used for clearing System properties during {@link #tearDown()}.
   */
  private static final String[] ALL_KEYS = { Configuration.ANALYZER_LOG_DIRECTORY, Configuration.CHECKER_LOG_DIRECTORY, Configuration.CLEANER_LOG_DIRECTORY,
      Configuration.CREATE_LINKS_FOR_CORRECT_WORDS, Configuration.getDataSourceKey( DataSource.IPI, Configuration.DATA_SOURCE_CLEANER_ACTIVE ),
      Configuration.getDataSourceKey( DataSource.IPI, Configuration.DATA_SOURCE_TERM_CONFIRMATION_PERIOD ),
      Configuration.getDataSourceKey( DataSource.IPI, Configuration.DATA_SOURCE_TERM_VALIDITY_PERIOD ) };

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( ConfigurationTest.class );

  /**
   * If a key must be inserted, it must also be deleted during {@link #tearDown()}.
   */
  private String keyToDelete;

  /**
   * The old value to backup and restore during {@link #tearDown()}
   */
  private Object valueBackup;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public ConfigurationTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getAnalyzerLogDirectory()}.
   * @throws Exception if the test fails
   */
  public void testGetAnalyzerLogDirectory() throws Exception
  {
    checkGetDirectory( ANALYZER_LOG_DIRECTORY, Configuration.ANALYZER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#setAnalyzerLogDirectory(File)}.
   * @throws Exception if the test fails
   */
  public void testSetAnalyzerLogDirectory() throws Exception
  {
    checkSetDirectory( ANALYZER_LOG_DIRECTORY, Configuration.ANALYZER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getNewAnalyzerLogFileName()}.
   * @throws Exception if the test fails
   */
  public void testGetNewAnalyzerLogFileName() throws Exception
  {
    File newLogFile = Configuration.getNewAnalyzerLogFileName();
    assertNotNull( "Checking a new log file exists", newLogFile );
    assertTrue( "Checking path", newLogFile.toString().contains( Configuration.getAnalyzerLogDirectory().toString() ) );
    assertTrue( "Checking file name", newLogFile.toString().contains( "_analyse_" ) );
    assertTrue( "Checking file type", newLogFile.toString().endsWith( ".txt" ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getCheckerLogDirectory()}.
   * @throws Exception if the test fails
   */
  public void testGetCheckerLogDirectory() throws Exception
  {
    checkGetDirectory( CHECKER_LOG_DIRECTORY, Configuration.CHECKER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#setCheckerLogDirectory(File)}.
   * @throws Exception if the test fails
   */
  public void testSetCheckerLogDirectory() throws Exception
  {
    checkSetDirectory( CHECKER_LOG_DIRECTORY, Configuration.CHECKER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getNewCheckerLogFileName()}.
   * @throws Exception if the test fails
   */
  public void testGetNewCheckerLogFileName() throws Exception
  {
    File newLogFile = Configuration.getNewCheckerLogFileName();
    assertNotNull( "Checking a new log file exists", newLogFile );
    assertTrue( "Checking path", newLogFile.toString().contains( Configuration.getCheckerLogDirectory().toString() ) );
    assertTrue( "Checking file name", newLogFile.toString().contains( "_checker_" ) );
    assertTrue( "Checking file type", newLogFile.toString().endsWith( ".txt" ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getNewCheckerSummaryFileName()}.
   * @throws Exception if the test fails
   */
  public void testGetNewCheckerSummaryFileName() throws Exception
  {
    File newLogFile = Configuration.getNewCheckerSummaryFileName();
    assertNotNull( "Checking a new log file exists", newLogFile );
    assertTrue( "Checking path", newLogFile.toString().contains( Configuration.getCheckerLogDirectory().toString() ) );
    assertTrue( "Checking file name", newLogFile.toString().contains( "_checker_summary_" ) );
    assertTrue( "Checking file type", newLogFile.toString().endsWith( ".csv" ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getCleanerLogDirectory()}.
   * @throws Exception if the test fails
   */
  public void testGetCleanerLogDirectory() throws Exception
  {
    checkGetDirectory( CLEANER_LOG_DIRECTORY, Configuration.CLEANER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#setCleanerLogDirectory(File)}.
   * @throws Exception if the test fails
   */
  public void testSetCleanerLogDirectory() throws Exception
  {
    checkSetDirectory( CLEANER_LOG_DIRECTORY, Configuration.CLEANER_LOG_DIRECTORY );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#getNewCleanerLogFileName()}.
   * @throws Exception if the test fails
   */
  public void testGetNewCleanerLogFileName() throws Exception
  {
    File newLogFile = Configuration.getNewCleanerLogFileName();
    assertNotNull( "Checking a new log file exists", newLogFile );
    assertTrue( "Checking path", newLogFile.toString().contains( Configuration.getCleanerLogDirectory().toString() ) );
    assertTrue( "Checking file name", newLogFile.toString().contains( "_cleaner_" ) );
    assertTrue( "Checking file type", newLogFile.toString().endsWith( ".txt" ) );
  }

  /**
   * Tests {@link Configuration#getTermConfirmationPeriod(DataSource)}.
   * @throws Exception if the test fails
   */
  public void testGetTermConfirmationPeriod() throws Exception
  {
    //as the configuration in db may change we have to read the current configuration (=expected value while no overriding system-property is set)
    DataSourceVO ipiDataSource = DAOPool.get( DataSourceDAO.class ).find( DataSource.IPI );
    checkGetPeriod( TERM_CONFIRMATION_PERIOD, Configuration.DATA_SOURCE_TERM_CONFIRMATION_PERIOD, ipiDataSource.getTermConfirmationPeriod() );
  }

  /**
   * Tests {@link Configuration#setTermConfirmationPeriod(DataSource, Integer)}.
   * @throws Exception if the test fails
   */
  public void testSetTermConfirmationPeriod() throws Exception
  {
    checkSetPeriod( TERM_CONFIRMATION_PERIOD );
  }

  /**
   * Tests {@link Configuration#getTermValidityPeriod(DataSource)}.
   * @throws Exception if the test fails
   */
  public void testGetTermValidityPeriod() throws Exception
  {
    //as the configuration in db may change we have to read the current configuration (=expected value while no overriding system-property is set)
    DataSourceVO ipiDataSource = DAOPool.get( DataSourceDAO.class ).find( DataSource.IPI );
    checkGetPeriod( TERM_VALIDITY_PERIOD, Configuration.DATA_SOURCE_TERM_VALIDITY_PERIOD, ipiDataSource.getTermValidityPeriod() );
  }

  /**
   * Tests {@link Configuration#setTermValidityPeriod(DataSource, Integer)}.
   * @throws Exception if the test fails
   */
  public void testSetTermValidityPeriod() throws Exception
  {
    checkSetPeriod( TERM_VALIDITY_PERIOD );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#createLinksForCorrectWords()}.
   */
  public void testCreateLinksForCorrectWords()
  {
    assertNotNull( "Checking links for correct words", Configuration.createLinksForCorrectWords() );

    System.setProperty( Configuration.CREATE_LINKS_FOR_CORRECT_WORDS, "false" );
    Configuration.readConfig(); // Normally, this isn't necessary, since System Properties are set before Configuration is created
    assertEquals( "Checking overridden links for correct words", false, Configuration.createLinksForCorrectWords().booleanValue() );

    System.setProperty( Configuration.CREATE_LINKS_FOR_CORRECT_WORDS, "true" );
    Configuration.readConfig();
    assertEquals( "Checking overridden links for correct words", true, Configuration.createLinksForCorrectWords().booleanValue() );

    // Clear the system property, since it's permanent and could influence other tests.
    System.setProperty( Configuration.CREATE_LINKS_FOR_CORRECT_WORDS, "" );
  }

  /**
   * Tests {@link ch.ige.wdltool.util.Configuration#setCreateLinksForCorrectWords(Boolean)}.
   */
  public void testSetCreateLinksForCorrectWords()
  {
    Boolean currentValue = Configuration.createLinksForCorrectWords();
    assertNotNull( "Checking links for correct words", currentValue );

    Configuration.setCreateLinksForCorrectWords( Boolean.FALSE );
    assertEquals( "Checking changed links for correct words", false, Configuration.createLinksForCorrectWords().booleanValue() );

    Configuration.setCreateLinksForCorrectWords( Boolean.TRUE );
    assertEquals( "Checking changed links for correct words", true, Configuration.createLinksForCorrectWords().booleanValue() );

    Configuration.setCreateLinksForCorrectWords( currentValue );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<DataSourceDAO> getDAOClass()
  {
    return DataSourceDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected boolean assertVOEqual( DataSourceVO originalVO, DataSourceVO serializedVO )
  {
    // Don't bother...
    return true;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected DataSourceVO getSerializableVO()
  {
    // Don't bother...
    return new DataSourceVO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    clearSystemPropertiesAndReloadConfiguration();
    if ( this.keyToDelete != null )
    {
      new ConfigurationDAO().delete( this.keyToDelete );
    }
  }

  /**
   * Does what it says...  ;-)
   */
  private void clearSystemPropertiesAndReloadConfiguration()
  {
    for ( String key: ALL_KEYS )
    {
      System.clearProperty( key );
    }
    Configuration.readConfig();
  }

  /**
   * Checks getting a directory.
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @param attributeKey The key used for the attribute, for instance {@link Configuration#ANALYZER_LOG_DIRECTORY} 
   * @throws Exception if the check fails
   */
  private void checkGetDirectory( String attribute, String attributeKey ) throws Exception
  {
    if ( getDirectory( attribute ) == null )
    {
      // Fake it if it doesn't exist, and delete it at the end
      setDirectory( attribute, new File( System.getProperty( "java.io.tmpdir" ) ) );
      this.keyToDelete = attributeKey;
    }

    File directory = getDirectory( attribute );
    assertTrue( "Checking " + directory + " is a directory", directory.isDirectory() );
    assertTrue( "Checking " + directory + " exists", directory.exists() );

    String newValue = "gugus3";
    System.setProperty( attributeKey, newValue );
    Configuration.readConfig(); // Normally, this isn't necessary, since System Properties are set before Configuration is created
    assertEquals( "Checking system property", newValue, System.getProperty( attributeKey ) );
    directory = getDirectory( attribute );
    assertEquals( "Checking the system property overrides", newValue, directory.toString() );
    assertTrue( "Checking it's not a directory", !directory.isDirectory() );
    assertTrue( "Checking it doesn't exists", !directory.exists() );
  }

  /**
   * Checks setting a directory.
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @param attributeKey The key used for the attribute, for instance {@link Configuration#ANALYZER_LOG_DIRECTORY} 
   * @throws Exception if the check fails
   */
  private void checkSetDirectory( String attribute, String attributeKey ) throws Exception
  {
    this.valueBackup = getDirectory( attribute );
    LOG.info( "Old directory: " + this.valueBackup );

    try
    {
      setDirectory( attribute, null );
      fail( "Null directory must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    String filename = "\\\\this\\is\\a\\test";
    File directory = new File( filename );
    setDirectory( attribute, directory );

    File newDirectory = getDirectory( attribute );
    assertNotNull( "Checking that configuration exists", newDirectory );
    assertEquals( "Checking directory", filename, newDirectory.toString() );

    if ( this.valueBackup != null )
    {
      LOG.info( "Restoring old configuration value" );
      setDirectory( attribute, (File) this.valueBackup );
    }
    else
    {
      this.keyToDelete = attributeKey;
    }
  }

  /**
   * Tests getting a period value.
   * @param attribute The attribute name, for instance <code>termConfirmationPeriod</code> 
   * @param attributeKey The key used for the attribute, for instance {@link Configuration#DATA_SOURCE_TERM_CONFIRMATION_PERIOD} 
   * @param expectedIpiValue The value to expect for the IPI datasource
   * @throws Exception if the check fails
   */
  private void checkGetPeriod( String attribute, String attributeKey, int expectedIpiValue ) throws Exception
  {
    try
    {
      getDataSourceInteger( null, attribute );
      fail( "Null data source must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDataSourceInteger( DataSource.NICE, attribute );
      fail( "Nice data source must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDataSourceInteger( DataSource.WIPO, attribute );
      fail( "WIPO data source must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    assertEquals( "Checking for IPI", expectedIpiValue, getDataSourceInteger( DataSource.IPI, attribute ).intValue() );

    // Override with system property
    int newValue = 8888;
    System.setProperty( Configuration.getDataSourceKey( DataSource.IPI, attributeKey ), Integer.toString( newValue ) );
    Configuration.readConfig(); // Normally, this isn't necessary, since System Properties are set before Configuration is created
    assertEquals( "Checking for IPI", newValue, getDataSourceInteger( DataSource.IPI, attribute ).intValue() );
  }

  /**
   * Tests setting a period.
   * @param attribute The attribute name, for instance <code>termConfirmationPeriod</code> 
   * @throws Exception if the check fails
   */
  public void checkSetPeriod( String attribute ) throws Exception
  {
    try
    {
      setDataSourceInteger( DataSource.NICE, attribute, 33 );
      fail( "Nice data source must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      setDataSourceInteger( DataSource.WIPO, attribute, 44 );
      fail( "WIPO data source must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      setDataSourceInteger( DataSource.IPI, attribute, null );
      fail( "Null value must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      setDataSourceInteger( DataSource.IPI, attribute, -1 );
      fail( "Negative value must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      setDataSourceInteger( DataSource.IPI, attribute, 0 );
      fail( "Zero value must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    this.valueBackup = getDataSourceInteger( DataSource.IPI, attribute );
    LOG.info( "Old period: " + this.valueBackup );

    Integer newPeriod = 9999;
    setDataSourceInteger( DataSource.IPI, attribute, newPeriod );
    Integer reloadedPeriod = getDataSourceInteger( DataSource.IPI, attribute );
    assertEquals( "Checking for updated configuration", newPeriod, reloadedPeriod );

    if ( this.valueBackup != null )
    {
      LOG.debug( "Restoring old period" );
      setDataSourceInteger( DataSource.IPI, attribute, (Integer) this.valueBackup );
    }
  }

  /**
   * Gets a directory with the given attribute name.
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @return The configured directory
   * @throws Exception if the method call fails
   */
  private File getDirectory( String attribute ) throws Exception
  {
    String getterName = "get" + StringUtils.capitalize( attribute );
    Method getterMethod = Configuration.class.getMethod( getterName );
    try
    {
      return (File) getterMethod.invoke( null );
    }
    catch ( InvocationTargetException e )
    {
      throw (Exception) e.getCause();
    }
  }

  /**
   * Sets a directory with the given attribute name.
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @param directory The directory
   * @throws Exception if the method call fails
   */
  private void setDirectory( String attribute, File directory ) throws Exception
  {
    String setterName = "set" + StringUtils.capitalize( attribute );
    Method setterMethod = Configuration.class.getMethod( setterName, File.class );
    try
    {
      setterMethod.invoke( null, directory );
    }
    catch ( InvocationTargetException e )
    {
      throw (Exception) e.getCause();
    }
  }

  /**
   * Gets an Integer with the given attribute name for a DataSource.
   * @param dataSource The data source whose configuration is fetched
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @return The Integer
   * @throws Exception if the method call fails
   */
  private Integer getDataSourceInteger( DataSource dataSource, String attribute ) throws Exception
  {
    String getterName = "get" + StringUtils.capitalize( attribute );
    Method getterMethod = Configuration.class.getMethod( getterName, DataSource.class );
    try
    {
      return (Integer) getterMethod.invoke( null, dataSource );
    }
    catch ( InvocationTargetException e )
    {
      throw (Exception) e.getCause();
    }
  }

  /**
   * Sets an Integer with the given attribute name.
   * @param dataSource The data source whose configuration is set
   * @param attribute The attribute name, for instance <code>analyzerLogDirectory</code> 
   * @param value The Integer value
   * @throws Exception if the method call fails
   */
  private void setDataSourceInteger( DataSource dataSource, String attribute, Integer value ) throws Exception
  {
    String setterName = "set" + StringUtils.capitalize( attribute );
    Method setterMethod = Configuration.class.getMethod( setterName, DataSource.class, Integer.class );
    try
    {
      setterMethod.invoke( null, dataSource, value );
    }
    catch ( InvocationTargetException e )
    {
      throw (Exception) e.getCause();
    }
  }
}
