package ch.ige.wdltool.validator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.hibernate.validator.ClassValidator;
import org.hibernate.validator.InvalidValue;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.vo.NiceTermVO;

/**
 * Validates all value objects.
 * Note: According to some bug reports, the Hibernate Validator executes <strong>all</strong>
 * validation annotations, starting with the base class. If the base class and the overriding
 * class contradict themselves, it has to be corrected in a special case.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: Validator.java,v 1.15 2008/05/08 10:46:33 lewis_lee Exp $
 */
public class Validator
{
  /**
   * The message used when a property must be null
   */
  public static final String MESSAGE_NULL = "validator.null";

  /**
   * The message used when a property must be not null
   */
  public static final String MESSAGE_NOT_NULL = "validator.notNull";

  /**
   * The message used when a property must be false
   */
  public static final String MESSAGE_FALSE = "validator.assertFalse";

  /**
   * The message used when a term already exists as stopword
   */
  public static final String MESSAGE_EXISTING_STOPWORD = "validator.existingStopword";

  /**
   * The message used when a term already exists
   */
  public static final String MESSAGE_EXISTING_TERM = "validator.existingTerm";

  /**
   * The message used when a term group number is invalid
   */
  public static final String MESSAGE_INVALID_TERM_GROUP_NUMBER = "validator.invalidTermGroupNumber";

  /**
   * Validates the class block
   */
  private static Map<Class<? extends AbstractVO>, ClassValidator<? extends AbstractVO>> validators = new HashMap<Class<? extends AbstractVO>, ClassValidator<? extends AbstractVO>>();

  /**
   * Gets the class validator for a VO class.
   * @param <V> The VO type
   * @param voClass The VO class
   * @return The class validator for the VO class
   */
  @SuppressWarnings( "unchecked" )
  static <V extends AbstractVO> ClassValidator<V> getValidator( Class<V> voClass )
  {
    if ( !validators.containsKey( voClass ) )
    {
      synchronized ( validators )
      {
        validators.put( voClass, new ClassValidator<V>( voClass ) );
      }
    }
    return (ClassValidator<V>) validators.get( voClass );
  }

  /**
   * Validates a value object.
   * @param valueObject The value object to validate
   * @param checkIdentifierExists If this is <code>true</code> and the value object is an
   *        {@link ch.ige.dbutils.vo.IdentifiableVO}, then its identifier must not be <code>null</code>
   * @throws WdlValidationException if the value object is invalid
   */
  @SuppressWarnings( "unchecked" )
  public static void validate( AbstractVO valueObject, boolean checkIdentifierExists ) throws WdlValidationException
  {
    Check.notNull( valueObject, "value object" );
    ClassValidator validator = getValidator( valueObject.getClass() );
    // Note: Arrays.asList returns a List that can't be changed
    List<InvalidValue> invalidValues = new ArrayList<InvalidValue>( Arrays.asList( validator.getInvalidValues( valueObject ) ) );

    // Since the identifier should have be @NotNull,
    // this checks whether according to the flag 'checkIdentifierExists'
    // the identifier should exist at all. If it doesn't need to exist,
    // the corresponding InvalidValue is wrong and thus removed.
    if ( !checkIdentifierExists )
    {
      if ( valueObject instanceof IdentifiableVO )
      {
        Iterator<InvalidValue> iter = invalidValues.iterator();
        while ( iter.hasNext() )
        {
          InvalidValue invalidValue = iter.next();
          // Check that the message really points to a 'not null' error
          if ( invalidValue.getPropertyName().equals( "identifier" ) && invalidValue.getMessage().equals( MESSAGE_NOT_NULL ) )
          {
            iter.remove();
          }

        }
      }
    }
    if ( !invalidValues.isEmpty() )
    {
      WdlValidationException exception = new WdlValidationException();
      exception.setValueObject( valueObject );
      exception.setInvalidValues( invalidValues );
      throw exception;
    }

    // Special validators, where member variables depend on each other
    if ( valueObject instanceof NiceTermVO )
    {
      validateTermGroupNumber( (NiceTermVO) valueObject );
    }
  }

  /**
   * Creates a validation exception and throws it, for all cases
   * when a simple validator annotation won't do.
   * @param messageKey The message key to use
   * @param valueObject The faulty value object
   * @param propertyName The name of the faulty property
   * @param value The faulty property value
   * @throws WdlValidationException always!
   */
  public static void throwWdlValidationException( String messageKey, AbstractVO valueObject, String propertyName, Object value ) throws WdlValidationException
  {
    List<InvalidValue> invalidValues = new ArrayList<InvalidValue>( 1 );
    invalidValues.add( new InvalidValue( messageKey, valueObject.getClass(), propertyName, value, valueObject ) );

    WdlValidationException exception = new WdlValidationException();
    exception.setValueObject( valueObject );
    exception.setInvalidValues( invalidValues );

    throw exception;
  }

  /**
   * Validates a Nice term's term group number.
   * @param niceTerm The Nice term to validate
   * @throws WdlValidationException if the Nice term group number is invalid
   */
  static void validateTermGroupNumber( NiceTermVO niceTerm ) throws WdlValidationException
  {
    boolean termGroupIdIsNull = ( niceTerm.getTermGroupId() == null );
    boolean termGroupNumberIsNull = ( niceTerm.getNiceTermGroupNumber() == null );
    if ( ( termGroupIdIsNull && !termGroupNumberIsNull ) || ( !termGroupIdIsNull && termGroupNumberIsNull ) )
    {
      throwWdlValidationExceptionForTermGroupNumber( niceTerm );
    }
    if ( !termGroupNumberIsNull && niceTerm.getClassNumber() == null )
    {
      throwWdlValidationExceptionForTermGroupNumber( niceTerm );
    }

    if ( niceTerm.getClassNumber() != null && niceTerm.getNiceTermGroupNumber() != null )
    {
      int lowerLimit = niceTerm.getClassNumber().intValue() * 10000;
      int upperLimit = lowerLimit + 9999;
      if ( niceTerm.getNiceTermGroupNumber().intValue() < lowerLimit || niceTerm.getNiceTermGroupNumber().intValue() > upperLimit )
      {
        throwWdlValidationExceptionForTermGroupNumber( niceTerm );
      }
    }
  }

  /**
   * Throws the WDL validation exception for a faulty term group number.
   * @param niceTerm The Nice term to validate
   * @throws WdlValidationException always
   */
  static void throwWdlValidationExceptionForTermGroupNumber( NiceTermVO niceTerm ) throws WdlValidationException
  {
    throwWdlValidationException( Validator.MESSAGE_INVALID_TERM_GROUP_NUMBER, niceTerm, "termGroupNumber", niceTerm.getNiceTermGroupNumber() );
  }
}
