package ch.ige.wdltool.vo;

import java.sql.Timestamp;

import org.hibernate.validator.Length;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;
import org.hibernate.validator.Past;
import org.hibernate.validator.Pattern;
import org.hibernate.validator.Range;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.enums.DataSource;

/**
 * A data source, which owns a number of terms.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: DataSourceVO.java,v 1.3 2007/08/09 07:40:08 lewis_lee Exp $
 */
public class DataSourceVO extends AbstractVO implements IdentifiableVO<DataSource>
{
  /**
   * The unique ID for this
   */
  private DataSource dataSourceId;

  /**
   * The data source's name
   */
  private String name;

  /**
   * The number of days that a term is valid
   */
  private Integer termValidityPeriod;

  /**
   * The number of days after which a valid term must be confirmed
   */
  private Integer termConfirmationPeriod;

  /**
   * Shows whether old terms are cleaned
   */
  private boolean isCleanerActive;

  /**
   * The date/time of the latest change
   */
  private Timestamp changeDate;

  // @see ch.ige.dbutils.vo.IdentifiableVO#getIdentifier()
  @NotNull
  public DataSource getIdentifier()
  {
    return this.dataSourceId;
  }

  // @see ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(java.io.Serializable)
  public void setIdentifier( DataSource id )
  {
    this.dataSourceId = id;
  }

  /**
   * Gets the name.
   * @return the name
   */
  @NotEmpty
  @Length(min=1,max=25)
  @Pattern(regex="[^\\s].*") // Must not start with a blank...
  public String getName()
  {
    return this.name;
  }

  /**
   * Sets the name.
   * @param name The name to set
   */
  public void setName( String name )
  {
    this.name = name;
  }

  /**
   * Gets the number of days that a term is valid.
   * @return The number of days that a term is valid
   */
  @Range(min=1,max=9999)
  // The validation also depends on the data source type
  public Integer getTermValidityPeriod()
  {
    return this.termValidityPeriod;
  }

  /**
   * Sets the number of days that a term is valid.
   * @param termValidityPeriod The number of days that a term is valid
   */
  public void setTermValidityPeriod( Integer termValidityPeriod )
  {
    this.termValidityPeriod = termValidityPeriod;
  }

  /**
   * Gets the number of days after which a valid term must be confirmed.
   * @return The number of days after which a valid term must be confirmed
   */
  @Range(min=1,max=9999)
  // The validation also depends on the data source type
  public Integer getTermConfirmationPeriod()
  {
    return this.termConfirmationPeriod;
  }

  /**
   * Sets the number of days after which a valid term must be confirmed.
   * @param termConfirmationPeriod The number of days after which a valid term must be confirmed
   */
  public void setTermConfirmationPeriod( Integer termConfirmationPeriod )
  {
    this.termConfirmationPeriod = termConfirmationPeriod;
  }

  /**
   * Shows whether old terms are cleaned.
   * @return <code>true</code> if old terms are cleaned, otherwise <code>false</code>
   */
  // The validation also depends on the data source type
  public boolean isCleanerActive()
  {
    return this.isCleanerActive;
  }

  /**
   * Sets the flag showing whether old terms are cleaned.
   * @param isCleanerActive The flag showing whether old terms are cleaned
   */
  public void setCleanerActive( boolean isCleanerActive )
  {
    this.isCleanerActive = isCleanerActive;
  }

  /**
   * Gets the date/time of the latest change.
   * @return The date/time of the latest change
   */
  @NotNull
  @Past
  public Timestamp getChangeDate()
  {
    return this.changeDate;
  }

  /**
   * Sets the date/time of the latest change.
   * @param changeDate The date/time of the latest change
   */
  public void setChangeDate( Timestamp changeDate )
  {
    this.changeDate = changeDate;
  }
}
